Package: gdata
Version: 3.0.1
Date: 2024-10-22
Title: Various R Programming Tools for Data Manipulation
Authors@R: c(person(c("Gregory", "R."), "Warnes", role="aut", email="greg@warnes.net"),
             person("Gregor", "Gorjanc", role="aut"),
             person("Arni", "Magnusson", role=c("aut","cre"), email="thisisarni@gmail.com"),
             person("Liviu", "Andronic", role="aut"),
             person("Jim", "Rogers", role="aut"),
             person("Don", "MacQueen", role="aut"),
             person("Ales", "Korosec", role="aut"),
             person("Ben", "Bolker", role="ctb"),
             person("Michael", "Chirico", role="ctb"),
             person("Gabor", "Grothendieck", role="ctb"),
             person("Thomas", "Lumley", role="ctb"),
             person("Brian", "Ripley", role="ctb"),
             person("inoui llc", role="fnd"))
Imports: gtools, methods, stats, utils
Suggests: RUnit
Description: Various R programming tools for data manipulation, including
  medical unit conversions, combining objects, character vector operations,
  factor manipulation, obtaining information about R objects, generating
  fixed-width format files, extracting components of date & time objects,
  operations on columns of data frames, matrix operations, operations on
  vectors, operations on data frames, value of last evaluated expression, and a
  resample() wrapper for sample() that ensures consistent behavior for both
  scalar and vector arguments.
License: GPL-3
URL: https://github.com/r-gregmisc/gdata
BugReports: https://github.com/r-gregmisc/gdata/issues
NeedsCompilation: no
Packaged: 2024-10-22 03:06:44 UTC; arnim
Author: Gregory R. Warnes [aut],
  Gregor Gorjanc [aut],
  Arni Magnusson [aut, cre],
  Liviu Andronic [aut],
  Jim Rogers [aut],
  Don MacQueen [aut],
  Ales Korosec [aut],
  Ben Bolker [ctb],
  Michael Chirico [ctb],
  Gabor Grothendieck [ctb],
  Thomas Lumley [ctb],
  Brian Ripley [ctb],
  inoui llc [fnd]
Maintainer: Arni Magnusson <thisisarni@gmail.com>
Repository: CRAN
Date/Publication: 2024-10-22 04:30:02 UTC
Built: R 4.4.0; ; 2026-07-18 04:58:35 UTC; unix

Package: gdata
Version: 3.0.1
Date: 2024-10-22
Title: Various R Programming Tools for Data Manipulation
Authors@R: c(person(c("Gregory", "R."), "Warnes", role="aut", email="greg@warnes.net"),
             person("Gregor", "Gorjanc", role="aut"),
             person("Arni", "Magnusson", role=c("aut","cre"), email="thisisarni@gmail.com"),
             person("Liviu", "Andronic", role="aut"),
             person("Jim", "Rogers", role="aut"),
             person("Don", "MacQueen", role="aut"),
             person("Ales", "Korosec", role="aut"),
             person("Ben", "Bolker", role="ctb"),
             person("Michael", "Chirico", role="ctb"),
             person("Gabor", "Grothendieck", role="ctb"),
             person("Thomas", "Lumley", role="ctb"),
             person("Brian", "Ripley", role="ctb"),
             person("inoui llc", role="fnd"))
Imports: gtools, methods, stats, utils
Suggests: RUnit
Description: Various R programming tools for data manipulation, including
  medical unit conversions, combining objects, character vector operations,
  factor manipulation, obtaining information about R objects, generating
  fixed-width format files, extracting components of date & time objects,
  operations on columns of data frames, matrix operations, operations on
  vectors, operations on data frames, value of last evaluated expression, and a
  resample() wrapper for sample() that ensures consistent behavior for both
  scalar and vector arguments.
License: GPL-3
URL: https://github.com/r-gregmisc/gdata
BugReports: https://github.com/r-gregmisc/gdata/issues
NeedsCompilation: no
Packaged: 2024-10-22 03:06:44 UTC; arnim
Author: Gregory R. Warnes [aut],
  Gregor Gorjanc [aut],
  Arni Magnusson [aut, cre],
  Liviu Andronic [aut],
  Jim Rogers [aut],
  Don MacQueen [aut],
  Ales Korosec [aut],
  Ben Bolker [ctb],
  Michael Chirico [ctb],
  Gabor Grothendieck [ctb],
  Thomas Lumley [ctb],
  Brian Ripley [ctb],
  inoui llc [fnd]
Maintainer: Arni Magnusson <thisisarni@gmail.com>
Repository: CRAN
Date/Publication: 2024-10-22 04:30:02 UTC

export(.onAttach,
       ans,
       Args,
       bindData,
       case,
       cbindX,
       centerText,
       combine,
       ConvertMedUnits,
       drop.levels,
       duplicated2,
       env,
       first,
       "first<-",
       frameApply,
       interleave,
       is.what,
       keep,
       last,
       "last<-",
       left,
       ll,
       ls.funs,
       lowerTriangle,
       "lowerTriangle<-",
       matchcols,
       nobs, n_obs,
       nPairs,
       mv,
       rename.vars,
       remove.vars,
       reorder.factor,
       resample,
       right,
       startsWith, starts_with,
       trim,
       trimSum,
       unmatrix,
       update.list,
       update.data.frame,
       upperTriangle,
       "upperTriangle<-",
       wideByFactor,
       write.fwf,

       ## Object size stuff
       object_size, as.object_sizes, is.object_sizes, humanReadable, object.size,

       ## getDateTime stuff
       getYear, getMonth, getDay, getHour, getMin, getSec,

       ## mapLevels stuff
       mapLevels, as.levelsMap, as.listLevelsMap, is.levelsMap, is.listLevelsMap, "mapLevels<-",

       ## unknown stuff
       isUnknown, unknownToNA, NAToUnknown
)

importFrom(stats, reorder)
importFrom(stats, nobs)
importFrom(gtools, mixedsort)
importFrom(methods, is, slot, slotNames)
importFrom(utils, data, head, read.table, tail, write.table)

S3method(reorder, factor)

S3method(summary, nPairs)

## drop.levels stuff
S3method(drop.levels, default)
S3method(drop.levels, factor)
S3method(drop.levels, list)
S3method(drop.levels, data.frame)

## getDateTime stuff
S3method(getYear, default)
S3method(getYear, Date)
S3method(getYear, POSIXct)
S3method(getYear, POSIXlt)

S3method(getMonth, default)
S3method(getMonth, Date)
S3method(getMonth, POSIXct)
S3method(getMonth, POSIXlt)

S3method(getDay, default)
S3method(getDay, Date)
S3method(getDay, POSIXct)
S3method(getDay, POSIXlt)

S3method(getHour, default)

S3method(getMin, default)

S3method(getSec, default)

## mapLevels stuff
S3method(mapLevels, default)
S3method(mapLevels, character)
S3method(mapLevels, factor)
S3method(mapLevels, list)
S3method(mapLevels, data.frame)

S3method("mapLevels<-", default)
S3method("mapLevels<-", list)
S3method("mapLevels<-", data.frame)

S3method(print, levelsMap)
S3method(print, listLevelsMap)

S3method("[", levelsMap)
S3method("[", listLevelsMap)

S3method(c, levelsMap)
S3method(c, listLevelsMap)

S3method(unique, levelsMap)
S3method(sort, levelsMap)

## nobs stuff
S3method(nobs, data.frame)
S3method(nobs, default)
S3method(nobs, lm)

## Object size stuff
S3method(c,      object_sizes)
S3method(format, object_sizes)
S3method(print,  object_sizes)

## unknown stuff
S3method(isUnknown, default)
S3method(isUnknown, POSIXlt)
S3method(isUnknown, list)
S3method(isUnknown, data.frame)
S3method(isUnknown, matrix)

S3method(unknownToNA, default)
S3method(unknownToNA, factor)
S3method(unknownToNA, list)
S3method(unknownToNA, data.frame)

S3method(NAToUnknown, default)
S3method(NAToUnknown, factor)
S3method(NAToUnknown, list)
S3method(NAToUnknown, data.frame)

## trim stuff
S3method(trim, character)
S3method(trim, default)
S3method(trim, factor)
S3method(trim, list)
S3method(trim, data.frame)

## left, right
S3method(left,  data.frame)
S3method(left,  matrix)
S3method(right, data.frame)
S3method(right, matrix)

# update methods for list, data.frame
S3method(update, list)
S3method(update, data.frame)

Spreadsheet-XLSC version 0.01
=============================

Spreadsheet::XLSX - Perl extension for reading MS Excel 2007 files;

INSTALLATION

To install this module type the following:

   perl Makefile.PL
   make
   make test
   make install

DEPENDENCIES

This module requires these other modules and libraries:

  Archive::Zip

COPYRIGHT AND LICENCE

Copyright (C) 2008 by Dmitry Ovsyanko

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself, either Perl version 5.8.8 or,
at your option, any later version of Perl 5 you may have available.

NAME
    Archive::Zip - Provide an interface to ZIP archive files.

SYNOPSIS
       # Create a Zip file
       use Archive::Zip qw( :ERROR_CODES :CONSTANTS );
       my $zip = Archive::Zip->new();
       
   # Add a directory
       my $dir_member = $zip->addDirectory( 'dirname/' );
       
   # Add a file from a string with compression
       my $string_member = $zip->addString( 'This is a test', 'stringMember.txt' );
       $string_member->desiredCompressionMethod( COMPRESSION_DEFLATED );
       
   # Add a file from disk
       my $file_member = $zip->addFile( 'xyz.pl', 'AnotherName.pl' );
       
   # Save the Zip file
       unless ( $zip->writeToFileNamed('someZip.zip') == AZ_OK ) {
           die 'write error';
       }
       
   # Read a Zip file
       my $somezip = Archive::Zip->new();
       unless ( $somezip->read( 'someZip.zip' ) == AZ_OK ) {
           die 'read error';
       }
       
   # Change the compression type for a file in the Zip
       my $member = $somezip->memberNamed( 'stringMember.txt' );
       $member->desiredCompressionMethod( COMPRESSION_STORED );
       unless ( $zip->writeToFileNamed( 'someOtherZip.zip' ) == AZ_OK ) {
           die 'write error';
       }

DESCRIPTION
    The Archive::Zip module allows a Perl program to create, manipulate,
    read, and write Zip archive files.

    Zip archives can be created, or you can read from existing zip files.

    Once created, they can be written to files, streams, or strings. Members
    can be added, removed, extracted, replaced, rearranged, and enumerated.
    They can also be renamed or have their dates, comments, or other
    attributes queried or modified. Their data can be compressed or
    uncompressed as needed.

    Members can be created from members in existing Zip files, or from
    existing directories, files, or strings.

    This module uses the Compress::Raw::Zlib library to read and write the
    compressed streams inside the files.

    One can use Archive::Zip::MemberRead to read the zip file archive
    members as if they were files.

  File Naming
    Regardless of what your local file system uses for file naming, names in
    a Zip file are in Unix format (*forward* slashes (/) separating
    directory names, etc.).

    "Archive::Zip" tries to be consistent with file naming conventions, and
    will translate back and forth between native and Zip file names.

    However, it can't guess which format names are in. So two rules control
    what kind of file name you must pass various routines:

    Names of files are in local format.
        "File::Spec" and "File::Basename" are used for various file
        operations. When you're referring to a file on your system, use its
        file naming conventions.

    Names of archive members are in Unix format.
        This applies to every method that refers to an archive member, or
        provides a name for new archive members. The "extract()" methods
        that can take one or two names will convert from local to zip names
        if you call them with a single name.

  Archive::Zip Object Model
  Overview
    Archive::Zip::Archive objects are what you ordinarily deal with. These
    maintain the structure of a zip file, without necessarily holding data.
    When a zip is read from a disk file, the (possibly compressed) data
    still lives in the file, not in memory. Archive members hold information
    about the individual members, but not (usually) the actual member data.
    When the zip is written to a (different) file, the member data is
    compressed or copied as needed. It is possible to make archive members
    whose data is held in a string in memory, but this is not done when a
    zip file is read. Directory members don't have any data.

  Inheritance
      Exporter
       Archive::Zip                            Common base class, has defs.
           Archive::Zip::Archive               A Zip archive.
           Archive::Zip::Member                Abstract superclass for all members.
               Archive::Zip::StringMember      Member made from a string
               Archive::Zip::FileMember        Member made from an external file
                   Archive::Zip::ZipFileMember Member that lives in a zip file
                   Archive::Zip::NewFileMember Member whose data is in a file
               Archive::Zip::DirectoryMember   Member that is a directory

EXPORTS
    :CONSTANTS
        Exports the following constants:

        FA_MSDOS FA_UNIX GPBF_ENCRYPTED_MASK GPBF_DEFLATING_COMPRESSION_MASK
        GPBF_HAS_DATA_DESCRIPTOR_MASK COMPRESSION_STORED
        COMPRESSION_DEFLATED IFA_TEXT_FILE_MASK IFA_TEXT_FILE
        IFA_BINARY_FILE COMPRESSION_LEVEL_NONE COMPRESSION_LEVEL_DEFAULT
        COMPRESSION_LEVEL_FASTEST COMPRESSION_LEVEL_BEST_COMPRESSION

    :MISC_CONSTANTS
        Exports the following constants (only necessary for extending the
        module):

        FA_AMIGA FA_VAX_VMS FA_VM_CMS FA_ATARI_ST FA_OS2_HPFS FA_MACINTOSH
        FA_Z_SYSTEM FA_CPM FA_WINDOWS_NTFS
        GPBF_IMPLODING_8K_SLIDING_DICTIONARY_MASK
        GPBF_IMPLODING_3_SHANNON_FANO_TREES_MASK
        GPBF_IS_COMPRESSED_PATCHED_DATA_MASK COMPRESSION_SHRUNK
        DEFLATING_COMPRESSION_NORMAL DEFLATING_COMPRESSION_MAXIMUM
        DEFLATING_COMPRESSION_FAST DEFLATING_COMPRESSION_SUPER_FAST
        COMPRESSION_REDUCED_1 COMPRESSION_REDUCED_2 COMPRESSION_REDUCED_3
        COMPRESSION_REDUCED_4 COMPRESSION_IMPLODED COMPRESSION_TOKENIZED
        COMPRESSION_DEFLATED_ENHANCED
        COMPRESSION_PKWARE_DATA_COMPRESSION_LIBRARY_IMPLODED

    :ERROR_CODES
        Explained below. Returned from most methods.

        AZ_OK AZ_STREAM_END AZ_ERROR AZ_FORMAT_ERROR AZ_IO_ERROR

ERROR CODES
    Many of the methods in Archive::Zip return error codes. These are
    implemented as inline subroutines, using the "use constant" pragma. They
    can be imported into your namespace using the ":ERROR_CODES" tag:

      use Archive::Zip qw( :ERROR_CODES );
      
  ...
      
  unless ( $zip->read( 'myfile.zip' ) == AZ_OK ) {
          die "whoops!";
      }

    AZ_OK (0)
        Everything is fine.

    AZ_STREAM_END (1)
        The read stream (or central directory) ended normally.

    AZ_ERROR (2)
        There was some generic kind of error.

    AZ_FORMAT_ERROR (3)
        There is a format error in a ZIP file being read.

    AZ_IO_ERROR (4)
        There was an IO error.

  Compression
    Archive::Zip allows each member of a ZIP file to be compressed (using
    the Deflate algorithm) or uncompressed.

    Other compression algorithms that some versions of ZIP have been able to
    produce are not supported. Each member has two compression methods: the
    one it's stored as (this is always COMPRESSION_STORED for string and
    external file members), and the one you desire for the member in the zip
    file.

    These can be different, of course, so you can make a zip member that is
    not compressed out of one that is, and vice versa.

    You can inquire about the current compression and set the desired
    compression method:

      my $member = $zip->memberNamed( 'xyz.txt' );
      $member->compressionMethod();    # return current compression
      
  # set to read uncompressed
      $member->desiredCompressionMethod( COMPRESSION_STORED );
      
  # set to read compressed
      $member->desiredCompressionMethod( COMPRESSION_DEFLATED );

    There are two different compression methods:

    COMPRESSION_STORED
        File is stored (no compression)

    COMPRESSION_DEFLATED
        File is Deflated

  Compression Levels
    If a member's desiredCompressionMethod is COMPRESSION_DEFLATED, you can
    choose different compression levels. This choice may affect the speed of
    compression and decompression, as well as the size of the compressed
    member data.

      $member->desiredCompressionLevel( 9 );

    The levels given can be:

    0 or COMPRESSION_LEVEL_NONE
        This is the same as saying

          $member->desiredCompressionMethod( COMPRESSION_STORED );

    1 .. 9
        1 gives the best speed and worst compression, and 9 gives the best
        compression and worst speed.

    COMPRESSION_LEVEL_FASTEST
        This is a synonym for level 1.

    COMPRESSION_LEVEL_BEST_COMPRESSION
        This is a synonym for level 9.

    COMPRESSION_LEVEL_DEFAULT
        This gives a good compromise between speed and compression, and is
        currently equivalent to 6 (this is in the zlib code). This is the
        level that will be used if not specified.

Archive::Zip Methods
    The Archive::Zip class (and its invisible subclass
    Archive::Zip::Archive) implement generic zip file functionality.
    Creating a new Archive::Zip object actually makes an
    Archive::Zip::Archive object, but you don't have to worry about this
    unless you're subclassing.

  Constructor
    new( [$fileName] )
        Make a new, empty zip archive.

            my $zip = Archive::Zip->new();

        If an additional argument is passed, new() will call read() to read
        the contents of an archive:

            my $zip = Archive::Zip->new( 'xyz.zip' );

        If a filename argument is passed and the read fails for any reason,
        new will return undef. For this reason, it may be better to call
        read separately.

  Zip Archive Utility Methods
    These Archive::Zip methods may be called as functions or as object
    methods. Do not call them as class methods:

        $zip = Archive::Zip->new();
        $crc = Archive::Zip::computeCRC32( 'ghijkl' );    # OK
        $crc = $zip->computeCRC32( 'ghijkl' );            # also OK
        $crc = Archive::Zip->computeCRC32( 'ghijkl' );    # NOT OK

    Archive::Zip::computeCRC32( $string [, $crc] )
        This is a utility function that uses the Compress::Raw::Zlib CRC
        routine to compute a CRC-32. You can get the CRC of a string:

            $crc = Archive::Zip::computeCRC32( $string );

        Or you can compute the running CRC:

            $crc = 0;
            $crc = Archive::Zip::computeCRC32( 'abcdef', $crc );
            $crc = Archive::Zip::computeCRC32( 'ghijkl', $crc );

    Archive::Zip::setChunkSize( $number )
        Report or change chunk size used for reading and writing. This can
        make big differences in dealing with large files. Currently, this
        defaults to 32K. This also changes the chunk size used for
        Compress::Raw::Zlib. You must call setChunkSize() before reading or
        writing. This is not exportable, so you must call it like:

            Archive::Zip::setChunkSize( 4096 );

        or as a method on a zip (though this is a global setting). Returns
        old chunk size.

    Archive::Zip::chunkSize()
        Returns the current chunk size:

            my $chunkSize = Archive::Zip::chunkSize();

    Archive::Zip::setErrorHandler( \&subroutine )
        Change the subroutine called with error strings. This defaults to
        \&Carp::carp, but you may want to change it to get the error
        strings. This is not exportable, so you must call it like:

            Archive::Zip::setErrorHandler( \&myErrorHandler );

        If myErrorHandler is undef, resets handler to default. Returns old
        error handler. Note that if you call Carp::carp or a similar routine
        or if you're chaining to the default error handler from your error
        handler, you may want to increment the number of caller levels that
        are skipped (do not just set it to a number):

            $Carp::CarpLevel++;

    Archive::Zip::tempFile( [$tmpdir] )
        Create a uniquely named temp file. It will be returned open for
        read/write. If $tmpdir is given, it is used as the name of a
        directory to create the file in. If not given, creates the file
        using "File::Spec::tmpdir()". Generally, you can override this
        choice using the

            $ENV{TMPDIR}

        environment variable. But see the File::Spec documentation for your
        system. Note that on many systems, if you're running in taint mode,
        then you must make sure that $ENV{TMPDIR} is untainted for it to be
        used. Will *NOT* create $tmpdir if it doesn't exist (this is a
        change from prior versions!). Returns file handle and name:

            my ($fh, $name) = Archive::Zip::tempFile();
            my ($fh, $name) = Archive::Zip::tempFile('myTempDir');
            my $fh = Archive::Zip::tempFile();  # if you don't need the name

  Zip Archive Accessors
    members()
        Return a copy of the members array

            my @members = $zip->members();

    numberOfMembers()
        Return the number of members I have

    memberNames()
        Return a list of the (internal) file names of the zip members

    memberNamed( $string )
        Return ref to member whose filename equals given filename or undef.
        $string must be in Zip (Unix) filename format.

    membersMatching( $regex )
        Return array of members whose filenames match given regular
        expression in list context. Returns number of matching members in
        scalar context.

            my @textFileMembers = $zip->membersMatching( '.*\.txt' );
            # or
            my $numberOfTextFiles = $zip->membersMatching( '.*\.txt' );

    diskNumber()
        Return the disk that I start on. Not used for writing zips, but
        might be interesting if you read a zip in. This should be 0, as
        Archive::Zip does not handle multi-volume archives.

    diskNumberWithStartOfCentralDirectory()
        Return the disk number that holds the beginning of the central
        directory. Not used for writing zips, but might be interesting if
        you read a zip in. This should be 0, as Archive::Zip does not handle
        multi-volume archives.

    numberOfCentralDirectoriesOnThisDisk()
        Return the number of CD structures in the zipfile last read in. Not
        used for writing zips, but might be interesting if you read a zip
        in.

    numberOfCentralDirectories()
        Return the number of CD structures in the zipfile last read in. Not
        used for writing zips, but might be interesting if you read a zip
        in.

    centralDirectorySize()
        Returns central directory size, as read from an external zip file.
        Not used for writing zips, but might be interesting if you read a
        zip in.

    centralDirectoryOffsetWRTStartingDiskNumber()
        Returns the offset into the zip file where the CD begins. Not used
        for writing zips, but might be interesting if you read a zip in.

    zipfileComment( [$string] )
        Get or set the zipfile comment. Returns the old comment.

            print $zip->zipfileComment();
            $zip->zipfileComment( 'New Comment' );

    eocdOffset()
        Returns the (unexpected) number of bytes between where the EOCD was
        found and where it expected to be. This is normally 0, but would be
        positive if something (a virus, perhaps) had added bytes somewhere
        before the EOCD. Not used for writing zips, but might be interesting
        if you read a zip in. Here is an example of how you can diagnose
        this:

          my $zip = Archive::Zip->new('somefile.zip');
          if ($zip->eocdOffset())
          {
            warn "A virus has added ", $zip->eocdOffset, " bytes of garbage\n";
          }

        The "eocdOffset()" is used to adjust the starting position of member
        headers, if necessary.

    fileName()
        Returns the name of the file last read from. If nothing has been
        read yet, returns an empty string; if read from a file handle,
        returns the handle in string form.

  Zip Archive Member Operations
    Various operations on a zip file modify members. When a member is passed
    as an argument, you can either use a reference to the member itself, or
    the name of a member. Of course, using the name requires that names be
    unique within a zip (this is not enforced).

    removeMember( $memberOrName )
        Remove and return the given member, or match its name and remove it.
        Returns undef if member or name doesn't exist in this Zip. No-op if
        member does not belong to this zip.

    replaceMember( $memberOrName, $newMember )
        Remove and return the given member, or match its name and remove it.
        Replace with new member. Returns undef if member or name doesn't
        exist in this Zip, or if $newMember is undefined.

        It is an (undiagnosed) error to provide a $newMember that is a
        member of the zip being modified.

            my $member1 = $zip->removeMember( 'xyz' );
            my $member2 = $zip->replaceMember( 'abc', $member1 );
            # now, $member2 (named 'abc') is not in $zip,
            # and $member1 (named 'xyz') is, having taken $member2's place.

    extractMember( $memberOrName [, $extractedName ] )
        Extract the given member, or match its name and extract it. Returns
        undef if member doesn't exist in this Zip. If optional second arg is
        given, use it as the name of the extracted member. Otherwise, the
        internal filename of the member is used as the name of the extracted
        file or directory. If you pass $extractedName, it should be in the
        local file system's format. All necessary directories will be
        created. Returns "AZ_OK" on success.

    extractMemberWithoutPaths( $memberOrName [, $extractedName ] )
        Extract the given member, or match its name and extract it. Does not
        use path information (extracts into the current directory). Returns
        undef if member doesn't exist in this Zip. If optional second arg is
        given, use it as the name of the extracted member (its paths will be
        deleted too). Otherwise, the internal filename of the member (minus
        paths) is used as the name of the extracted file or directory.
        Returns "AZ_OK" on success.

    addMember( $member )
        Append a member (possibly from another zip file) to the zip file.
        Returns the new member. Generally, you will use addFile(),
        addDirectory(), addFileOrDirectory(), addString(), or read() to add
        members.

            # Move member named 'abc' to end of zip:
            my $member = $zip->removeMember( 'abc' );
            $zip->addMember( $member );

    updateMember( $memberOrName, $fileName )
        Update a single member from the file or directory named $fileName.
        Returns the (possibly added or updated) member, if any; "undef" on
        errors. The comparison is based on "lastModTime()" and (in the case
        of a non-directory) the size of the file.

    addFile( $fileName [, $newName ] )
        Append a member whose data comes from an external file, returning
        the member or undef. The member will have its file name set to the
        name of the external file, and its desiredCompressionMethod set to
        COMPRESSION_DEFLATED. The file attributes and last modification time
        will be set from the file. If the name given does not represent a
        readable plain file or symbolic link, undef will be returned.
        $fileName must be in the format required for the local file system.
        The optional $newName argument sets the internal file name to
        something different than the given $fileName. $newName, if given,
        must be in Zip name format (i.e. Unix). The text mode bit will be
        set if the contents appears to be text (as returned by the "-T" perl
        operator).

        *NOTE* that you shouldn't (generally) use absolute path names in zip
        member names, as this will cause problems with some zip tools as
        well as introduce a security hole and make the zip harder to use.

    addDirectory( $directoryName [, $fileName ] )
        Append a member created from the given directory name. The directory
        name does not have to name an existing directory. If the named
        directory exists, the file modification time and permissions are set
        from the existing directory, otherwise they are set to now and
        permissive default permissions. $directoryName must be in local file
        system format. The optional second argument sets the name of the
        archive member (which defaults to $directoryName). If given, it must
        be in Zip (Unix) format. Returns the new member.

    addFileOrDirectory( $name [, $newName ] )
        Append a member from the file or directory named $name. If $newName
        is given, use it for the name of the new member. Will add or remove
        trailing slashes from $newName as needed. $name must be in local
        file system format. The optional second argument sets the name of
        the archive member (which defaults to $name). If given, it must be
        in Zip (Unix) format.

    addString( $stringOrStringRef, $name )
        Append a member created from the given string or string reference.
        The name is given by the second argument. Returns the new member.
        The last modification time will be set to now, and the file
        attributes will be set to permissive defaults.

            my $member = $zip->addString( 'This is a test', 'test.txt' );

    contents( $memberOrMemberName [, $newContents ] )
        Returns the uncompressed data for a particular member, or undef.

            print "xyz.txt contains " . $zip->contents( 'xyz.txt' );

        Also can change the contents of a member:

            $zip->contents( 'xyz.txt', 'This is the new contents' );

        If called expecting an array as the return value, it will include
        the status as the second value in the array.

            ($content, $status) = $zip->contents( 'xyz.txt');

  Zip Archive I/O operations
    A Zip archive can be written to a file or file handle, or read from one.

    writeToFileNamed( $fileName )
        Write a zip archive to named file. Returns "AZ_OK" on success.

            my $status = $zip->writeToFileNamed( 'xx.zip' );
            die "error somewhere" if $status != AZ_OK;

        Note that if you use the same name as an existing zip file that you
        read in, you will clobber ZipFileMembers. So instead, write to a
        different file name, then delete the original. If you use the
        "overwrite()" or "overwriteAs()" methods, you can re-write the
        original zip in this way. $fileName should be a valid file name on
        your system.

    writeToFileHandle( $fileHandle [, $seekable] )
        Write a zip archive to a file handle. Return AZ_OK on success. The
        optional second arg tells whether or not to try to seek backwards to
        re-write headers. If not provided, it is set if the Perl "-f" test
        returns true. This could fail on some operating systems, though.

            my $fh = IO::File->new( 'someFile.zip', 'w' );
            unless ( $zip->writeToFileHandle( $fh ) == AZ_OK ) {
                # error handling
            }

        If you pass a file handle that is not seekable (like if you're
        writing to a pipe or a socket), pass a false second argument:

            my $fh = IO::File->new( '| cat > somefile.zip', 'w' );
            $zip->writeToFileHandle( $fh, 0 );   # fh is not seekable

        If this method fails during the write of a member, that member and
        all following it will return false from "wasWritten()". See
        writeCentralDirectory() for a way to deal with this. If you want,
        you can write data to the file handle before passing it to
        writeToFileHandle(); this could be used (for instance) for making
        self-extracting archives. However, this only works reliably when
        writing to a real file (as opposed to STDOUT or some other possible
        non-file).

        See examples/selfex.pl for how to write a self-extracting archive.

    writeCentralDirectory( $fileHandle [, $offset ] )
        Writes the central directory structure to the given file handle.

        Returns AZ_OK on success. If given an $offset, will seek to that
        point before writing. This can be used for recovery in cases where
        writeToFileHandle or writeToFileNamed returns an IO error because of
        running out of space on the destination file.

        You can truncate the zip by seeking backwards and then writing the
        directory:

            my $fh = IO::File->new( 'someFile.zip', 'w' );
                my $retval = $zip->writeToFileHandle( $fh );
            if ( $retval == AZ_IO_ERROR ) {
                my @unwritten = grep { not $_->wasWritten() } $zip->members();
                if (@unwritten) {
                    $zip->removeMember( $member ) foreach my $member ( @unwritten );
                    $zip->writeCentralDirectory( $fh,
                    $unwritten[0]->writeLocalHeaderRelativeOffset());
                }
            }

    overwriteAs( $newName )
        Write the zip to the specified file, as safely as possible. This is
        done by first writing to a temp file, then renaming the original if
        it exists, then renaming the temp file, then deleting the renamed
        original if it exists. Returns AZ_OK if successful.

    overwrite()
        Write back to the original zip file. See overwriteAs() above. If the
        zip was not ever read from a file, this generates an error.

    read( $fileName )
        Read zipfile headers from a zip file, appending new members. Returns
        "AZ_OK" or error code.

            my $zipFile = Archive::Zip->new();
            my $status = $zipFile->read( '/some/FileName.zip' );

    readFromFileHandle( $fileHandle, $filename )
        Read zipfile headers from an already-opened file handle, appending
        new members. Does not close the file handle. Returns "AZ_OK" or
        error code. Note that this requires a seekable file handle; reading
        from a stream is not yet supported.

            my $fh = IO::File->new( '/some/FileName.zip', 'r' );
            my $zip1 = Archive::Zip->new();
            my $status = $zip1->readFromFileHandle( $fh );
            my $zip2 = Archive::Zip->new();
            $status = $zip2->readFromFileHandle( $fh );

  Zip Archive Tree operations
    These used to be in Archive::Zip::Tree but got moved into Archive::Zip.
    They enable operation on an entire tree of members or files. A usage
    example:

      use Archive::Zip;
      my $zip = Archive::Zip->new();
      
  # add all readable files and directories below . as xyz/*
      $zip->addTree( '.', 'xyz' );
      
  # add all readable plain files below /abc as def/*
      $zip->addTree( '/abc', 'def', sub { -f && -r } );
      
  # add all .c files below /tmp as stuff/*
      $zip->addTreeMatching( '/tmp', 'stuff', '\.c$' );
      
  # add all .o files below /tmp as stuff/* if they aren't writable
      $zip->addTreeMatching( '/tmp', 'stuff', '\.o$', sub { ! -w } );
      
  # add all .so files below /tmp that are smaller than 200 bytes as stuff/*
      $zip->addTreeMatching( '/tmp', 'stuff', '\.o$', sub { -s < 200 } );
      
  # and write them into a file
      $zip->writeToFileNamed('xxx.zip');
      
  # now extract the same files into /tmpx
      $zip->extractTree( 'stuff', '/tmpx' );

    $zip->addTree( $root, $dest [,$pred] ) -- Add tree of files to a zip
        $root is the root of the tree of files and directories to be added.
        It is a valid directory name on your system. $dest is the name for
        the root in the zip file (undef or blank means to use relative
        pathnames). It is a valid ZIP directory name (that is, it uses
        forward slashes (/) for separating directory components). $pred is
        an optional subroutine reference to select files: it is passed the
        name of the prospective file or directory using $_, and if it
        returns true, the file or directory will be included. The default is
        to add all readable files and directories. For instance, using

          my $pred = sub { /\.txt/ };
          $zip->addTree( '.', '', $pred );

        will add all the .txt files in and below the current directory,
        using relative names, and making the names identical in the zipfile:

          original name           zip member name
          ./xyz                   xyz
          ./a/                    a/
          ./a/b                   a/b

        To translate absolute to relative pathnames, just pass them in:
        $zip->addTree( '/c/d', 'a' );

          original name           zip member name
          /c/d/xyz                a/xyz
          /c/d/a/                 a/a/
          /c/d/a/b                a/a/b

        Returns AZ_OK on success. Note that this will not follow symbolic
        links to directories. Note also that this does not check for the
        validity of filenames.

        Note that you generally *don't* want to make zip archive member
        names absolute.

    $zip->addTreeMatching( $root, $dest, $pattern [,$pred] )
        $root is the root of the tree of files and directories to be added
        $dest is the name for the root in the zip file (undef means to use
        relative pathnames) $pattern is a (non-anchored) regular expression
        for filenames to match $pred is an optional subroutine reference to
        select files: it is passed the name of the prospective file or
        directory in $_, and if it returns true, the file or directory will
        be included. The default is to add all readable files and
        directories. To add all files in and below the current dirctory
        whose names end in ".pl", and make them extract into a subdirectory
        named "xyz", do this:

          $zip->addTreeMatching( '.', 'xyz', '\.pl$' )

        To add all *writable* files in and below the dirctory named "/abc"
        whose names end in ".pl", and make them extract into a subdirectory
        named "xyz", do this:

          $zip->addTreeMatching( '/abc', 'xyz', '\.pl$', sub { -w } )

        Returns AZ_OK on success. Note that this will not follow symbolic
        links to directories.

    $zip->updateTree( $root, [ $dest, [ $pred [, $mirror]]] );
        Update a zip file from a directory tree.

        "updateTree()" takes the same arguments as "addTree()", but first
        checks to see whether the file or directory already exists in the
        zip file, and whether it has been changed.

        If the fourth argument $mirror is true, then delete all my members
        if corresponding files weren't found.

        Returns an error code or AZ_OK if all is well.

    $zip->extractTree()
    $zip->extractTree( $root )
    $zip->extractTree( $root, $dest )
    $zip->extractTree( $root, $dest, $volume )
        If you don't give any arguments at all, will extract all the files
        in the zip with their original names.

        If you supply one argument for $root, "extractTree" will extract all
        the members whose names start with $root into the current directory,
        stripping off $root first. $root is in Zip (Unix) format. For
        instance,

          $zip->extractTree( 'a' );

        when applied to a zip containing the files: a/x a/b/c ax/d/e d/e
        will extract:

        a/x as ./x

        a/b/c as ./b/c

        If you give two arguments, "extractTree" extracts all the members
        whose names start with $root. It will translate $root into $dest to
        construct the destination file name. $root and $dest are in Zip
        (Unix) format. For instance,

           $zip->extractTree( 'a', 'd/e' );

        when applied to a zip containing the files: a/x a/b/c ax/d/e d/e
        will extract:

        a/x to d/e/x

        a/b/c to d/e/b/c and ignore ax/d/e and d/e

        If you give three arguments, "extractTree" extracts all the members
        whose names start with $root. It will translate $root into $dest to
        construct the destination file name, and then it will convert to
        local file system format, using $volume as the name of the
        destination volume.

        $root and $dest are in Zip (Unix) format.

        $volume is in local file system format.

        For instance, under Windows,

           $zip->extractTree( 'a', 'd/e', 'f:' );

        when applied to a zip containing the files: a/x a/b/c ax/d/e d/e
        will extract:

        a/x to f:d/e/x

        a/b/c to f:d/e/b/c and ignore ax/d/e and d/e

        If you want absolute paths (the prior example used paths relative to
        the current directory on the destination volume, you can specify
        these in $dest:

           $zip->extractTree( 'a', '/d/e', 'f:' );

        when applied to a zip containing the files: a/x a/b/c ax/d/e d/e
        will extract:

        a/x to f:\d\e\x

        a/b/c to f:\d\e\b\c and ignore ax/d/e and d/e

        Returns an error code or AZ_OK if everything worked OK.

MEMBER OPERATIONS
  Member Class Methods
    Several constructors allow you to construct members without adding them
    to a zip archive. These work the same as the addFile(), addDirectory(),
    and addString() zip instance methods described above, but they don't add
    the new members to a zip.

    Archive::Zip::Member->newFromString( $stringOrStringRef [, $fileName] )
        Construct a new member from the given string. Returns undef on
        error.

            my $member = Archive::Zip::Member->newFromString( 'This is a test',
                                                         'xyz.txt' );

    newFromFile( $fileName )
        Construct a new member from the given file. Returns undef on error.

            my $member = Archive::Zip::Member->newFromFile( 'xyz.txt' );

    newDirectoryNamed( $directoryName [, $zipname ] )
        Construct a new member from the given directory. $directoryName must
        be a valid name on your file system; it doesn't have to exist.

        If given, $zipname will be the name of the zip member; it must be a
        valid Zip (Unix) name. If not given, it will be converted from
        $directoryName.

        Returns undef on error.

            my $member = Archive::Zip::Member->newDirectoryNamed( 'CVS/' );

  Member Simple accessors
    These methods get (and/or set) member attribute values.

    versionMadeBy()
        Gets the field from the member header.

    fileAttributeFormat( [$format] )
        Gets or sets the field from the member header. These are "FA_*"
        values.

    versionNeededToExtract()
        Gets the field from the member header.

    bitFlag()
        Gets the general purpose bit field from the member header. This is
        where the "GPBF_*" bits live.

    compressionMethod()
        Returns the member compression method. This is the method that is
        currently being used to compress the member data. This will be
        COMPRESSION_STORED for added string or file members, or any of the
        "COMPRESSION_*" values for members from a zip file. However, this
        module can only handle members whose data is in COMPRESSION_STORED
        or COMPRESSION_DEFLATED format.

    desiredCompressionMethod( [$method] )
        Get or set the member's "desiredCompressionMethod". This is the
        compression method that will be used when the member is written.
        Returns prior desiredCompressionMethod. Only COMPRESSION_DEFLATED or
        COMPRESSION_STORED are valid arguments. Changing to
        COMPRESSION_STORED will change the member desiredCompressionLevel to
        0; changing to COMPRESSION_DEFLATED will change the member
        desiredCompressionLevel to COMPRESSION_LEVEL_DEFAULT.

    desiredCompressionLevel( [$method] )
        Get or set the member's desiredCompressionLevel This is the method
        that will be used to write. Returns prior desiredCompressionLevel.
        Valid arguments are 0 through 9, COMPRESSION_LEVEL_NONE,
        COMPRESSION_LEVEL_DEFAULT, COMPRESSION_LEVEL_BEST_COMPRESSION, and
        COMPRESSION_LEVEL_FASTEST. 0 or COMPRESSION_LEVEL_NONE will change
        the desiredCompressionMethod to COMPRESSION_STORED. All other
        arguments will change the desiredCompressionMethod to
        COMPRESSION_DEFLATED.

    externalFileName()
        Return the member's external file name, if any, or undef.

    fileName()
        Get or set the member's internal filename. Returns the (possibly
        new) filename. Names will have backslashes converted to forward
        slashes, and will have multiple consecutive slashes converted to
        single ones.

    lastModFileDateTime()
        Return the member's last modification date/time stamp in MS-DOS
        format.

    lastModTime()
        Return the member's last modification date/time stamp, converted to
        unix localtime format.

            print "Mod Time: " . scalar( localtime( $member->lastModTime() ) );

    setLastModFileDateTimeFromUnix()
        Set the member's lastModFileDateTime from the given unix time.

            $member->setLastModFileDateTimeFromUnix( time() );

    internalFileAttributes()
        Return the internal file attributes field from the zip header. This
        is only set for members read from a zip file.

    externalFileAttributes()
        Return member attributes as read from the ZIP file. Note that these
        are NOT UNIX!

    unixFileAttributes( [$newAttributes] )
        Get or set the member's file attributes using UNIX file attributes.
        Returns old attributes.

            my $oldAttribs = $member->unixFileAttributes( 0666 );

        Note that the return value has more than just the file permissions,
        so you will have to mask off the lowest bits for comparisions.

    localExtraField( [$newField] )
        Gets or sets the extra field that was read from the local header.
        This is not set for a member from a zip file until after the member
        has been written out. The extra field must be in the proper format.

    cdExtraField( [$newField] )
        Gets or sets the extra field that was read from the central
        directory header. The extra field must be in the proper format.

    extraFields()
        Return both local and CD extra fields, concatenated.

    fileComment( [$newComment] )
        Get or set the member's file comment.

    hasDataDescriptor()
        Get or set the data descriptor flag. If this is set, the local
        header will not necessarily have the correct data sizes. Instead, a
        small structure will be stored at the end of the member data with
        these values. This should be transparent in normal operation.

    crc32()
        Return the CRC-32 value for this member. This will not be set for
        members that were constructed from strings or external files until
        after the member has been written.

    crc32String()
        Return the CRC-32 value for this member as an 8 character printable
        hex string. This will not be set for members that were constructed
        from strings or external files until after the member has been
        written.

    compressedSize()
        Return the compressed size for this member. This will not be set for
        members that were constructed from strings or external files until
        after the member has been written.

    uncompressedSize()
        Return the uncompressed size for this member.

    isEncrypted()
        Return true if this member is encrypted. The Archive::Zip module
        does not currently create or extract encrypted members.

    isTextFile( [$flag] )
        Returns true if I am a text file. Also can set the status if given
        an argument (then returns old state). Note that this module does not
        currently do anything with this flag upon extraction or storage.
        That is, bytes are stored in native format whether or not they came
        from a text file.

    isBinaryFile()
        Returns true if I am a binary file. Also can set the status if given
        an argument (then returns old state). Note that this module does not
        currently do anything with this flag upon extraction or storage.
        That is, bytes are stored in native format whether or not they came
        from a text file.

    extractToFileNamed( $fileName )
        Extract me to a file with the given name. The file will be created
        with default modes. Directories will be created as needed. The
        $fileName argument should be a valid file name on your file system.
        Returns AZ_OK on success.

    isDirectory()
        Returns true if I am a directory.

    writeLocalHeaderRelativeOffset()
        Returns the file offset in bytes the last time I was written.

    wasWritten()
        Returns true if I was successfully written. Reset at the beginning
        of a write attempt.

  Low-level member data reading
    It is possible to use lower-level routines to access member data
    streams, rather than the extract* methods and contents(). For instance,
    here is how to print the uncompressed contents of a member in chunks
    using these methods:

        my ( $member, $status, $bufferRef );
        $member = $zip->memberNamed( 'xyz.txt' );
        $member->desiredCompressionMethod( COMPRESSION_STORED );
        $status = $member->rewindData();
        die "error $status" unless $status == AZ_OK;
        while ( ! $member->readIsDone() )
        {
        ( $bufferRef, $status ) = $member->readChunk();
        die "error $status"
                    if $status != AZ_OK && $status != AZ_STREAM_END;
        # do something with $bufferRef:
        print $$bufferRef;
        }
        $member->endRead();

    readChunk( [$chunkSize] )
        This reads the next chunk of given size from the member's data
        stream and compresses or uncompresses it as necessary, returning a
        reference to the bytes read and a status. If size argument is not
        given, defaults to global set by Archive::Zip::setChunkSize. Status
        is AZ_OK on success until the last chunk, where it returns
        AZ_STREAM_END. Returns "( \$bytes, $status)".

            my ( $outRef, $status ) = $self->readChunk();
            print $$outRef if $status != AZ_OK && $status != AZ_STREAM_END;

    rewindData()
        Rewind data and set up for reading data streams or writing zip
        files. Can take options for "inflateInit()" or "deflateInit()", but
        this isn't likely to be necessary. Subclass overrides should call
        this method. Returns "AZ_OK" on success.

    endRead()
        Reset the read variables and free the inflater or deflater. Must be
        called to close files, etc. Returns AZ_OK on success.

    readIsDone()
        Return true if the read has run out of data or errored out.

    contents()
        Return the entire uncompressed member data or undef in scalar
        context. When called in array context, returns "( $string, $status
        )"; status will be AZ_OK on success:

            my $string = $member->contents();
            # or
            my ( $string, $status ) = $member->contents();
            die "error $status" unless $status == AZ_OK;

        Can also be used to set the contents of a member (this may change
        the class of the member):

            $member->contents( "this is my new contents" );

    extractToFileHandle( $fh )
        Extract (and uncompress, if necessary) the member's contents to the
        given file handle. Return AZ_OK on success.

Archive::Zip::FileMember methods
    The Archive::Zip::FileMember class extends Archive::Zip::Member. It is
    the base class for both ZipFileMember and NewFileMember classes. This
    class adds an "externalFileName" and an "fh" member to keep track of the
    external file.

    externalFileName()
        Return the member's external filename.

    fh()
        Return the member's read file handle. Automatically opens file if
        necessary.

Archive::Zip::ZipFileMember methods
    The Archive::Zip::ZipFileMember class represents members that have been
    read from external zip files.

    diskNumberStart()
        Returns the disk number that the member's local header resides in.
        Should be 0.

    localHeaderRelativeOffset()
        Returns the offset into the zip file where the member's local header
        is.

    dataOffset()
        Returns the offset from the beginning of the zip file to the
        member's data.

REQUIRED MODULES
    Archive::Zip requires several other modules:

    Carp

    Compress::Raw::Zlib

    Cwd

    File::Basename

    File::Copy

    File::Find

    File::Path

    File::Spec

    IO::File

    IO::Seekable

    Time::Local

BUGS AND CAVEATS
  When not to use Archive::Zip
    If you are just going to be extracting zips (and/or other archives) you
    are recommended to look at using Archive::Extract instead, as it is much
    easier to use and factors out archive-specific functionality.

  Try to avoid IO::Scalar
    One of the most common ways to use Archive::Zip is to generate Zip files
    in-memory. Most people have use IO::Scalar for this purpose.

    Unfortunately, as of 1.11 this module no longer works with IO::Scalar as
    it incorrectly implements seeking.

    Anybody using IO::Scalar should consider porting to IO::String, which is
    smaller, lighter, and is implemented to be perfectly compatible with
    regular seekable filehandles.

    Support for IO::Scalar most likely will not be restored in the future,
    as IO::Scalar itself cannot change the way it is implemented due to
    back-compatibility issues.

TO DO
    * auto-choosing storing vs compression

    * extra field hooks (see notes.txt)

    * check for dups on addition/renaming?

    * Text file extraction (line end translation)

    * Reading zip files from non-seekable inputs (Perhaps by proxying
    through IO::String?)

    * separate unused constants into separate module

    * cookbook style docs

    * Handle tainted paths correctly

    * Work on better compatability with other IO:: modules

SUPPORT
    Bugs should be reported via the CPAN bug tracker

    <http://rt.cpan.org/NoAuth/ReportBug.html?Queue=Archive-Zip>

    For other issues contact the maintainer

AUTHOR
    Adam Kennedy <adamk@cpan.org>

    Previously maintained by Steve Peters <steve@fisharerojo.org>.

    File attributes code by Maurice Aubrey <maurice@lovelyfilth.com>.

    Originally by Ned Konz <nedkonz@cpan.org>.

COPYRIGHT
    Some parts copyright 2006 - 2009 Adam Kennedy.

    Some parts copyright 2005 Steve Peters.

    Original work copyright 2000 - 2004 Ned Konz.

    This program is free software; you can redistribute it and/or modify it
    under the same terms as Perl itself.

SEE ALSO
    Look at Archive::Zip::MemberRead which is a wrapper that allows one to
    read Zip archive members as if they were files.

    Compress::Raw::Zlib, Archive::Tar, Archive::Extract

    There is a Japanese translation of this document at
    <http://www.memb.jp/~deq/perl/doc-ja/Archive-Zip.html> that was done by
    DEQ <deq@oct.zaq.ne.jp> . Thanks!

NAME
    OLE::Storage_Lite - Simple Class for OLE document interface.

DESCRIPTION
    This module allows you to read and write an OLE-Structured file.

    The module will work on the majority of Windows, UNIX and
    Macintosh platforms.

REQUIREMENT
     Perl 5.005 or later.

INSTALLATION
    The module can be installed using the standard Perl procedure:

        perl Makefile.PL
        make
        make test
        make install    # You may need to be root
        make clean      # or make realclean

    Windows users without a working "make" can get nmake from:
        ftp://ftp.microsoft.com/Softlib/MSLFILES/nmake15.exe

SAMPLE
    Samples scripts are in sample directory.

    smplls.pl : displays PPS structure of specified file
                (subset of "lls" distributed with OLE::Storage)

    smpsv.pl  : creates and save a sample OLE-file(tsv.dat).

AUTHOR
    Kawai Takanori (kwitknr@cpan.org)

NAME
    Spreadsheet::ParseExcel - Extract information from Excel file.

DESCRIPTION
    This module allows you to extract information from Excel file.

    This module can handle files in Excel95, 97 and 2000 format.

    The module will work on the majority of Windows, UNIX and
    Macintosh platforms.

REQUIREMENT
    This module requires these modules:
        OLE::Storage_Lite
        Jcode.pm      (if you are using FmtJapan, or FmtJapan2)
        Unicode::Map  (if you are using FmtJapan2 or FmtUnicode)
        IO::Scalar    (if PERLIO is not available)
        Spreadsheet::WriteExcel (to use Spreadsheet::ParseExcel::SaveParser)

INSTALLATION
    The module can be installed using the standard Perl procedure:

        perl Build.PL
        ./Build
        ./Build test
        ./Bbuild install

      or

        perl Makefile.PL
        make
        make test
        make install    # You may need to be root
        make clean      # or make realclean

      or using CPAN.pm or CPANPLUS.pm

        cpan Spreadsheet::ParseExcel

    Windows users without a working "make" can get nmake from:
        ftp://ftp.microsoft.com/Softlib/MSLFILES/nmake15.exe

For FmtJapan2
  If you use FmtJapan2, you must do following actions:

  (1) Copy "CP932Excel.map" included with this distribution to an
      applicatable directry.

  (2) To add "CP932Excel" as map name, append following lines to
      REGISTRY in the Unicode::Map hierarchy
      (changing map directy to applicatable directry):

name:    CP932Excel
srcURL:  $SrcUnicode/VENDORS/MICSFT/WINDOWS/CP932.TXT
src:     $DestUnicode/VENDORS/MICSFT/WINDOWS/CP932.TXT
map:     (which you copied directry)/CP932Excel.map
# Don't remove this line


AUTHOR

    Current maintainer 0.40+: John McNamara jmcnamara@cpan.org

    Maintainer 0.27-0.33: Gabor Szabo szabgab@cpan.org

    Original author: Kawai Takanori (Hippo2000) kwitknr@cpan.org


SUPPORT
    Before you report problems, please check the "KNOWN PROBLEMS" section
    in the manual page for Spreadsheet::ParseExcel.

    There is a Google group for discussing and asking questions about
    Spreadsheet::ParseExcel.
    http://groups-beta.google.com/group/spreadsheet-parseexcel/

    Bugs can be reported via rt.cpan.org. See the following for instructions:
    http://rt.cpan.org/Public/Dist/Display.html?Name=Spreadsheet-ParseExcel

    Test cases are always welcome.

## Test setup
if(FALSE) {
  library("RUnit")
  library("gdata")
}

test.drop.levels <- function()
{
  f <- factor(c("A", "B", "C", "D"))[1:3]
  fDrop <- factor(c("A", "B", "C"))

  l <- list(f=f, i=1:3, c=c("A", "B", "D"))
  lDrop <- list(f=fDrop, i=1:3, c=c("A", "B", "D"))

  df <- as.data.frame(l)
  dfDrop <- as.data.frame(lDrop)

  checkIdentical(drop.levels(f), fDrop)
  checkIdentical(drop.levels(l), lDrop)
  checkIdentical(drop.levels(df), dfDrop)
}

## Test setup
if(FALSE) {
  library("RUnit")
  library("gdata")
}

test.trim <- function()
{
  tmp <- Sys.getlocale(category="LC_COLLATE")
  Sys.setlocale(category="LC_COLLATE", locale="C")

  sTrim <- "    this is an example string    "
  sTrimR <- "this is an example string"

  fTrim <- factor(c(sTrim, sTrim, " A", " B ", "  C ", "D "))
  fTrimR <- factor(c(sTrimR, sTrimR, "A", "B", "C", "D"))

  lTrim <- list(s=rep(sTrim, times=6), f=fTrim, i=1:6)
  lTrimR <- list(s=rep(sTrimR, times=6), f=fTrimR, i=1:6)

  dfTrim <- as.data.frame(lTrim)
  dfTrimR <- as.data.frame(lTrimR)

  checkIdentical(trim(sTrim), sTrimR)
  checkIdentical(trim(fTrim), fTrimR)
  checkIdentical(levels(trim(fTrim, recode.factor=FALSE)),
                 c("this is an example string", "C", "A", "B", "D"))
  checkIdentical(trim(lTrim), lTrimR)
  checkIdentical(trim(dfTrim), dfTrimR)

  Sys.setlocale(category="LC_COLLATE", locale=tmp)
}

## The setup seems to be quite messy, but it is so to enable use of this in
## several ways as shown below.

## 1. "R CMD check" should be the most authoritative way to run the RUnit
## tests for a developer. RUnit tests are issued during R CMD check of the
## package due to example section of .runRUnitTests() function. If any test
## fails or if there are any R errors during RUnit testing, R CMD check fails.
## These are variable values specific for this way:
##  - .path DEVEL/PATH/PKG.Rcheck/PKG/unitTests
##  - .way  function

## 2. Running ".runRUnitTests()" from within R after library(PKG) is handy for
## package useRs, since it enables useRs to be sure that all tests pass for
## their installation. This is just a convenient wrapper function to run
## the RUnit testing suite. These are variable values specific for this
## way:
##  - .path INSTALL/PATH/PKG/unitTests
##  - .way  function

## 3. The "Shell" is another possibility, mainly for a developer in order to
## skip possibly lengthy R CMD check and perform just RUnit testing with an
## installed version of a pcakage. These are variable values specific for
## this way:
##  - .path DEVEL/PATH/PKG/inst/unitTests
##  - .way  shell
##
## Rscript runRUnitTests.R
## R CMD BATCH runRUnitTests.R
## make
## make all

PKG  <- "gdata"

if(require("RUnit", quietly=TRUE)) {
  path <- normalizePath("unitTests")

  cat("\nRunning unit tests\n")
  print(list(pkg=PKG, getwd=getwd(), pathToUnitTests=path))

  library(package=PKG, character.only=TRUE)

  testFileRegexp <- "^runit.+\\.[rR]$"

  ## Debugging echo
  cat("\nRunning RUnit tests\n")
  print(list(pkg=PKG, getwd=getwd(), pathToRUnitTests=path))

  ## Define tests
  testSuite <- defineTestSuite(name=paste(PKG, "RUnit testing"), dirs=path,
                               testFileRegexp=testFileRegexp)

  ## Run
  tests <- runTestSuite(testSuite)

  if(file.access(path, 02) != 0) {
    ## Cannot write to path -> use writable one
    tdir <- tempfile(paste(PKG, "RUnitTests", sep="_"))
    dir.create(tdir)
    pathReport <- file.path(tdir, "report")
  } else {
    pathReport <- file.path(path, "report")
  }

  ## Print results:
  printTextProtocol(tests)
  printTextProtocol(tests, fileName=paste0(pathReport, ".txt"))

  ## Print HTML Version of results:
  printHTMLProtocol(tests, fileName=paste0(pathReport, ".html"))

  cat("\nRUnit reports also written to\n",
      pathReport, ".(txt|html)\n\n", sep="")

  ## Return stop() to cause R CMD check stop in case of
  ##  - failures i.e. FALSE to RUnit tests or
  ##  - errors i.e. R errors
  tmp <- getErrors(tests)
  if(tmp$nFail > 0 || tmp$nErr > 0) {
    stop("\n\nRUnit testing failed:\n",
         " - #test failures: ", tmp$nFail, "\n",
         " - #R errors: ",  tmp$nErr, "\n\n")
  }
} else {
  cat("R package 'RUnit' cannot be loaded - no unit tests run\n",
      "for package", PKG, "\n")
}

library(gdata)

options(humanReadable=FALSE)

set.seed(123456)

baseSI <- 10
powerSI <- seq(from=0, to=27, by=3)
SI0 <- (baseSI)^powerSI
k <- length(SI0) - 1
SI1 <- SI0 - SI0 / c(2, runif(n=k, min=1.01, max=5.99))
SI2 <- SI0 + SI0 / c(2, runif(n=k, min=1.01, max=5.99))

baseIEC <- 2
powerIEC <- seq(from=0, to=90, by=10)
IEC0 <- (baseIEC)^powerIEC
IEC1 <- IEC0 - IEC0 / c(2, runif(n=k, min=1.01, max=5.99))
IEC2 <- IEC0 + IEC0 / c(2, runif(n=k, min=1.01, max=5.99))

# Auto units, specify width
cbind(humanReadable(x=SI2,  standard="SI",   width=7),
      humanReadable(x=SI2,  standard="SI",   width=5),
      humanReadable(x=SI2,  standard="SI",   width=3),
      humanReadable(x=IEC2, standard="IEC",  width=7),
      humanReadable(x=IEC2, standard="IEC",  width=5),
      humanReadable(x=IEC2, standard="IEC",  width=3),
      humanReadable(x=IEC2, standard="Unix", width=7),
      humanReadable(x=IEC2, standard="Unix", width=5),
      humanReadable(x=IEC2, standard="Unix", width=3))

# Auto units, specify digits
cbind(humanReadable(x=SI2,  standard="SI",   width=NULL, digits=7),
      humanReadable(x=SI2,  standard="SI",   width=NULL, digits=3),
      humanReadable(x=SI2,  standard="SI",   width=NULL, digits=2),
      humanReadable(x=SI2,  standard="SI",   width=NULL, digits=1),
      humanReadable(x=IEC2, standard="IEC",  width=NULL, digits=7),
      humanReadable(x=IEC2, standard="IEC",  width=NULL, digits=3),
      humanReadable(x=IEC2, standard="IEC",  width=NULL, digits=2),
      humanReadable(x=IEC2, standard="IEC",  width=NULL, digits=1),
      humanReadable(x=IEC2, standard="Unix", width=NULL, digits=7),
      humanReadable(x=IEC2, standard="Unix", width=NULL, digits=3),
      humanReadable(x=IEC2, standard="Unix", width=NULL, digits=2),
      humanReadable(x=IEC2, standard="Unix", width=NULL, digits=1))

# Single unit, specify width
cbind(humanReadable(x=SI1,  units="GB",  standard="SI",   width=7),
      humanReadable(x=SI1,  units="GB",  standard="SI",   width=5),
      humanReadable(x=SI1,  units="GB",  standard="SI",   width=3),
      humanReadable(x=IEC1, units="GiB", standard="IEC",  width=7),
      humanReadable(x=IEC1, units="GiB", standard="IEC",  width=5),
      humanReadable(x=IEC1, units="GiB", standard="IEC",  width=3),
      humanReadable(x=IEC1, units="G",   standard="Unix", width=7),
      humanReadable(x=IEC1, units="G",   standard="Unix", width=5),
      humanReadable(x=IEC1, units="G",   standard="Unix", width=3))

# Single unit, specify digits
cbind(humanReadable(x=SI1,  units="GB",  standard="SI",   width=NULL, digits=7),
      humanReadable(x=SI1,  units="GB",  standard="SI",   width=NULL, digits=3),
      humanReadable(x=SI1,  units="GB",  standard="SI",   width=NULL, digits=2),
      humanReadable(x=SI1,  units="GB",  standard="SI",   width=NULL, digits=1),
      humanReadable(x=IEC1, units="GiB", standard="IEC",  width=NULL, digits=7),
      humanReadable(x=IEC1, units="GiB", standard="IEC",  width=NULL, digits=3),
      humanReadable(x=IEC1, units="GiB", standard="IEC",  width=NULL, digits=2),
      humanReadable(x=IEC1, units="GiB", standard="IEC",  width=NULL, digits=1),
      humanReadable(x=IEC1, units="G",   standard="Unix", width=NULL, digits=7),
      humanReadable(x=IEC1, units="G",   standard="Unix", width=NULL, digits=3),
      humanReadable(x=IEC1, units="G",   standard="Unix", width=NULL, digits=2),
      humanReadable(x=IEC1, units="G",   standard="Unix", width=NULL, digits=1))

stopifnot(is.object_sizes(as.object_sizes(2^(1:30))))
stopifnot(format(as.object_sizes(124)) == "124 bytes")
stopifnot(format(as.object_sizes(124e8), units="auto") == "11.5 GiB")
stopifnot(format(as.object_sizes(124e8), humanReadable=TRUE) == "11.5 GiB")
stopifnot(format(as.object_sizes(124e8), units="bytes") == "1.24e+10 bytes")

tools::assertError(as.object_sizes(-1))
tools::assertError(as.object_sizes("a"))
tools::assertError(as.object_sizes(list()))
tools::assertError(as.object_sizes(NULL))
tools::assertError(as.object_sizes(0+1i))

stopifnot(format(as.object_sizes(1e40)) == "1e+40 bytes")
stopifnot(format(as.object_sizes(1e40), units="auto") == "8.271806e+15 YiB")
stopifnot(format(as.object_sizes(1e40), units="bytes") == "1e+40 bytes")
stopifnot(format(as.object_sizes(1e40), humanReadable=TRUE) == "8.271806e+15 YiB")
stopifnot(format(as.object_sizes(1e40), humanReadable=FALSE) == "1e+40 bytes")

options(humanReadable=TRUE)
stopifnot(format(as.object_sizes(1e40)) == "8.271806e+15 YiB")
options(humanReadable=FALSE)

## Test setup
if(FALSE) {
  library("RUnit")
  library("gdata")
}

test.reorder.factor <- function()
{
  tmp <- Sys.getlocale(category="LC_COLLATE")
  Sys.setlocale(category="LC_COLLATE", locale="C")

  ## Create a 4 level example factor
  levs <- c("PLACEBO", "300 MG", "600 MG", "1200 MG")
  trt <- factor(rep(x=levs, times=c(22, 24, 28, 26)))

  ## Change the order to something useful
  ## default "mixedsort" ordering
  trt2 <- reorder(trt)
  levsTest <- c("300 MG", "600 MG", "1200 MG", "PLACEBO")
  checkIdentical(levels(trt2), levsTest)

  ## Using indexes
  trt3 <- reorder(trt, new.order=c(4, 2, 3, 1))
  levsTest <- c("PLACEBO", "300 MG", "600 MG", "1200 MG")
  checkIdentical(levels(trt3), levsTest)

  ## Using label names
  trt4 <- reorder(trt, new.order=c("PLACEBO", "300 MG", "600 MG", "1200 MG"))
  levsTest <- c("PLACEBO", "300 MG", "600 MG", "1200 MG")
  checkIdentical(levels(trt4), levsTest)

  ## Using frequency
  trt5 <- reorder(trt, X=as.numeric(trt), FUN=length)
  levsTest <- c("PLACEBO", "300 MG", "1200 MG", "600 MG")
  checkIdentical(levels(trt5), levsTest)

  ## Drop out the '300 MG' level
  trt6 <- reorder(trt, new.order=c("PLACEBO", "600 MG", "1200 MG"))
  levsTest <- c("PLACEBO", "600 MG", "1200 MG")
  checkIdentical(levels(trt6), levsTest)

  Sys.setlocale(category="LC_COLLATE", locale=tmp)
}

## Test setup
if(FALSE) {
  library("RUnit")
  library("gdata")
}

num <- 1
cha <- "a"
fac <- factor("A")

tYear <- as.character(c(2006, 1995, 1005, 3067))
tMonth <- c("01", "04", "06", "12")
tDay <- c("01", "12", "22", "04")
tDate <- paste(paste(tYear, tMonth, tDay, sep="-"), "GMT")

tHour <- c("05", "16", "20", "03")
tMin <- c("16", "40", "06", "52")
tSec <- c("56", "34", "05", "15")
tTime <- paste(tHour, tMin, tSec, sep=":")

cDate <- as.Date(tDate)
cDatePOSIXct <- as.POSIXct(tDate)
cDatePOSIXlt <- as.POSIXlt(tDate)

test.getYear <- function()
{
  checkException(getYear(x=num))
  checkException(getYear(x=cha))
  checkException(getYear(x=fac))

  checkIdentical(getYear(x=cDate), tYear)
  checkIdentical(getYear(x=cDatePOSIXct), tYear)
  checkIdentical(getYear(x=cDatePOSIXlt), tYear)
}

test.getMonth <- function()
{
  checkException(getMonth(x=num))
  checkException(getMonth(x=cha))
  checkException(getMonth(x=fac))

  checkIdentical(getMonth(x=cDate), tMonth)
  checkIdentical(getMonth(x=cDatePOSIXct), tMonth)
  checkIdentical(getMonth(x=cDatePOSIXlt), tMonth)
}

test.getDay <- function()
{
  checkException(getDay(x=num))
  checkException(getDay(x=cha))
  checkException(getDay(x=fac))

  checkIdentical(getDay(x=cDate), tDay)
  checkIdentical(getDay(x=cDatePOSIXct), tDay)
  checkIdentical(getDay(x=cDatePOSIXlt), tDay)
}

test.getHour <- function()
{
  checkException(getHour(x=num))
  checkException(getHour(x=cha))
  checkException(getHour(x=fac))
}

test.getMin <- function()
{
  checkException(getMin(x=num))
  checkException(getMin(x=cha))
  checkException(getMin(x=fac))
}

test.getSec <- function()
{
  checkException(getSec(x=num))
  checkException(getSec(x=cha))
  checkException(getSec(x=fac))
}

library(gdata)

## Test data

num <- round(c(733070.345678, 1214213.78765456, 553823.798765678,
               1085022.8876545678, 571063.88765456, 606718.3876545678,
               1053686.6, 971024.187656, 631193.398765456, 879431.1), digits=3)

testData <- data.frame(num1=c(1:10, NA),
                       num2=c(NA, seq(from=1, to=5.5, by=0.5)),
                       num3=c(NA, num),
                       int1=c(as.integer(1:4), NA, as.integer(4:9)),
                       fac1=factor(c(NA, letters[1:9], "hjh")),
                       fac2=factor(c(letters[6:15], NA)),
                       cha1=c(letters[17:26], NA),
                       cha2=c(NA, "longer", letters[25:17]),
                       stringsAsFactors=FALSE)
levels(testData$fac1) <- c(levels(testData$fac1), "unusedLevel")
testData$Date <- as.Date("1900-1-1")
testData$Date[2] <- NA
testData$POSIXt <- as.POSIXct(strptime("1900-1-1 01:01:01",
                                       format="%Y-%m-%d %H:%M:%S"))
testData$POSIXt[5] <- NA

## Tests

## Default
write.fwf(testData)

## NA should be -
write.fwf(x=testData, na="-")
## NA should be -NA-
write.fwf(x=testData, na="-NA-")

## Some other separator than space
write.fwf(testData[, 1:4], sep="-mySep-")

## Justify right for character columns
write.fwf(testData, justify="right")

## Justify right for character columns, track na values
write.fwf(testData, justify="right", na="!")

## With quotes
write.fwf(testData, quote=TRUE)

## With rownames
write.fwf(testData, rownames=TRUE)

## Without colnames
write.fwf(testData, colnames=FALSE)

## Without colnames, with rownames
write.fwf(testData, colnames=FALSE, rownames=TRUE)

## With rownames and colnames and rowCol
write.fwf(testData, rownames=TRUE, rowCol="HI!")

## Test setup
if(FALSE) {
  library("RUnit")
  library("gdata")
}

test.mapLevels <- function()
{
  ## Integer and numeric
  checkException(mapLevels(1:3))  # wrong class(x)
  checkException(mapLevels(1.5))  # wrong class(x)

  ## Factor
  f <- factor(c("B", "C", "A"))
  fMapInt <- list(A=as.integer(1), B=as.integer(2), C=as.integer(3))
  fMapInt1 <- list(B=as.integer(1), C=as.integer(2))
  fMapCha <- list(A="A", B="B", C="C")
  fMapInt <- as.levelsMap(fMapInt)
  fMapInt1 <- as.levelsMap(fMapInt1)
  fMapCha <- as.levelsMap(fMapCha)
  fMapCha1 <- fMapCha[c(1, 3)]  # this will test also [.levelsMap
  checkIdentical(mapLevels(f), fMapInt)
  checkTrue(is.levelsMap(mapLevels(f)))  # test for is.levelsMap
  checkTrue(is.levelsMap(fMapInt))       # test for as.levelsMap
  checkTrue(!gdata:::.isCharacterMap(fMapInt))
  checkIdentical(mapLevels(f, sort=FALSE), fMapInt)  # sort is not used for factors
  checkIdentical(mapLevels(f[1:2], drop=TRUE), fMapInt1)
  checkIdentical(mapLevels(f, codes=FALSE), fMapCha)
  checkIdentical(mapLevels(f[c(2, 3)], drop=TRUE, codes=FALSE), fMapCha1)

  ## Character
  cha <- c("Z", "M", "A")
  chaMapInt <- list(A=as.integer(1), M=as.integer(2), Z=as.integer(3))
  chaMapIntO <- list(Z=as.integer(1), M=as.integer(2), A=as.integer(3))
  chaMapInt1 <- list(M=as.integer(1), Z=as.integer(2))
  chaMapCha <- list(A="A", M="M", Z="Z")
  chaMapInt <- as.levelsMap(chaMapInt)
  chaMapIntO <- as.levelsMap(chaMapIntO)
  chaMapInt1 <- as.levelsMap(chaMapInt1)
  chaMapCha <- as.levelsMap(chaMapCha)
  checkIdentical(mapLevels(cha), chaMapInt)
  checkIdentical(mapLevels(cha, sort=FALSE), chaMapIntO)  # sort works for characters
  checkIdentical(mapLevels(cha[1:2], drop=TRUE), chaMapInt1)
  checkIdentical(mapLevels(cha, codes=FALSE), chaMapCha)

  ## List
  l <- list(f=f, cha=cha)
  l1 <- list(cha=cha, f=f)
  l2 <- list(cha=cha, f=f, i=1:10)
  lMapInt <- list(f=fMapInt, cha=chaMapInt)
  lMapCha <- list(f=fMapCha, cha=chaMapCha)
  lMapInt <- as.listLevelsMap(lMapInt)
  lMapCha <- as.listLevelsMap(lMapCha)
  lMapChaC <- as.list(sort(unique(c(cha, as.character(f)))))
  lMapChaCO <- as.list(unique(c(cha, as.character(f))))
  names(lMapChaC) <- unlist(lMapChaC)
  names(lMapChaCO) <- unlist(lMapChaCO)
  lMapChaC <- as.levelsMap(lMapChaC)
  lMapChaCO <- as.levelsMap(lMapChaCO)
  checkIdentical(mapLevels(l), lMapInt)
  checkTrue(is.listLevelsMap(mapLevels(l)))  # test for is.listLevelsMap
  checkTrue(is.listLevelsMap(lMapInt))       # test for as.listLevelsMap
  checkIdentical(mapLevels(l, codes=FALSE), lMapCha)
  checkException(mapLevels(l, combine=TRUE))  # can not combine integer maps
  checkIdentical(mapLevels(l, codes=FALSE, combine=TRUE), lMapChaC)
  checkIdentical(mapLevels(l1, codes=FALSE, combine=TRUE), lMapChaC)
  checkIdentical(mapLevels(l1, codes=FALSE, combine=TRUE, sort=FALSE), lMapChaCO)
  checkException(mapLevels(l2))  # only char and factor

  ## Data.frame
  df <- data.frame(f1=factor(c("G", "Abc", "Abc", "D", "F")),
                   f2=factor(c("Abc", "Abc", "B", "D", "K")),
                   cha=c("jkl", "A", "D", "K", "L"),
                   int=1:5)
  dfMapInt <- list(f1=mapLevels(df$f1), f2=mapLevels(df$f2), cha=mapLevels(df$cha))
  dfMapInt <- as.listLevelsMap(dfMapInt)
  dfMapInt1 <- dfMapInt[c(1, 3)]  # this will test also [.listLevelsMap
  checkException(mapLevels(df))   # wrong class of int
  checkIdentical(mapLevels(df[, 1:3]), dfMapInt)
  checkIdentical(mapLevels(df[, c(1, 3)]), dfMapInt1)
}

test.checkLevelsMap <- function(x)
{
  ## levelsMap
  ## not a list
  checkException(gdata:::.checkLevelsMap(x="A", method="raw"))
  ## list without names
  checkException(gdata:::.checkLevelsMap(x=list("A"), method="raw"))
  fMapInt <- list(A=as.integer(1), B=as.integer(2), C=as.integer(3))
  ## x should be levelsMap
  checkException(gdata:::.checkLevelsMap(x=fMapInt, method="class"))

  ## listLevelsMap
  map <- list(as.levelsMap(fMapInt), as.levelsMap(fMapInt))
  map1 <- list(fMapInt, fMapInt)
  class(map1) <- "listLevelsMap"
  ## x should be a listLevelsMap
  checkException(gdata:::.checkListLevelsMap(x=map, method="class"))
  ## x should be also a list of levelsMaps
  checkException(gdata:::.checkListLevelsMap(x=map1, method="class"))
  ## the rest is done with levelsMap tests
}

test.cLevelsMap <- function()
{
  f1 <- factor(letters[c(2, 1)])
  f2 <- factor(letters[c(3, 1, 2)])
  mapCha1 <- mapLevels(f1, codes=FALSE)  # get maps
  mapCha2 <- mapLevels(f2, codes=FALSE)
  mapCha1S <- mapLevels(as.character(f1), codes=FALSE, sort=FALSE)
  mapCha2S <- mapLevels(as.character(f2), codes=FALSE, sort=FALSE)
  mapChaTest <- list(a="a", b="b")
  mapChaTest1 <- list(a="a", b="b", c="c")
  mapChaTest2 <- list(c="c", a="a", b="b")
  class(mapChaTest) <- class(mapChaTest1) <- class(mapChaTest2) <- "levelsMap"
  mapChaTest3 <- list(mapChaTest, mapChaTest1, mapChaTest, mapChaTest1)
  class(mapChaTest3)  <- "listLevelsMap"
  checkIdentical(c(mapCha1), mapChaTest)
  checkIdentical(c(mapCha2, mapCha1), mapChaTest1)
  checkIdentical(c(mapCha2S, mapCha1S, sort=FALSE), mapChaTest2)

  l <- list(f1, f2)
  mapCha <- mapLevels(l, codes=FALSE)
  checkIdentical(c(mapCha, mapCha), mapChaTest3)
  checkIdentical(c(mapCha, recursive=TRUE), mapChaTest1)

  checkException(c(mapLevels(f1)))  # can not combine integer "levelsMaps"

  ## Example with maps of different length of components
  map1 <- list(A=c("a", "e", "i", "o", "u"), B="b", C="c", C="m",
               D=c("d", "e"), F="f")
  map2 <- list(A=c("a", "z", "w", "y", "x"), F="f", G=c("g", "h", "j"),
               i="i", k=c("k", "l"), B="B")
  map0Test <- list(A=c("a", "e", "i", "o", "u"), B="b", C="c", C="m",
                   D=c("d", "e"), F="f",
                   A=c("z", "w", "y", "x"), G=c("g", "h", "j"),
                   i="i", k=c("k", "l"), B="B")
  map0Test <- as.levelsMap(map0Test)
  mapTest <- sort(map0Test)
  map1 <- as.levelsMap(map1)
  map2 <- as.levelsMap(map2)
  map <- c(map1, map2)
  map0 <- c(map1, map2, sort=FALSE)
  checkIdentical(map, mapTest)
  checkIdentical(map0, map0Test)
}

test.uniqueLevelsMap <- function()
{
  map <- list(A=c(1, 2, 1, 3), B=4, C=1, C=5, D=c(6, 8), E=7, B=4,
              D=c(6, 8))
  map1 <- map
  map1[[1]] <- map[[1]][c(1, 2, 4)]
  map1[[7]] <- NULL  # remove B=4
  map1[[7]] <- NULL  # remove D=c(6, 8)
  ## unique (used in as.levelsMap), will remove duplicates (A=1)
  checkIdentical(as.levelsMap(map1), as.levelsMap(map))
}

"test.mapLevels<-" <- function()
{
  ## Some errors
  checkException("mapLevels<-"(1.1, value=2))  # wrong class(x)
  checkException("mapLevels<-"(complex(1.1), value=2))  # wrong class(x)

  f <- factor(c("A", "B", "C"))
  fMapInt <- mapLevels(f)
  ## Can not apply integer "levelsMap" to "character"
  checkException("mapLevels<-"(as.character(f), value=fMapInt))

  fMapCha <- mapLevels(f, codes=FALSE)
  ## Can not apply character levelsMap to "integer"
  checkException("mapLevels<-"(as.integer(f), value=chaMapCha))

  fMapFuzz <- fMapInt
  fMapFuzz[[1]] <- "A"
  ## All components of 'value' must be of the same class
  checkException("mapLevels<-"(as.character(f), value=fMapFuzz))
  checkException("mapLevels<-"(as.integer(f), value=fMapFuzz))

  ## x integer, value integer levelsMap
  f <- factor(letters[c(10, 15, 1, 2)])
  fMapInt <- mapLevels(f)
  fInt <- as.integer(f)
  mapLevels(fInt) <- fMapInt
  checkIdentical(fInt, f)

  ## x factor, value integer levelsMap
  fInt <- factor(as.integer(f))
  mapLevels(fInt) <- fMapInt
  checkIdentical(fInt, f)

  ## Above is essentially the same as levels<-.factor
  fInt1 <- factor(as.integer(f))
  levels(fInt1) <- fMapInt
  checkIdentical(fInt1, f)

  ## x character, value character levelsMap
  cha <- c("B", "A", "C")
  chaMapCha <- as.levelsMap(list(A1="A", B2="B", C3="C"))
  mapLevels(cha) <- chaMapCha
  chaTest <- factor(c("B2", "A1", "C3"))
  checkIdentical(cha, chaTest)
  ## and a bit more for components of length > 1
  cha <- c("G", "I", "B", "A", "C", "D", "Z")
  chaMapCha <- as.levelsMap(list(A1=c("A", "G", "I"), B2="B", C3=c("C", "D")))
  mapLevels(cha) <- chaMapCha
  chaTest <- factor(c("A1", "A1", "B2", "A1", "C3", "C3", NA))
  checkIdentical(cha, chaTest)

  ## x factor, value character levelsMap
  f <- factor(c("G", "I", "B", "A", "C", "D", "Z"))
  fMapCha <- as.levelsMap(list(A1=c("A", "G", "I"), B2="B", C3=c("C", "D")))
  mapLevels(f) <- fMapCha
  fTest <- factor(c("A1", "A1", "B2", "A1", "C3", "C3", NA))
  checkIdentical(f, fTest)

  ## Two factors and character map
  f1 <- factor(letters[1:10])
  f2 <- factor(letters[5:14])
  checkIdentical(as.integer(f1), as.integer(f2))  # the same integer codes
  mapCha1 <- mapLevels(f1, codes=FALSE)  # get maps
  mapCha2 <- mapLevels(f2, codes=FALSE)
  mapCha <- c(mapCha1, mapCha2)          # combine maps
  ## apply map
  mapLevels(f1) <- mapCha  # the same as levels(f1) <- mapCha
  mapLevels(f2) <- mapCha  # the same as levels(f2) <- mapCha
  checkIdentical(as.integer(f1), 1:10)  # \ internal codes are now
  checkIdentical(as.integer(f2), 5:14)  # / "consistent" among factors

  ## The same with list
  l <- list(f1=f1, f2=f2)
  mapCha <- mapLevels(l, codes=FALSE, combine=TRUE)
  mapLevels(l) <- mapCha
  checkIdentical(as.integer(l$f1), 1:10)  # \ internal codes are now
  checkIdentical(as.integer(l$f2), 5:14)  # / "consistent" among factors

  ## and data.frame
  df <- data.frame(f1=f1, f2=f2)
  mapCha <- mapLevels(df, codes=FALSE, combine=TRUE)
  mapLevels(df) <- mapCha
  checkIdentical(as.integer(df$f1), 1:10)  # \ internal codes are now
  checkIdentical(as.integer(df$f2), 5:14)  # / "consistent" among factors
}

humanReadable <- function(x, units="auto", standard=c("IEC", "SI", "Unix"),
                          digits=1, width=NULL, sep=" ",
                          justify = c("right", "left"))
{
  ## Setup
  suffix.SI   <- c("B",  "kB",  "MB",  "GB",  "TB",  "PB",  "EB",  "ZB",  "YB")
  suffix.IEC  <- c("B", "KiB", "MiB", "GiB", "TiB", "PiB", "EiB", "ZiB", "YiB")
  suffix.Unix <- c("B" ,  "K",   "M",   "G",   "T",   "P",   "E",   "Z",   "Y")

  standard <- match.arg(standard)
  if(length(justify) == 1)
    justify <- c(justify, justify)

  ## Functions
  .applyHuman <- function(x, base, suffix, digits, width, sep)
  {
    ## Which suffix should we use?
    n <- length(suffix)
    i <- pmax(pmin(floor(log(x, base)), n-1),0)
    if(!is.finite(i))
      i <- 0
    x <- x / base^i
    ## Formatting
    if(is.null(width))
    {
      ## the same formatting for all
      x <- format(round(x=x, digits=digits), nsmall=digits)
    }
    else
    {
      ## similar to ls, du, and df
      lenX <- nchar(x)
      if(lenX > width) {
        digits <- pmax(width - nchar(round(x)) - 1, 0)
      }
      if(i == 0) digits <- 0
      x <- round(x, digits=digits)
    }
    c(x, suffix[i+1])
  }

  ## Work
  if(any(x < 0)) stop("'x' must be positive")
  if(standard == "SI")
  {
    suffix <- suffix.SI
    base <- 10^3
  }
  else if (standard=="IEC")
  {
    suffix <- suffix.IEC
    base <- 2^10
  }
  else  # (standard=="Unix)
  {
    suffix <- suffix.Unix
    base <- 2^10
  }

  if(!missing(units) && units=="bytes")
  {
    retval <- rbind(x, "bytes")
  }
  else if(!missing(units) && units!="auto")
  {
    units <- suffix[match(toupper(units), toupper(suffix))]
    power <- match(units, suffix) -1
    X <- x / (base^power)
    X <- format.default(x=X, digits=digits, nsmall=digits)
    retval <- rbind(X, rep(units, length(X)))
  }
  else
  {
    retval <- sapply(X=x, FUN=".applyHuman", base=base, suffix=suffix,
                     digits=digits, width=width, sep=sep)
  }

  if(all(justify == "none"))
  {
    paste(trim(retval[1,]), trim(retval[2,]), sep=sep)
  }
  else
  {
    paste(format(trim(retval[1,]), justify=justify[1]),
          format(trim(retval[2,]), justify=justify[2]), sep=sep)
  }
}

interleave <- function(..., append.source=TRUE, sep=": ", drop=FALSE)
{
  sources <- list(...)

  sources[sapply(sources, is.null)] <- NULL

  sources <- lapply(sources, function(x)
    if(is.matrix(x) || is.data.frame(x))
      x else t(x))

  nrows <- sapply(sources, nrow)
  mrows <- max(nrows)
  if(any(nrows!=mrows & nrows!=1))
    stop("arguments have differing numbers of rows")

  sources <- lapply(sources, function(x)
    if(nrow(x)==1) x[rep(1,mrows),,drop=drop] else x)

  tmp <- do.call("rbind", sources)

  nsources <- length(sources)
  indexes <- outer((0:(nsources-1)) * mrows, 1:mrows, "+")

  retval <- tmp[indexes,,drop=drop]

  if(append.source && !is.null(names(sources)))
  {
    if(!is.null(row.names(tmp)))
    {
      row.names(retval) <- paste(format(row.names(retval)),
                                 format(names(sources)), sep=sep)
    }
    else
    {
      row.names(retval) <- rep(names(sources), mrows)
    }
  }

  retval
}

update.data.frame <- function(object, new, by, by.x=by, by.y=by, append=TRUE,
                              verbose=FALSE, ...)
{
  x <- object
  y <- new

  retval <- x
  x.by <- x[[by.x]]
  y.by <- y[[by.y]]

  matches.x <- match(y.by, x.by)
  matches.y <- which(!is.na(matches.x))
  nomatch.y <- which(is.na(matches.x))
  matches.x <- matches.x[!is.na(matches.x)]

  if(length(matches.x)>0)
    retval[matches.x, ] <- y[matches.y,]

  if(length(nomatch.y) && append)
    retval <- rbind(retval, y[nomatch.y,])

  if(verbose)
  {
    cat("\n")
    cat("Number of rows in x     :", nrow(x),           "\n")
    cat("Number of rows in y     :", nrow(y),           "\n")
    cat("\n")
    cat("Number of rows replaced :", length(matches.x), "\n")
    cat("Number of rows appended :", length(nomatch.y), "\n")
    cat("\n")
    cat("Number of rows in result:", nrow(retval),      "\n")
    cat("\n")
  }
  retval
}

frameApply <- function(x, by = NULL, on = by[1],
                       fun = function(xi) c(Count = nrow(xi)) , subset = TRUE,
                       simplify = TRUE, byvar.sep = "\\$\\@\\$", ...)
{
  subset <- eval(substitute(subset), x, parent.frame())
  x <- x[subset, , drop = FALSE]
  if(!is.null(by)) {
    x[by] <- drop.levels(x[by])
    for(i in seq(along = by))
      if(length(grep(byvar.sep, as.character(x[[by[i]]]))))
        stop("Choose a different value for byvar.sep.")
    byvars <- unique(x[by])
    BYVAR <- do.call("paste", c(as.list(x[by]), sep = byvar.sep))
    byvars <- byvars[order(unique(BYVAR)), , drop = FALSE]
    splx <- split(x[on], BYVAR)
    splres <- lapply(splx, fun, ...)
    if(!simplify) out <- list(by = byvars, result = splres)
    else {
      i <- 1 ; nres <- length(splres)
      while(inherits(splres[[i]], "try-error") & i < nres) i <- i + 1
      nms <- names(splres[[i]])
      splres <- lapply(splres, function(xi) {
        if(inherits(xi, "try-error")) {
          return(rep(NA, length(nms)))
        }
        else xi
      })
      res <- do.call("rbind", splres)
      res <- as.data.frame(res)
      names(res) <- nms
      if(length(intersect(names(byvars), names(res))))
        stop("names of 'by' variables are the same as names of result elements")
      out <- data.frame(byvars, res)
    }
  }
  else {
    out <- fun(x[on])
    if(simplify) out <- as.data.frame(as.list(out))
  }
  out
}

ls.funs <- function (...)
{
  mycall <- match.call()
  mycall[[1]] <- as.name("ls")
  nameList <- eval.parent(mycall)
  if(length(nameList) > 0)
  {
    funcFlags <- sapply(nameList, function(x) is.function(get(x)))
    return(nameList[funcFlags])
  }
  else
  {
    return(list())
  }
}

getYear <- function(x, format, ...)
  UseMethod("getYear")

getYear.default <- function(x, format, ...)
  stop("'getYear' can only be used on objects of a date/time class")

getYear.Date <-
getYear.POSIXct <-
getYear.POSIXlt <- function(x, format="%Y", ...)
  format(x=x, format=format, ...)

getMonth <- function(x, format, ...)
  UseMethod("getMonth")

getMonth.default <- function(x, format, ...)
  stop("'getMonth' can only be used on objects of a date/time class")

getMonth.Date <-
getMonth.POSIXct <-
getMonth.POSIXlt <- function(x, format="%m", ...)
  format(x=x, format=format)

getDay <- function(x, format, ...)
  UseMethod("getDay")

getDay.default <- function(x, format, ...)
  stop("'getDay' can only be used on objects of a date/time class")

getDay.Date <-
getDay.POSIXct <-
getDay.POSIXlt <- function(x, format="%d", ...)
  format(x=x, format=format)

getHour <- function(x, format, ...)
  UseMethod("getHour")

getHour.default <- function(x, format, ...)
  stop("'getHour' can only be used on objects of a date/time class")

getMin <- function(x, format, ...)
  UseMethod("getMin")

getMin.default <- function(x, format, ...)
  stop("'getMin' can only be used on objects of a date/time class")

getSec <- function(x, format, ...)
  UseMethod("getSec")

getSec.default <- function(x, format, ...)
  stop("'getSec' can only be used on objects of a date/time class")

matchcols <- function(object, with, without, method=c("and","or"), ...)
{
  method <- match.arg(method)
  cols <- colnames(object)

  # Include columns matching 'with' pattern(s)
  if(method == "and")
  {
    for(i in 1:length(with))
    {
      if(length(cols)>0)
        cols <- grep(with[i], cols, value=TRUE, ...)
    }
  }
  else if(!missing(with))
  {
    if(length(cols)>0)
      cols <- sapply(with, grep, x=cols, value=TRUE, ...)
  }

  # Exclude columns matching 'without' pattern(s)
  if(!missing(without))
  {
    for(i in 1:length(without))
    {
      if(length(cols) > 0)
      {
        omit <- grep(without[i], cols, ...)
        if(length(omit) > 0)
          cols <- cols[-omit]
      }
    }
  }

  cols
}

upperTriangle <- function(x, diag=FALSE, byrow=FALSE)
{
  if(byrow)
    t(x)[rev(upper.tri(x, diag=diag))]
  else
    x[upper.tri(x, diag=diag)]
}

"upperTriangle<-" <- function(x, diag=FALSE, byrow=FALSE, value)
{
  if(byrow) {
    ret <- t(x)
    ret[rev(upper.tri(x, diag=diag))] <- value
    t(ret)
  }
  else {
    x[upper.tri(x, diag=diag)] <- value
    x
  }
}

lowerTriangle <- function(x, diag=FALSE, byrow=FALSE)
{
  if(byrow)
    t(x)[rev(lower.tri(x, diag=diag))]
  else
    x[lower.tri(x, diag=diag)]
}

"lowerTriangle<-" <- function(x, diag=FALSE, byrow=FALSE, value)
{
  if(byrow) {
    ret <- t(x)
    ret[rev(lower.tri(x, diag=diag))] <- value
    t(ret)
  }
  else {
    x[lower.tri(x, diag=diag)] <- value
    x
  }
}

## s$Id: read.xls.R 1342 2009-07-16 02:49:11Z warnes $

## findPerl attempts to locate a valid perl executable.  If the 'perl' argument is missing, 


findPerl <- function(perl, verbose = "FALSE")
{

  errorMsg <- "perl executable not found. Use perl= argument to specify the correct path."
      
  if (missing(perl))
    {
      perl = "perl"
    }

  perl = Sys.which(perl)
  if (perl=="" || perl=="perl")
    stop(errorMsg)

  if (.Platform$OS == "windows") {
    if (length(grep("rtools", tolower(perl))) > 0) {
      perl.ftype <- shell("ftype perl", intern = TRUE)
      if (length(grep("^perl=", perl.ftype)) > 0) {
        perl <- sub('^perl="([^"]*)".*', "\\1", perl.ftype)
      }
    }
      }
  
  if (verbose) cat("Using perl at", perl, "\n")
  
  perl
}

## s$Id: read.xls.R 1423 2010-02-21 17:12:30Z ggrothendieck2 $

xlsFormats <- function(perl="perl", verbose=FALSE)
{
  ## determine proper path to perl executable
  perl <- if (missing(perl))
    findPerl(verbose = verbose)
  else
    findPerl(perl, verbose = verbose)
  
  ##
  ## directories
  package.dir <- find.package('gdata')
  perl.dir <- file.path(package.dir,'perl')
  ##
  ##

  cmd <- "supportedFormats.pl"
  sc <- file.path(perl.dir, cmd)
  
  ##
  ##

  ##
  ## execution command

  cmd <- paste(shQuote(perl), shQuote(sc), sep=" ")

  ##

  if(verbose)
    {
      cat("\n")
      cat("Determining supported formats...\n")
      cat("\n")
    }

  ##

  output <- system(cmd, intern=TRUE)

  ##

  if(verbose) cat("Results: ", output, "\n")

  ##

  retval <- unlist( strsplit(output," "))
  retval <- retval[ -c(1,2) ]

  return(retval)
}

cbindX <- function(...)
{
  ## Setup
  x <- list(...)

  ## Are all objects matrices or data.frames?
  test <- sapply(x, function(z) is.matrix(z) | is.data.frame(z))
  if(any(!test)) stop("only matrices and data.frames can be used")

  ## Get maximum number of rows
  tmp <- sapply(x, nrow)
  maxi <- which.max(tmp)
  test <- tmp < tmp[maxi]

  ## Core
  ## Adding additional 'empty' rows so all objects have same number of rows
  for(i in 1:length(tmp)) {
    if(test[i]) {
      add <- matrix(nrow=tmp[maxi] - tmp[i], ncol=ncol(x[[i]]))
      if(is.data.frame(x[[i]])) {
        add <- as.data.frame(add)
      }
      colnames(add) <- colnames(x[[i]])
      x[[i]] <- rbind(x[[i]], add)
    }
  }

  ## Column-bind all objects
  ret <- x[[1]]
  for(i in 2:length(tmp)) {
    ret <- cbind(ret, x[[i]])
  }

  ## Return
  ret
}

## Test setup
if(FALSE) {
  library("RUnit")
  library("gdata")
}

test.cbindX <- function()
{
  df1 <- data.frame(a=1:3, b=c("A", "B", "C"))
  df2 <- data.frame(c=as.character(1:5), a=5:1)

  ma1 <- matrix(as.character(1:4), nrow=2, ncol=2)
  ma2 <- matrix(1:6, nrow=3, ncol=2)

  df12test <- cbindX(df1, df2)
  df12stand <- data.frame(a=c(1:3, NA, NA),
                          b=c("A", "B", "C", NA, NA),
                          c=as.character(1:5),
                          a=5:1)
  names(df12stand)[4] <- "a"
  checkEquals(df12test, df12stand)

  ma12test <- cbindX(ma1, ma2)
  ma12stand <- matrix(as.character(c(1,   3, 1, 4,
                                     2,   4, 2, 5,
                                     NA, NA, 3, 6)), nrow=3, ncol=4, byrow=TRUE)
  checkEquals(ma12test, ma12stand)

  da11test <- cbindX(df1, ma1)
  da11stand <- data.frame(a=1:3,
                          b=c("A", "B", "C"),
                          as.character(c(1:2, NA)),
                          as.character(c(3:4, NA)))
  names(da11stand)[3:4] <- c("1", "2")
  checkEquals(da11test, da11stand)

  tmpTest <- cbindX(df1, df2, ma1, ma2)
  tmpStand <- data.frame(a=c(1:3, NA, NA),
                         b=c("A", "B", "C", NA, NA),
                         c=as.character(1:5),
                         a=5:1,
                         as.character(c(1:2, NA, NA, NA)),
                         as.character(c(3:4, NA, NA, NA)),
                         c(1:3, NA, NA),
                         c(4:6, NA, NA))
  names(tmpStand)[4:8] <- c("a", "1", "2", "1", "2")
  checkEquals(tmpTest, tmpStand)

  tmpTest <- cbindX(ma1, ma2, df1, df2)
  tmpStand <- data.frame(as.character(c(1:2, NA, NA, NA)),
                         as.character(c(3:4, NA, NA, NA)),
                         as.character(c(1:3, NA, NA)),
                         as.character(c(4:6, NA, NA)),
                         a=c(1:3, NA, NA),
                         b=c("A", "B", "C", NA, NA),
                         c=as.character(1:5),
                         a=5:1)
  names(tmpStand)[c(1:4, 8)] <- c("1", "2", "3", "4", "a")
  checkEquals(tmpTest, tmpStand)
}

trimSum <- function(x, n, right=TRUE, na.rm=FALSE, ...)
{
  ## Setup
  if(!is.vector(x) | is.list(x))
    stop("'x' must be a vector - for now")
  if(!is.numeric(x))
    stop("'x' must be numeric")
  if(length(x) <= n)
    stop("'n' must be smaller than the length of x")

  ## Trim
  N <- length(x)
  if(right) {
    x2 <- x[1:n]
    x2[n] <- sum(x[n:N], na.rm=na.rm)
  } else {
    k <- (N - n + 1)
    x2 <- x[k:N]
    x2[1] <- sum(x[1:k], na.rm=na.rm)
  }

  x2
}

## Test setup
if(FALSE) {
  library("RUnit")
  library("gdata")
}

test.trimSum <- function()
{
  ## 'x' must be a vector - for now
  checkException(trimSum(matrix(1:10)))
  checkException(trimSum(data.frame(1:10)))
  checkException(trimSum(list(1:10)))

  ## 'x' must be numeric
  checkException(trimSum(letters))

  ## 'n' must be smaller than the length of x
  checkException(trimSum(x=1:10, n=11))
  checkException(trimSum(x=1, n=1))

  ## Default
  x <- trimSum(x=1:10, n=5)
  x2 <- c(1:4, 45)
  checkEquals(x, x2)

  ## Left
  x <- trimSum(x=1:10, n=5, right=FALSE)
  x2 <- c(21, 7:10)
  checkEquals(x, x2)

  ## NA
  x <- trimSum(x=c(1:9, NA), n=5)
  x2 <- c(1:4, NA)
  checkEquals(x, x2)

  x <- trimSum(x=c(1:9, NA), n=5, na.rm=TRUE)
  x2 <- c(1:4, 35)
  checkEquals(x, x2)
}

keep <- function(..., list=character(0), all=FALSE, sure=FALSE)
{
  if(missing(...) && missing(list))
  {
    warning("keep something, or use rm(list=ls()) to clear workspace - ",
            "nothing was removed")
    return(invisible(NULL))
  }
  names <- as.character(substitute(list(...)))[-1]
  list <- c(list, names)
  keep.elements <- match(list, ls(1,all.names=all))
  if(any(is.na(keep.elements)))
  {
    warning("you tried to keep \"", list[which(is.na(keep.elements))[1]],
            "\" which doesn't exist in workspace - nothing was removed", sep="")
    return(invisible(NULL))
  }

  if(sure)
    rm(list=ls(1,all.names=all)[-keep.elements], pos=1)
  else
    return(ls(1,all.names=all)[-keep.elements])
}

unmatrix <- function(x, byrow=FALSE)
{
  rnames <- rownames(x)
  cnames <- colnames(x)
  if(is.null(rnames)) rnames <- paste("r",1:nrow(x),sep='')
  if(is.null(cnames)) cnames <- paste("c",1:ncol(x),sep='')
  nmat <- outer(rnames, cnames, paste, sep=":")

  if(byrow)
  {
    vlist <- c(t(x))
    names(vlist) <- c(t(nmat))
  }
  else
  {
    vlist <- c(x)
    names(vlist) <- c(nmat)
  }

  return(vlist)
}

library(gdata)

options(humanReadable=FALSE)

set.seed(123456)

baseSI <- 10
powerSI <- seq(from=0, to=27, by=3)
SI0 <- (baseSI)^powerSI
k <- length(SI0) - 1
SI1 <- SI0 - SI0 / c(2, runif(n=k, min=1.01, max=5.99))
SI2 <- SI0 + SI0 / c(2, runif(n=k, min=1.01, max=5.99))

baseIEC <- 2
powerIEC <- seq(from=0, to=90, by=10)
IEC0 <- (baseIEC)^powerIEC
IEC1 <- IEC0 - IEC0 / c(2, runif(n=k, min=1.01, max=5.99))
IEC2 <- IEC0 + IEC0 / c(2, runif(n=k, min=1.01, max=5.99))

# Auto units, specify width
cbind(humanReadable(x=SI2,  standard="SI",   width=7),
      humanReadable(x=SI2,  standard="SI",   width=5),
      humanReadable(x=SI2,  standard="SI",   width=3),
      humanReadable(x=IEC2, standard="IEC",  width=7),
      humanReadable(x=IEC2, standard="IEC",  width=5),
      humanReadable(x=IEC2, standard="IEC",  width=3),
      humanReadable(x=IEC2, standard="Unix", width=7),
      humanReadable(x=IEC2, standard="Unix", width=5),
      humanReadable(x=IEC2, standard="Unix", width=3))

# Auto units, specify digits
cbind(humanReadable(x=SI2,  standard="SI",   width=NULL, digits=7),
      humanReadable(x=SI2,  standard="SI",   width=NULL, digits=3),
      humanReadable(x=SI2,  standard="SI",   width=NULL, digits=2),
      humanReadable(x=SI2,  standard="SI",   width=NULL, digits=1),
      humanReadable(x=IEC2, standard="IEC",  width=NULL, digits=7),
      humanReadable(x=IEC2, standard="IEC",  width=NULL, digits=3),
      humanReadable(x=IEC2, standard="IEC",  width=NULL, digits=2),
      humanReadable(x=IEC2, standard="IEC",  width=NULL, digits=1),
      humanReadable(x=IEC2, standard="Unix", width=NULL, digits=7),
      humanReadable(x=IEC2, standard="Unix", width=NULL, digits=3),
      humanReadable(x=IEC2, standard="Unix", width=NULL, digits=2),
      humanReadable(x=IEC2, standard="Unix", width=NULL, digits=1))

# Single unit, specify width
cbind(humanReadable(x=SI1,  units="GB",  standard="SI",   width=7),
      humanReadable(x=SI1,  units="GB",  standard="SI",   width=5),
      humanReadable(x=SI1,  units="GB",  standard="SI",   width=3),
      humanReadable(x=IEC1, units="GiB", standard="IEC",  width=7),
      humanReadable(x=IEC1, units="GiB", standard="IEC",  width=5),
      humanReadable(x=IEC1, units="GiB", standard="IEC",  width=3),
      humanReadable(x=IEC1, units="G",   standard="Unix", width=7),
      humanReadable(x=IEC1, units="G",   standard="Unix", width=5),
      humanReadable(x=IEC1, units="G",   standard="Unix", width=3)
      )

# Single unit, specify digits
cbind(humanReadable(x=SI1, units="GB", standard="SI", width=NULL, digits=7),
      humanReadable(x=SI1, units="GB", standard="SI", width=NULL, digits=3),
      humanReadable(x=SI1, units="GB", standard="SI", width=NULL, digits=2),
      humanReadable(x=SI1, units="GB", standard="SI", width=NULL, digits=1),
      humanReadable(x=IEC1, units="GiB", standard="IEC", width=NULL, digits=7),
      humanReadable(x=IEC1, units="GiB", standard="IEC", width=NULL, digits=3),
      humanReadable(x=IEC1, units="GiB", standard="IEC", width=NULL, digits=2),
      humanReadable(x=IEC1, units="GiB", standard="IEC", width=NULL, digits=1),
      humanReadable(x=IEC1, units="G", standard="Unix", width=NULL, digits=7),
      humanReadable(x=IEC1, units="G", standard="Unix", width=NULL, digits=3),
      humanReadable(x=IEC1, units="G", standard="Unix", width=NULL, digits=2),
      humanReadable(x=IEC1, units="G", standard="Unix", width=NULL, digits=1)
      )


stopifnot( is.object_sizes(as.object_sizes( 2^(1:30) ) ) )
stopifnot( format(as.object_sizes(124)) == "124 bytes")
stopifnot( format(as.object_sizes(124e8), units="auto") == "11.5 GiB")
stopifnot( format(as.object_sizes(124e8), humanReadable=TRUE) == "11.5 GiB")
stopifnot( format(as.object_sizes(124e8), units="bytes") == "1.24e+10 bytes")

tools::assertError( as.object_sizes(-1) )
tools::assertError( as.object_sizes('a') )
tools::assertError( as.object_sizes(list()) )
tools::assertError( as.object_sizes(NULL) )
tools::assertError( as.object_sizes(0+1i) )

stopifnot( format(as.object_sizes(1e40)               ) == "1e+40 bytes"     )
stopifnot( format(as.object_sizes(1e40), units="auto" ) == "8.271806e+15 YiB")
stopifnot( format(as.object_sizes(1e40), units="bytes") == "1e+40 bytes"     )
stopifnot( format(as.object_sizes(1e40), humanReadable=TRUE) == "8.271806e+15 YiB")
stopifnot( format(as.object_sizes(1e40), humanReadable=FALSE) ==  "1e+40 bytes")

options(humanReadable=TRUE)
stopifnot( format(as.object_sizes(1e40) ) == "8.271806e+15 YiB")
options(humanReadable=FALSE)

object_size <- function(...)
{
  structure(sapply(list(...), utils::object.size),
            class=c("object_sizes", "numeric"))
}

print.object_sizes <- function(x, quote=FALSE,
                               humanReadable=getOption("humanReadable"),
                               standard="IEC", units, digits=1, width=NULL,
                               sep=" ", justify = c("right", "left"), ...)
{
  print(format(x, humanReadable=humanReadable, standard=standard, units=units,
               digits=digits, width=width, sep=sep,
               justify=justify), quote=quote, ...)
  invisible(x)
}

format.object_sizes <- function(x, humanReadable=getOption("humanReadable"),
                                standard="IEC", units, digits=1, width=NULL,
                                sep=" ", justify = c("right", "left"), ...)
{
  if(!missing(units))
  {
    if (units=="bytes")
      paste(x, "bytes")
    else
      humanReadable(x, standard=standard, units=units, digits=digits,
                    width=width, sep=sep, justify=justify)
  }
  else if(is.null(humanReadable) || humanReadable==FALSE)
  {
    paste(x, "bytes")
  }
  else
  {
    humanReadable(x, standard=standard, units=units, digits=digits, width=width,
                  sep=sep, justify=justify)
  }
}

is.object_sizes <- function(x)
  inherits(x, what="object_sizes")

as.object_sizes <- function(x)
{
  if(!is.numeric(x) || any(x<0))
    stop("'x' must be a positive numeric vector")
  class(x) <- c("object_sizes", "numeric")
  x
}

c.object_sizes <- function(..., recursive=FALSE)
{
  x <- NextMethod()
  if(is.numeric(x)) class(x) <- c("object_sizes", "numeric")
  x
}

object.size <- function(...)
{
  .Deprecated(old="gdata::object.size",
              new="object_size() or utils::object.size()")
  object_size(...)
}

### bindData.R
###------------------------------------------------------------------------
### What: Bind two data frames - code
### $Id$
### Time-stamp: <2008-12-30 22:01:00 ggorjan>
###------------------------------------------------------------------------

bindData <- function(x, y, common)
{
  ## --- Setup ---
  if(!is.data.frame(x)) stop("'x' must be a data frame")
  if(!is.data.frame(y)) stop("'y' must be a data frame")

  ## --- New data frame ---

  ## First add common column and a dataset indicator column
  z <- rbind(x[common], y[common])

  ## Other columns
  ## - remove common columns in x and y
  namesz <- names(z)
  otherx <- names(x)
  otherx <- otherx[!(otherx %in% namesz)]
  othery <- names(y)
  othery <- othery[!(othery %in% namesz)]

  ## - add all other columns but as a set for each input data frame
  rx <- nrow(x); cx <- length(otherx)
  ry <- nrow(y); cy <- length(othery)
  
  z <- cbind(z, rbind(x[otherx], matrix(rep(NA, times=(ry * cx)), nrow=ry, ncol=cx, dimnames=list(NULL, otherx))))
  z <- cbind(z, rbind(matrix(rep(NA, times=(rx * cy)), nrow=rx, ncol=cy, dimnames=list(NULL, othery)), y[othery]))

  z
}

###------------------------------------------------------------------------
### bindData.R ends here

## Test setup
if(FALSE) {
  library("RUnit")
  library("gdata")
}

test.bindData <- function()
{
  ## 'x'/'y' must be a data.frame
  checkException(bindData(x=1:10, y=1:10))
  checkException(bindData(x=matrix(1:10), y=matrix(1:10)))

  n1 <- 6; n2 <- 12; n3 <- 4
  ## Single trait 1
  num <- c(5:n1, 10:13)
  tmp1 <- data.frame(y1=rnorm(n=n1),
                     f1=factor(rep(c("A", "B"), n1/2)),
                     ch=letters[num],
                     fa=factor(letters[num]),
                     nu=(num) + 0.5,
                     id=factor(num), stringsAsFactors=FALSE)

  ## Single trait 2 with repeated records, some subjects also in tmp1
  num <- 4:9
  tmp2 <- data.frame(y2=rnorm(n=n2),
                     f2=factor(rep(c("C", "D"), n2/2)),
                     ch=letters[rep(num, times=2)],
                     fa=factor(letters[rep(c(num), times=2)]),
                     nu=c((num) + 0.5, (num) + 0.25),
                     id=factor(rep(num, times=2)), stringsAsFactors=FALSE)

  ## Single trait 3 with completely distinct set of subjects
  num <- 1:4
  tmp3 <- data.frame(y3=rnorm(n=n3),
                     f3=factor(rep(c("E", "F"), n3/2)),
                     ch=letters[num],
                     fa=factor(letters[num]),
                     nu=(num) + 0.5,
                     id=factor(num), stringsAsFactors=FALSE)

  ## Combine all datasets
  tmp12 <- bindData(x=tmp1, y=tmp2, common=c("id", "nu", "ch", "fa"))
  tmp123 <- bindData(x=tmp12, y=tmp3, common=c("id", "nu", "ch", "fa"))

  checkEquals(names(tmp123), c("id", "nu", "ch", "fa", "y1", "f1", "y2", "f2", "y3", "f3"))
  checkEquals(rbind(tmp1["id"], tmp2["id"], tmp3["id"]), tmp123["id"])
  checkEquals(rbind(tmp1["fa"], tmp2["fa"], tmp3["fa"]), tmp123["fa"])
  checkEquals(is.na(tmp123$y1), c(rep(FALSE, times=n1), rep(TRUE, times=n2+n3)))
  checkEquals(is.na(tmp123$f1), c(rep(FALSE, times=n1), rep(TRUE, times=n2+n3)))
  checkEquals(is.na(tmp123$y2), c(rep(TRUE, times=n1), rep(FALSE, times=n2), rep(TRUE, times=n3)))
  checkEquals(is.na(tmp123$f2), c(rep(TRUE, times=n1), rep(FALSE, times=n2), rep(TRUE, times=n3)))
  checkEquals(is.na(tmp123$y3), c(rep(TRUE, times=n1+n2), rep(FALSE, times=n3)))
  checkEquals(is.na(tmp123$f3), c(rep(TRUE, times=n1+n2), rep(FALSE, times=n3)))
}

### runit.bindData.R
###------------------------------------------------------------------------
### What: Bind two data frames - unit tests
### $Id$
### Time-stamp: <2008-12-30 11:58:50 ggorjan>
###------------------------------------------------------------------------

### {{{ --- Test setup ---

if(FALSE) {
  library("RUnit")
  library("gdata")
}

### }}}
### {{{ --- bindData ---

test.bindData <- function()
{
  ## 'x'/'y' must be a data.frame
  checkException(bindData(x=1:10, y=1:10))
  checkException(bindData(x=matrix(1:10), y=matrix(1:10)))
  
  n1 <- 6; n2 <- 12; n3 <- 4
  ## Single trait 1
  num <- c(5:n1, 10:13)
  tmp1 <- data.frame(y1=rnorm(n=n1),
                     f1=factor(rep(c("A", "B"), n1/2)),
                     ch=letters[num],
                     fa=factor(letters[num]),
                     nu=(num) + 0.5,
                     id=factor(num), stringsAsFactors=FALSE)

  ## Single trait 2 with repeated records, some subjects also in tmp1 
  num <- 4:9
  tmp2 <- data.frame(y2=rnorm(n=n2),
                     f2=factor(rep(c("C", "D"), n2/2)),
                     ch=letters[rep(num, times=2)],
                     fa=factor(letters[rep(c(num), times=2)]),
                     nu=c((num) + 0.5, (num) + 0.25),
                     id=factor(rep(num, times=2)), stringsAsFactors=FALSE)

  ## Single trait 3 with completely distinct set of subjects
  num <- 1:4
  tmp3 <- data.frame(y3=rnorm(n=n3),
                     f3=factor(rep(c("E", "F"), n3/2)),
                     ch=letters[num],
                     fa=factor(letters[num]),
                     nu=(num) + 0.5,
                     id=factor(num), stringsAsFactors=FALSE)

  ## Combine all datasets
  tmp12 <- bindData(x=tmp1, y=tmp2, common=c("id", "nu", "ch", "fa"))
  tmp123 <- bindData(x=tmp12, y=tmp3, common=c("id", "nu", "ch", "fa"))

  checkEquals(names(tmp123), c("id", "nu", "ch", "fa", "y1", "f1", "y2", "f2", "y3", "f3"))
  checkEquals(rbind(tmp1["id"], tmp2["id"], tmp3["id"]), tmp123["id"])
  checkEquals(rbind(tmp1["fa"], tmp2["fa"], tmp3["fa"]), tmp123["fa"])
  checkEquals(is.na(tmp123$y1), c(rep(FALSE, times=n1), rep(TRUE, times=n2+n3)))
  checkEquals(is.na(tmp123$f1), c(rep(FALSE, times=n1), rep(TRUE, times=n2+n3)))
  checkEquals(is.na(tmp123$y2), c(rep(TRUE, times=n1), rep(FALSE, times=n2), rep(TRUE, times=n3)))
  checkEquals(is.na(tmp123$f2), c(rep(TRUE, times=n1), rep(FALSE, times=n2), rep(TRUE, times=n3)))
  checkEquals(is.na(tmp123$y3), c(rep(TRUE, times=n1+n2), rep(FALSE, times=n3)))
  checkEquals(is.na(tmp123$f3), c(rep(TRUE, times=n1+n2), rep(FALSE, times=n3)))
}

### }}}
### {{{ Dear Emacs
## Local variables:
## folded-file: t
## End:
### }}}

###------------------------------------------------------------------------
### runit.bindData.R ends here

### runit.mapLevels.R
###------------------------------------------------------------------------
### What: Unit tests for mapLevels et al.
### $Id: runit.mapLevels.R 993 2006-10-30 17:10:08Z ggorjan $
### Time-stamp: <2006-10-29 16:41:41 ggorjan>
###------------------------------------------------------------------------

### {{{ --- Test setup ---

if(FALSE) {
  library("RUnit")
  library("gdata")
}

### }}}
### {{{ mapLevels, is.*, as.*, [.*

test.mapLevels <- function()
{
  ## Integer and numeric
  checkException(mapLevels(1:3)) # wrong class(x)
  checkException(mapLevels(1.5)) # wrong class(x)

  ## Factor
  f <- factor(c("B", "C", "A"))
  fMapInt <- list(A=as.integer(1), B=as.integer(2), C=as.integer(3))
  fMapInt1 <- list(B=as.integer(1), C=as.integer(2))
  fMapCha <- list(A="A", B="B", C="C")
  fMapInt <- as.levelsMap(fMapInt)
  fMapInt1 <- as.levelsMap(fMapInt1)
  fMapCha <- as.levelsMap(fMapCha)
  fMapCha1 <- fMapCha[c(1, 3)] # this will test also [.levelsMap
  checkIdentical(mapLevels(f), fMapInt)
  checkTrue(is.levelsMap(mapLevels(f))) # test for is.levelsMap
  checkTrue(is.levelsMap(fMapInt))      # test for as.levelsMap
  checkTrue(!gdata:::.isCharacterMap(fMapInt))
  checkIdentical(mapLevels(f, sort=FALSE), fMapInt) # sort is not used for factors
  checkIdentical(mapLevels(f[1:2], drop=TRUE), fMapInt1)
  checkIdentical(mapLevels(f, codes=FALSE), fMapCha)
  checkIdentical(mapLevels(f[c(2, 3)], drop=TRUE, codes=FALSE), fMapCha1)

  ## Character
  cha <- c("Z", "M", "A")
  chaMapInt <- list(A=as.integer(1), M=as.integer(2), Z=as.integer(3))
  chaMapIntO <- list(Z=as.integer(1), M=as.integer(2), A=as.integer(3))
  chaMapInt1 <- list(M=as.integer(1), Z=as.integer(2))
  chaMapCha <- list(A="A", M="M", Z="Z")
  chaMapInt <- as.levelsMap(chaMapInt)
  chaMapIntO <- as.levelsMap(chaMapIntO)
  chaMapInt1 <- as.levelsMap(chaMapInt1)
  chaMapCha <- as.levelsMap(chaMapCha)
  checkIdentical(mapLevels(cha), chaMapInt)
  checkIdentical(mapLevels(cha, sort=FALSE), chaMapIntO) # sort works for characters
  checkIdentical(mapLevels(cha[1:2], drop=TRUE), chaMapInt1)
  checkIdentical(mapLevels(cha, codes=FALSE), chaMapCha)

  ## List
  l <- list(f=f, cha=cha)
  l1 <- list(cha=cha, f=f)
  l2 <- list(cha=cha, f=f, i=1:10)
  lMapInt <- list(f=fMapInt, cha=chaMapInt)
  lMapCha <- list(f=fMapCha, cha=chaMapCha)
  lMapInt <- as.listLevelsMap(lMapInt)
  lMapCha <- as.listLevelsMap(lMapCha)
  lMapChaC <- as.list(sort(unique(c(cha, as.character(f)))))
  lMapChaCO <- as.list(unique(c(cha, as.character(f))))
  names(lMapChaC) <- unlist(lMapChaC)
  names(lMapChaCO) <- unlist(lMapChaCO)
  lMapChaC <- as.levelsMap(lMapChaC)
  lMapChaCO <- as.levelsMap(lMapChaCO)
  checkIdentical(mapLevels(l), lMapInt)
  checkTrue(is.listLevelsMap(mapLevels(l))) # test for is.listLevelsMap
  checkTrue(is.listLevelsMap(lMapInt))      # test for as.listLevelsMap
  checkIdentical(mapLevels(l, codes=FALSE), lMapCha)
  checkException(mapLevels(l, combine=TRUE)) # can not combine integer maps
  checkIdentical(mapLevels(l, codes=FALSE, combine=TRUE), lMapChaC)
  checkIdentical(mapLevels(l1, codes=FALSE, combine=TRUE), lMapChaC)
  checkIdentical(mapLevels(l1, codes=FALSE, combine=TRUE, sort=FALSE), lMapChaCO)
  checkException(mapLevels(l2)) # only char and factor

  ## Data.frame
  df <- data.frame(f1=factor(c("G", "Abc", "Abc", "D", "F")),
                   f2=factor(c("Abc", "Abc", "B", "D", "K")),
                         cha=c("jkl", "A", "D", "K", "L"),
                   int=1:5)
  dfMapInt <- list(f1=mapLevels(df$f1), f2=mapLevels(df$f2), cha=mapLevels(df$cha))
  dfMapInt <- as.listLevelsMap(dfMapInt)
  dfMapInt1 <- dfMapInt[c(1, 3)] # this will test also [.listLevelsMap
  checkException(mapLevels(df)) # wrong class of int
  checkIdentical(mapLevels(df[, 1:3]), dfMapInt)
  checkIdentical(mapLevels(df[, c(1, 3)]), dfMapInt1)
}

### }}}
### {{{ .check*

test.checkLevelsMap <- function(x)
{
  ## --- levelsMap ---

  ## not a list
  checkException(gdata:::.checkLevelsMap(x="A", method="raw"))
  ## list without names
  checkException(gdata:::.checkLevelsMap(x=list("A"), method="raw"))
  fMapInt <- list(A=as.integer(1), B=as.integer(2), C=as.integer(3))
  ## x should be levelsMap
  checkException(gdata:::.checkLevelsMap(x=fMapInt, method="class"))

  ## --- listLevelsMap ---

  map <- list(as.levelsMap(fMapInt), as.levelsMap(fMapInt))
  map1 <- list(fMapInt, fMapInt)
  class(map1) <- "listLevelsMap"
  ## x should be a listLevelsMap
  checkException(gdata:::.checkListLevelsMap(x=map, method="class"))
  ## x should be also a list of levelsMaps
  checkException(gdata:::.checkListLevelsMap(x=map1, method="class"))
  ## the rest is done with levelsMap tests
}

### }}}
### {{{ c.*

test.cLevelsMap <- function()
{
  f1 <- factor(letters[c(2, 1)])
  f2 <- factor(letters[c(3, 1, 2)])
  mapCha1 <- mapLevels(f1, codes=FALSE) # get maps
  mapCha2 <- mapLevels(f2, codes=FALSE)
  mapCha1S <- mapLevels(as.character(f1), codes=FALSE, sort=FALSE)
  mapCha2S <- mapLevels(as.character(f2), codes=FALSE, sort=FALSE)
  mapChaTest <- list(a="a", b="b")
  mapChaTest1 <- list(a="a", b="b", c="c")
  mapChaTest2 <- list(c="c", a="a", b="b")
  class(mapChaTest) <- class(mapChaTest1) <- class(mapChaTest2) <- "levelsMap"
  mapChaTest3 <- list(mapChaTest, mapChaTest1, mapChaTest, mapChaTest1)
  class(mapChaTest3)  <- "listLevelsMap"
  checkIdentical(c(mapCha1), mapChaTest)
  checkIdentical(c(mapCha2, mapCha1), mapChaTest1)
  checkIdentical(c(mapCha2S, mapCha1S, sort=FALSE), mapChaTest2)

  l <- list(f1, f2)
  mapCha <- mapLevels(l, codes=FALSE)
  checkIdentical(c(mapCha, mapCha), mapChaTest3)
  checkIdentical(c(mapCha, recursive=TRUE), mapChaTest1)

  checkException(c(mapLevels(f1))) # can not combine integer “levelsMaps”

  ## Example with maps of different length of components
  map1 <- list(A=c("a", "e", "i", "o", "u"), B="b", C="c", C="m",
               D=c("d", "e"), F="f")
  map2 <- list(A=c("a", "z", "w", "y", "x"), F="f", G=c("g", "h", "j"),
               i="i", k=c("k", "l"), B="B")
  map0Test <- list(A=c("a", "e", "i", "o", "u"), B="b", C="c", C="m",
                   D=c("d", "e"), F="f",
                   A=c("z", "w", "y", "x"), G=c("g", "h", "j"),
                   i="i", k=c("k", "l"), B="B")
  map0Test <- as.levelsMap(map0Test)
  mapTest <- sort(map0Test)
  map1 <- as.levelsMap(map1)
  map2 <- as.levelsMap(map2)
  map <- c(map1, map2)
  map0 <- c(map1, map2, sort=FALSE)
  checkIdentical(map, mapTest)
  checkIdentical(map0, map0Test)
}

### }}}
### {{{ unique

test.uniqueLevelsMap <- function()
{
  map <- list(A=c(1, 2, 1, 3), B=4, C=1, C=5, D=c(6, 8), E=7, B=4,
              D=c(6, 8))
  map1 <- map
  map1[[1]] <- map[[1]][c(1, 2, 4)]
  map1[[7]] <- NULL # remove B=4
  map1[[7]] <- NULL # remove D=c(6, 8)
  ## unique (used in as.levelsMap), will remove duplicates (A=1)
  checkIdentical(as.levelsMap(map1), as.levelsMap(map))
}

### }}}
### {{{ mapLevels<-

"test.mapLevels<-" <- function()
{
  ## Some errors
  checkException("mapLevels<-"(1.1, value=2))  # wrong class(x)
  checkException("mapLevels<-"(complex(1.1), value=2)) # wrong class(x)

  f <- factor(c("A", "B", "C"))
  fMapInt <- mapLevels(f)
  ## can not apply integer "levelsMap" to "character"
  checkException("mapLevels<-"(as.character(f), value=fMapInt))

  fMapCha <- mapLevels(f, codes=FALSE)
  ## can not apply character levelsMap to "integer"
  checkException("mapLevels<-"(as.integer(f), value=chaMapCha))

  fMapFuzz <- fMapInt
  fMapFuzz[[1]] <- "A"
  ## all components of 'value' must be of the same class
  checkException("mapLevels<-"(as.character(f), value=fMapFuzz))
  checkException("mapLevels<-"(as.integer(f), value=fMapFuzz))

  ## x integer, value integer levelsMap
  f <- factor(letters[c(10, 15, 1, 2)])
  fMapInt <- mapLevels(f)
  fInt <- as.integer(f)
  mapLevels(fInt) <- fMapInt
  checkIdentical(fInt, f)

  ## x factor, value integer levelsMap
  fInt <- factor(as.integer(f))
  mapLevels(fInt) <- fMapInt
  checkIdentical(fInt, f)

  ## above is essentially the same as levels<-.factor
  fInt1 <- factor(as.integer(f))
  levels(fInt1) <- fMapInt
  checkIdentical(fInt1, f)

  ## x character, value character levelsMap
  cha <- c("B", "A", "C")
  chaMapCha <- as.levelsMap(list(A1="A", B2="B", C3="C"))
  mapLevels(cha) <- chaMapCha
  chaTest <- factor(c("B2", "A1", "C3"))
  checkIdentical(cha, chaTest)
  ## and a bit more for components of length > 1
  cha <- c("G", "I", "B", "A", "C", "D", "Z")
  chaMapCha <- as.levelsMap(list(A1=c("A", "G", "I"), B2="B", C3=c("C", "D")))
  mapLevels(cha) <- chaMapCha
  chaTest <- factor(c("A1", "A1", "B2", "A1", "C3", "C3", NA))
  checkIdentical(cha, chaTest)

  ## x factor, value character levelsMap
  f <- factor(c("G", "I", "B", "A", "C", "D", "Z"))
  fMapCha <- as.levelsMap(list(A1=c("A", "G", "I"), B2="B", C3=c("C", "D")))
  mapLevels(f) <- fMapCha
  fTest <- factor(c("A1", "A1", "B2", "A1", "C3", "C3", NA))
  checkIdentical(f, fTest)

  ## Two factors and character map
  f1 <- factor(letters[1:10])
  f2 <- factor(letters[5:14])
  checkIdentical(as.integer(f1), as.integer(f2)) # the same integer codes
  mapCha1 <- mapLevels(f1, codes=FALSE) # get maps
  mapCha2 <- mapLevels(f2, codes=FALSE)
  mapCha <- c(mapCha1, mapCha2)         # combine maps
  ## apply map
  mapLevels(f1) <- mapCha # the same as levels(f1) <- mapCha
  mapLevels(f2) <- mapCha # the same as levels(f2) <- mapCha
  checkIdentical(as.integer(f1), 1:10) # \ internal codes are now
  checkIdentical(as.integer(f2), 5:14) # / "consistent" among factors

  ## The same with list
  l <- list(f1=f1, f2=f2)
  mapCha <- mapLevels(l, codes=FALSE, combine=TRUE)
  mapLevels(l) <- mapCha
  checkIdentical(as.integer(l$f1), 1:10) # \ internal codes are now
  checkIdentical(as.integer(l$f2), 5:14) # / "consistent" among factors

  ## and data.frame
  df <- data.frame(f1=f1, f2=f2)
  mapCha <- mapLevels(df, codes=FALSE, combine=TRUE)
  mapLevels(df) <- mapCha
  checkIdentical(as.integer(df$f1), 1:10) # \ internal codes are now
  checkIdentical(as.integer(df$f2), 5:14) # / "consistent" among factors

}

### }}}
### {{{ Dear Emacs
## Local variables:
## folded-file: t
## End:
### }}}

###------------------------------------------------------------------------
### runit.mapLevels.R ends here

case <- function(x, ..., default=NA)
{
  magic <- "....default...."
  alternatives <- c(..., "....default...."=magic)

  x <- as.character(x)
  retval <- factor(x, levels=alternatives, labels=names(alternatives))
  levels(retval)[length(alternatives)] <- as.character(default)
  retval[is.na(retval) & !is.na(x)] <- default

  retval
}

reorder.factor <- function(x, X, FUN, ..., order=is.ordered(x), new.order,
                           sort=mixedsort)
{
  constructor <- if (order) ordered else factor

  if(!missing(X) || !missing(FUN))
  {
    if(missing(FUN)) FUN <- 'mean'

    ## I would prefer to call stats::reorder.default directly,
    ## but it is not exported from stats, so the relevant code is
    ## replicated here:
    ## -->
    scores <- tapply(X = X, INDEX = x, FUN = FUN, ...)
    levels <- names(base::sort(scores, na.last = TRUE))
    if(order)
      ans <- ordered(x, levels=levels)
    else
      ans <- factor(x, levels=levels)
    attr(ans, "scores") <- scores
    ## <--
    return(ans)
  }
  else if (!missing(new.order))
  {
    if (is.numeric(new.order))
      new.order <- levels(x)[new.order]
    else
      new.order <- new.order
  }
  else
    new.order <- sort(levels(x))

  constructor(x, levels=new.order)
}

## s$Id: read.xls.R 1423 2010-02-21 17:12:30Z ggrothendieck2 $

installXLSXsupport <- function(perl="perl", verbose=FALSE)
{
  ## determine proper path to perl executable
  perl <- if (missing(perl))
    findPerl(verbose = verbose)
  else
    findPerl(perl, verbose = verbose)
  
  ##
  ## directories
  package.dir <- find.package('gdata')
  perl.dir <- file.path(package.dir,'perl')
  ##
  ##

  cmd <- "install_modules.pl"
  sc <- file.path(perl.dir, cmd)
  
  ##
  ##

  ##
  ## execution command

  cmd <- paste(shQuote(perl), shQuote(sc), sep=" ")

  ##

  if(verbose)
    {
      cat("\n")
      cat("Attempting to automaticall install Perl libraries to support XLSX (Excel 2007+) file format...\n")
      cat("\n")
    }

  ##

  output <- system(cmd, intern=TRUE)

  ##

  if(verbose) cat("Results: ", output, "\n")

  ##

  if( "XLSX" %in% xlsFormats(perl=perl, verbose=verbose) )
	{
		cat("\nPerl XLSX support libraries successfully installed.\n\n")
		invisible(TRUE)
	}
  else
	{
        	stop("\nUnable to install Perl XLSX support libraries.\n\n")
		invisible(FALSE)                
	}		
}

## s$Id: xls2sep.R 1639 2013-01-14 20:47:57Z warnes $

xls2csv <- function(xls, sheet=1, verbose=FALSE, blank.lines.skip=TRUE,
                    ..., perl="perl")
  xls2sep(xls=xls, sheet=sheet, verbose=verbose,
          blank.lines.skip=blank.lines.skip, ..., method="csv",
          perl=perl)

xls2tab <- function(xls, sheet=1, verbose=FALSE, blank.lines.skip=TRUE,
                    ..., perl="perl")
  xls2sep(xls=xls, sheet=sheet, verbose=verbose,
          blank.lines.skip=blank.lines.skip, ..., method="tab",
          perl=perl) 

xls2tsv <- function(xls, sheet=1, verbose=FALSE, blank.lines.skip=TRUE,
                    ..., perl="perl")
  xls2sep(xls=xls, sheet=sheet, verbose=verbose,
          blank.lines.skip=blank.lines.skip, ..., method="tsv",
          perl=perl) 

xls2sep <- function(xls,
                    sheet=1,
                    verbose=FALSE,
                    blank.lines.skip=TRUE,
                    ...,
                    method=c("csv","tsv","tab"),
                    perl = perl)
  {
    
    method <- match.arg(method)
    
	perl <- if (missing(perl))
		findPerl(verbose = verbose)
	else
		findPerl(perl, verbose = verbose)

    ##
    ## directories
    package.dir <- find.package('gdata')
    perl.dir <- file.path(package.dir,'perl')
    ##

    ##
    ## filesheet
    tf <- NULL
    if ( substring(xls, 1, 7) == "http://" ||
         substring(xls, 1, 6) == "ftp://" )
      {
        tf <- paste(tempfile(), "xls", sep = ".")
        if(verbose)
          cat("Downloading",
              dQuote(xls), " to ",
              dQuote(tf), "...\n")
        download.file(xls, tf, mode = "wb")
        if(verbose) cat("Done.\n")
        xls <- tf
      }

    if(method=="csv")
      {
        script <- file.path(perl.dir,'xls2csv.pl')
        targetFile <- paste(tempfile(), "csv", sep = ".")
      }
    else if(method=="tab")
      {
        script <- file.path(perl.dir,'xls2tab.pl')
        targetFile <- paste(tempfile(), "tab", sep = ".")
      }
    else if(method=="tsv")
      {
        script <- file.path(perl.dir,'xls2tsv.pl')
        targetFile <- paste(tempfile(), "tsv", sep = ".")
      }
    else
      {
        stop("Unknown method", method)
      }
    
    ##
    ##

    ##
    ## blank.lines.skip
    ##
    if (blank.lines.skip)
      skipBlank=""
    else
      skipBlank="-s"

    ##
    ## execution command
    cmd <- paste(shQuote(perl),
                 shQuote(script),
                 skipBlank,  # flag is not quoted
                 shQuote(xls),
                 shQuote(targetFile),
                 shQuote(sheet),
                 sep=" ")
    ##
    ##

    if(verbose)
      {
        cat("\n")
        cat("Converting xls file\n")
        cat("   ", dQuote(xls), "\n")
        cat("to", method, " file \n")
        cat("   ", dQuote(targetFile), "\n")
        cat("... \n\n")
      }
    
    ##
    ## do the translation
    if(verbose)  cat("Executing '", cmd, "'... \n\n")

    results <- try(system(cmd, intern=!verbose))

    if(inherits(results, "try-error"))
      stop( "Unable to read xls file '", xls, "':", results )
    
    if(verbose) cat(results,"\n\n")
    if (verbose) cat("Done.\n\n")

    ##
    ## check that the target file was created
    ##
    if(!file.exists(targetFile))
      stop( "Intermediate file '", targetFile, "' missing!" )

    ## Creae a file object to hand to the next stage..
    retval <- try(file(targetFile))
    if(inherits(retval, "try-error"))

      stop("Unable to open intermediate file '", targetFile, "':",
           retval)
    
    return(retval)
    
  }

sheetCount <- function(xls, verbose = FALSE, perl = "perl") {
  perl <- if (missing(perl)) findPerl(verbose = verbose)
  else findPerl(perl, verbose = verbose)
  sheetCmd(xls, cmd="sheetCount.pl", verbose=verbose, perl=perl)
}
  
sheetNames <- function(xls, verbose = FALSE, perl = "perl") {
  perl <- if (missing(perl)) findPerl(verbose = verbose)
  else findPerl(perl, verbose = verbose)
  sheetCmd(xls, cmd="sheetNames.pl", verbose=verbose, perl=perl)
}
  
sheetCmd <- function(xls, cmd="sheetCount.pl", verbose=FALSE, perl="perl")
{

  ##
  ## directories
  package.dir <- find.package('gdata')
  perl.dir <- file.path(package.dir,'perl')
  ##
  ##

  ##
  ## files
  tf <- NULL
  if ( substring(xls, 1, 7) == "http://" ||
      substring(xls, 1, 6) == "ftp://" )
    {
      tf <- paste(tempfile(), "xls", sep = ".")
      if(verbose)
        cat("Downloading",
            dQuote(xls), " to ",
            dQuote(tf), "...\n")
      else
        cat("Downloading...\n")
      download.file(xls, tf, mode = "wb")
      cat("Done.\n")
      xls <- tf
    }
  ##
  
  sc <- file.path(perl.dir, cmd)
  
  ##
  ##

  ##
  ## execution command

  cmd <- paste(shQuote(perl), shQuote(sc), shQuote(xls), sep=" ")
  ##
  ##

  ##
  ## do the translation
  if(verbose)
    {
      cat("\n")
      cat("Extracting sheet information from\n")
      cat("   ", dQuote(xls), "\n")
      cat("... \n\n")
    }
  ##
  output <- system(cmd, intern=TRUE, ignore.stderr=TRUE)
  if(verbose) cat("Results: ", output, "\n")
  ##
  tc <- textConnection(output)
  results <- read.table(tc, as.is=TRUE, header=FALSE)
  close(tc)
  results <- unlist(results)
  names(results) <- NULL
  ##
  if (verbose) cat("Done.\n\n")
  
  results
}

update.list <- function(object, new, unnamed=FALSE, ...)
{
  retval <- object

  for(name in names(new))
    retval[[name]] <- new[[name]]

  if(unnamed)
  {
    if(is.null(names(new)))
      names(new) <- rep("", length=length(new))
    for(i in (1:length(new))[names(new)==""])
      retval <- append(retval, new[[i]])
  }

  retval
}

ll <- function(pos=1, unit="KB", digits=0, dim=FALSE, sort=FALSE, class=NULL,
               invert=FALSE, ...)
{
  get.object.class <- function(object.name, pos)
  {
    object <- get(object.name, pos=pos)
    class <- class(object)[1]
    return(class)
  }

  get.object.dim <- function(object.name, pos)
  {
    object <- get(object.name, pos=pos)
    if(class(object)[1] == "function")
      dim <- ""
    else if(!is.null(dim(object)))
      dim <- paste(dim(object), collapse=" x ")
    else
      dim <- length(object)
    return(dim)
  }

  get.object.size <- function(object.name, pos)
  {
    object <- get(object.name, pos=pos)
    size <- try(unclass(object.size(object)), silent=TRUE)
    if(inherits(size,"try-error"))
      size <- 0
    return(size)
  }

  ## 1  Set unit, denominator, original.rank
  unit <- match.arg(unit, c("bytes","KB","MB"))
  denominator <- switch(unit, "KB"=1024, "MB"=1024^2, 1)
  original.rank <- NULL

  ## 2  Detect what 'pos' is like, then get class, size, dim
  if(is.character(pos))  # pos is an environment name
    pos <- match(pos, search())
  if(is.list(pos))  # pos is a list-like object
  {
    if(is.null(names(pos)))
      stop("All elements of a list must be named")
    original.rank <- rank(names(pos))
    pos <- as.environment(pos)
  }
  if(length(ls(pos,...)) == 0)  # pos is an empty environment
  {
    object.frame <- data.frame()
  }
  else if(environmentName(as.environment(pos)) == "Autoloads")
  {
    object.frame <- data.frame(rep("function",length(ls(pos,...))),
                               rep(0,length(ls(pos,...))),
                               row.names=ls(pos,...))
    if(dim)
    {
      object.frame <- cbind(object.frame, rep("",nrow(object.frame)))
      names(object.frame) <- c("Class", unit, "Dim")
    }
    else
      names(object.frame) <- c("Class", unit)
  }
  else
  {
    class.vector <- sapply(ls(pos,...), get.object.class, pos=pos)
    size.vector <- sapply(ls(pos,...), get.object.size, pos=pos)
    size.vector <- round(size.vector/denominator, digits)
    object.frame <- data.frame(class.vector=class.vector,
                               size.vector=size.vector,
                               row.names=names(size.vector))
    names(object.frame) <- c("Class", unit)
    if(dim)
      object.frame <- cbind(object.frame,
                            Dim=sapply(ls(pos,...),get.object.dim,pos=pos))
  }

  ## 3  Retain original order of list elements
  if(!sort && !is.null(original.rank))
    object.frame <- object.frame[original.rank,]

  ## 4  Filter results given class
  if(!is.null(class))
  {
    include <- object.frame$Class %in% class
    if(invert)
      include <- !include
    object.frame <- object.frame[include,]
  }

  return(object.frame)
}

rename.vars <- function(data, from="", to="", info=TRUE)
{
  dsn <- deparse(substitute(data))
  dfn <- names(data)

  if (length(from) != length(to)) {
    cat('--------- from and to not same length ---------\n')
    stop()
  }

  if (length(dfn) < length(to)) {
    cat('--------- too many new names ---------\n')
    stop()
  }

  chng <- match(from, dfn)

  frm.in <- from %in% dfn
  if (!all(frm.in)) {
    cat('---------- some of the from names not found in', dsn, '\n')
    stop()
  }

  if (length(to) != length(unique(to))) {
    cat('---------- New names not unique\n')
    stop()
  }

  dfn.new <- dfn
  dfn.new[chng] <- to
  if (info)
    cat('\nChanging in', dsn)
  tmp <- rbind(from, to)
  dimnames(tmp)[[1]] <- c('From:', 'To:')
  dimnames(tmp)[[2]] <- rep('', length(from))
  if (info)
  {
    print(tmp,quote=FALSE)
    cat("\n")
  }
  names(data) <- dfn.new
  data
}

remove.vars <- function(data, names, info=TRUE)
{
  dsn <- deparse(substitute(data))
  if (info) cat('\nChanging in', dsn, "\n")

  flag <- names %in% colnames(data)
  if(any(!flag))
    warning("Variable(s) not found: ", paste(names[!flag], collapse=", "))
  if(any(flag))
  {
    if(info)
      cat("Dropping variables:", paste(names[flag], collapse=", "), "\n\n")
    for(var in names[flag])
      data[[var]] <- NULL
  }
  data
}

### runit.trim.R
###------------------------------------------------------------------------
### What: Tests for trim
### $Id: runit.trim.R 1784 2014-04-05 02:23:45Z warnes $
### Time-stamp: <2006-08-29 14:21:02 ggorjan>
###------------------------------------------------------------------------

### {{{ --- Test setup ---

if(FALSE) {
  library("RUnit")
  library("gdata")
}

### }}}
### {{{ --- trim ---

test.trim <- function()
{
  tmp <- Sys.getlocale(category="LC_COLLATE")
  Sys.setlocale(category="LC_COLLATE", locale="C")

  sTrim <- "    this is an example string    "
  sTrimR <- "this is an example string"

  fTrim <- factor(c(sTrim, sTrim, " A", " B ", "  C ", "D "))
  fTrimR <- factor(c(sTrimR, sTrimR, "A", "B", "C", "D"))

  lTrim <- list(s=rep(sTrim, times=6), f=fTrim, i=1:6)
  lTrimR <- list(s=rep(sTrimR, times=6), f=fTrimR, i=1:6)

  dfTrim <- as.data.frame(lTrim)
  dfTrimR <- as.data.frame(lTrimR)

  checkIdentical(trim(sTrim), sTrimR)
  checkIdentical(trim(fTrim), fTrimR)
  checkIdentical(
                 levels(trim(fTrim, recode.factor=FALSE)),
                 c("this is an example string", "C", "A", "B", "D")
                 )
  checkIdentical(trim(lTrim), lTrimR)
  checkIdentical(trim(dfTrim), dfTrimR)

  Sys.setlocale(category="LC_COLLATE", locale=tmp)  
}

### }}}
### {{{ Dear Emacs
## Local variables:
## folded-file: t
## End:
### }}}

###------------------------------------------------------------------------
### runit.trim.R ends here

left  <- function(x, n=6L, ...) UseMethod("left")
right <- function(x, n=6L, ...) UseMethod("right")

.leftright <- function(x, n=6L, add.col.nums = TRUE,
                       direction = c("left", "right"), ...)
{
  stopifnot(length(n) == 1L)

  ncx <- ncol(x)

  if (add.col.nums && is.null(colnames(x)))
    colnames(x) <- paste0("[", 1:ncx, ",]")

  if (n < 0L)
    n <- max(ncx + n, 0L)
  else
    n <- min(n, ncx)

  if (direction == "left")
    sel <- seq.int(from=1, length.out = n)
  else  # direction="right"
    sel <- seq.int(to = ncx, length.out = n)

  x[, sel, drop = FALSE]
}

left.data.frame <- function(x, n=6L, add.col.nums=TRUE, ...)
  .leftright(x, n=n, add.col.nums=add.col.nums, direction="left")

right.data.frame <- function(x, n=6L, add.col.nums=TRUE, ...)
  .leftright(x, n=n, add.col.nums=add.col.nums, direction="right")

left.matrix <- left.data.frame
right.matrix <- right.data.frame

### runit.trim.R
###------------------------------------------------------------------------
### What: Tests for trim
### $Id: runit.trim.R 993 2006-10-30 17:10:08Z ggorjan $
### Time-stamp: <2006-08-29 14:21:02 ggorjan>
###------------------------------------------------------------------------

### {{{ --- Test setup ---

if(FALSE) {
  library("RUnit")
  library("gdata")
}

### }}}
### {{{ --- trim ---

test.trim <- function()
{
  tmp <- Sys.getlocale(category="LC_COLLATE")
  Sys.setlocale(category="LC_COLLATE", locale="C")

  sTrim <- "    this is an example string    "
  sTrimR <- "this is an example string"

  fTrim <- factor(c(sTrim, sTrim, " A", " B ", "  C ", "D "))
  fTrimR <- factor(c(sTrimR, sTrimR, "A", "B", "C", "D"))

  lTrim <- list(s=rep(sTrim, times=6), f=fTrim, i=1:6)
  lTrimR <- list(s=rep(sTrimR, times=6), f=fTrimR, i=1:6)

  dfTrim <- as.data.frame(lTrim)
  dfTrimR <- as.data.frame(lTrimR)

  checkIdentical(trim(sTrim), sTrimR)
  checkIdentical(trim(fTrim), fTrimR)
  checkIdentical(
                 levels(trim(fTrim, recode.factor=FALSE)),
                 c("this is an example string", "C", "A", "B", "D")
                 )
  checkIdentical(trim(lTrim), lTrimR)
  checkIdentical(trim(dfTrim), dfTrimR)

  Sys.setlocale(category="LC_COLLATE", locale=tmp)  
}

### }}}
### {{{ Dear Emacs
## Local variables:
## folded-file: t
## End:
### }}}

###------------------------------------------------------------------------
### runit.trim.R ends here

env <- function(unit="KB", digits=0)
{
  get.object.size <- function(object.name, pos)
  {
    object <- get(object.name, pos=pos)
    size <- try(unclass(object.size(object)), silent=TRUE)
    if(inherits(size, "try-error"))
      size <- 0
    return(size)
  }

  get.environment.size <- function(pos)
  {
    if(search()[pos]=="Autoloads" || length(ls(pos,all.names=TRUE))==0)
      size <- 0
    else
      size <- sum(sapply(ls(pos,all.names=TRUE), get.object.size, pos=pos))
    return(size)
  }

  get.environment.nobjects <- function(pos)
  {
    nobjects <- length(ls(pos,all.names=TRUE))
    return(nobjects)
  }

  unit <- match.arg(unit, c("bytes","KB","MB"))
  denominator <- switch(unit, "KB"=1024, "MB"=1024^2, 1)
  size.vector <- sapply(seq(along=search()), get.environment.size)
  size.vector <- round(size.vector/denominator, digits)
  nobjects.vector <- sapply(seq(along=search()), get.environment.nobjects)
  env.frame <- data.frame(search(), nobjects.vector, size.vector)
  names(env.frame) <- c("Environment", "Objects", unit)

  print(env.frame, right=FALSE)
  invisible(env.frame)
}

## Redefine here, so that the locally defined methods (particularly
## nobs.default) take precedence over the ones now defined in the
## stats package

nobs <- function(object, ...)
  UseMethod("nobs")

nobs.default <- function(object, ...)
{
  if(is.numeric(object) || is.logical(object))
    sum(!is.na(object))
  else
    stats::nobs(object, ...)
}


nobs.data.frame <- function(object, ...)
  sapply(object, nobs.default)

## Now provided by 'stats' package, so provide alias to satisfy
## dependencies
nobs.lm <- stats:::nobs.lm

n_obs <- function(object, ...) nobs(object, ...)

ConvertMedUnits <- function(x, measurement, abbreviation,
                            to=c("Conventional","SI","US"),
                            exact=!missing(abbreviation))
{
  MedUnits <- NULL  # define to avoid R CMD check warning
  data(MedUnits, package='gdata', envir=environment())

  to=match.arg(to)
  if(!missing(measurement) && missing(abbreviation))
  {
    if(exact)
      matchUnits <-
        MedUnits[tolower(MedUnits$Measurement) == tolower(measurement),]
    else
      matchUnits <- MedUnits[grep(measurement, MedUnits$Measurement,
                                  ignore.case=TRUE),]
  }
  else if(missing(measurement) && !missing(abbreviation))
  {
    if(exact)
      matchUnits <-
        MedUnits[tolower(MedUnits$Abbreviation) == tolower(abbreviation),]
    else
      matchUnits <- MedUnits[grep(match, MedUnits$Abbrevation,
                                  ignore.case=TRUE),]
  }
  else  # both missing or both specified
    stop("one of `measurement' or `abbreviation' must be specified")

  if(nrow(matchUnits) > 1)
    stop(
      paste("More than one matching row.  Please use 'exact=TRUE' ",
            "and supply one of these matching strings:",
            paste('\t"',matchUnits$Measurement, '"', sep='', collapse="\n\t"),
            sep="\n\t"))
  else if (nrow(matchUnits) < 1)
    stop("No match")

  if (to %in% c("Convetional", "US"))
  {
    retval <- x / matchUnits$Conversion
    attr(retval,"units") <- matchUnits$ConventionalUnits
  }
  else
  {
    retval <- x * matchUnits$Conversion
    attr(retval,"units") <- matchUnits$SIUnits
  }
  retval
}

wideByFactor <- function(x, factor, common, sort=TRUE, keepFactor=TRUE)
{
  ## Setup
  if(!is.data.frame(x))
    stop("'x' must be a data frame")
  if(length(factor) != 1)
    stop("'factor' can be only of length one")
  if(!is.factor(x[[factor]]))
    stop("column defined in 'factor' must be a factor")
  if(sort) x <- x[order(x[[factor]]), ]

  ## Extend by factors levels
  y <- x[common]
  if(keepFactor) y[factor] <- x[factor]
  levs <- levels(x[[factor]])

  ## Remove common and factor from the list of column names
  other <- names(x)
  other <- other[!(other %in% common) & !(other %in% factor)]

  ## Add all other columns but as a set for each level of a factor
  for(level in levs) {
    for(col in other) {
      ## Add a column col
      y[paste(col, level, sep=".")] <- x[col]
      ## Fill with NA for other levels than level
      y[x[factor] != level, paste(col, level, sep=".")] <- NA
      ## This filling might be inefficient if there is large number
      ## of levels, since there will be quite a lot of filling.
    }
  }
  y
}

## Test setup
if(FALSE) {
  library("RUnit")
  library("gdata")
}

test.wideByFactor <- function()
{
  n <- 10
  f <- 2
  tmp <- data.frame(y1=(1:n)/2, y2=(n:1)*2,
                    f1=factor(rep(letters[1:f], n/2)),
                    f2=factor(c(rep(c("M"), n/2), rep(c("F"), n/2))),
                    c1=1:n, c2=2*(1:n))

  ## 'x' must be a data.frame
  checkException(wideByFactor(x=1:10))
  checkException(wideByFactor(x=matrix(1:10)))
  ## 'factor' can be only of length one
  checkException(wideByFactor(x=tmp, factor=c("f1", "f2")))
  ## Column defined in 'factor' must be a factor
  checkException(wideByFactor(x=tmp, factor="c1"))

  tmp2 <- wideByFactor(x=tmp, factor="f1", common=c("c1", "c2"), sort=FALSE)
  checkEquals(tmp2[c("c1", "c2")], tmp[c("c1", "c2")])
  checkEquals(names(tmp2), c("c1", "c2", "f1", "y1.a", "y2.a", "f2.a", "y1.b", "y2.b", "f2.b"))
  checkEquals(tmp2$y1.a, c(0.5, NA, 1.5, NA, 2.5, NA, 3.5, NA, 4.5, NA))
  checkEquals(tmp2$f2.a, factor(c("M", NA, "M", NA, "M", NA, "F", NA, "F", NA)))
  tmp2 <- wideByFactor(x=tmp, factor="f1", common=c("c1", "c2"), sort=TRUE, keepFactor=FALSE)
  checkEquals(tmp2$f2.a, factor(c("M", "M", "M", "F", "F", NA, NA, NA, NA, NA)))
  checkEquals(names(tmp2), c("c1", "c2", "y1.a", "y2.a", "f2.a", "y1.b", "y2.b", "f2.b"))
}

### runit.wideByFactor.R
###------------------------------------------------------------------------
### What: Reshape by factor levels - unit tests
### $Id$
### Time-stamp: <2008-12-30 11:58:50 ggorjan>
###------------------------------------------------------------------------

### {{{ --- Test setup ---

if(FALSE) {
  library("RUnit")
  library("gdata")
}

### }}}
### {{{ --- wideByFactor ---

test.wideByFactor <- function()
{
  n <- 10
  f <- 2
  tmp <- data.frame(y1=(1:n)/2,
                    y2=(n:1)*2,
                    f1=factor(rep(letters[1:f], n/2)),
                    f2=factor(c(rep(c("M"), n/2), rep(c("F"), n/2))),
                    c1=1:n,
                    c2=2*(1:n))
  
  ## 'x' must be a data.frame
  checkException(wideByFactor(x=1:10))
  checkException(wideByFactor(x=matrix(1:10)))
  ## 'factor' can be only of length one
  checkException(wideByFactor(x=tmp, factor=c("f1", "f2")))
  ## column defined in 'factor' must be a factor
  checkException(wideByFactor(x=tmp, factor="c1"))

  tmp2 <- wideByFactor(x=tmp, factor="f1", common=c("c1", "c2"), sort=FALSE)
  checkEquals(tmp2[c("c1", "c2")], tmp[c("c1", "c2")])
  checkEquals(names(tmp2), c("c1", "c2", "f1", "y1.a", "y2.a", "f2.a", "y1.b", "y2.b", "f2.b"))
  checkEquals(tmp2$y1.a, c(0.5, NA, 1.5, NA, 2.5, NA, 3.5, NA, 4.5, NA))
  checkEquals(tmp2$f2.a, factor(c("M", NA, "M", NA, "M", NA, "F", NA, "F", NA)))
  tmp2 <- wideByFactor(x=tmp, factor="f1", common=c("c1", "c2"), sort=TRUE, keepFactor=FALSE)
  checkEquals(tmp2$f2.a, factor(c("M", "M", "M", "F", "F", NA, NA, NA, NA, NA)))
  checkEquals(names(tmp2), c("c1", "c2", "y1.a", "y2.a", "f2.a", "y1.b", "y2.b", "f2.b"))
}

### }}}
### {{{ Dear Emacs
## Local variables:
## folded-file: t
## End:
### }}}

###------------------------------------------------------------------------
### runit.wideByFactor.R ends here

startsWith <- function(x, prefix, trim=FALSE, ignore.case=FALSE)
{
  if(trim) x <- trim(x)

  if(ignore.case)
  {
    x <- toupper(x)
    prefix <- toupper(prefix)
  }

  base::startsWith(x, prefix)
}

starts_with <- function(...) startsWith(...)

## The base R 'sample' function behaves differently if it is passed a
## sampling vector of length 1 than if it is passed a
## vector of length greater than 1.  For the 1-element
## case it samples from the list 1:x, instead of from the contents
## of x.  This function remove the special case: it always samples from
## the provided argument, no matter the length.

resample <- function(x, size, replace = FALSE, prob = NULL)
{
  if(length(x)<1)
  {
    if(!missing(size) && size>0)
      stop("Requested sample of size ", size, " from list of length 0")
    else
      x[FALSE]
  }
  else if(length(x)==1)
  {
    if(missing(size) || size==1)
      x
    else if(size>=1 && replace==TRUE)
      rep(x, size)
    else if(size < 1)
      x[FALSE]
    else
      stop("cannot cannot take a sample larger than the population",
           " when 'replace = FALSE'")
  }
  else
  {
    sample(x, size, replace, prob)
  }
}

### unknown.R
###------------------------------------------------------------------------
### What: Change given unknown value to NA and vice versa
### $Id: unknown.R 1797 2014-04-05 18:19:49Z warnes $
### Time-stamp: <2007-04-26 13:16:10 ggorjan>
###------------------------------------------------------------------------

### {{{ isUnknown

###------------------------------------------------------------------------

isUnknown <- function(x, unknown=NA, ...)
  UseMethod("isUnknown")

isUnknown.default <- function(x, unknown=NA, ...)
{
  if(is.list(unknown)) unknown <- unlist(unknown)
  ret <- x %in% unknown
  if(any(is.na(unknown))) ret <- ret | is.na(x)
  ret
}

isUnknown.POSIXlt <- function(x, unknown=NA, ...)
{
  ## FIXME: codetools say
  ## isUnknown.POSIXlt: wrong number of arguments to as.character
  if(is.list(unknown) && !inherits(x=unknown, what="POSIXlt")) {
    unknown <- lapply(unknown, FUN=as.character, ...)
  } else {
    unknown <- as.character(x=unknown, ...)
  }

  if(is.list(x) && !inherits(x=x, what="POSIXlt")) {
    x <- lapply(x, FUN=as.character, ...)
  } else {
    x <- as.character(x=x, ...)
  }

  isUnknown.default(x=as.character(x), unknown=as.character(unknown))
}

isUnknown.list <- function(x, unknown=NA, ...) {
  unknown <- .unknownList(x=x, unknown=unknown)
  x <- mapply(FUN="isUnknown", x=x, unknown=unknown, ..., SIMPLIFY=FALSE)
  x
}

isUnknown.data.frame <- function(x, unknown=NA, ...)
{
  x[] <- isUnknown.list(x, unknown=unknown, ...)
  x
}

isUnknown.matrix <- function(x, unknown=NA, ...)
  apply(X=x, MARGIN=ifelse(ncol(x) > nrow(x), 1, 2), FUN=isUnknown,
        unknown=unknown)

### }}}
### {{{ unknownToNA

###------------------------------------------------------------------------

unknownToNA <- function(x, unknown, warning=FALSE, ...)
  UseMethod("unknownToNA")

unknownToNA.default <- function(x, unknown, warning=FALSE, ...)
{
  if(warning) {
    if(any(is.na(x)))
      warning("'x' already has NA")
  }
  is.na(x) <- isUnknown(x=x, unknown=unknown)
  x
}

unknownToNA.factor <- function(x, unknown, warning=FALSE, ...)
{
  ## could put this func into default method, but I need unlisted unknown
  ## for levels handling
  if(warning) {
    if(any(is.na(x)))
      warning("'x' already has NA")
  }
  if(is.list(unknown)) unknown <- unlist(unknown)
  ## Levels handling - read help page on this
  levs <- levels(x)
  levs <- levs[!(levs %in% unknown)]
  factor(x, levels=levs)
}

unknownToNA.list <- function(x, unknown, warning=FALSE, ...)
{
  unknown <- .unknownList(x=x, unknown=unknown)
  x <- mapply(FUN="unknownToNA", x=x, unknown=unknown, warning=warning,
              SIMPLIFY=FALSE)
  return(x)
}

unknownToNA.data.frame <- function(x, unknown, warning=FALSE, ...)
{
  x[] <- unknownToNA.list(x=x, unknown=unknown, warning=warning)
  x
}

### }}}
### {{{ NAToUnknown

###------------------------------------------------------------------------

NAToUnknown <- function(x, unknown, force=FALSE, call.=FALSE, ...)
  UseMethod("NAToUnknown")

NAToUnknown.default <- function(x, unknown, force=FALSE, call.=FALSE, ...)
{
  if(length(as.character(unknown)) != 1) # as.character allows also POSIXlt
    stop("'unknown' must be a single value")
  if(any(isUnknown(x, unknown=unknown)) && !force)
    stop(sprintf("'x' already has value %s", dQuote(unknown)))
  classX <- class(x)[1]
  classUnk <- class(unknown)[1]
  if(classX != classUnk) {
    tmp <- c("integer", "numeric")
    if(!(classX %in% tmp && classUnk %in% tmp)) {
      warning(sprintf("'unknown' should be %s for %s 'x' - will try to coerce",
                      dQuote(classX), dQuote(classX)), call.=call.)
    }
    unknown <- do.call(paste("as.", classX, sep=""), args=list(unknown))
  }
  x[is.na(x)] <- unknown
  x
}

NAToUnknown.factor <- function(x, unknown, force=FALSE, call.=FALSE, ...)
{
  if(length(unknown) != 1)
    stop("'unknown' must be a single value")
  if(any(isUnknown(x, unknown=unknown))) {
    if(!force) stop(sprintf("'x' already has level %s", dQuote(unknown)))
  } else {
    mapLevels(x) <- c(mapLevels(x, codes=FALSE),
                      mapLevels(as.character(unknown), codes=FALSE))
  }
  x[is.na(x)] <- unknown
  if(!force)
    warning(sprintf("new level is introduced: %s", unknown), call.=call.)
  x
}

NAToUnknown.list <- function(x, unknown, force=FALSE, call.=FALSE, ...)
{
  unknown <- .unknownList(x=x, unknown=unknown)
  x <- mapply(FUN="NAToUnknown", x=x, unknown=unknown, force=force,
              call.=call., SIMPLIFY=FALSE)
  x
}

NAToUnknown.data.frame <- function(x, unknown, force=FALSE, call.=FALSE, ...)
{
  x[] <- NAToUnknown.list(x=x, unknown=unknown, force=force, call.=call.)
  x
}

### }}}
### {{{ .unknownList
###------------------------------------------------------------------------

.unknownList <- function(x, unknown)
{
  ## --- Setup ---

  n <- length(x)
  unkN <- length(unknown)
  namesX <- names(x)
  namesXNullTest <- is.null(namesX)
  unkNames <- names(unknown)
  unkNamesNullTest <- is.null(unkNames)
  defInNames <- ".default" %in% unkNames
  defInd <- unkNames %in% ".default"
  def <- unknown[defInd]

  if(defInNames) { ## Remove default
    unkN <- unkN - 1
    unkNames <- unkNames[!defInd]
    unknown <- unknown[!defInd]
  }

  if(!namesXNullTest) { ## Check for nonexistent name
    test <- !(unkNames %in% namesX)
    if(any(test)) stop(sprintf("name(s) %s not in names of 'x'",
                       paste(sQuote(unkNames[test]), collapse=" ")))
  }

  ## --- Recycle ---

  if(unkN < n) {
    if(unkNamesNullTest | defInNames) {
      if(defInNames) { # handling .default
        names(def) <- NULL
        unknownDef <- rep(def, length=(n - unkN))
        names(unknownDef) <- namesX[!(namesX %in% unkNames)]
        unknown <- c(unknownDef, unknown)
      } else {
        unknownDef <- unknown
        unknown <- rep(unknownDef, length=n)
      }
    } else {
      stop("can not propely recycle named 'unknown'")
    }
  }

  ## --- Names ---

  if(!namesXNullTest) { ## no need if namesX NULL
    if(unkNamesNullTest) { ## missing unkNames
      names(unknown) <- namesX
    } else {                ## unkNames known
      unknown <- unknown[match(namesX, names(unknown))]
    }
  }

  unknown
}

### }}}
### {{{ Dear Emacs
### Local variables:
### folded-file: t
### End:
### }}}

###------------------------------------------------------------------------
### unknown.R ends here

## Test setup
library("RUnit")
library("gdata")

## Vectors

intUnk <- 9999
xInt        <- as.integer(c(NA,     1:2, NA,     5, 6, 7, 8, 9))
xIntUnk     <- as.integer(c(intUnk, 1:2, intUnk, 5, 6, 7, 8, 9))
xIntUnkTest <- xIntUnk %in% intUnk

numUnk <- 0
xNum        <- c(9999, NA, 1.5, NA, 5, 6, 7, 8, 9)
xNumUnk     <- c(9999, 0,  1.5, 0,  5, 6, 7, 8, 9)
xNumUnkTest <- xNumUnk %in% numUnk

chaUnk <- "notAvail"
chaUnk1 <- "-"
xCha        <- c("A", "B", NA,      "C", NA,      "-", "7", "8", "9")
xChaUnk     <- c("A", "B", chaUnk,  "C", chaUnk,  "-", "7", "8", "9")
xChaUnk1    <- c("A", "B", chaUnk1, "C", chaUnk1, "-", "7", "8", "9")
xChaUnkTest  <- xChaUnk %in% chaUnk
xChaUnk1Test <- xChaUnk %in% chaUnk1

facUnk <- "notAvail"
facUnk1 <- "NA"
xFac     <- factor(c("A", "0", 0, "NA", "NA", intUnk, numUnk, "-", NA))
xFacUnk  <- factor(c("A", "0", 0, "NA", "NA", intUnk, numUnk, "-", facUnk))
xFacUnk1 <- factor(c("A", "0", 0, "NA", "NA", intUnk, numUnk, "-", facUnk1))
xFacUnkTest <-     c(0,   0,   0, 0,    0,    0,      0,      0,   1)
xFacUnkTest <- as.logical(xFacUnkTest)
xFacUnk1Test <-    c(0,   0,   0, 1,    1,    0,      0,      0,   1)
xFacUnk1Test <- as.logical(xFacUnk1Test)
xFac1    <- factor(c("A", "0", 0, NA, NA, intUnk, numUnk, "-", NA))

facLev <- "A"
xFacUnkLev <- factor(c("A", "0", 0, "NA", "NA", intUnk, numUnk, "-", "A"))
xFacUnkLevTest <-    c(1,    0,  0, 0,    0,    0,      0,      0,   1)
xFacUnkLevTest <- as.logical(xFacUnkLevTest)

dateUnk <- as.Date("2006-08-14")
tmp <- as.Date("2006-08-15")
xDate     <- c(tmp, NA)
xDateUnk  <- c(tmp,   dateUnk)
xDateTest <- c(FALSE, TRUE)

xDate1Unk  <- c(tmp,   dateUnk, NA)
xDate1Test <- c(FALSE, TRUE,    FALSE)

POSIXltUnk <- strptime("2006-08-14", format="%Y-%m-%d")
tmp <- strptime("2006-08-15", format="%Y-%m-%d")
xPOSIXlt     <- c(tmp, NA)
xPOSIXltUnk  <- c(tmp,   POSIXltUnk)
xPOSIXltTest <- c(FALSE, TRUE)

xPOSIXlt1Unk  <- c(tmp,   POSIXltUnk, NA)
xPOSIXlt1Test <- c(FALSE, TRUE,       FALSE)

POSIXctUnk <- as.POSIXct(strptime("2006-08-14 01:01:01", format="%Y-%m-%d %H:%M:%S"))
tmp <- as.POSIXct(strptime("2006-08-15 01:01:01", format="%Y-%m-%d %H:%M:%S"))
xPOSIXct     <- c(tmp, NA)
xPOSIXctUnk  <- c(tmp, POSIXctUnk)
xPOSIXctTest <- xPOSIXltTest

xPOSIXct1Unk  <- c(tmp, POSIXctUnk, NA)
xPOSIXct1Test <- xPOSIXlt1Test

## Lists and data frames

xList <- list(xInt, xCha, xNum, xFac)
xListN <- list(int=xInt, cha=xCha, num=xNum, fac=xFac)
xListUnk <- list(xIntUnk, xChaUnk, xNumUnk, xFacUnk)
xListUnkTest <- list(xIntUnkTest, xChaUnkTest, xNumUnkTest, xFacUnkTest)
xListNUnk <- list(int=xIntUnk, cha=xChaUnk, num=xNumUnk, fac=xFacUnk)
xListNUnkTest <- list(int=xIntUnkTest, cha=xChaUnkTest, num=xNumUnkTest, fac=xFacUnkTest)

xDF <- as.data.frame(xListN)
xDF$cha <- as.character(xDF$cha)
xDFUnk <- as.data.frame(xListNUnk)
xDFUnk$cha <- as.character(xDFUnk$cha)
xDFUnkTest <- as.data.frame(xListNUnkTest)

unkC <- c(intUnk, chaUnk, numUnk, facUnk)
unkL <- list(intUnk, chaUnk, numUnk, facUnk)
unkLN <- list(num=numUnk, cha=chaUnk, fac=facUnk, int=intUnk)  # mixed as it is named
unkLMN <- list(cha=chaUnk, int=intUnk, num=c(intUnk, numUnk),
               fac=c(chaUnk1, facUnk))

xListMNUnkF <- list(int=as.integer(c(9999, 1, 2, 9999, 5, 6, 7, 8, 9)),
                    cha=c("A", "B", "notAvail", "C", "notAvail", "-", "7", "8", "9"),
                    num=c(9999, 0, 1.5, 0, 5, 6, 7, 8, 9),
                    fac=factor(c("A", "0", "0", "NA", "NA", 9999, "0", "-", "notAvail")))
xListMNUnkFTest <- list(int=c(1, 0, 0, 1, 0, 0, 0, 0, 0),
                        cha=c(0, 0, 1, 0, 1, 0, 0, 0, 0),
                        num=c(1, 1, 0, 1, 0, 0, 0, 0, 0),
                        fac=c(0, 0, 0, 0, 0, 0, 0, 1, 1))
xListMNUnkFTest <- lapply(xListMNUnkFTest, as.logical)
xListMNF <- list(int=as.integer(c(NA, 1, 2, NA, 5, 6, 7, 8, 9)),
                 cha=c("A", "B", NA, "C", NA, "-", "7", "8", "9"),
                 num=c(NA, NA, 1.5, NA, 5, 6, 7, 8, 9),
                 fac=factor(c("A", "0", "0", "NA", "NA", "9999", "0", NA, NA)))

xDFMUnkF <- as.data.frame(xListMNUnkF)
xDFMUnkF$cha <- as.character(xDFMUnkF$cha)
xDFMUnkFTest <- as.data.frame(xListMNUnkFTest)
xDFMF <- as.data.frame(xListMNF)
xDFMF$cha <- as.character(xDFMF$cha)

unk1 <- 555555
xListUnk1 <- list(as.integer(c(unk1, 1, 2, unk1, 5, 6, 7, 8, 9)),
                  c("A", "B", unk1, "C", unk1, "-", "7", "8", "9"),
                  c(9999, unk1, 1.5, unk1, 5, 6, 7, 8, 9),
                  factor(c("A", "0", "0", "NA", "NA", "9999", "0", "-", unk1)))
xListUnk1Test <- lapply(xListUnk1, function(x) x %in% unk1)
xListNUnk1 <- xListUnk1
names(xListNUnk1) <- c("int", "cha", "num", "fac")
xDFUnk1 <- as.data.frame(xListNUnk1)
xDFUnk1$cha <- as.character(xDFUnk1$cha)
xDFUnk1Test <- as.data.frame(xListUnk1Test)
names(xDFUnk1Test) <- names(xListNUnk1)

unkC2 <- c(0, "notAvail")
xListUnk2 <- list(as.integer(c(unkC2[1], 1, 2, unkC2[1], 5, 6, 7, 8, 9)),
                  c("A", "B", unkC2[2], "C", unkC2[2], "-", "7", "8", "9"),
                  c(9999, as.numeric(unkC2[1]), 1.5, as.numeric(unkC2[1]), 5, 6, 7, 8, 9),
                  factor(c("A", "0", "0", "NA", "NA", "9999", "0", "-", unkC2[2])))
xListNUnk2 <- xListUnk2
names(xListNUnk2) <- c("int", "cha", "num", "fac")
xDFUnk2 <- as.data.frame(xListNUnk2)
xDFUnk2$cha <- as.character(xDFUnk2$cha)

xListUnk2Test <- xListUnk2
xListUnk2Test[[1]] <- xListUnk2Test[[1]] %in% unkC2[1]
xListUnk2Test[[2]] <- xListUnk2Test[[2]] %in% unkC2[2]
xListUnk2Test[[3]] <- xListUnk2Test[[3]] %in% unkC2[1]
xListUnk2Test[[4]] <- xListUnk2Test[[4]] %in% unkC2[2]
xListNUnk2Test <- xListUnk2Test
names(xListNUnk2Test) <- names(xListNUnk2)
xDFUnk2Test <- as.data.frame(xListNUnk2Test)

unkL2 <- as.list(unkC2)
unkLN2 <- unkL2[c(2, 1)]
names(unkLN2) <- c("cha", "int")
xListUnk2a <- list(as.integer(c(NA, 1, 2, NA, 5, 6, 7, 8, 9)),
                   c("A", "B", unkLN2[[2]], "C", unkLN2[[2]], "-", "7", "8", "9"),
                   c(9999, NA, 1.5, NA, 5, 6, 7, 8, 9),
                   factor(c("A", "0", "0", "NA", "NA", "9999", "0", "-", unkLN2[[2]])))
xListUnk2aTest <- xListUnk2a
xListUnk2aTest[[1]] <- xListUnk2aTest[[1]] %in% unkLN2[1]
xListUnk2aTest[[2]] <- xListUnk2aTest[[2]] %in% unkLN2[2]
xListUnk2aTest[[3]] <- xListUnk2aTest[[3]] %in% unkLN2[1]
xListUnk2aTest[[4]] <- xListUnk2aTest[[4]] %in% unkLN2[2]

xList2a <- list(xListUnk2a[[1]],
                c("A", "B", NA, "C", NA, "-", "7", "8", "9"),
                xListUnk2a[[3]],
                factor(c("A", NA, NA, "NA", "NA", 9999, NA, "-", NA)))

## Matrix

matUnk <- 9999
mat        <- matrix(1:25, nrow=5, ncol=5)
mat[1, 2] <- NA; mat[1, 4] <- NA; mat[2, 2] <- NA;
mat[3, 2] <- NA; mat[3, 5] <- NA; mat[5, 4] <- NA;
matUnk1       <- mat
matUnk1[1, 2] <- matUnk; matUnk1[1, 4] <- matUnk; matUnk1[2, 2] <- matUnk;
matUnk1[3, 2] <- matUnk; matUnk1[3, 5] <- matUnk; matUnk1[5, 4] <- matUnk;
matUnkTest <- matUnk1Test <- is.na(mat)

matUnk2Test <- matUnkTest | mat == 1

## Use of unknown=list(.default=, ...) or similarly named vector

D1 <- "notAvail"
unkLND1 <- list(.default=D1)
xListUnkD1 <- list(as.integer(c(NA, 1:2, NA, 5, 6, 7, 8, 9)),
                   c("A", "B", D1, "C", D1, "-", "7", "8", "9"),
                   c(9999, NA, 1.5, NA, 5, 6, 7, 8, 9),
                   factor(c("A", "0", 0, "NA", "NA", intUnk, numUnk, "-", D1)))
xListUnkD1Test <- lapply(xListUnkD1, function(x) x %in% D1)
xListD1 <- xList

xListNUnkD1     <- xListUnkD1
xListNUnkD1Test <- xListUnkD1Test
names(xListNUnkD1) <- names(xListNUnkD1Test) <- names(xListNUnk1)
xListND1 <- xListN

DSO2 <- c("notAvail", 5678)
unkLNDSO2 <- as.list(DSO2)
names(unkLNDSO2) <- c(".default", "someOther")
xListUnkDSO2 <- list(as.integer(c(NA, 1:2, NA, 5, 6, 7, 8, 9)),
                     c("A", "B", DSO2[1], "C", DSO2[1], "-", "7", "8", "9"),
                     c(9999, NA, 1.5, NA, 5, 6, 7, 8, 9),
                     factor(c("A", "0", 0, "NA", "NA", intUnk, numUnk, "-", DSO2[2])))
xListUnkDSO2Test <- lapply(xListUnkDSO2, function(x) x %in% DSO2)

unkLND3 <- list(.default="notAvail", num=0, int=9999)
xListNUnkD3 <- list(int=as.integer(c(unkLND3[[3]], 1:2, unkLND3[[3]], 5, 6, 7, 8, 9)),
                    cha=c("A", "B", unkLND3[[1]], "C", unkLND3[[1]], "-", "7", "8", "9"),
                    num=c(9999, unkLND3[[2]], 1.5, unkLND3[[2]], 5, 6, 7, 8, 9),
                    fac=factor(c("A", "0", 0, "NA", "NA", intUnk, numUnk, "-", unkLND3[[1]])))
xListNUnkD3Test <- xListNUnkD3
xListNUnkD3Test$int <- xListNUnkD3Test$int %in% unkLND3[[3]]
xListNUnkD3Test$cha <- xListNUnkD3Test$cha %in% unkLND3[[1]]
xListNUnkD3Test$num <- xListNUnkD3Test$num %in% unkLND3[[2]]
xListNUnkD3Test$fac <- xListNUnkD3Test$fac %in% unkLND3[[1]]

unkLND2E <- list(.default="notAvail", 9999)

## isUnknown

test.isUnknown <- function()
{
  ## Base methods for vectors
  checkIdentical(isUnknown(xIntUnk, unknown=as.integer(intUnk)), xIntUnkTest)
  checkIdentical(isUnknown(xIntUnk, unknown=intUnk),             xIntUnkTest)
  checkIdentical(isUnknown(xNumUnk, unknown=numUnk),             xNumUnkTest)
  checkIdentical(isUnknown(xNumUnk, unknown=as.integer(numUnk)), xNumUnkTest)
  checkIdentical(isUnknown(xChaUnk, unknown=chaUnk),             xChaUnkTest)
  checkIdentical(isUnknown(xFacUnk, unknown=facUnk),             xFacUnkTest)

  ## Multiple values are allowed for vector methods in vector or list form
  checkIdentical(isUnknown(xIntUnk, unknown=unkC), xIntUnkTest)
  checkIdentical(isUnknown(xIntUnk, unknown=unkL), xIntUnkTest)

  ## NA's in factors
  checkIdentical(isUnknown(xFacUnk1, unknown=facUnk1), xFacUnk1Test)
  facNA <- factor(c("0", 1, 2, 3, NA, "NA"))
  facNATest <- c(FALSE, FALSE, FALSE, FALSE, TRUE, FALSE)
  checkIdentical(isUnknown(facNA), facNATest)

  ## Date-time classes
  checkIdentical(isUnknown(xDateUnk, unknown=dateUnk), xDateTest)
  checkIdentical(isUnknown(xDate1Unk, unknown=dateUnk), xDate1Test)
  checkIdentical(isUnknown(xPOSIXltUnk, unknown=POSIXltUnk), xPOSIXltTest)
  checkIdentical(isUnknown(xPOSIXlt1Unk, unknown=POSIXltUnk), xPOSIXlt1Test)
  checkIdentical(isUnknown(xPOSIXctUnk, unknown=POSIXctUnk), xPOSIXctTest)
  checkIdentical(isUnknown(xPOSIXct1Unk, unknown=POSIXctUnk), xPOSIXct1Test)

  ## Lists and data frames

  ## with vector of single unknown values
  checkIdentical(isUnknown(xListUnk, unknown=unkC), xListUnkTest)
  checkIdentical(isUnknown(xDFUnk, unknown=unkC),   xDFUnkTest)

  ## with list of single unknown values
  checkIdentical(isUnknown(xListUnk, unknown=unkL), xListUnkTest)
  checkIdentical(isUnknown(xDFUnk, unknown=unkL),   xDFUnkTest)

  ## with named list of single unknown values
  checkIdentical(isUnknown(xListNUnk, unknown=unkLN), xListNUnkTest)
  checkIdentical(isUnknown(xDFUnk, unknown=unkLN),    xDFUnkTest)

  ## with named list of multiple unknown values - valid here
  checkIdentical(isUnknown(xListMNUnkF, unknown=unkLMN), xListMNUnkFTest)
  checkIdentical(isUnknown(xDFMUnkF, unknown=unkLMN), xDFMUnkFTest)

  ## with single unknown value - recycling
  checkIdentical(isUnknown(xListUnk1, unknown=unk1), xListUnk1Test)
  checkIdentical(isUnknown(xDFUnk1,   unknown=unk1), xDFUnk1Test)

  ## with vector of two unknown values - recycling
  checkIdentical(isUnknown(xListUnk2, unknown=unkC2), xListUnk2Test)
  checkIdentical(isUnknown(xDFUnk2,   unknown=unkC2), xDFUnk2Test)

  ## with list of two unknown values - recycling
  checkIdentical(isUnknown(xListUnk2, unknown=unkL2), xListUnk2Test)
  checkIdentical(isUnknown(xDFUnk2,   unknown=unkL2), xDFUnk2Test)

  ## list(.default=)
  checkIdentical(isUnknown(x=xListUnkD1, unknown=unkLND1), xListUnkD1Test)
  ## list(.default=, someOther=) we do not know someOther, but should work
  ## as x is not named
  checkIdentical(isUnknown(x=xListUnkDSO2, unknown=unkLNDSO2), xListUnkDSO2Test)
  ## list(.default=) in named list
  checkIdentical(isUnknown(x=xListNUnkD1, unknown=unkLND1), xListNUnkD1Test)
  ## list(.default=, someOther=) OK if someOther is in the named list
  checkIdentical(isUnknown(x=xListNUnkD3, unknown=unkLND3), xListNUnkD3Test)
  ## list(.default=, 99) ERROR as we do not know where to apply 99
  checkException(isUnknown(x=xListNUnk, unknown=unkLND2E))

  ## Matrix
  checkIdentical(isUnknown(x=mat, unknown=NA), matUnkTest)
  checkIdentical(isUnknown(x=matUnk1, unknown=matUnk), matUnkTest)
  checkIdentical(isUnknown(x=matUnk1, unknown=c(1, matUnk)), matUnk2Test)
}

## unknownToNA

test.unknownToNA <- function()
{
  ## Base methods for vectors
  checkIdentical(unknownToNA(xIntUnk, as.integer(intUnk)), xInt)
  checkIdentical(unknownToNA(xIntUnk, intUnk),             xInt)  # with numeric
  checkIdentical(unknownToNA(xNumUnk, numUnk),             xNum)
  checkIdentical(unknownToNA(xNumUnk, as.integer(numUnk)), xNum)
  checkIdentical(unknownToNA(xChaUnk, chaUnk),             xCha)
  checkIdentical(unknownToNA(xChaUnk, chaUnk),             xCha)
  checkIdentical(unknownToNA(xFacUnk, facUnk),             xFac)

  ## Multiple values are allowed for vector methods in vector or list form
  checkIdentical(unknownToNA(xIntUnk, unknown=unkC), xInt)
  checkIdentical(unknownToNA(xIntUnk, unknown=unkL), xInt)

  ## NA's in factors
  checkIdentical(unknownToNA(xFacUnk1, unknown=facUnk1), xFac1)
  facNA <- factor(c("0", 1, 2, 3, NA, "NA"))
  facNATest <- factor(c("0", 1, 2, 3, NA, NA))
  checkIdentical(unknownToNA(x=facNA, unknown="NA"), facNATest)

  ## Date-time classes
  checkIdentical(unknownToNA(xDateUnk, unknown=dateUnk), xDate)
  checkIdentical(unknownToNA(xPOSIXctUnk, unknown=POSIXctUnk), xPOSIXct)

  ## Per Brian Ripley on 2014-01-15:
  ##
  ## On platforms where POSIXlt has a gmtoff component, it does not need to be set.  So
  ##
  ## > z$gmtoff
  ## [1] 3600   NA
  ## > xPOSIXltUnk$gmtoff
  ## [1] 3600 3600
  ##
  ## (or sometimes 0, not NA).
  ##
  ## So although identical() correctly reports that they differ, this
  ## is allowed for optional components.
  ##
  ## It would also be wrong to use identical() to compare isdst
  ## components: isdst = -1 means unknown.
  ##
  ## Replaced:
  ##   checkIdentical(unknownToNA(xPOSIXltUnk, unknown=POSIXltUnk), xPOSIXlt)
  ## With:
  tmp_unknownToNA <- unknownToNA(xPOSIXltUnk, unknown=POSIXltUnk)
  tmp_xPOSIXlt   <- xPOSIXlt

  tmp_unknownToNA$gmtoff <- NULL  # Remove $gmtoff to avoid comparison
  tmp_xPOSIXlt$gmtoff   <- NULL

  isdst.unknown <- unique(
    c(which(is.na(tmp_unknownToNA$isdst) | tmp_unknownToNA$isdst==-1)),
    c(which(is.na(tmp_xPOSIXlt$isdst) | tmp_xPOSIXlt$isdst==-1)))

  checkIdentical(tmp_unknownToNA$isdst[!isdst.unknown],
                 tmp_xPOSIXlt$isds[!isdst.unknown])

  tmp_unknownToNA$isdst <- NULL   # remove $isdst to avoid comparison
  tmp_xPOSIXlt$isdst   <- NULL    # by checkIdentical

  checkIdentical(tmp_unknownToNA, tmp_xPOSIXlt)

  ## Lists and data frames

  ## with vector of single unknown values
  checkIdentical(unknownToNA(xListUnk, unknown=unkC), xList)
  checkIdentical(unknownToNA(xDFUnk, unknown=unkC), xDF)

  ## with list of single unknown values
  checkIdentical(unknownToNA(xListUnk, unknown=unkL), xList)
  checkIdentical(unknownToNA(xDFUnk, unknown=unkL), xDF)

  ## with named list of single unknown values
  checkIdentical(unknownToNA(xListNUnk, unknown=unkLN), xListN)
  checkIdentical(unknownToNA(xDFUnk, unknown=unkLN), xDF)

  ## with names list of multiple unknown values - must be an error
  checkIdentical(unknownToNA(xListMNUnkF, unknown=unkLMN), xListMNF)
  checkIdentical(unknownToNA(xDFMUnkF, unknown=unkLMN), xDFMF)

  ## with single unknown value - recycling
  checkIdentical(unknownToNA(xListUnk1, unknown=unk1), xList)
  checkIdentical(unknownToNA(xDFUnk1,   unknown=unk1), xDF)

  ## with vector of two unknown values - recycling
  checkIdentical(unknownToNA(xListUnk2, unknown=unkC2), xList)
  checkIdentical(unknownToNA(xDFUnk2,   unknown=unkC2), xDF)

  ## with list of two unknown values - recycling
  checkIdentical(unknownToNA(xListUnk2, unknown=unkL2), xList)
  checkIdentical(unknownToNA(xDFUnk2,   unknown=unkL2), xDF)

  ## with named list of two unknown values but x is not named so named list
  ## does not have any effect --> error as we do not know how to recycle
  checkException(unknownToNA(xListUnk2a, unknown=unkLN2))

  ## but we should get some results with named x
  checkIdentical(unknownToNA(xListNUnk2, unknown=unkL2), xListN)
  ## not also necesarily with recycling of names lists, as it is
  ## not clear how to properly recycle named lists (only names that match
  ## can be really properly recycled)
  checkException(unknownToNA(xListNUnk2, unknown=unkLN2))
  checkIdentical(unknownToNA(xDFUnk2, unknown=unkL2), xDF)
  checkException(unknownToNA(xDFUnk2, unknown=unkLN2))

  ## list(.default=)
  checkIdentical(unknownToNA(x=xListUnkD1, unknown=unkLND1), xListD1)
  ## list(.default=, someOther=) we do not know someOther, but should work
  ## as x is not named
  checkIdentical(unknownToNA(x=xListUnkDSO2, unknown=unkLNDSO2), xList)
  ## list(.default=) in named list
  checkIdentical(unknownToNA(x=xListNUnkD1, unknown=unkLND1), xListND1)
  ## list(.default=, someOther=) OK if someOther is in the named list
  checkIdentical(unknownToNA(x=xListNUnkD3, unknown=unkLND3), xListN)
  ## list(.default=, 99) ERROR as we do not know where to apply 99
  checkException(unknownToNA(x=xListNUnk, unknown=unkLND2E))

  ## Matrix
  checkEquals(unknownToNA(x=matUnk1, unknown=matUnk), mat)
}

## NAToUnknown

test.NAToUnknown <- function()
{
  ## Base methods for vectors
  checkIdentical(NAToUnknown(xInt, as.integer(intUnk)), xIntUnk)
  checkIdentical(NAToUnknown(xInt, intUnk),             xIntUnk)  # with numeric
  checkIdentical(NAToUnknown(xNum, numUnk),             xNumUnk)
  checkIdentical(NAToUnknown(xNum, as.integer(numUnk)), xNumUnk)
  checkIdentical(NAToUnknown(xCha, chaUnk),             xChaUnk)
  checkIdentical(NAToUnknown(xCha, chaUnk),             xChaUnk)
  checkIdentical(NAToUnknown(xFac, facUnk),             xFacUnk)

  ## only single values are allowed for vector methods
  checkException(NAToUnknown(xInt, unknown=unkC))
  checkException(NAToUnknown(xInt, unknown=unkL))

  ## and they should not already be in x unless force=TRUE
  checkException(NAToUnknown(xCha, unknown=chaUnk1))
  checkIdentical(NAToUnknown(xCha, unknown=chaUnk1, force=TRUE), xChaUnk1)

  checkException(NAToUnknown(xFac, unknown=facLev))
  checkIdentical(NAToUnknown(xFac, unknown=facLev, force=TRUE), xFacUnkLev)

  ## NA's in factors
  checkIdentical(NAToUnknown(xFac, unknown=facUnk1, force=TRUE), xFacUnk1)
  facNA <- factor(c("0", 1, 2, 3, NA, NA))
  facNATest <- factor(c("0", 1, 2, 3, "NA", "NA"))
  checkIdentical(NAToUnknown(x=facNA, unknown="NA"), facNATest)

  ## Date-time classes
  checkIdentical(NAToUnknown(xDate, unknown=dateUnk), xDateUnk)
  checkIdentical(NAToUnknown(xPOSIXct, unknown=POSIXctUnk), xPOSIXctUnk)

  ## Per Brian Ripley on 2014-01-15:
  ##
  ## On platforms where POSIXlt has a gmtoff component, it does not need to be set.  So
  ##
  ## > z$gmtoff
  ## [1] 3600   NA
  ## > xPOSIXltUnk$gmtoff
  ## [1] 3600 3600
  ##
  ## (or sometimes 0, not NA).
  ##
  ## So although identical() correctly reports that they differ, this
  ## is allowed for optional components.
  ##
  ## It would also be wrong to use identical() to compare isdst
  ## components: isdst = -1 means unknown.
  ##
  ## Replaced:
  ##   checkIdentical(NAToUnknown(xPOSIXlt, unknown=POSIXltUnk), xPOSIXltUnk)
  ## With:
  tmp_NAToUnknown <- NAToUnknown(xPOSIXlt, unknown=POSIXltUnk)
  tmp_xPOSIXltUnk   <- xPOSIXltUnk

  tmp_NAToUnknown$gmtoff <- NULL  # remove $gmtoff to avoid comparison
  tmp_xPOSIXltUnk$gmtoff   <- NULL

  isdst.unknown <- unique(
    c(which(is.na(tmp_NAToUnknown$isdst) | tmp_NAToUnknown$isdst==-1)),
    c(which(is.na(tmp_xPOSIXltUnk$isdst) | tmp_xPOSIXltUnk$isdst==-1)))

  checkIdentical(tmp_NAToUnknown$isdst[!isdst.unknown],
                 tmp_xPOSIXltUnk$isds[!isdst.unknown])

  tmp_NAToUnknown$isdst <- NULL  # remove $isdst to avoid comparison
  tmp_xPOSIXltUnk$isdst <- NULL  # by checkIdentical

  checkIdentical(tmp_NAToUnknown, tmp_xPOSIXltUnk)

  ## Lists and data frames

  ## with vector of single unknown values
  checkIdentical(NAToUnknown(xList, unknown=unkC), xListUnk)
  checkIdentical(NAToUnknown(xDF, unknown=unkC), xDFUnk)

  ## with list of single unknown values
  checkIdentical(NAToUnknown(xList, unknown=unkL), xListUnk)
  checkIdentical(NAToUnknown(xDF, unknown=unkL), xDFUnk)

  ## with named list of single unknown values
  checkIdentical(NAToUnknown(xListN, unknown=unkLN), xListNUnk)
  checkIdentical(NAToUnknown(xDF, unknown=unkLN), xDFUnk)

  ## with names list of multiple unknown values - must be an error
  checkException(NAToUnknown(xListN, unknown=unkLMN))
  checkException(NAToUnknown(xDF, unknown=unkLMN))

  ## with single unknown value - recycling
  checkIdentical(NAToUnknown(xList, unknown=unk1), xListUnk1)
  checkIdentical(NAToUnknown(xDF,   unknown=unk1), xDFUnk1)

  ## with vector of two unknown values - recycling
  checkIdentical(NAToUnknown(xList, unknown=unkC2), xListUnk2)
  checkIdentical(NAToUnknown(xDF,   unknown=unkC2), xDFUnk2)

  ## with list of two unknown values - recycling
  checkIdentical(NAToUnknown(xList, unknown=unkL2), xListUnk2)
  checkIdentical(NAToUnknown(xDF,   unknown=unkL2), xDFUnk2)

  ## with named list of two unknown values but x is not named so named list
  ## does not have any effect --> error as we do not know how to recycle
  checkException(NAToUnknown(xList, unknown=unkLN2))

  ## but we should get some results with named x
  checkIdentical(NAToUnknown(xListN, unknown=unkL2), xListNUnk2)
  ## not also necesarilly with recycling of names lists, as it is
  ## not clear how to properly recycle named lists (only names that match
  ## can be really properly recycled)
  checkException(NAToUnknown(xListN, unknown=unkLN2))
  checkIdentical(NAToUnknown(xDF, unknown=unkL2), xDFUnk2)
  checkException(NAToUnknown(xDF, unknown=unkLN2))

  ## list(.default=)
  checkIdentical(NAToUnknown(x=xList, unknown=unkLND1), xListUnkD1)
  ## list(.default=, someOther=) we do not know someOther, but should work
  ## as x is not named
  checkIdentical(NAToUnknown(x=xList, unknown=unkLNDSO2), xListUnkDSO2)
  ## list(.default=) in named list
  checkIdentical(NAToUnknown(x=xListN, unknown=unkLND1), xListNUnkD1)
  ## list(.default=, someOther=) OK if someOther is in the named list
  checkIdentical(NAToUnknown(x=xListN, unknown=unkLND3), xListNUnkD3)
  ## list(.default=, 99) ERROR as we do not know where to apply 99
  checkException(NAToUnknown(x=xListN, unknown=unkLND2E))

  ## Matrix
  checkEquals(NAToUnknown(x=mat, unknown=matUnk), matUnk1)
}

### runit.unknown.R
###------------------------------------------------------------------------
### What: Tests for Change given unknown value to NA and vice versa methods
### $Id: runit.unknown.R 1801 2014-04-05 21:08:41Z warnes $
### Time-stamp: <2006-10-30 17:46:21 ggorjan>
###------------------------------------------------------------------------

### {{{ --- Test setup ---

library("RUnit")
library("gdata")


### {{{ --- Vectors ---

intUnk <- 9999
xInt        <- as.integer(c(NA,     1:2, NA,     5, 6, 7, 8, 9))
xIntUnk     <- as.integer(c(intUnk, 1:2, intUnk, 5, 6, 7, 8, 9))
xIntUnkTest <- xIntUnk %in% intUnk

numUnk <- 0
xNum        <- c(9999, NA, 1.5, NA, 5, 6, 7, 8, 9)
xNumUnk     <- c(9999, 0,  1.5, 0,  5, 6, 7, 8, 9)
xNumUnkTest <- xNumUnk %in% numUnk

chaUnk <- "notAvail"
chaUnk1 <- "-"
xCha        <- c("A", "B", NA,      "C", NA,      "-", "7", "8", "9")
xChaUnk     <- c("A", "B", chaUnk,  "C", chaUnk,  "-", "7", "8", "9")
xChaUnk1    <- c("A", "B", chaUnk1, "C", chaUnk1, "-", "7", "8", "9")
xChaUnkTest  <- xChaUnk %in% chaUnk
xChaUnk1Test <- xChaUnk %in% chaUnk1

facUnk <- "notAvail"
facUnk1 <- "NA"
xFac     <- factor(c("A", "0", 0, "NA", "NA", intUnk, numUnk, "-", NA))
xFacUnk  <- factor(c("A", "0", 0, "NA", "NA", intUnk, numUnk, "-", facUnk))
xFacUnk1 <- factor(c("A", "0", 0, "NA", "NA", intUnk, numUnk, "-", facUnk1))
xFacUnkTest <-     c(0,   0,   0, 0,    0,    0,      0,      0,   1)
xFacUnkTest <- as.logical(xFacUnkTest)
xFacUnk1Test <-    c(0,   0,   0, 1,    1,    0,      0,      0,   1)
xFacUnk1Test <- as.logical(xFacUnk1Test)
xFac1    <- factor(c("A", "0", 0, NA, NA, intUnk, numUnk, "-", NA))

facLev <- "A"
xFacUnkLev <- factor(c("A", "0", 0, "NA", "NA", intUnk, numUnk, "-", "A"))
xFacUnkLevTest <-    c(1,    0,  0, 0,    0,    0,      0,      0,   1)
xFacUnkLevTest <- as.logical(xFacUnkLevTest)

dateUnk <- as.Date("2006-08-14")
tmp <- as.Date("2006-08-15")
xDate     <- c(tmp, NA)
xDateUnk  <- c(tmp,   dateUnk)
xDateTest <- c(FALSE, TRUE)

xDate1Unk  <- c(tmp,   dateUnk, NA)
xDate1Test <- c(FALSE, TRUE,    FALSE)

POSIXltUnk <- strptime("2006-08-14", format="%Y-%m-%d")
tmp <- strptime("2006-08-15", format="%Y-%m-%d")
xPOSIXlt     <- c(tmp, NA)
xPOSIXltUnk  <- c(tmp,   POSIXltUnk)
xPOSIXltTest <- c(FALSE, TRUE)

xPOSIXlt1Unk  <- c(tmp,   POSIXltUnk, NA)
xPOSIXlt1Test <- c(FALSE, TRUE,       FALSE)

POSIXctUnk <- as.POSIXct(strptime("2006-08-14 01:01:01", format="%Y-%m-%d %H:%M:%S"))
tmp <- as.POSIXct(strptime("2006-08-15 01:01:01", format="%Y-%m-%d %H:%M:%S"))
xPOSIXct     <- c(tmp, NA)
xPOSIXctUnk  <- c(tmp, POSIXctUnk)
xPOSIXctTest <- xPOSIXltTest

xPOSIXct1Unk  <- c(tmp, POSIXctUnk, NA)
xPOSIXct1Test <- xPOSIXlt1Test

### }}}
### {{{ --- Lists and data.frames ---

xList <- list(xInt, xCha, xNum, xFac)
xListN <- list(int=xInt, cha=xCha, num=xNum, fac=xFac)
xListUnk <- list(xIntUnk, xChaUnk, xNumUnk, xFacUnk)
xListUnkTest <- list(xIntUnkTest, xChaUnkTest, xNumUnkTest, xFacUnkTest)
xListNUnk <- list(int=xIntUnk, cha=xChaUnk, num=xNumUnk, fac=xFacUnk)
xListNUnkTest <- list(int=xIntUnkTest, cha=xChaUnkTest, num=xNumUnkTest, fac=xFacUnkTest)

xDF <- as.data.frame(xListN)
xDF$cha <- as.character(xDF$cha)
xDFUnk <- as.data.frame(xListNUnk)
xDFUnk$cha <- as.character(xDFUnk$cha)
xDFUnkTest <- as.data.frame(xListNUnkTest)

unkC <- c(intUnk, chaUnk, numUnk, facUnk)
unkL <- list(intUnk, chaUnk, numUnk, facUnk)
unkLN <- list(num=numUnk, cha=chaUnk, fac=facUnk, int=intUnk) ## mixed as it is named
unkLMN <- list(cha=chaUnk, int=intUnk, num=c(intUnk, numUnk),
               fac=c(chaUnk1, facUnk))

xListMNUnkF <- list(int=as.integer(c(9999, 1, 2, 9999, 5, 6, 7, 8, 9)),
                   cha=c("A", "B", "notAvail", "C", "notAvail", "-", "7", "8", "9"),
                   num=c(9999, 0, 1.5, 0, 5, 6, 7, 8, 9),
                   fac=factor(c("A", "0", "0", "NA", "NA", 9999, "0", "-", "notAvail")))
xListMNUnkFTest <- list(int=c(1, 0, 0, 1, 0, 0, 0, 0, 0),
                        cha=c(0, 0, 1, 0, 1, 0, 0, 0, 0),
                        num=c(1, 1, 0, 1, 0, 0, 0, 0, 0),
                        fac=c(0, 0, 0, 0, 0, 0, 0, 1, 1))
xListMNUnkFTest <- lapply(xListMNUnkFTest, as.logical)
xListMNF <- list(int=as.integer(c(NA, 1, 2, NA, 5, 6, 7, 8, 9)),
                 cha=c("A", "B", NA, "C", NA, "-", "7", "8", "9"),
                 num=c(NA, NA, 1.5, NA, 5, 6, 7, 8, 9),
                 fac=factor(c("A", "0", "0", "NA", "NA", "9999", "0", NA, NA)))

xDFMUnkF <- as.data.frame(xListMNUnkF)
xDFMUnkF$cha <- as.character(xDFMUnkF$cha)
xDFMUnkFTest <- as.data.frame(xListMNUnkFTest)
xDFMF <- as.data.frame(xListMNF)
xDFMF$cha <- as.character(xDFMF$cha)

unk1 <- 555555
xListUnk1 <- list(as.integer(c(unk1, 1, 2, unk1, 5, 6, 7, 8, 9)),
                  c("A", "B", unk1, "C", unk1, "-", "7", "8", "9"),
                  c(9999, unk1, 1.5, unk1, 5, 6, 7, 8, 9),
                  factor(c("A", "0", "0", "NA", "NA", "9999", "0", "-", unk1)))
xListUnk1Test <- lapply(xListUnk1, function(x) x %in% unk1)
xListNUnk1 <- xListUnk1
names(xListNUnk1) <- c("int", "cha", "num", "fac")
xDFUnk1 <- as.data.frame(xListNUnk1)
xDFUnk1$cha <- as.character(xDFUnk1$cha)
xDFUnk1Test <- as.data.frame(xListUnk1Test)
names(xDFUnk1Test) <- names(xListNUnk1)

unkC2 <- c(0, "notAvail")
xListUnk2 <- list(as.integer(c(unkC2[1], 1, 2, unkC2[1], 5, 6, 7, 8, 9)),
                  c("A", "B", unkC2[2], "C", unkC2[2], "-", "7", "8", "9"),
                  c(9999, as.numeric(unkC2[1]), 1.5, as.numeric(unkC2[1]), 5, 6, 7, 8, 9),
                  factor(c("A", "0", "0", "NA", "NA", "9999", "0", "-", unkC2[2])))
xListNUnk2 <- xListUnk2
names(xListNUnk2) <- c("int", "cha", "num", "fac")
xDFUnk2 <- as.data.frame(xListNUnk2)
xDFUnk2$cha <- as.character(xDFUnk2$cha)

xListUnk2Test <- xListUnk2
xListUnk2Test[[1]] <- xListUnk2Test[[1]] %in% unkC2[1]
xListUnk2Test[[2]] <- xListUnk2Test[[2]] %in% unkC2[2]
xListUnk2Test[[3]] <- xListUnk2Test[[3]] %in% unkC2[1]
xListUnk2Test[[4]] <- xListUnk2Test[[4]] %in% unkC2[2]
xListNUnk2Test <- xListUnk2Test
names(xListNUnk2Test) <- names(xListNUnk2)
xDFUnk2Test <- as.data.frame(xListNUnk2Test)

unkL2 <- as.list(unkC2)
unkLN2 <- unkL2[c(2, 1)]
names(unkLN2) <- c("cha", "int")
xListUnk2a <- list(as.integer(c(NA, 1, 2, NA, 5, 6, 7, 8, 9)),
                  c("A", "B", unkLN2[[2]], "C", unkLN2[[2]], "-", "7", "8", "9"),
                  c(9999, NA, 1.5, NA, 5, 6, 7, 8, 9),
                  factor(c("A", "0", "0", "NA", "NA", "9999", "0", "-", unkLN2[[2]])))
xListUnk2aTest <- xListUnk2a
xListUnk2aTest[[1]] <- xListUnk2aTest[[1]] %in% unkLN2[1]
xListUnk2aTest[[2]] <- xListUnk2aTest[[2]] %in% unkLN2[2]
xListUnk2aTest[[3]] <- xListUnk2aTest[[3]] %in% unkLN2[1]
xListUnk2aTest[[4]] <- xListUnk2aTest[[4]] %in% unkLN2[2]

xList2a <- list(xListUnk2a[[1]],
                c("A", "B", NA, "C", NA, "-", "7", "8", "9"),
                xListUnk2a[[3]],
                factor(c("A", NA, NA, "NA", "NA", 9999, NA, "-", NA)))

### }}}
### {{{ --- Matrix ---

matUnk <- 9999
mat        <- matrix(1:25, nrow=5, ncol=5)
mat[1, 2] <- NA; mat[1, 4] <- NA; mat[2, 2] <- NA;
mat[3, 2] <- NA; mat[3, 5] <- NA; mat[5, 4] <- NA;
matUnk1       <- mat
matUnk1[1, 2] <- matUnk; matUnk1[1, 4] <- matUnk; matUnk1[2, 2] <- matUnk;
matUnk1[3, 2] <- matUnk; matUnk1[3, 5] <- matUnk; matUnk1[5, 4] <- matUnk;
matUnkTest <- matUnk1Test <- is.na(mat)

matUnk2Test <- matUnkTest | mat == 1

### }}}
### {{{ --- Use of unknown=list(.default=, ...) or similarly named vector ---

D1 <- "notAvail"
unkLND1 <- list(.default=D1)
xListUnkD1 <- list(as.integer(c(NA, 1:2, NA, 5, 6, 7, 8, 9)),
                   c("A", "B", D1, "C", D1, "-", "7", "8", "9"),
                   c(9999, NA, 1.5, NA, 5, 6, 7, 8, 9),
                   factor(c("A", "0", 0, "NA", "NA", intUnk, numUnk, "-", D1)))
xListUnkD1Test <- lapply(xListUnkD1, function(x) x %in% D1)
xListD1 <- xList

xListNUnkD1     <- xListUnkD1
xListNUnkD1Test <- xListUnkD1Test
names(xListNUnkD1) <- names(xListNUnkD1Test) <- names(xListNUnk1)
xListND1 <- xListN

DSO2 <- c("notAvail", 5678)
unkLNDSO2 <- as.list(DSO2)
names(unkLNDSO2) <- c(".default", "someOther")
xListUnkDSO2 <- list(as.integer(c(NA, 1:2, NA, 5, 6, 7, 8, 9)),
                      c("A", "B", DSO2[1], "C", DSO2[1], "-", "7", "8", "9"),
                      c(9999, NA, 1.5, NA, 5, 6, 7, 8, 9),
                      factor(c("A", "0", 0, "NA", "NA", intUnk, numUnk, "-", DSO2[2])))
xListUnkDSO2Test <- lapply(xListUnkDSO2, function(x) x %in% DSO2)

unkLND3 <- list(.default="notAvail", num=0, int=9999)
xListNUnkD3 <- list(int=as.integer(c(unkLND3[[3]], 1:2, unkLND3[[3]], 5, 6, 7, 8, 9)),
                    cha=c("A", "B", unkLND3[[1]], "C", unkLND3[[1]], "-", "7", "8", "9"),
                    num=c(9999, unkLND3[[2]], 1.5, unkLND3[[2]], 5, 6, 7, 8, 9),
                    fac=factor(c("A", "0", 0, "NA", "NA", intUnk, numUnk, "-", unkLND3[[1]])))
xListNUnkD3Test <- xListNUnkD3
xListNUnkD3Test$int <- xListNUnkD3Test$int %in% unkLND3[[3]]
xListNUnkD3Test$cha <- xListNUnkD3Test$cha %in% unkLND3[[1]]
xListNUnkD3Test$num <- xListNUnkD3Test$num %in% unkLND3[[2]]
xListNUnkD3Test$fac <- xListNUnkD3Test$fac %in% unkLND3[[1]]

unkLND2E <- list(.default="notAvail", 9999)

### }}}

### }}}
### {{{ --- isUnknown ---

test.isUnknown <- function()
{
  ## --- base methods for vectors ---

  ## base ...
  checkIdentical(isUnknown(xIntUnk, unknown=as.integer(intUnk)), xIntUnkTest)
  checkIdentical(isUnknown(xIntUnk, unknown=intUnk),             xIntUnkTest)
  checkIdentical(isUnknown(xNumUnk, unknown=numUnk),             xNumUnkTest)
  checkIdentical(isUnknown(xNumUnk, unknown=as.integer(numUnk)), xNumUnkTest)
  checkIdentical(isUnknown(xChaUnk, unknown=chaUnk),             xChaUnkTest)
  checkIdentical(isUnknown(xFacUnk, unknown=facUnk),             xFacUnkTest)

  ## multiple values are allowed for vector methods in vector or list form
  checkIdentical(isUnknown(xIntUnk, unknown=unkC), xIntUnkTest)
  checkIdentical(isUnknown(xIntUnk, unknown=unkL), xIntUnkTest)

  ## NA's in factors
  checkIdentical(isUnknown(xFacUnk1, unknown=facUnk1), xFacUnk1Test)
  facNA <- factor(c("0", 1, 2, 3, NA, "NA"))
  facNATest <- c(FALSE, FALSE, FALSE, FALSE, TRUE, FALSE)
  checkIdentical(isUnknown(facNA), facNATest)

  ## Date-time classes
  checkIdentical(isUnknown(xDateUnk, unknown=dateUnk), xDateTest)
  checkIdentical(isUnknown(xDate1Unk, unknown=dateUnk), xDate1Test)
  checkIdentical(isUnknown(xPOSIXltUnk, unknown=POSIXltUnk), xPOSIXltTest)
  checkIdentical(isUnknown(xPOSIXlt1Unk, unknown=POSIXltUnk), xPOSIXlt1Test)
  checkIdentical(isUnknown(xPOSIXctUnk, unknown=POSIXctUnk), xPOSIXctTest)
  checkIdentical(isUnknown(xPOSIXct1Unk, unknown=POSIXctUnk), xPOSIXct1Test)

  ## --- lists and data.frames ---

  ## with vector of single unknown values
  checkIdentical(isUnknown(xListUnk, unknown=unkC), xListUnkTest)
  checkIdentical(isUnknown(xDFUnk, unknown=unkC),   xDFUnkTest)

  ## with list of single unknown values
  checkIdentical(isUnknown(xListUnk, unknown=unkL), xListUnkTest)
  checkIdentical(isUnknown(xDFUnk, unknown=unkL),   xDFUnkTest)

  ## with named list of single unknown values
  checkIdentical(isUnknown(xListNUnk, unknown=unkLN), xListNUnkTest)
  checkIdentical(isUnknown(xDFUnk, unknown=unkLN),    xDFUnkTest)

  ## with named list of multiple unknown values - valid here
  checkIdentical(isUnknown(xListMNUnkF, unknown=unkLMN), xListMNUnkFTest)
  checkIdentical(isUnknown(xDFMUnkF, unknown=unkLMN), xDFMUnkFTest)

  ## with single unknown value - recycling
  checkIdentical(isUnknown(xListUnk1, unknown=unk1), xListUnk1Test)
  checkIdentical(isUnknown(xDFUnk1,   unknown=unk1), xDFUnk1Test)

  ## with vector of two unknown values - recycling
  checkIdentical(isUnknown(xListUnk2, unknown=unkC2), xListUnk2Test)
  checkIdentical(isUnknown(xDFUnk2,   unknown=unkC2), xDFUnk2Test)

  ## with list of two unknown values - recycling
  checkIdentical(isUnknown(xListUnk2, unknown=unkL2), xListUnk2Test)
  checkIdentical(isUnknown(xDFUnk2,   unknown=unkL2), xDFUnk2Test)

  ## list(.default=)
  checkIdentical(isUnknown(x=xListUnkD1, unknown=unkLND1), xListUnkD1Test)
  ## list(.default=, someOther=) we do not know someOther, but should work
  ## as x is not named
  checkIdentical(isUnknown(x=xListUnkDSO2, unknown=unkLNDSO2), xListUnkDSO2Test)
  ## list(.default=) in named list
  checkIdentical(isUnknown(x=xListNUnkD1, unknown=unkLND1), xListNUnkD1Test)
  ## list(.default=, someOther=) OK if someOther is in the named list
  checkIdentical(isUnknown(x=xListNUnkD3, unknown=unkLND3), xListNUnkD3Test)
  ## list(.default=, 99) ERROR as we do not know where to apply 99
  checkException(isUnknown(x=xListNUnk, unknown=unkLND2E))

  ## --- matrix ---

  checkIdentical(isUnknown(x=mat, unknown=NA), matUnkTest)
  checkIdentical(isUnknown(x=matUnk1, unknown=matUnk), matUnkTest)
  checkIdentical(isUnknown(x=matUnk1, unknown=c(1, matUnk)), matUnk2Test)
}

### }}}
### {{{ --- unknownToNA ---

test.unknownToNA <- function()
{
  ## --- base methods for vectors ---

  ## base ...
  checkIdentical(unknownToNA(xIntUnk, as.integer(intUnk)), xInt)
  checkIdentical(unknownToNA(xIntUnk, intUnk),             xInt) ## with numeric
  checkIdentical(unknownToNA(xNumUnk, numUnk),             xNum)
  checkIdentical(unknownToNA(xNumUnk, as.integer(numUnk)), xNum)
  checkIdentical(unknownToNA(xChaUnk, chaUnk),             xCha)
  checkIdentical(unknownToNA(xChaUnk, chaUnk),             xCha)
  checkIdentical(unknownToNA(xFacUnk, facUnk),             xFac)

  ## multiple values are allowed for vector methods in vector or list form
  checkIdentical(unknownToNA(xIntUnk, unknown=unkC), xInt)
  checkIdentical(unknownToNA(xIntUnk, unknown=unkL), xInt)

  ## NA's in factors
  checkIdentical(unknownToNA(xFacUnk1, unknown=facUnk1), xFac1)
  facNA <- factor(c("0", 1, 2, 3, NA, "NA"))
  facNATest <- factor(c("0", 1, 2, 3, NA, NA))
  checkIdentical(unknownToNA(x=facNA, unknown="NA"), facNATest)

  ## Date-time classes
  checkIdentical(unknownToNA(xDateUnk, unknown=dateUnk), xDate)
  checkIdentical(unknownToNA(xPOSIXctUnk, unknown=POSIXctUnk), xPOSIXct)

  ####
  ## Per Brian Ripley on 2014-01-15:
  ##
  ## On platforms where POSIXlt has a gmtoff component, it does not need to be set.  So
  ##
  ## > z$gmtoff
  ## [1] 3600   NA
  ## > xPOSIXltUnk$gmtoff
  ## [1] 3600 3600
  ##
  ## (or sometimes 0, not NA).
  ##
  ## So although identical() correctly reports that they differ, this
  ## is allowed for optional components.
  ##
  ## It would also be wrong to use identical() to compare isdst
  ## components: isdst = -1 means unknown.
  ##
  ## Replaced:
  ##   checkIdentical(unknownToNA(xPOSIXltUnk, unknown=POSIXltUnk), xPOSIXlt)
  ## With:
  tmp_unknownToNA <- unknownToNA(xPOSIXltUnk, unknown=POSIXltUnk)
  tmp_xPOSIXlt   <- xPOSIXlt
  ##
  tmp_unknownToNA$gmtoff <- NULL  # Remove $gmtoff to avoid comparison
  tmp_xPOSIXlt$gmtoff   <- NULL
  ##
  isdst.unknown <- unique(
      c(which(is.na(tmp_unknownToNA$isdst) |
              tmp_unknownToNA$isdst==-1
              )
        )
      ,
      c(which(is.na(tmp_xPOSIXlt$isdst) |
              tmp_xPOSIXlt$isdst==-1
              )
        )

      )
  ##
  checkIdentical(tmp_unknownToNA$isdst[!isdst.unknown],
                 tmp_xPOSIXlt$isds[!isdst.unknown])
  ##
  tmp_unknownToNA$isdst <- NULL   # Remove $isdst to avoid comparison
  tmp_xPOSIXlt$isdst   <- NULL    # by checkIdentical
  ##
  checkIdentical(tmp_unknownToNA, tmp_xPOSIXlt)
  ####


  ## --- lists and data.frames ---

  ## with vector of single unknown values
  checkIdentical(unknownToNA(xListUnk, unknown=unkC), xList)
  checkIdentical(unknownToNA(xDFUnk, unknown=unkC), xDF)

  ## with list of single unknown values
  checkIdentical(unknownToNA(xListUnk, unknown=unkL), xList)
  checkIdentical(unknownToNA(xDFUnk, unknown=unkL), xDF)

  ## with named list of single unknown values
  checkIdentical(unknownToNA(xListNUnk, unknown=unkLN), xListN)
  checkIdentical(unknownToNA(xDFUnk, unknown=unkLN), xDF)

  ## with names list of multiple unknown values - must be an error
  checkIdentical(unknownToNA(xListMNUnkF, unknown=unkLMN), xListMNF)
  checkIdentical(unknownToNA(xDFMUnkF, unknown=unkLMN), xDFMF)

  ## with single unknown value - recycling
  checkIdentical(unknownToNA(xListUnk1, unknown=unk1), xList)
  checkIdentical(unknownToNA(xDFUnk1,   unknown=unk1), xDF)

  ## with vector of two unknown values - recycling
  checkIdentical(unknownToNA(xListUnk2, unknown=unkC2), xList)
  checkIdentical(unknownToNA(xDFUnk2,   unknown=unkC2), xDF)

  ## with list of two unknown values - recycling
  checkIdentical(unknownToNA(xListUnk2, unknown=unkL2), xList)
  checkIdentical(unknownToNA(xDFUnk2,   unknown=unkL2), xDF)

  ## with named list of two unknown values but x is not named so named list
  ## does not have any effect --> error as we do not know how to recycle
  checkException(unknownToNA(xListUnk2a, unknown=unkLN2))

  ## but we should get some results with named x
  checkIdentical(unknownToNA(xListNUnk2, unknown=unkL2), xListN)
  ## not also necesarilly with recycling of names lists, as it is
  ## not clear how to properly recycle named lists (only names that match
  ## can be really properly recycled)
  checkException(unknownToNA(xListNUnk2, unknown=unkLN2))
  checkIdentical(unknownToNA(xDFUnk2, unknown=unkL2), xDF)
  checkException(unknownToNA(xDFUnk2, unknown=unkLN2))

  ## list(.default=)
  checkIdentical(unknownToNA(x=xListUnkD1, unknown=unkLND1), xListD1)
  ## list(.default=, someOther=) we do not know someOther, but should work
  ## as x is not named
  checkIdentical(unknownToNA(x=xListUnkDSO2, unknown=unkLNDSO2), xList)
  ## list(.default=) in named list
  checkIdentical(unknownToNA(x=xListNUnkD1, unknown=unkLND1), xListND1)
  ## list(.default=, someOther=) OK if someOther is in the named list
  checkIdentical(unknownToNA(x=xListNUnkD3, unknown=unkLND3), xListN)
  ## list(.default=, 99) ERROR as we do not know where to apply 99
  checkException(unknownToNA(x=xListNUnk, unknown=unkLND2E))

  ## --- matrix ---

  checkEquals(unknownToNA(x=matUnk1, unknown=matUnk), mat)
}

### }}}
### {{{ --- NAToUnknown ---

test.NAToUnknown <- function()
{
  ## --- base methods for vectors ---

  ## base ...
  checkIdentical(NAToUnknown(xInt, as.integer(intUnk)), xIntUnk)
  checkIdentical(NAToUnknown(xInt, intUnk),             xIntUnk) ## with numeric
  checkIdentical(NAToUnknown(xNum, numUnk),             xNumUnk)
  checkIdentical(NAToUnknown(xNum, as.integer(numUnk)), xNumUnk)
  checkIdentical(NAToUnknown(xCha, chaUnk),             xChaUnk)
  checkIdentical(NAToUnknown(xCha, chaUnk),             xChaUnk)
  checkIdentical(NAToUnknown(xFac, facUnk),             xFacUnk)

  ## only single values are allowed for vector methods
  checkException(NAToUnknown(xInt, unknown=unkC))
  checkException(NAToUnknown(xInt, unknown=unkL))

  ## and they should not already be in x unless force=TRUE
  checkException(NAToUnknown(xCha, unknown=chaUnk1))
  checkIdentical(NAToUnknown(xCha, unknown=chaUnk1, force=TRUE), xChaUnk1)

  checkException(NAToUnknown(xFac, unknown=facLev))
  checkIdentical(NAToUnknown(xFac, unknown=facLev, force=TRUE), xFacUnkLev)

  ## NA's in factors
  checkIdentical(NAToUnknown(xFac, unknown=facUnk1, force=TRUE), xFacUnk1)
  facNA <- factor(c("0", 1, 2, 3, NA, NA))
  facNATest <- factor(c("0", 1, 2, 3, "NA", "NA"))
  checkIdentical(NAToUnknown(x=facNA, unknown="NA"), facNATest)

  ## Date-time classes
  checkIdentical(NAToUnknown(xDate, unknown=dateUnk), xDateUnk)
  checkIdentical(NAToUnknown(xPOSIXct, unknown=POSIXctUnk), xPOSIXctUnk)


  ####
  ## Per Brian Ripley on 2014-01-15:
  ##
  ## On platforms where POSIXlt has a gmtoff component, it does not need to be set.  So
  ##
  ## > z$gmtoff
  ## [1] 3600   NA
  ## > xPOSIXltUnk$gmtoff
  ## [1] 3600 3600
  ##
  ## (or sometimes 0, not NA).
  ##
  ## So although identical() correctly reports that they differ, this
  ## is allowed for optional components.
  ##
  ## It would also be wrong to use identical() to compare isdst
  ## components: isdst = -1 means unknown.
  ##
  ## Replaced:
  ##   checkIdentical(NAToUnknown(xPOSIXlt, unknown=POSIXltUnk), xPOSIXltUnk)
  ## With:
  tmp_NAToUnknown <- NAToUnknown(xPOSIXlt, unknown=POSIXltUnk)
  tmp_xPOSIXltUnk   <- xPOSIXltUnk
  ##
  tmp_NAToUnknown$gmtoff <- NULL  # Remove $gmtoff to avoid comparison
  tmp_xPOSIXltUnk$gmtoff   <- NULL
  ##
  isdst.unknown <- unique(
      c(which(is.na(tmp_NAToUnknown$isdst) |
              tmp_NAToUnknown$isdst==-1
              )
        )
      ,
      c(which(is.na(tmp_xPOSIXltUnk$isdst) |
              tmp_xPOSIXltUnk$isdst==-1
              )
        )

      )
  ##
  checkIdentical(tmp_NAToUnknown$isdst[!isdst.unknown],
                 tmp_xPOSIXltUnk$isds[!isdst.unknown])
  ##
  tmp_NAToUnknown$isdst <- NULL   # Remove $isdst to avoid comparison
  tmp_xPOSIXltUnk$isdst   <- NULL    # by checkIdentical
  ##
  checkIdentical(tmp_NAToUnknown, tmp_xPOSIXltUnk)
  ####


  ## --- lists and data.frames ---

  ## with vector of single unknown values
  checkIdentical(NAToUnknown(xList, unknown=unkC), xListUnk)
  checkIdentical(NAToUnknown(xDF, unknown=unkC), xDFUnk)

  ## with list of single unknown values
  checkIdentical(NAToUnknown(xList, unknown=unkL), xListUnk)
  checkIdentical(NAToUnknown(xDF, unknown=unkL), xDFUnk)

  ## with named list of single unknown values
  checkIdentical(NAToUnknown(xListN, unknown=unkLN), xListNUnk)
  checkIdentical(NAToUnknown(xDF, unknown=unkLN), xDFUnk)

  ## with names list of multiple unknown values - must be an error
  checkException(NAToUnknown(xListN, unknown=unkLMN))
  checkException(NAToUnknown(xDF, unknown=unkLMN))

  ## with single unknown value - recycling
  checkIdentical(NAToUnknown(xList, unknown=unk1), xListUnk1)
  checkIdentical(NAToUnknown(xDF,   unknown=unk1), xDFUnk1)

  ## with vector of two unknown values - recycling
  checkIdentical(NAToUnknown(xList, unknown=unkC2), xListUnk2)
  checkIdentical(NAToUnknown(xDF,   unknown=unkC2), xDFUnk2)

  ## with list of two unknown values - recycling
  checkIdentical(NAToUnknown(xList, unknown=unkL2), xListUnk2)
  checkIdentical(NAToUnknown(xDF,   unknown=unkL2), xDFUnk2)

  ## with named list of two unknown values but x is not named so named list
  ## does not have any effect --> error as we do not know how to recycle
  checkException(NAToUnknown(xList, unknown=unkLN2))

  ## but we should get some results with named x
  checkIdentical(NAToUnknown(xListN, unknown=unkL2), xListNUnk2)
  ## not also necesarilly with recycling of names lists, as it is
  ## not clear how to properly recycle named lists (only names that match
  ## can be really properly recycled)
  checkException(NAToUnknown(xListN, unknown=unkLN2))
  checkIdentical(NAToUnknown(xDF, unknown=unkL2), xDFUnk2)
  checkException(NAToUnknown(xDF, unknown=unkLN2))

  ## list(.default=)
  checkIdentical(NAToUnknown(x=xList, unknown=unkLND1), xListUnkD1)
  ## list(.default=, someOther=) we do not know someOther, but should work
  ## as x is not named
  checkIdentical(NAToUnknown(x=xList, unknown=unkLNDSO2), xListUnkDSO2)
  ## list(.default=) in named list
  checkIdentical(NAToUnknown(x=xListN, unknown=unkLND1), xListNUnkD1)
  ## list(.default=, someOther=) OK if someOther is in the named list
  checkIdentical(NAToUnknown(x=xListN, unknown=unkLND3), xListNUnkD3)
  ## list(.default=, 99) ERROR as we do not know where to apply 99
  checkException(NAToUnknown(x=xListN, unknown=unkLND2E))

  ## --- matrix ---

  checkEquals(NAToUnknown(x=mat, unknown=matUnk), matUnk1)
}

### }}}
### {{{ Dear Emacs
### Local variables:
### folded-file: t
### End:
### }}}

###------------------------------------------------------------------------
### runit.unknown.R ends here

### runit.unknown.R
###------------------------------------------------------------------------
### What: Tests for Change given unknown value to NA and vice versa methods
### $Id: runit.unknown.R 993 2006-10-30 17:10:08Z ggorjan $
### Time-stamp: <2006-10-30 17:46:21 ggorjan>
###------------------------------------------------------------------------

### {{{ --- Test setup ---

if(FALSE) {
  library("RUnit")
  library("gdata")
}

### {{{ --- Vectors ---

intUnk <- 9999
xInt        <- as.integer(c(NA,     1:2, NA,     5, 6, 7, 8, 9))
xIntUnk     <- as.integer(c(intUnk, 1:2, intUnk, 5, 6, 7, 8, 9))
xIntUnkTest <- xIntUnk %in% intUnk

numUnk <- 0
xNum        <- c(9999, NA, 1.5, NA, 5, 6, 7, 8, 9)
xNumUnk     <- c(9999, 0,  1.5, 0,  5, 6, 7, 8, 9)
xNumUnkTest <- xNumUnk %in% numUnk

chaUnk <- "notAvail"
chaUnk1 <- "-"
xCha        <- c("A", "B", NA,      "C", NA,      "-", "7", "8", "9")
xChaUnk     <- c("A", "B", chaUnk,  "C", chaUnk,  "-", "7", "8", "9")
xChaUnk1    <- c("A", "B", chaUnk1, "C", chaUnk1, "-", "7", "8", "9")
xChaUnkTest  <- xChaUnk %in% chaUnk
xChaUnk1Test <- xChaUnk %in% chaUnk1

facUnk <- "notAvail"
facUnk1 <- "NA"
xFac     <- factor(c("A", "0", 0, "NA", "NA", intUnk, numUnk, "-", NA))
xFacUnk  <- factor(c("A", "0", 0, "NA", "NA", intUnk, numUnk, "-", facUnk))
xFacUnk1 <- factor(c("A", "0", 0, "NA", "NA", intUnk, numUnk, "-", facUnk1))
xFacUnkTest <-     c(0,   0,   0, 0,    0,    0,      0,      0,   1)
xFacUnkTest <- as.logical(xFacUnkTest)
xFacUnk1Test <-    c(0,   0,   0, 1,    1,    0,      0,      0,   1)
xFacUnk1Test <- as.logical(xFacUnk1Test)
xFac1    <- factor(c("A", "0", 0, NA, NA, intUnk, numUnk, "-", NA))

facLev <- "A"
xFacUnkLev <- factor(c("A", "0", 0, "NA", "NA", intUnk, numUnk, "-", "A"))
xFacUnkLevTest <-    c(1,    0,  0, 0,    0,    0,      0,      0,   1)
xFacUnkLevTest <- as.logical(xFacUnkLevTest)

dateUnk <- as.Date("2006-08-14")
tmp <- as.Date("2006-08-15")
xDate     <- c(tmp, NA)
xDateUnk  <- c(tmp,   dateUnk)
xDateTest <- c(FALSE, TRUE)

xDate1Unk  <- c(tmp,   dateUnk, NA)
xDate1Test <- c(FALSE, TRUE,    FALSE)

POSIXltUnk <- strptime("2006-08-14", format="%Y-%m-%d")
tmp <- strptime("2006-08-15", format="%Y-%m-%d")
xPOSIXlt     <- c(tmp, NA)
xPOSIXltUnk  <- c(tmp,   POSIXltUnk)
xPOSIXltTest <- c(FALSE, TRUE)

xPOSIXlt1Unk  <- c(tmp,   POSIXltUnk, NA)
xPOSIXlt1Test <- c(FALSE, TRUE,       FALSE)

POSIXctUnk <- as.POSIXct(strptime("2006-08-14 01:01:01", format="%Y-%m-%d %H:%M:%S"))
tmp <- as.POSIXct(strptime("2006-08-15 01:01:01", format="%Y-%m-%d %H:%M:%S"))
xPOSIXct     <- c(tmp, NA)
xPOSIXctUnk  <- c(tmp, POSIXctUnk)
xPOSIXctTest <- xPOSIXltTest

xPOSIXct1Unk  <- c(tmp, POSIXctUnk, NA)
xPOSIXct1Test <- xPOSIXlt1Test

### }}}
### {{{ --- Lists and data.frames ---

xList <- list(xInt, xCha, xNum, xFac)
xListN <- list(int=xInt, cha=xCha, num=xNum, fac=xFac)
xListUnk <- list(xIntUnk, xChaUnk, xNumUnk, xFacUnk)
xListUnkTest <- list(xIntUnkTest, xChaUnkTest, xNumUnkTest, xFacUnkTest)
xListNUnk <- list(int=xIntUnk, cha=xChaUnk, num=xNumUnk, fac=xFacUnk)
xListNUnkTest <- list(int=xIntUnkTest, cha=xChaUnkTest, num=xNumUnkTest, fac=xFacUnkTest)

xDF <- as.data.frame(xListN)
xDF$cha <- as.character(xDF$cha)
xDFUnk <- as.data.frame(xListNUnk)
xDFUnk$cha <- as.character(xDFUnk$cha)
xDFUnkTest <- as.data.frame(xListNUnkTest)

unkC <- c(intUnk, chaUnk, numUnk, facUnk)
unkL <- list(intUnk, chaUnk, numUnk, facUnk)
unkLN <- list(num=numUnk, cha=chaUnk, fac=facUnk, int=intUnk) ## mixed as it is named
unkLMN <- list(cha=chaUnk, int=intUnk, num=c(intUnk, numUnk),
               fac=c(chaUnk1, facUnk))

xListMNUnkF <- list(int=as.integer(c(9999, 1, 2, 9999, 5, 6, 7, 8, 9)),
                   cha=c("A", "B", "notAvail", "C", "notAvail", "-", "7", "8", "9"),
                   num=c(9999, 0, 1.5, 0, 5, 6, 7, 8, 9),
                   fac=factor(c("A", "0", "0", "NA", "NA", 9999, "0", "-", "notAvail")))
xListMNUnkFTest <- list(int=c(1, 0, 0, 1, 0, 0, 0, 0, 0),
                        cha=c(0, 0, 1, 0, 1, 0, 0, 0, 0),
                        num=c(1, 1, 0, 1, 0, 0, 0, 0, 0),
                        fac=c(0, 0, 0, 0, 0, 0, 0, 1, 1))
xListMNUnkFTest <- lapply(xListMNUnkFTest, as.logical)
xListMNF <- list(int=as.integer(c(NA, 1, 2, NA, 5, 6, 7, 8, 9)),
                 cha=c("A", "B", NA, "C", NA, "-", "7", "8", "9"),
                 num=c(NA, NA, 1.5, NA, 5, 6, 7, 8, 9),
                 fac=factor(c("A", "0", "0", "NA", "NA", "9999", "0", NA, NA)))

xDFMUnkF <- as.data.frame(xListMNUnkF)
xDFMUnkF$cha <- as.character(xDFMUnkF$cha)
xDFMUnkFTest <- as.data.frame(xListMNUnkFTest)
xDFMF <- as.data.frame(xListMNF)
xDFMF$cha <- as.character(xDFMF$cha)

unk1 <- 555555
xListUnk1 <- list(as.integer(c(unk1, 1, 2, unk1, 5, 6, 7, 8, 9)),
                  c("A", "B", unk1, "C", unk1, "-", "7", "8", "9"),
                  c(9999, unk1, 1.5, unk1, 5, 6, 7, 8, 9),
                  factor(c("A", "0", "0", "NA", "NA", "9999", "0", "-", unk1)))
xListUnk1Test <- lapply(xListUnk1, function(x) x %in% unk1)
xListNUnk1 <- xListUnk1
names(xListNUnk1) <- c("int", "cha", "num", "fac")
xDFUnk1 <- as.data.frame(xListNUnk1)
xDFUnk1$cha <- as.character(xDFUnk1$cha)
xDFUnk1Test <- as.data.frame(xListUnk1Test)
names(xDFUnk1Test) <- names(xListNUnk1)

unkC2 <- c(0, "notAvail")
xListUnk2 <- list(as.integer(c(unkC2[1], 1, 2, unkC2[1], 5, 6, 7, 8, 9)),
                  c("A", "B", unkC2[2], "C", unkC2[2], "-", "7", "8", "9"),
                  c(9999, as.numeric(unkC2[1]), 1.5, as.numeric(unkC2[1]), 5, 6, 7, 8, 9),
                  factor(c("A", "0", "0", "NA", "NA", "9999", "0", "-", unkC2[2])))
xListNUnk2 <- xListUnk2
names(xListNUnk2) <- c("int", "cha", "num", "fac")
xDFUnk2 <- as.data.frame(xListNUnk2)
xDFUnk2$cha <- as.character(xDFUnk2$cha)

xListUnk2Test <- xListUnk2
xListUnk2Test[[1]] <- xListUnk2Test[[1]] %in% unkC2[1]
xListUnk2Test[[2]] <- xListUnk2Test[[2]] %in% unkC2[2]
xListUnk2Test[[3]] <- xListUnk2Test[[3]] %in% unkC2[1]
xListUnk2Test[[4]] <- xListUnk2Test[[4]] %in% unkC2[2]
xListNUnk2Test <- xListUnk2Test
names(xListNUnk2Test) <- names(xListNUnk2)
xDFUnk2Test <- as.data.frame(xListNUnk2Test)

unkL2 <- as.list(unkC2)
unkLN2 <- unkL2[c(2, 1)]
names(unkLN2) <- c("cha", "int")
xListUnk2a <- list(as.integer(c(NA, 1, 2, NA, 5, 6, 7, 8, 9)),
                  c("A", "B", unkLN2[[2]], "C", unkLN2[[2]], "-", "7", "8", "9"),
                  c(9999, NA, 1.5, NA, 5, 6, 7, 8, 9),
                  factor(c("A", "0", "0", "NA", "NA", "9999", "0", "-", unkLN2[[2]])))
xListUnk2aTest <- xListUnk2a
xListUnk2aTest[[1]] <- xListUnk2aTest[[1]] %in% unkLN2[1]
xListUnk2aTest[[2]] <- xListUnk2aTest[[2]] %in% unkLN2[2]
xListUnk2aTest[[3]] <- xListUnk2aTest[[3]] %in% unkLN2[1]
xListUnk2aTest[[4]] <- xListUnk2aTest[[4]] %in% unkLN2[2]

xList2a <- list(xListUnk2a[[1]],
                c("A", "B", NA, "C", NA, "-", "7", "8", "9"),
                xListUnk2a[[3]],
                factor(c("A", NA, NA, "NA", "NA", 9999, NA, "-", NA)))

### }}}
### {{{ --- Matrix ---

matUnk <- 9999
mat        <- matrix(1:25, nrow=5, ncol=5)
mat[1, 2] <- NA; mat[1, 4] <- NA; mat[2, 2] <- NA;
mat[3, 2] <- NA; mat[3, 5] <- NA; mat[5, 4] <- NA;
matUnk1       <- mat
matUnk1[1, 2] <- matUnk; matUnk1[1, 4] <- matUnk; matUnk1[2, 2] <- matUnk;
matUnk1[3, 2] <- matUnk; matUnk1[3, 5] <- matUnk; matUnk1[5, 4] <- matUnk;
matUnkTest <- matUnk1Test <- is.na(mat)

matUnk2Test <- matUnkTest | mat == 1

### }}}
### {{{ --- Use of unknown=list(.default=, ...) or similarly named vector ---

D1 <- "notAvail"
unkLND1 <- list(.default=D1)
xListUnkD1 <- list(as.integer(c(NA, 1:2, NA, 5, 6, 7, 8, 9)),
                   c("A", "B", D1, "C", D1, "-", "7", "8", "9"),
                   c(9999, NA, 1.5, NA, 5, 6, 7, 8, 9),
                   factor(c("A", "0", 0, "NA", "NA", intUnk, numUnk, "-", D1)))
xListUnkD1Test <- lapply(xListUnkD1, function(x) x %in% D1)
xListD1 <- xList

xListNUnkD1     <- xListUnkD1
xListNUnkD1Test <- xListUnkD1Test
names(xListNUnkD1) <- names(xListNUnkD1Test) <- names(xListNUnk1)
xListND1 <- xListN

DSO2 <- c("notAvail", 5678)
unkLNDSO2 <- as.list(DSO2)
names(unkLNDSO2) <- c(".default", "someOther")
xListUnkDSO2 <- list(as.integer(c(NA, 1:2, NA, 5, 6, 7, 8, 9)),
                      c("A", "B", DSO2[1], "C", DSO2[1], "-", "7", "8", "9"),
                      c(9999, NA, 1.5, NA, 5, 6, 7, 8, 9),
                      factor(c("A", "0", 0, "NA", "NA", intUnk, numUnk, "-", DSO2[2])))
xListUnkDSO2Test <- lapply(xListUnkDSO2, function(x) x %in% DSO2)

unkLND3 <- list(.default="notAvail", num=0, int=9999)
xListNUnkD3 <- list(int=as.integer(c(unkLND3[[3]], 1:2, unkLND3[[3]], 5, 6, 7, 8, 9)),
                    cha=c("A", "B", unkLND3[[1]], "C", unkLND3[[1]], "-", "7", "8", "9"),
                    num=c(9999, unkLND3[[2]], 1.5, unkLND3[[2]], 5, 6, 7, 8, 9),
                    fac=factor(c("A", "0", 0, "NA", "NA", intUnk, numUnk, "-", unkLND3[[1]])))
xListNUnkD3Test <- xListNUnkD3
xListNUnkD3Test$int <- xListNUnkD3Test$int %in% unkLND3[[3]]
xListNUnkD3Test$cha <- xListNUnkD3Test$cha %in% unkLND3[[1]]
xListNUnkD3Test$num <- xListNUnkD3Test$num %in% unkLND3[[2]]
xListNUnkD3Test$fac <- xListNUnkD3Test$fac %in% unkLND3[[1]]

unkLND2E <- list(.default="notAvail", 9999)

### }}}

### }}}
### {{{ --- isUnknown ---

test.isUnknown <- function()
{
  ## --- base methods for vectors ---

  ## base ...
  checkIdentical(isUnknown(xIntUnk, unknown=as.integer(intUnk)), xIntUnkTest)
  checkIdentical(isUnknown(xIntUnk, unknown=intUnk),             xIntUnkTest)
  checkIdentical(isUnknown(xNumUnk, unknown=numUnk),             xNumUnkTest)
  checkIdentical(isUnknown(xNumUnk, unknown=as.integer(numUnk)), xNumUnkTest)
  checkIdentical(isUnknown(xChaUnk, unknown=chaUnk),             xChaUnkTest)
  checkIdentical(isUnknown(xFacUnk, unknown=facUnk),             xFacUnkTest)

  ## multiple values are allowed for vector methods in vector or list form
  checkIdentical(isUnknown(xIntUnk, unknown=unkC), xIntUnkTest)
  checkIdentical(isUnknown(xIntUnk, unknown=unkL), xIntUnkTest)

  ## NA's in factors
  checkIdentical(isUnknown(xFacUnk1, unknown=facUnk1), xFacUnk1Test)
  facNA <- factor(c("0", 1, 2, 3, NA, "NA"))
  facNATest <- c(FALSE, FALSE, FALSE, FALSE, TRUE, FALSE)
  checkIdentical(isUnknown(facNA), facNATest)

  ## Date-time classes
  checkIdentical(isUnknown(xDateUnk, unknown=dateUnk), xDateTest)
  checkIdentical(isUnknown(xDate1Unk, unknown=dateUnk), xDate1Test)
  checkIdentical(isUnknown(xPOSIXltUnk, unknown=POSIXltUnk), xPOSIXltTest)
  checkIdentical(isUnknown(xPOSIXlt1Unk, unknown=POSIXltUnk), xPOSIXlt1Test)
  checkIdentical(isUnknown(xPOSIXctUnk, unknown=POSIXctUnk), xPOSIXctTest)
  checkIdentical(isUnknown(xPOSIXct1Unk, unknown=POSIXctUnk), xPOSIXct1Test)

  ## --- lists and data.frames ---

  ## with vector of single unknown values
  checkIdentical(isUnknown(xListUnk, unknown=unkC), xListUnkTest)
  checkIdentical(isUnknown(xDFUnk, unknown=unkC),   xDFUnkTest)

  ## with list of single unknown values
  checkIdentical(isUnknown(xListUnk, unknown=unkL), xListUnkTest)
  checkIdentical(isUnknown(xDFUnk, unknown=unkL),   xDFUnkTest)

  ## with named list of single unknown values
  checkIdentical(isUnknown(xListNUnk, unknown=unkLN), xListNUnkTest)
  checkIdentical(isUnknown(xDFUnk, unknown=unkLN),    xDFUnkTest)

  ## with named list of multiple unknown values - valid here
  checkIdentical(isUnknown(xListMNUnkF, unknown=unkLMN), xListMNUnkFTest)
  checkIdentical(isUnknown(xDFMUnkF, unknown=unkLMN), xDFMUnkFTest)

  ## with single unknown value - recycling
  checkIdentical(isUnknown(xListUnk1, unknown=unk1), xListUnk1Test)
  checkIdentical(isUnknown(xDFUnk1,   unknown=unk1), xDFUnk1Test)

  ## with vector of two unknown values - recycling
  checkIdentical(isUnknown(xListUnk2, unknown=unkC2), xListUnk2Test)
  checkIdentical(isUnknown(xDFUnk2,   unknown=unkC2), xDFUnk2Test)

  ## with list of two unknown values - recycling
  checkIdentical(isUnknown(xListUnk2, unknown=unkL2), xListUnk2Test)
  checkIdentical(isUnknown(xDFUnk2,   unknown=unkL2), xDFUnk2Test)

  ## list(.default=)
  checkIdentical(isUnknown(x=xListUnkD1, unknown=unkLND1), xListUnkD1Test)
  ## list(.default=, someOther=) we do not know someOther, but should work
  ## as x is not named
  checkIdentical(isUnknown(x=xListUnkDSO2, unknown=unkLNDSO2), xListUnkDSO2Test)
  ## list(.default=) in named list
  checkIdentical(isUnknown(x=xListNUnkD1, unknown=unkLND1), xListNUnkD1Test)
  ## list(.default=, someOther=) OK if someOther is in the named list
  checkIdentical(isUnknown(x=xListNUnkD3, unknown=unkLND3), xListNUnkD3Test)
  ## list(.default=, 99) ERROR as we do not know where to apply 99
  checkException(isUnknown(x=xListNUnk, unknown=unkLND2E))

  ## --- matrix ---

  checkIdentical(isUnknown(x=mat, unknown=NA), matUnkTest)
  checkIdentical(isUnknown(x=matUnk1, unknown=matUnk), matUnkTest)
  checkIdentical(isUnknown(x=matUnk1, unknown=c(1, matUnk)), matUnk2Test)
}

### }}}
### {{{ --- unknownToNA ---

test.unknownToNA <- function()
{
  ## --- base methods for vectors ---

  ## base ...
  checkIdentical(unknownToNA(xIntUnk, as.integer(intUnk)), xInt)
  checkIdentical(unknownToNA(xIntUnk, intUnk),             xInt) ## with numeric
  checkIdentical(unknownToNA(xNumUnk, numUnk),             xNum)
  checkIdentical(unknownToNA(xNumUnk, as.integer(numUnk)), xNum)
  checkIdentical(unknownToNA(xChaUnk, chaUnk),             xCha)
  checkIdentical(unknownToNA(xChaUnk, chaUnk),             xCha)
  checkIdentical(unknownToNA(xFacUnk, facUnk),             xFac)

  ## multiple values are allowed for vector methods in vector or list form
  checkIdentical(unknownToNA(xIntUnk, unknown=unkC), xInt)
  checkIdentical(unknownToNA(xIntUnk, unknown=unkL), xInt)

  ## NA's in factors
  checkIdentical(unknownToNA(xFacUnk1, unknown=facUnk1), xFac1)
  facNA <- factor(c("0", 1, 2, 3, NA, "NA"))
  facNATest <- factor(c("0", 1, 2, 3, NA, NA))
  checkIdentical(unknownToNA(x=facNA, unknown="NA"), facNATest)

  ## Date-time classes
  checkIdentical(unknownToNA(xDateUnk, unknown=dateUnk), xDate)
  checkIdentical(unknownToNA(xPOSIXltUnk, unknown=POSIXltUnk), xPOSIXlt)
  checkIdentical(unknownToNA(xPOSIXctUnk, unknown=POSIXctUnk), xPOSIXct)

  ## --- lists and data.frames ---

  ## with vector of single unknown values
  checkIdentical(unknownToNA(xListUnk, unknown=unkC), xList)
  checkIdentical(unknownToNA(xDFUnk, unknown=unkC), xDF)

  ## with list of single unknown values
  checkIdentical(unknownToNA(xListUnk, unknown=unkL), xList)
  checkIdentical(unknownToNA(xDFUnk, unknown=unkL), xDF)

  ## with named list of single unknown values
  checkIdentical(unknownToNA(xListNUnk, unknown=unkLN), xListN)
  checkIdentical(unknownToNA(xDFUnk, unknown=unkLN), xDF)

  ## with names list of multiple unknown values - must be an error
  checkIdentical(unknownToNA(xListMNUnkF, unknown=unkLMN), xListMNF)
  checkIdentical(unknownToNA(xDFMUnkF, unknown=unkLMN), xDFMF)

  ## with single unknown value - recycling
  checkIdentical(unknownToNA(xListUnk1, unknown=unk1), xList)
  checkIdentical(unknownToNA(xDFUnk1,   unknown=unk1), xDF)

  ## with vector of two unknown values - recycling
  checkIdentical(unknownToNA(xListUnk2, unknown=unkC2), xList)
  checkIdentical(unknownToNA(xDFUnk2,   unknown=unkC2), xDF)

  ## with list of two unknown values - recycling
  checkIdentical(unknownToNA(xListUnk2, unknown=unkL2), xList)
  checkIdentical(unknownToNA(xDFUnk2,   unknown=unkL2), xDF)

  ## with named list of two unknown values but x is not named so named list
  ## does not have any effect --> error as we do not know how to recycle
  checkException(unknownToNA(xListUnk2a, unknown=unkLN2))

  ## but we should get some results with named x
  checkIdentical(unknownToNA(xListNUnk2, unknown=unkL2), xListN)
  ## not also necesarilly with recycling of names lists, as it is
  ## not clear how to properly recycle named lists (only names that match
  ## can be really properly recycled)
  checkException(unknownToNA(xListNUnk2, unknown=unkLN2))
  checkIdentical(unknownToNA(xDFUnk2, unknown=unkL2), xDF)
  checkException(unknownToNA(xDFUnk2, unknown=unkLN2))

  ## list(.default=)
  checkIdentical(unknownToNA(x=xListUnkD1, unknown=unkLND1), xListD1)
  ## list(.default=, someOther=) we do not know someOther, but should work
  ## as x is not named
  checkIdentical(unknownToNA(x=xListUnkDSO2, unknown=unkLNDSO2), xList)
  ## list(.default=) in named list
  checkIdentical(unknownToNA(x=xListNUnkD1, unknown=unkLND1), xListND1)
  ## list(.default=, someOther=) OK if someOther is in the named list
  checkIdentical(unknownToNA(x=xListNUnkD3, unknown=unkLND3), xListN)
  ## list(.default=, 99) ERROR as we do not know where to apply 99
  checkException(unknownToNA(x=xListNUnk, unknown=unkLND2E))

  ## --- matrix ---

  checkEquals(unknownToNA(x=matUnk1, unknown=matUnk), mat)
}

### }}}
### {{{ --- NAToUnknown ---

test.NAToUnknown <- function()
{
  ## --- base methods for vectors ---

  ## base ...
  checkIdentical(NAToUnknown(xInt, as.integer(intUnk)), xIntUnk)
  checkIdentical(NAToUnknown(xInt, intUnk),             xIntUnk) ## with numeric
  checkIdentical(NAToUnknown(xNum, numUnk),             xNumUnk)
  checkIdentical(NAToUnknown(xNum, as.integer(numUnk)), xNumUnk)
  checkIdentical(NAToUnknown(xCha, chaUnk),             xChaUnk)
  checkIdentical(NAToUnknown(xCha, chaUnk),             xChaUnk)
  checkIdentical(NAToUnknown(xFac, facUnk),             xFacUnk)

  ## only single values are allowed for vector methods
  checkException(NAToUnknown(xInt, unknown=unkC))
  checkException(NAToUnknown(xInt, unknown=unkL))

  ## and they should not already be in x unless force=TRUE
  checkException(NAToUnknown(xCha, unknown=chaUnk1))
  checkIdentical(NAToUnknown(xCha, unknown=chaUnk1, force=TRUE), xChaUnk1)

  checkException(NAToUnknown(xFac, unknown=facLev))
  checkIdentical(NAToUnknown(xFac, unknown=facLev, force=TRUE), xFacUnkLev)

  ## NA's in factors
  checkIdentical(NAToUnknown(xFac, unknown=facUnk1, force=TRUE), xFacUnk1)
  facNA <- factor(c("0", 1, 2, 3, NA, NA))
  facNATest <- factor(c("0", 1, 2, 3, "NA", "NA"))
  checkIdentical(NAToUnknown(x=facNA, unknown="NA"), facNATest)

  ## Date-time classes
  checkIdentical(NAToUnknown(xDate, unknown=dateUnk), xDateUnk)
  checkIdentical(NAToUnknown(xPOSIXlt, unknown=POSIXltUnk), xPOSIXltUnk)
  checkIdentical(NAToUnknown(xPOSIXct, unknown=POSIXctUnk), xPOSIXctUnk)

  ## --- lists and data.frames ---

  ## with vector of single unknown values
  checkIdentical(NAToUnknown(xList, unknown=unkC), xListUnk)
  checkIdentical(NAToUnknown(xDF, unknown=unkC), xDFUnk)

  ## with list of single unknown values
  checkIdentical(NAToUnknown(xList, unknown=unkL), xListUnk)
  checkIdentical(NAToUnknown(xDF, unknown=unkL), xDFUnk)

  ## with named list of single unknown values
  checkIdentical(NAToUnknown(xListN, unknown=unkLN), xListNUnk)
  checkIdentical(NAToUnknown(xDF, unknown=unkLN), xDFUnk)

  ## with names list of multiple unknown values - must be an error
  checkException(NAToUnknown(xListN, unknown=unkLMN))
  checkException(NAToUnknown(xDF, unknown=unkLMN))

  ## with single unknown value - recycling
  checkIdentical(NAToUnknown(xList, unknown=unk1), xListUnk1)
  checkIdentical(NAToUnknown(xDF,   unknown=unk1), xDFUnk1)

  ## with vector of two unknown values - recycling
  checkIdentical(NAToUnknown(xList, unknown=unkC2), xListUnk2)
  checkIdentical(NAToUnknown(xDF,   unknown=unkC2), xDFUnk2)

  ## with list of two unknown values - recycling
  checkIdentical(NAToUnknown(xList, unknown=unkL2), xListUnk2)
  checkIdentical(NAToUnknown(xDF,   unknown=unkL2), xDFUnk2)

  ## with named list of two unknown values but x is not named so named list
  ## does not have any effect --> error as we do not know how to recycle
  checkException(NAToUnknown(xList, unknown=unkLN2))

  ## but we should get some results with named x
  checkIdentical(NAToUnknown(xListN, unknown=unkL2), xListNUnk2)
  ## not also necesarilly with recycling of names lists, as it is
  ## not clear how to properly recycle named lists (only names that match
  ## can be really properly recycled)
  checkException(NAToUnknown(xListN, unknown=unkLN2))
  checkIdentical(NAToUnknown(xDF, unknown=unkL2), xDFUnk2)
  checkException(NAToUnknown(xDF, unknown=unkLN2))

  ## list(.default=)
  checkIdentical(NAToUnknown(x=xList, unknown=unkLND1), xListUnkD1)
  ## list(.default=, someOther=) we do not know someOther, but should work
  ## as x is not named
  checkIdentical(NAToUnknown(x=xList, unknown=unkLNDSO2), xListUnkDSO2)
  ## list(.default=) in named list
  checkIdentical(NAToUnknown(x=xListN, unknown=unkLND1), xListNUnkD1)
  ## list(.default=, someOther=) OK if someOther is in the named list
  checkIdentical(NAToUnknown(x=xListN, unknown=unkLND3), xListNUnkD3)
  ## list(.default=, 99) ERROR as we do not know where to apply 99
  checkException(NAToUnknown(x=xListN, unknown=unkLND2E))

  ## --- matrix ---

  checkEquals(NAToUnknown(x=mat, unknown=matUnk), matUnk1)
}

### }}}
### {{{ Dear Emacs
### Local variables:
### folded-file: t
### End:
### }}}

###------------------------------------------------------------------------
### runit.unknown.R ends here

# $Id: elem.R 625 2005-06-09 14:20:30Z nj7w $

elem <- function(object=1, unit=c("KB","MB","bytes"), digits=0,
                 dimensions=FALSE)
{
  .Deprecated("ll", package="gdata")
  ll(pos=object, unit=unit, digits=digits, dimensions=dimensions)
}

combine <- function(..., names=NULL)
{
  tmp <- list(...)
  if(is.null(names))
    names <- names(tmp)
  if(is.null(names))
    names <- sapply(as.list(match.call()), deparse)[-1]

  if(any(sapply(tmp, is.matrix) | sapply(tmp, is.data.frame)))
  {
    len <- sapply(tmp, function(x) c(dim(x),1)[1])
    len[is.null(len)] <- 1
    data <- rbind(...)
  }
  else
  {
    len <- sapply(tmp,length)
    data <- unlist(tmp)

  }

  namelist <- factor(rep(names, len), levels=names)

  return(data.frame(data, source=namelist))
}

centerText <- function(x, width=getOption("width"))
{
  retval <- vector(length=length(x), mode="character")
  for(i in 1:length(x))
  {
    text <- trim(x[i])
    textWidth  <- nchar(text)
    nspaces <- floor((width - textWidth)/2)
    spaces <- paste(rep(" ",nspaces), sep="", collapse="")
    retval[i] <- paste(spaces, text, sep="", collapse="\n")
  }
  retval
}

### runit.getDateTimeParts.R
###------------------------------------------------------------------------
### What: Extract date and time parts from ... - unit tests
### $Id$
### Time-stamp: <2008-12-30 22:41:18 ggorjan>
###------------------------------------------------------------------------

### {{{ --- Test setup ---

if(FALSE) {
  library("RUnit")
  library("gdata")
}

num <- 1
cha <- "a"
fac <- factor(c("A"))

tYear <- as.character(c(2006, 1995, 1005, 3067))
tMonth <- c("01", "04", "06", "12")
tDay <- c("01", "12", "22", "04")
tDate <- paste( paste(tYear, tMonth, tDay, sep="-"), "GMT" )

tHour <- c("05", "16", "20", "03")
tMin <- c("16", "40", "06", "52")
tSec <- c("56", "34", "05", "15")
tTime <- paste(tHour, tMin, tSec, sep=":")

cDate <- as.Date(tDate)
cDatePOSIXct <- as.POSIXct(tDate)
cDatePOSIXlt <- as.POSIXlt(tDate)

### }}}
### {{{ --- getYear ---

test.getYear <- function()
{
  checkException(getYear(x=num))
  checkException(getYear(x=cha))
  checkException(getYear(x=fac))

  checkIdentical(getYear(x=cDate), tYear)
  checkIdentical(getYear(x=cDatePOSIXct), tYear)
  checkIdentical(getYear(x=cDatePOSIXlt), tYear)
}

### }}}
### {{{ --- getMonth ---

test.getMonth <- function()
{
  checkException(getMonth(x=num))
  checkException(getMonth(x=cha))
  checkException(getMonth(x=fac))

  checkIdentical(getMonth(x=cDate), tMonth)
  checkIdentical(getMonth(x=cDatePOSIXct), tMonth)
  checkIdentical(getMonth(x=cDatePOSIXlt), tMonth)
}

### }}}
### {{{ --- getDay ---

test.getDay <- function()
{
  checkException(getDay(x=num))
  checkException(getDay(x=cha))
  checkException(getDay(x=fac))

  checkIdentical(getDay(x=cDate), tDay)
  checkIdentical(getDay(x=cDatePOSIXct), tDay)
  checkIdentical(getDay(x=cDatePOSIXlt), tDay)
}

### }}}
### {{{ --- getHour ---

test.getHour <- function()
{
  checkException(getHour(x=num))
  checkException(getHour(x=cha))
  checkException(getHour(x=fac))

##  checkIdentical(getHour(x=cDate), tHour)
}

### }}}
### {{{ --- getMin ---

test.getMin <- function()
{
  checkException(getMin(x=num))
  checkException(getMin(x=cha))
  checkException(getMin(x=fac))

##  checkIdentical(getMin(x=cDate), tMin)
}

### }}}
### {{{ --- getSec ---

test.getSec <- function()
{
  checkException(getSec(x=num))
  checkException(getSec(x=cha))
  checkException(getSec(x=fac))

##  checkIdentical(getSec(x=cDate), tSec)
}

### }}}
### {{{ Dear Emacs
### Local variables:
### folded-file: t
### end:
### }}}

###------------------------------------------------------------------------
### runit.getDateTimeParts.R ends here

### tests.write.fwf.R
###------------------------------------------------------------------------
### What: Tests for write.fwf
### $Id: tests.write.fwf.R 1300 2008-08-05 11:47:18Z ggorjan $
### Time-stamp: <2008-08-05 12:25:05 ggorjan>
###------------------------------------------------------------------------

library(gdata)

## --- Test data ---

num <- round(c(733070.345678, 1214213.78765456, 553823.798765678,
               1085022.8876545678,  571063.88765456, 606718.3876545678,
               1053686.6, 971024.187656, 631193.398765456, 879431.1),
             digits=3)

testData <- data.frame(num1=c(1:10, NA),
                       num2=c(NA, seq(from=1, to=5.5, by=0.5)),
                       num3=c(NA, num),
                       int1=c(as.integer(1:4), NA, as.integer(4:9)),
                       fac1=factor(c(NA, letters[1:9], "hjh")),
                       fac2=factor(c(letters[6:15], NA)),
                       cha1=c(letters[17:26], NA),
                       cha2=c(NA, "longer", letters[25:17]),
                       stringsAsFactors=FALSE)
levels(testData$fac1) <- c(levels(testData$fac1), "unusedLevel")
testData$Date <- as.Date("1900-1-1")
testData$Date[2] <- NA
testData$POSIXt <- as.POSIXct(strptime("1900-1-1 01:01:01", format="%Y-%m-%d %H:%M:%S"))
testData$POSIXt[5] <- NA

## --- Tests ---

## Default
write.fwf(testData)

## NA should be -
write.fwf(x=testData, na="-")
## NA should be -NA-
write.fwf(x=testData, na="-NA-")

## Some other separator than space
write.fwf(testData[, 1:4], sep="-mySep-")

## Justify right for character columns
write.fwf(testData, justify="right")

## Justify right for character columns, track na values
write.fwf(testData, justify="right", na="!")

## With quotes
write.fwf(testData, quote=TRUE)

## With rownames
write.fwf(testData, rownames=TRUE)

## Without colnames
write.fwf(testData, colnames=FALSE)

## Without colnames, with rownames
write.fwf(testData, colnames=FALSE, rownames=TRUE)

## With rownames and colnames and rowCol
write.fwf(testData, rownames=TRUE, rowCol="HI!")

## formatInfo is tested with unit tests

###------------------------------------------------------------------------
### tests.write.fwf.R ends

### runit.mapLevels.R
###------------------------------------------------------------------------
### What: Unit tests for mapLevels et al.
### $Id: runit.mapLevels.R 1784 2014-04-05 02:23:45Z warnes $
### Time-stamp: <2006-10-29 16:41:41 ggorjan>
###------------------------------------------------------------------------

### {{{ --- Test setup ---

if(FALSE) {
  library("RUnit")
  library("gdata")
}

### }}}
### {{{ mapLevels, is.*, as.*, [.*

test.mapLevels <- function()
{
  ## Integer and numeric
  checkException(mapLevels(1:3)) # wrong class(x)
  checkException(mapLevels(1.5)) # wrong class(x)

  ## Factor
  f <- factor(c("B", "C", "A"))
  fMapInt <- list(A=as.integer(1), B=as.integer(2), C=as.integer(3))
  fMapInt1 <- list(B=as.integer(1), C=as.integer(2))
  fMapCha <- list(A="A", B="B", C="C")
  fMapInt <- as.levelsMap(fMapInt)
  fMapInt1 <- as.levelsMap(fMapInt1)
  fMapCha <- as.levelsMap(fMapCha)
  fMapCha1 <- fMapCha[c(1, 3)] # this will test also [.levelsMap
  checkIdentical(mapLevels(f), fMapInt)
  checkTrue(is.levelsMap(mapLevels(f))) # test for is.levelsMap
  checkTrue(is.levelsMap(fMapInt))      # test for as.levelsMap
  checkTrue(!gdata:::.isCharacterMap(fMapInt))
  checkIdentical(mapLevels(f, sort=FALSE), fMapInt) # sort is not used for factors
  checkIdentical(mapLevels(f[1:2], drop=TRUE), fMapInt1)
  checkIdentical(mapLevels(f, codes=FALSE), fMapCha)
  checkIdentical(mapLevels(f[c(2, 3)], drop=TRUE, codes=FALSE), fMapCha1)

  ## Character
  cha <- c("Z", "M", "A")
  chaMapInt <- list(A=as.integer(1), M=as.integer(2), Z=as.integer(3))
  chaMapIntO <- list(Z=as.integer(1), M=as.integer(2), A=as.integer(3))
  chaMapInt1 <- list(M=as.integer(1), Z=as.integer(2))
  chaMapCha <- list(A="A", M="M", Z="Z")
  chaMapInt <- as.levelsMap(chaMapInt)
  chaMapIntO <- as.levelsMap(chaMapIntO)
  chaMapInt1 <- as.levelsMap(chaMapInt1)
  chaMapCha <- as.levelsMap(chaMapCha)
  checkIdentical(mapLevels(cha), chaMapInt)
  checkIdentical(mapLevels(cha, sort=FALSE), chaMapIntO) # sort works for characters
  checkIdentical(mapLevels(cha[1:2], drop=TRUE), chaMapInt1)
  checkIdentical(mapLevels(cha, codes=FALSE), chaMapCha)

  ## List
  l <- list(f=f, cha=cha)
  l1 <- list(cha=cha, f=f)
  l2 <- list(cha=cha, f=f, i=1:10)
  lMapInt <- list(f=fMapInt, cha=chaMapInt)
  lMapCha <- list(f=fMapCha, cha=chaMapCha)
  lMapInt <- as.listLevelsMap(lMapInt)
  lMapCha <- as.listLevelsMap(lMapCha)
  lMapChaC <- as.list(sort(unique(c(cha, as.character(f)))))
  lMapChaCO <- as.list(unique(c(cha, as.character(f))))
  names(lMapChaC) <- unlist(lMapChaC)
  names(lMapChaCO) <- unlist(lMapChaCO)
  lMapChaC <- as.levelsMap(lMapChaC)
  lMapChaCO <- as.levelsMap(lMapChaCO)
  checkIdentical(mapLevels(l), lMapInt)
  checkTrue(is.listLevelsMap(mapLevels(l))) # test for is.listLevelsMap
  checkTrue(is.listLevelsMap(lMapInt))      # test for as.listLevelsMap
  checkIdentical(mapLevels(l, codes=FALSE), lMapCha)
  checkException(mapLevels(l, combine=TRUE)) # can not combine integer maps
  checkIdentical(mapLevels(l, codes=FALSE, combine=TRUE), lMapChaC)
  checkIdentical(mapLevels(l1, codes=FALSE, combine=TRUE), lMapChaC)
  checkIdentical(mapLevels(l1, codes=FALSE, combine=TRUE, sort=FALSE), lMapChaCO)
  checkException(mapLevels(l2)) # only char and factor

  ## Data.frame
  df <- data.frame(f1=factor(c("G", "Abc", "Abc", "D", "F")),
                   f2=factor(c("Abc", "Abc", "B", "D", "K")),
                         cha=c("jkl", "A", "D", "K", "L"),
                   int=1:5)
  dfMapInt <- list(f1=mapLevels(df$f1), f2=mapLevels(df$f2), cha=mapLevels(df$cha))
  dfMapInt <- as.listLevelsMap(dfMapInt)
  dfMapInt1 <- dfMapInt[c(1, 3)] # this will test also [.listLevelsMap
  checkException(mapLevels(df)) # wrong class of int
  checkIdentical(mapLevels(df[, 1:3]), dfMapInt)
  checkIdentical(mapLevels(df[, c(1, 3)]), dfMapInt1)
}

### }}}
### {{{ .check*

test.checkLevelsMap <- function(x)
{
  ## --- levelsMap ---

  ## not a list
  checkException(gdata:::.checkLevelsMap(x="A", method="raw"))
  ## list without names
  checkException(gdata:::.checkLevelsMap(x=list("A"), method="raw"))
  fMapInt <- list(A=as.integer(1), B=as.integer(2), C=as.integer(3))
  ## x should be levelsMap
  checkException(gdata:::.checkLevelsMap(x=fMapInt, method="class"))

  ## --- listLevelsMap ---

  map <- list(as.levelsMap(fMapInt), as.levelsMap(fMapInt))
  map1 <- list(fMapInt, fMapInt)
  class(map1) <- "listLevelsMap"
  ## x should be a listLevelsMap
  checkException(gdata:::.checkListLevelsMap(x=map, method="class"))
  ## x should be also a list of levelsMaps
  checkException(gdata:::.checkListLevelsMap(x=map1, method="class"))
  ## the rest is done with levelsMap tests
}

### }}}
### {{{ c.*

test.cLevelsMap <- function()
{
  f1 <- factor(letters[c(2, 1)])
  f2 <- factor(letters[c(3, 1, 2)])
  mapCha1 <- mapLevels(f1, codes=FALSE) # get maps
  mapCha2 <- mapLevels(f2, codes=FALSE)
  mapCha1S <- mapLevels(as.character(f1), codes=FALSE, sort=FALSE)
  mapCha2S <- mapLevels(as.character(f2), codes=FALSE, sort=FALSE)
  mapChaTest <- list(a="a", b="b")
  mapChaTest1 <- list(a="a", b="b", c="c")
  mapChaTest2 <- list(c="c", a="a", b="b")
  class(mapChaTest) <- class(mapChaTest1) <- class(mapChaTest2) <- "levelsMap"
  mapChaTest3 <- list(mapChaTest, mapChaTest1, mapChaTest, mapChaTest1)
  class(mapChaTest3)  <- "listLevelsMap"
  checkIdentical(c(mapCha1), mapChaTest)
  checkIdentical(c(mapCha2, mapCha1), mapChaTest1)
  checkIdentical(c(mapCha2S, mapCha1S, sort=FALSE), mapChaTest2)

  l <- list(f1, f2)
  mapCha <- mapLevels(l, codes=FALSE)
  checkIdentical(c(mapCha, mapCha), mapChaTest3)
  checkIdentical(c(mapCha, recursive=TRUE), mapChaTest1)

  checkException(c(mapLevels(f1))) # can not combine integer “levelsMaps”

  ## Example with maps of different length of components
  map1 <- list(A=c("a", "e", "i", "o", "u"), B="b", C="c", C="m",
               D=c("d", "e"), F="f")
  map2 <- list(A=c("a", "z", "w", "y", "x"), F="f", G=c("g", "h", "j"),
               i="i", k=c("k", "l"), B="B")
  map0Test <- list(A=c("a", "e", "i", "o", "u"), B="b", C="c", C="m",
                   D=c("d", "e"), F="f",
                   A=c("z", "w", "y", "x"), G=c("g", "h", "j"),
                   i="i", k=c("k", "l"), B="B")
  map0Test <- as.levelsMap(map0Test)
  mapTest <- sort(map0Test)
  map1 <- as.levelsMap(map1)
  map2 <- as.levelsMap(map2)
  map <- c(map1, map2)
  map0 <- c(map1, map2, sort=FALSE)
  checkIdentical(map, mapTest)
  checkIdentical(map0, map0Test)
}

### }}}
### {{{ unique

test.uniqueLevelsMap <- function()
{
  map <- list(A=c(1, 2, 1, 3), B=4, C=1, C=5, D=c(6, 8), E=7, B=4,
              D=c(6, 8))
  map1 <- map
  map1[[1]] <- map[[1]][c(1, 2, 4)]
  map1[[7]] <- NULL # remove B=4
  map1[[7]] <- NULL # remove D=c(6, 8)
  ## unique (used in as.levelsMap), will remove duplicates (A=1)
  checkIdentical(as.levelsMap(map1), as.levelsMap(map))
}

### }}}
### {{{ mapLevels<-

"test.mapLevels<-" <- function()
{
  ## Some errors
  checkException("mapLevels<-"(1.1, value=2))  # wrong class(x)
  checkException("mapLevels<-"(complex(1.1), value=2)) # wrong class(x)

  f <- factor(c("A", "B", "C"))
  fMapInt <- mapLevels(f)
  ## can not apply integer "levelsMap" to "character"
  checkException("mapLevels<-"(as.character(f), value=fMapInt))

  fMapCha <- mapLevels(f, codes=FALSE)
  ## can not apply character levelsMap to "integer"
  checkException("mapLevels<-"(as.integer(f), value=chaMapCha))

  fMapFuzz <- fMapInt
  fMapFuzz[[1]] <- "A"
  ## all components of 'value' must be of the same class
  checkException("mapLevels<-"(as.character(f), value=fMapFuzz))
  checkException("mapLevels<-"(as.integer(f), value=fMapFuzz))

  ## x integer, value integer levelsMap
  f <- factor(letters[c(10, 15, 1, 2)])
  fMapInt <- mapLevels(f)
  fInt <- as.integer(f)
  mapLevels(fInt) <- fMapInt
  checkIdentical(fInt, f)

  ## x factor, value integer levelsMap
  fInt <- factor(as.integer(f))
  mapLevels(fInt) <- fMapInt
  checkIdentical(fInt, f)

  ## above is essentially the same as levels<-.factor
  fInt1 <- factor(as.integer(f))
  levels(fInt1) <- fMapInt
  checkIdentical(fInt1, f)

  ## x character, value character levelsMap
  cha <- c("B", "A", "C")
  chaMapCha <- as.levelsMap(list(A1="A", B2="B", C3="C"))
  mapLevels(cha) <- chaMapCha
  chaTest <- factor(c("B2", "A1", "C3"))
  checkIdentical(cha, chaTest)
  ## and a bit more for components of length > 1
  cha <- c("G", "I", "B", "A", "C", "D", "Z")
  chaMapCha <- as.levelsMap(list(A1=c("A", "G", "I"), B2="B", C3=c("C", "D")))
  mapLevels(cha) <- chaMapCha
  chaTest <- factor(c("A1", "A1", "B2", "A1", "C3", "C3", NA))
  checkIdentical(cha, chaTest)

  ## x factor, value character levelsMap
  f <- factor(c("G", "I", "B", "A", "C", "D", "Z"))
  fMapCha <- as.levelsMap(list(A1=c("A", "G", "I"), B2="B", C3=c("C", "D")))
  mapLevels(f) <- fMapCha
  fTest <- factor(c("A1", "A1", "B2", "A1", "C3", "C3", NA))
  checkIdentical(f, fTest)

  ## Two factors and character map
  f1 <- factor(letters[1:10])
  f2 <- factor(letters[5:14])
  checkIdentical(as.integer(f1), as.integer(f2)) # the same integer codes
  mapCha1 <- mapLevels(f1, codes=FALSE) # get maps
  mapCha2 <- mapLevels(f2, codes=FALSE)
  mapCha <- c(mapCha1, mapCha2)         # combine maps
  ## apply map
  mapLevels(f1) <- mapCha # the same as levels(f1) <- mapCha
  mapLevels(f2) <- mapCha # the same as levels(f2) <- mapCha
  checkIdentical(as.integer(f1), 1:10) # \ internal codes are now
  checkIdentical(as.integer(f2), 5:14) # / "consistent" among factors

  ## The same with list
  l <- list(f1=f1, f2=f2)
  mapCha <- mapLevels(l, codes=FALSE, combine=TRUE)
  mapLevels(l) <- mapCha
  checkIdentical(as.integer(l$f1), 1:10) # \ internal codes are now
  checkIdentical(as.integer(l$f2), 5:14) # / "consistent" among factors

  ## and data.frame
  df <- data.frame(f1=f1, f2=f2)
  mapCha <- mapLevels(df, codes=FALSE, combine=TRUE)
  mapLevels(df) <- mapCha
  checkIdentical(as.integer(df$f1), 1:10) # \ internal codes are now
  checkIdentical(as.integer(df$f2), 5:14) # / "consistent" among factors

}

### }}}
### {{{ Dear Emacs
## Local variables:
## folded-file: t
## End:
### }}}

###------------------------------------------------------------------------
### runit.mapLevels.R ends here

duplicated2 <- function(x, bothWays=TRUE, ...)
    {
        if(!bothWays) {
            return(duplicated(x, ...))
        } else if(bothWays) {
            return((duplicated(x, ...) | duplicated(x, fromLast=TRUE, ...)))
        }
    }

trim <- function(s, recode.factor=TRUE, ...)
  UseMethod("trim", s)

trim.default <- function(s, recode.factor=TRUE, ...)
  s

trim.character <- function(s, recode.factor=TRUE, ...)
{
  s <- sub(pattern="^[[:blank:]]+", replacement="", x=s)
  s <- sub(pattern="[[:blank:]]+$", replacement="", x=s)
  s
}

trim.factor <- function(s, recode.factor=TRUE, ...)
{
  levels(s) <- trim(levels(s))
  if(recode.factor) {
    dots <- list(x=s, ...)
    if(is.null(dots$sort)) dots$sort <- sort
    s <- do.call(what="reorder.factor", args=dots)
  }
  s
}

trim.list <- function(s, recode.factor=TRUE, ...)
  lapply(s, trim, recode.factor=recode.factor, ...)

trim.data.frame <- function(s, recode.factor=TRUE, ...)
{
  s[] <- trim.list(s, recode.factor=recode.factor, ...)
  s
}

.onAttach <- function(libname, pkgname)
  {
    show <- function(...)
      packageStartupMessage(
         paste(
            strwrap(x = list(...), 
                    prefix = "gdata: "), 
            collapse="\n",sep="\n"
            )
      ) 

    try(
        {
          
          ## 1 - Can we access perl?
          hasPerl <- try( findPerl(), silent=TRUE)
          if(inherits(hasPerl, "try-error"))
            show(
        "
        Unable to locate valid perl interpreter
        \n
        \n
        read.xls() will be unable to read Excel XLS and XLSX files
        unless the 'perl=' argument is used to specify the location of
        a valid perl intrpreter.
        \n
        \n
        (To avoid display of this message in the future, please ensure perl
         is installed and available on the executable search path.)
        ")


          formats <- try(xlsFormats(),silent=TRUE)
          msg <- FALSE
          ## 2 - Are the libraries for XLS present?
          if( !("XLS" %in% formats) )
            {
              show(
                   "Unable to load perl libaries needed by read.xls()",
                   " to support 'XLX' (Excel 97-2004) files."
                   )
              msg <- TRUE
            }
          else
            {
              show(
                   "read.xls support for 'XLS'  (Excel 97-2004) files ENABLED.")
            }

          show("\n")
          
    ## 3 - Are the libbaries for XLSX present?
    if( !("XLSX" %in% formats) )
      {
        show(
            "Unable to load perl libaries needed by read.xls()",
            " to support 'XLSX' (Excel 2007+) files."
             )
        msg <- TRUE
      }
    else
      {
        show(
            "read.xls support for 'XLSX' (Excel 2007+)   files ENABLED."
             )
      }


          
    if(msg)
      {
        show("\n")
        
        show(
              " Run the function 'installXLSXsupport()'",
              " to automatically download and install the perl",
              " libaries needed to support Excel XLS and XLSX formats."
           )
    }
        
  })
        
  }

Args <- function(name, sort=FALSE)
{
  a <- formals(get(as.character(substitute(name)), pos=1))
  if(is.null(a))
    return(NULL)
  arg.labels <- names(a)
  arg.values <- as.character(a)
  char <- sapply(a, is.character)
  arg.values[char] <- paste("\"", arg.values[char], "\"", sep="")

  if(sort)
  {
    ord <- order(arg.labels)
    if(any(arg.labels == "..."))
      ord <- c(ord[-which(arg.labels[ord]=="...")],
               which(arg.labels=="..."))
    arg.labels <- arg.labels[ord]
    arg.values <- arg.values[ord]
  }

  output <- data.frame(value=I(arg.values), row.names=arg.labels)
  print(output, right=FALSE)

  invisible(output)
}

## s$Id: read.xls.R 1596 2012-08-22 15:45:22Z warnes $

read.xls <- function(xls, sheet = 1, verbose=FALSE, pattern, 
                     na.strings = c("NA","#DIV/0!"), ...,
                     method=c("csv","tsv","tab"),
                     perl="perl")
{
  con <- tfn <- NULL
  on.exit({ 
	err <- FALSE
	if (inherits(con, "connection")) {
		tryCatch(op <- isOpen(con), error = function(x) err <<- TRUE)
		if (!err && op) close(con)
	}
    if (file.exists(tfn)) file.remove(tfn)
  })

  method <- match.arg(method)
  
  ## expand file path, translating ~ to user's home directory, etc.
  xls <- path.expand(xls)


  ## translate from xls to csv/tsv/tab format (returns name of created file)
  perl <- if (missing(perl))
    findPerl(verbose = verbose)
  else
    findPerl(perl, verbose = verbose)
  
  con <- xls2sep(xls, sheet, verbose=verbose, ..., method=method, perl = perl)
  ## While xls2sep returns a connection, we are better off directly
  ## opening the file, so that R can properly handle the encoding.  So,
  ## just grab the full file path to use later, and close the connection.
  tfn <- summary(con)$description  
  close(con)

  if (missing(pattern))
    {
      if(verbose)
        cat("Reading", method, "file ", dQuote(tfn), "...\n")
      
      if(method=="csv")
        retval <- read.csv(tfn, na.strings=na.strings, ...)
      else if (method %in% c("tsv","tab") )
        retval <- read.delim(tfn, na.strings=na.strings, ...)
      else
        stop("Unknown method", method)
        
      if(verbose)
        cat("Done.\n")
    }
  else {
    if(verbose)
      cat("Searching for lines tfntaining pattern ", pattern, "... ")
    idx <- grep(pattern, readLines(tfn))
    if (length(idx) == 0) {
      warning("pattern not found")
      return(NULL)
    }
   if(verbose)
     cat("Done.\n")
    
    if(verbose)
      cat("Reading", method, "file ", dQuote(tfn), "...\n")

    if(method=="csv")
      retval <- read.csv(tfn, skip = idx[1]-1, na.strings=na.strings, ...)
    else if (method %in% c("tsv","tab") )
      retval <- read.delim(tfn, skip = idx[1]-1, na.strings=na.strings, ...)
    else
      stop("Unknown method", method)

    if(verbose)
      cat("Done.\n")     
  }

  retval
}

# $Id: aggregate.table.R 1681 2013-06-28 20:26:56Z warnes $

aggregate.table <- function(x, by1, by2, FUN=mean, ...)
  {
    warning("'aggregate.table' is deprecated and will be removed in a future version of the gdata package. ",
            "Please use 'tapply(X=",
            deparse(substitute(x)),
            ", INDEX=list(",
            deparse(substitute(by1)),
            ", ",
            deparse(substitute(by2)),
            "), FUN=",
            deparse(substitute(FUN)),
            if(length(list(...))>0)
            {
              l <- list(...)
              paste(", ",
                    paste(names(l),"=",
                          deparse(substitute(...)),
                          sep="",
                          collapse=", ")
                    )
            },
            ")' instead.")
    tapply(X=x, INDEX=list(by1, by2), FUN=FUN, ...)
  }

## aggregate.table <- function(x, by1, by2, FUN=mean, ... )
## {
##    
##     tab <- matrix( nrow=nlevels(by1), ncol=nlevels(by2) )
##     dimnames(tab) <- list(levels(by1),levels(by2))
##
##     for(i in 1:nrow(ag))
##       tab[ as.character(ag[i,1]), as.character(ag[i,2]) ] <- ag[i,3]
##     tab
##   }

### nPairs.R
###------------------------------------------------------------------------
### What: Number of variable pairs - code
### $Id$
### Time-stamp: <2008-12-30 18:29:58 ggorjan>
###------------------------------------------------------------------------

nPairs <- function(x, margin=FALSE, names=TRUE, abbrev=TRUE, ...)
{
  ## --- Setup ---
  if(!is.data.frame(x) & !is.matrix(x)) stop("'x' must be a data.frame or a matrix")
  k <- ncol(x)
  if(!margin) {
    ret <- matrix(nrow=k, ncol=k)
  } else {
    ret <- matrix(nrow=k, ncol=k + 1)
  }

  ## --- Count ---
  diag(ret)[1:k] <- apply(X=x, MARGIN=2, FUN=function(x) sum(!is.na(x)))
  for(i in 1:k) {
    for(j in i:k) {
      ret[i, j] <- ret[j, i] <- sum(!is.na(x[, i]) & !is.na(x[, j]))
      if(margin) {
        if(i == 1) {
          ret[i, (k + 1)] <- ret[1, 1]
        } else {
          ret[i, (k + 1)] <- sum(rowSums(!is.na(x[, c(1:i)])) == i)
        }
      }
    }
  }
  
  ## --- Names ---
  if(names) {
    tmp <- colnames(x)
    if(abbrev) tmp <- as.character(abbreviate(tmp, ...))
    rownames(ret) <- tmp
    if(margin) {
      colnames(ret) <- c(tmp, "all")
    } else {
      colnames(ret) <- tmp
    }
  }
  class(ret) <- c("nPairs", class(ret))
  ret
}

summary.nPairs <- function(object, ...)
{
  n <- nrow(object)
  ret <- matrix(data=0, nrow=n, ncol=n)
  for(i in 1:n) {
    tmp <- 1:n
    tmp <- tmp[!(tmp == i)]
    ret[i, tmp] <- object[i, i] - object[i, tmp]
  }
  dimnames(ret) <- dimnames(object)
  ret
}

###------------------------------------------------------------------------
### nPairs.R ends here

## Test setup
if(FALSE) {
  library("RUnit")
  library("gdata")
}

test.nPairs <- function()
{
  ## 'x' must be a data.frame or a matrix
  x <- rpois(100, lambda=10)
  checkException(nPairs(x=x))
  checkException(nPairs(x=table(x)))

  test <- data.frame(V1=c(1, 2, 3, 4, 5),
                     V2=c(NA, 2, 3, 4, 5),
                     V3=c(1, NA, NA, NA, NA),
                     V4=c(1, 2, 3, NA, NA))
  testCheck <- matrix(data=as.integer(c(5, 4, 1, 3,
                                        4, 4, 0, 2,
                                        1, 0, 1, 1,
                                        3, 2, 1, 3)),
                      nrow=4, ncol=4, byrow=TRUE)
  class(testCheck) <- c("nPairs", class(testCheck))

  testCheckNames <- testCheck
  colnames(testCheckNames) <- rownames(testCheckNames) <- colnames(test)

  checkIdentical(nPairs(x=test), testCheckNames)
  checkIdentical(nPairs(x=test, names=FALSE), testCheck)
  checkIdentical(nPairs(x=as.matrix(test)), testCheckNames)
  checkIdentical(nPairs(x=as.matrix(test), names=FALSE), testCheck)

  testCheck <- cbind(testCheckNames, as.integer(c(5, 4, 0, 0)))
  class(testCheck) <- class(testCheckNames)
  colnames(testCheck) <- c(colnames(test), "all")
  checkIdentical(nPairs(x=test, margin=TRUE), testCheck)

  testCheckSumm <- matrix(data=as.integer(c(0, 1, 4, 2,
                                            0, 0, 4, 2,
                                            0, 1, 0, 0,
                                            0, 1, 2, 0)),
                          nrow=4, ncol=4, byrow=TRUE)
  dimnames(testCheckSumm) <- dimnames(testCheckNames)
  tmp <- summary(nPairs(x=test))
  checkEquals(tmp, testCheckSumm)
}

### runit.nPairs.R
###------------------------------------------------------------------------
### What: Number of variable pairs - unit tests
### $Id$
### Time-stamp: <2008-12-30 18:24:59 ggorjan>
###------------------------------------------------------------------------

### {{{ --- Test setup ---

if(FALSE) {
  library("RUnit")
  library("gdata")
}

### }}}
### {{{ --- nPairs ---

test.nPairs <- function()
{
  ## 'x' must be a data.frame or a matrix
  x <- rpois(100, lambda=10)
  checkException(nPairs(x=x))
  checkException(nPairs(x=table(x)))

  test <- data.frame(V1=c(1, 2, 3, 4, 5),
                     V2=c(NA, 2, 3, 4, 5),
                     V3=c(1, NA, NA, NA, NA),
                     V4=c(1, 2, 3, NA, NA))
  testCheck <- matrix(data=as.integer(c(5, 4, 1, 3,
                                        4, 4, 0, 2,
                                        1, 0, 1, 1,
                                        3, 2, 1, 3)),
                      nrow=4, ncol=4, byrow=TRUE)
  class(testCheck) <- c("nPairs", class(testCheck))

  testCheckNames <- testCheck
  colnames(testCheckNames) <- rownames(testCheckNames) <- colnames(test)

  checkIdentical(nPairs(x=test), testCheckNames)
  checkIdentical(nPairs(x=test, names=FALSE), testCheck)
  checkIdentical(nPairs(x=as.matrix(test)), testCheckNames)
  checkIdentical(nPairs(x=as.matrix(test), names=FALSE), testCheck)

  testCheck <- cbind(testCheckNames, as.integer(c(5, 4, 0, 0)))
  class(testCheck) <- class(testCheckNames)
  colnames(testCheck) <- c(colnames(test), "all")
  checkIdentical(nPairs(x=test, margin=TRUE), testCheck)

  testCheckSumm <- matrix(data=as.integer(c(0, 1, 4, 2,
                                            0, 0, 4, 2,
                                            0, 1, 0, 0,
                                            0, 1, 2, 0)),
                          nrow=4, ncol=4, byrow=TRUE)
  dimnames(testCheckSumm) <- dimnames(testCheckNames)
  tmp <- summary(nPairs(x=test))
  checkEquals(tmp, testCheckSumm)
}


### }}}
### {{{ Dear Emacs
### Local variables:
### folded-file: t
### end:
### }}}

###------------------------------------------------------------------------
### runit.nPairs.R ends here

library(gdata)

if ( ! 'XLSX' %in% xlsFormats() )
  {
    try( installXLSXsupport() )
  }

# iris.xls is included in the gregmisc package for use as an example
xlsfile <- file.path(path.package('gdata'),'xls','iris.xls')

iris.1 <- read.xls(xlsfile) # defaults to csv format
iris.1

iris.2 <- read.xls(xlsfile,method="csv") # specify csv format
iris.2

iris.3 <- read.xls(xlsfile,method="tab") # specify tab format
iris.3

stopifnot(all.equal(iris.1, iris.2))
stopifnot(all.equal(iris.1, iris.3))

exampleFile <- file.path(path.package('gdata'),'xls',
                         'ExampleExcelFile.xls')

exampleFileX <- file.path(path.package('gdata'),'xls',
                         'ExampleExcelFile.xlsx')

# see the number and names of sheets:
sheetCount(exampleFile)

if(! 'XLSX' %in% xlsFormats() )
  {
    cat("************************************************************\n")
    cat("** DIFF IN THIS SECTION IS EXPECTED BECAUSE PERL PACKAGES **\n")
    cat("** FOR SUPPORTING XLSX ARE NOT INSTALLED                  **\n")
    cat("************************************************************\n")
  } else {
    sheetCount(exampleFileX)
  }


sheetNames(exampleFile)

if(! 'XLSX' %in% xlsFormats() )
  {
    cat("************************************************************\n")
    cat("** DIFF IN THIS SECTION IS EXPECTED BECAUSE PERL PACKAGES **\n")
    cat("** FOR SUPPORTING XLSX ARE NOT INSTALLED                  **\n")
    cat("************************************************************\n")
  } else {
    sheetNames(exampleFileX)
  }


example.1 <- read.xls(exampleFile, sheet=1) # default is first worksheet
example.1

example.2 <- read.xls(exampleFile, sheet=2) # second worksheet by number
example.2

example.3 <- read.xls(exampleFile, sheet=3, header=FALSE) # third worksheet by number
example.3

example.4 <- read.xls(exampleFile, sheet=4, header=FALSE) # fourth worksheet by number
example.4

if(! 'XLSX' %in% xlsFormats() )
  {
    cat("************************************************************\n")
    cat("** DIFF IN THIS SECTION IS EXPECTED BECAUSE PERL PACKAGES **\n")
    cat("** FOR SUPPORTING XLSX ARE NOT INSTALLED                  **\n")
    cat("************************************************************\n")
  } else {
    example.x.1 <- read.xls(exampleFileX, sheet=1) # default is first worksheet
    print(example.x.1)

    example.x.2 <- read.xls(exampleFileX, sheet=2) # second worksheet by number
    print(example.x.2)

    example.x.3 <- read.xls(exampleFileX, sheet=3, header=FALSE) # third worksheet by number
    print(example.x.3)

    example.x.4 <- read.xls(exampleFileX, sheet=4, header=FALSE) # fourth worksheet by number
    print(example.x.4)

    data <- read.xls(exampleFileX, sheet="Sheet Second") # and by name
    print(data)

    # load the third worksheet, skipping the first two non-data lines...
    data <- read.xls(exampleFileX, sheet="Sheet with initial text", skip=2)
    print(data)
  }

## Check handling of skip.blank.lines=FALSE

example.skip <- read.xls(exampleFile, sheet=2, blank.lines.skip=FALSE)
example.skip

if(! 'XLSX' %in% xlsFormats() )
  {
    cat("************************************************************\n")
    cat("** DIFF IN THIS SECTION IS EXPECTED BECAUSE PERL PACKAGES **\n")
    cat("** FOR SUPPORTING XLSX ARE NOT INSTALLED                  **\n")
    cat("************************************************************\n")
  } else {
    example.x.skip <- read.xls(exampleFileX, sheet=2, blank.lines.skip=FALSE)
    example.x.skip
  }



## Check handing of fileEncoding for latin-1 characters

latin1File  <- file.path(path.package('gdata'),'xls', 'latin-1.xls' )
latin1FileX <- file.path(path.package('gdata'),'xls', 'latin-1.xlsx')

if(.Platform$OS.type=="unix")
  {
      example.latin1 <- read.xls(latin1File,
                                 fileEncoding='latin1',
                                 encoding='latin1',
                                 stringsAsFactors=FALSE)
  } else {
      example.latin1 <- read.xls(latin1File,
                                        #fileEncoding='latin1',
                                 encoding='latin1',
                                 stringsAsFactors=FALSE)
  }

if(! 'XLSX' %in% xlsFormats() )
  {
    cat("************************************************************\n")
    cat("** DIFF IN THIS SECTION IS EXPECTED BECAUSE PERL PACKAGES **\n")
    cat("** FOR SUPPORTING XLSX ARE NOT INSTALLED                  **\n")
    cat("************************************************************\n")
  } else {
      if(.Platform$OS.type=="unix")
          {
              example.latin1.x <- read.xls(latin1FileX,
                                         fileEncoding='latin1',
                                         encoding='latin1',
                                         stringsAsFactors=FALSE)
          } else {
              example.latin1.x <- read.xls(latin1FileX,
                                        #fileEncoding='latin1',
                                         encoding='latin1',
                                         stringsAsFactors=FALSE)
          }
  }


## Check handling of very wide file

wideFile  <- file.path(path.package('gdata'),'xls', 'wide.xls' )
wideFileX <- file.path(path.package('gdata'),'xls', 'wide.xlsx')

example.wide <- read.xls(wideFile)
stopifnot(dim(example.wide)==c(0,256))

if( !'XLSX' %in% xlsFormats() )
  {
    cat("************************************************************\n")
    cat("** DIFF IN THIS SECTION IS EXPECTED BECAUSE PERL PACKAGES **\n")
    cat("** FOR SUPPORTING XLSX ARE NOT INSTALLED                  **\n")
    cat("************************************************************\n")
  } else {
    example.wide.x <- read.xls(wideFileX)
    stopifnot(dim(example.wide.x)==c(0,16384))
  }

## Check handling of files with dates calulcated relative to
## 1900-01-01 and 1904-01-01

file.1900  <- file.path(path.package('gdata'),'xls', 'ExampleExcelFile_1900.xls' )
file.1904  <- file.path(path.package('gdata'),'xls', 'ExampleExcelFile_1904.xls' )
fileX.1900 <- file.path(path.package('gdata'),'xls', 'ExampleExcelFile_1900.xlsx')
fileX.1904 <- file.path(path.package('gdata'),'xls', 'ExampleExcelFile_1904.xlsx')

example.1900 <- read.xls(file.1900, sheet=3, header=FALSE)
example.1900

example.1904 <- read.xls(file.1904, sheet=3, header=FALSE)
example.1904

exampleX.1900 <- read.xls(file.1900, sheet=3, header=FALSE)
exampleX.1900

exampleX.1904 <- read.xls(file.1904, sheet=3, header=FALSE)
exampleX.1904

# all colmns should be identical
stopifnot( na.omit(example.1900  == exampleX.1900) )
stopifnot( na.omit(example.1904  == exampleX.1904) )

# column 8 will differ by 1462 due to different date baselines (1900 vs 1904)
stopifnot( na.omit(example.1900 [,-8] == example.1904 [,-8]) )
stopifnot( na.omit(exampleX.1900[,-8] == exampleX.1904[,-8]) )

stopifnot( na.omit(example.1900 [,8] - example.1904 [,8]) == 1462 )
stopifnot( na.omit(exampleX.1900[,8] - exampleX.1904[,8]) == 1462 )

### runRUnitTests.R
###------------------------------------------------------------------------
### What: Run RUnit tests (wrapper function) - R code
### $Id$
### Time-stamp: <2008-12-30 20:59:11 ggorjan>
###------------------------------------------------------------------------

.runRUnitTestsGdata <- function(testFileRegexp="^runit.+\\.[rR]$")
{
  ## Setup
  .pkg <- environmentName(environment(.runRUnitTestsGdata))
  .path <- system.file("unitTests", package=.pkg)
  .suite <- file.path(.path, "runRUnitTests.R")

  ## Some checks
  stopifnot(file.exists(.path),
            file.info(path.expand(.path))$isdir,
            file.exists(.suite))

  ## Run the suite
  .way <- "function"
  source(.suite, local=TRUE)
  ## local=TRUE since .pkg and other vars do not exists in .suite environment
}

###------------------------------------------------------------------------
### runRUnitTests.R ends here

###------------------------------------------------------------------------
### What: Print object size in human readable format - code
###------------------------------------------------------------------------
object.size <- function(...)
{
  structure(sapply(list(...),
                   utils::object.size),
                   class=c("object_sizes", "numeric"))
}

print.object_sizes <- function(x,
                               quote=FALSE,
                               humanReadable=getOption("humanReadable"),
                               standard="IEC",
                               units,
                               digits=1,
                               width=NULL,
                               sep=" ",
                               justify = c("right", "left"),
                               ...)
{
    print(format(x,
                 humanReadable=humanReadable,
                 standard=standard,
                 units=units,
                 digits=digits,
                 width=width,
                 sep=sep,
                 justify=justify),
          quote=quote,
          ...)


    invisible(x)
}

format.object_sizes <- function(x,
                                humanReadable=getOption("humanReadable"),
                                standard="IEC",
                                units,
                                digits=1,
                                width=NULL,
                                sep=" ",
                                justify = c("right", "left"),
                                ...)
{
    if( !missing(units) )
       {
           if (units=="bytes")
               paste(x, "bytes")
           else
               humanReadable(x,
                             standard=standard,
                             units=units,
                             digits=digits,
                             width=width,
                             sep=sep,
                             justify=justify
                             )
       }
    else if( is.null(humanReadable) || humanReadable==FALSE )
        paste(x, "bytes")
    else
        humanReadable(x,
                      standard=standard,
                      units=units,
                      digits=digits,
                      width=width,
                      sep=sep,
                      justify=justify)

}



is.object_sizes <- function(x) inherits(x, what="object_sizes")

as.object_sizes <- function(x)
{
  if(!is.numeric(x) || any(x<0)) stop("'x' must be a positive numeric vector")

  class(x) <- c("object_sizes", "numeric")
  x
}

c.object_sizes <- function(..., recursive=FALSE)
{
  x <- NextMethod()
  if(is.numeric(x)) class(x) <- c("object_sizes", "numeric")
  x
}

###------------------------------------------------------------------------
### object.size.R ends here

### mapLevels.R
###------------------------------------------------------------------------
### What: Mapping levels
### $Id: mapLevels.R 1991 2015-04-29 03:27:50Z warnes $
### Time-stamp: <2007-04-26 13:16:18 ggorjan>
###------------------------------------------------------------------------

### {{{ mapLevels

###------------------------------------------------------------------------

mapLevels <- function(x, codes=TRUE, sort=TRUE, drop=FALSE,
                      combine=FALSE, ...)
{
  UseMethod("mapLevels")
}

mapLevels.default <- function(x, codes=TRUE, sort=TRUE, drop=FALSE,
                              combine=FALSE, ...)
{
  stop(sprintf("mapLevels can only be used on %s and %s atomic 'x'",
               dQuote("factor"), dQuote("character")))
}

mapLevels.character <- function(x, codes=TRUE, sort=TRUE, drop=FALSE,
                                combine=FALSE, ...)
{
  mapLevels.factor(x=x, codes=codes, sort=sort, drop=drop, ...)
}

## Could coerce character to factor and then use factor method, but that
## is more expensive than simple unique and length used bellow in factor
## method

mapLevels.factor <- function(x, codes=TRUE, sort=TRUE, drop=FALSE,
                             combine=FALSE, ...)
{
  ## --- Argument actions ----

  if(is.factor(x)) { # factor
    if(drop) x <- factor(x)
    nlevs <- nlevels(x)
    levs <- levels(x)
  } else {           # character
    levs <- unique(x)
    nlevs <- length(levs)
    if(sort) levs <- sort(levs, ...)
  }

  ## --- Create a map ---

  map <- vector(mode="list", length=nlevs)
  names(map) <- levs
  if(codes) {
    map[1:nlevs] <- 1:nlevs
  } else {
    map[1:nlevs] <- levs
  }
  class(map) <- "levelsMap"
  map
}

mapLevels.list <- function(x, codes=TRUE, sort=TRUE, drop=FALSE,
                           combine=FALSE, ...)
{
  map <- lapply(x, mapLevels, codes=codes, sort=sort, drop=drop, ...)
  class(map) <- "listLevelsMap"
  if(combine) {
    if(!codes) {
      return(c(map, sort=sort, recursive=TRUE))
    } else {
      stop(sprintf("can not combine integer %s", dQuote("levelsMaps")))
    }
  }
  map
}

mapLevels.data.frame <- function(x, codes=TRUE, sort=TRUE, drop=FALSE,
                                 combine=FALSE, ...)
{
  mapLevels.list(x, codes=codes, sort=sort, drop=drop, combine=combine, ...)
}

### }}}
### {{{ print.*
###------------------------------------------------------------------------

.unlistLevelsMap <- function(x, ind=FALSE)
{
  y <- unlist(x, use.names=FALSE)
  len <- sapply(x, FUN=length)
  names(y) <- rep(names(x), times=len)
  if(ind) {
    return(list(y, rep(1:length(x), times=len), len))
  } else {
    return(y)
  }
}

print.levelsMap <- function(x, ...)
{
  x <- .unlistLevelsMap(x)
  print(x, ...)
}

print.listLevelsMap <- function(x, ...)
{
  class(x) <- "list"
  print(x, ...)
}

### }}}
### {{{ [.*
###------------------------------------------------------------------------

## We need these two since [.list method drops class

"[.levelsMap" <- function(x, i)
{
  classX <- class(x)
  class(x) <- "list"
  x <- x[i]
  class(x) <- classX
  x
}

"[.listLevelsMap" <- function(x, i)
{
  classX <- class(x)
  class(x) <- "list"
  x <- x[i]
  class(x) <- classX
  x
}

### }}}
### {{{ is.*
###------------------------------------------------------------------------

is.levelsMap <- function(x)
  inherits(x=x, what="levelsMap")

is.listLevelsMap <- function(x)
  inherits(x=x, what="listLevelsMap")

.isCharacterMap <- function(x)
{
  if(is(x) == "levelsMap") {
    return(inherits(x=unlist(x), what="character"))
  } else {
    stop(sprintf("can be used only on %s", dQuote("levelsMap")))
  }
}

### }}}
### {{{ as.*
###------------------------------------------------------------------------

as.levelsMap <- function(x, check=TRUE, ...)
{
  if(check)
    .checkLevelsMap(x, method="raw")
  class(x) <- "levelsMap"
  unique(x, ...)
}

as.listLevelsMap <- function(x, check=TRUE)
{
  if(check)
    .checkListLevelsMap(x, method="raw")
  class(x) <- "listLevelsMap"
  x
}

### }}}
### {{{ .check*
###------------------------------------------------------------------------

.checkLevelsMap <- function(x, method) {
  xLab <- deparse(substitute(x))
  also <- "\b"
  if(method == "class") {
    also <- "also"
    if(!is.levelsMap(x))
      stop(sprintf("'%s' must be a %s", xLab, dQuote("levelsMap")))
  }
  if(!is.list(x) || is.null(names(x)))
    stop(sprintf("'%s' must be %s a named list", xLab, also))

  ## Components can be of different length
  ##  if(!all(sapply(x, FUN=length) == 1))
  ##  stop(sprintf("all components of '%s' must have length 1", xLab))
}

.checkListLevelsMap <- function(x, method) {
  xLab <- deparse(substitute(x))
  also <- "\b"
  if(method == "class") {
    also <- "also"
    if(!is.listLevelsMap(x))
      stop(sprintf("'%s' must be a %s", xLab, dQuote("listLevelsMap")))
  }
  if(!is.list(x) || any(!sapply(x, FUN=is.levelsMap)))
    stop(sprintf("'%s' must be %s a list of %s", xLab, also,
                 dQuote("levelsMap")))
  lapply(x, FUN=.checkLevelsMap, method=method)
}

### }}}
### {{{ c.*
###------------------------------------------------------------------------

c.levelsMap <- function(..., sort=TRUE, recursive=FALSE)
{
  x <- list(...)
  class(x) <- "listLevelsMap"
  ## we use recursive=TRUE here because ... is a lists of lists
  c(x, sort=sort, recursive=TRUE)
}

c.listLevelsMap <- function(..., sort=TRUE, recursive=FALSE)
{
  x <- list(...)
  lapply(x, FUN=.checkListLevelsMap, method="class")
  x <- unlist(x, recursive=FALSE)
  if(!recursive) {
    class(x) <- "listLevelsMap"
  } else {
    if(any(!sapply(x, FUN=.isCharacterMap)))
      stop(sprintf("can not combine integer %s", dQuote("levelsMaps")))
    if(!is.null(names(x))) names(x) <- NULL
    x <- unlist(x, recursive=FALSE)
    ## how to merge components with the same name?
    class(x) <- "levelsMap"
    if(sort) x <- sort(x)
    x <- unique(x)
  }
  x
}

### }}}
### {{{ sort
###------------------------------------------------------------------------

sort.levelsMap <- function(x, decreasing=FALSE, na.last=TRUE, ...)
  x[order(names(x), na.last=na.last, decreasing=decreasing)]

### }}}
### {{{ unique
###------------------------------------------------------------------------

unique.levelsMap <- function(x, incomparables=FALSE, ...)
{
  ## Find duplicates
  y <- .unlistLevelsMap(x, ind=TRUE)
  ## Duplicates for values and names combinations
  test <- duplicated(cbind(y[[1]], names(y[[1]])),
                     incomparables=incomparables, ...)
  if(any(test)) {
    if(any(y[[3]] > 1)) { # work with the same structure as in x
      j <- 1
      k <- y[[3]][1]
      empty <- NULL
      for(i in seq(along=x)) { # how slow is this loop?
        tmp <- !test[j:k]
        if(all(!tmp)) { # these components will be empty
          empty <- c(empty, i)
        } else {
          x[[i]] <- x[[i]][tmp]
        }
        j <- j + y[[3]][i]
        k <- k + y[[3]][i + 1]
      }
      if(!is.null(empty))
        x[empty] <- NULL
    } else { # simple one-length components
      x <- x[!test]
    }
  }
  x
}

### }}}
### {{{ mapLevels<-

###------------------------------------------------------------------------

"mapLevels<-" <- function(x, value)
  UseMethod("mapLevels<-")

"mapLevels<-.default" <- function(x, value)
{
  ## --- Checks ---

  classX <- c("integer", "character", "factor")
  if(any(!(class(x) %in% classX)))
    stop(sprintf("'x' must be either: %s", paste(dQuote(classX), collapse=", ")))

  .checkLevelsMap(x=value, method="class")

  ## --- Mapping levels in x ---

  char <- all(sapply(value, is.character))
  int <- all(sapply(value, is.integer))

  if(int) { # codes=TRUE
    if(is.integer(x)) x <- factor(x)
    if(is.factor(x)) levels(x) <- value
    if(is.character(x))
      stop(sprintf("can not apply integer %s to %s",
                   dQuote("levelsMap"), dQuote("character")))
  } else {  # codes=FALSE
    if(!char)
      stop("all components of 'value' must be of the same class")
    if(is.character(x)) x <- factor(x)
    if(is.factor(x)) levels(x) <- value
    if(is.integer(x))
      stop(sprintf("can not apply character %s to %s",
                   dQuote("levelsMap"), dQuote("integer")))
  }
  x
}

"mapLevels<-.list" <- function(x, value)
{
  if(!is.listLevelsMap(value)) {
    if(is.levelsMap(value)) {
      value <- as.listLevelsMap(list(value), check=FALSE)
      ## no need for check as default method does checking anyway
    } else {
      stop(sprintf("'x' must be either %s or %s",
                   dQuote("listLevelsMap"), dQuote("levelsMap")))
    }
  }
  x <- mapply(FUN="mapLevels<-", x=x, value=value, SIMPLIFY=FALSE)
  x
}

"mapLevels<-.data.frame" <- function(x, value)
{
  x[] <- "mapLevels<-.list"(x, value)
  x
}

### }}}
### {{{ Dear Emacs
## Local variables:
## folded-file: t
## End:
### }}}

###------------------------------------------------------------------------
### mapLevels.R ends here

### runit.reorder.factor.R
###------------------------------------------------------------------------
### What: Tests for reorder.factor
### $Id: runit.reorder.factor.R 1784 2014-04-05 02:23:45Z warnes $
### Time-stamp: <2006-10-30 18:25:05 ggorjan>
###------------------------------------------------------------------------

### {{{ --- Test setup ---

if(FALSE) {
  library("RUnit")
  library("gdata")
}

### }}}
### {{{ --- reorder.factor ---

test.reorder.factor <- function()
{
  tmp <- Sys.getlocale(category="LC_COLLATE")
  Sys.setlocale(category="LC_COLLATE", locale="C")

  ## Create a 4 level example factor
  levs <- c("PLACEBO", "300 MG", "600 MG", "1200 MG")
  trt <- factor(rep(x=levs, times=c(22, 24, 28, 26)))

  ## Change the order to something useful
  ## default "mixedsort" ordering
  trt2 <- reorder(trt)
  levsTest <- c("300 MG", "600 MG", "1200 MG", "PLACEBO")
  checkIdentical(levels(trt2), levsTest)

  ## using indexes:
  trt3 <- reorder(trt, new.order=c(4, 2, 3, 1))
  levsTest <- c("PLACEBO", "300 MG", "600 MG", "1200 MG")
  checkIdentical(levels(trt3), levsTest)

  ## using label names:
  trt4 <- reorder(trt, new.order=c("PLACEBO", "300 MG", "600 MG", "1200 MG"))
  levsTest <- c("PLACEBO", "300 MG", "600 MG", "1200 MG")
  checkIdentical(levels(trt4), levsTest)

  ## using frequency
  trt5 <- reorder(trt, X=as.numeric(trt), FUN=length)
  levsTest <- c("PLACEBO", "300 MG", "1200 MG", "600 MG")
  checkIdentical(levels(trt5), levsTest)

  ## drop out the '300 MG' level
  trt6 <- reorder(trt, new.order=c("PLACEBO", "600 MG", "1200 MG"))
  levsTest <- c("PLACEBO", "600 MG", "1200 MG")
  checkIdentical(levels(trt6), levsTest)

  Sys.setlocale(category="LC_COLLATE", locale=tmp)
}

### }}}
### {{{ Dear Emacs
## Local variables:
## folded-file: t
## End:
### }}}

###------------------------------------------------------------------------
### runit.reorder.factor.R ends here

is.what <- function(object, verbose=FALSE)
{
  do.test <- function(test, object)
  {
    result <- try(get(test)(object), silent=TRUE)
    if(!is.logical(result) || length(result)!=1 || is.na(result))
      result <- NULL
    return(result)
  }

  ## Get all names starting with "is."
  is.names <- unlist(sapply(search(), function(name) ls(name,pattern="^is\\.")))

  ## Narrow to functions
  is.functions <- is.names[sapply(is.names, function(x) is.function(get(x)))]

  tests <- sort(unique(is.functions[is.functions!="is.what"]))
  results <- suppressWarnings(unlist(sapply(tests, do.test, object=object)))

  if(verbose)
    output <- data.frame(is=ifelse(results,"T","."))
  else
    output <- names(results)[results]

  return(output)
}

mv <- function(from, to, envir=parent.frame())
{
  if( !is.character(from) || !exists(from, envir=envir, inherits = TRUE) )
    stop("`from` must be a character string specifying the name of an object.") 
  if( !is.character(to)   ) 
    stop("`to` must be a characater string.")
  value <- get(from, envir=envir) 
  assign(x=to, value=value, envir=envir)
  rm(list=from, envir=envir)
}

# Simply call 'first' or 'last' with a different default value for 'n'.
first <- function(x, n=1, ...) head(x, n=n, ...)
last  <- function(x, n=1, ...) tail(x, n=n, ...)

"first<-" <- function(x, n=1, ..., value )
{
  x[1:n] <- value[1:n]
  x
}

"last<-" <- function(x, n=1, ..., value )
{
  index <- seq( length(x)-n+1, length(x) )
  x[index] <- value[1:n]
  x
}

drop.levels  <- function(x, reorder=TRUE, ...)
  UseMethod("drop.levels")

drop.levels.default <- function(x, reorder=TRUE, ...)
  x

drop.levels.factor <- function(x, reorder=TRUE, ...)
{
  x <- x[, drop=TRUE]
  if(reorder) x <- reorder(x, ...)
  x
}

drop.levels.list <- function(x, reorder=TRUE, ...)
{
  lapply(x, drop.levels, reorder=reorder, ...)
}

drop.levels.data.frame <- function(x, reorder=TRUE, ...)
{
  x[] <- drop.levels.list(x, reorder=reorder, ...)
  x
}

### write.fwf.R
###------------------------------------------------------------------------
### What: Write fixed width format - code
### $Id: write.fwf.R 1967 2015-04-25 16:24:41Z warnes $
### Time-stamp: <2008-08-05 12:11:27 ggorjan>
###------------------------------------------------------------------------

write.fwf <- function(x,
                      file="",
                      append=FALSE,
                      quote=FALSE,
                      sep=" ",
                      na="",
                      rownames=FALSE,
                      colnames=TRUE,
                      rowCol=NULL,
                      justify="left",
                      formatInfo=FALSE,
                      quoteInfo=TRUE,
                      width=NULL,
                      eol="\n",
                      qmethod=c("escape", "double"),
                      scientific=TRUE,
                      ...)
{
  ## --- Setup ---

  dapply <- function(x, FUN, ..., simplify=TRUE)
      {
          if(is.data.frame(x))
              return(sapply(x, FUN, ..., simplify=simplify))
          else if(is.matrix(x))
              return(apply(x, 2, FUN, ...))
          else
              stop("x must be a data.frame or a matrix")
      }

  if(!(is.data.frame(x) || is.matrix(x)))
    stop("'x' must be a data.frame or matrix")
  if(length(na) > 1)
    stop("only single value can be defined for 'na'")

  if(!scientific)
      {
          option.scipen <- getOption("scipen")
          on.exit( function() options("scipen"=option.scipen) )
          options("scipen"=100)
      }


  if(rownames) {
    x <- as.data.frame(x)
    x <- cbind(rownames(x), x)
    rowColVal <- ifelse(!is.null(rowCol), rowCol, "row")
    colnames(x)[1] <- rowColVal
  }
  colnamesMy <- colnames(x)
  if(length(colnamesMy)==0)
      colnamesMy <- paste( "V", 1:ncol(x), sep="")

  nRow <- nrow(x)
  nCol <- length(colnamesMy)

  widthNULL <- is.null(width)
  if(!widthNULL && length(width) != nCol) {
    warning("recycling 'width'")
    widthOld <- width
    width <- integer(length=nCol)
    width[] <- widthOld
  }

  ## --- Format info ---

  retFormat <- data.frame(colname=colnamesMy,
                          nlevels=0,
                          position=0,
                          width=0,
                          digits=0,
                          exp=0,
                          stringsAsFactors=FALSE)

  ## Which columns are numeric like
  isNum <- dapply(x, is.numeric)
  ## is.numeric picks also Date and POSIXt
  isNum <- isNum & !(dapply(x, inherits, what="Date") |
                     dapply(x, inherits, what="POSIXt"))

  ## Which columns are factors --> convert them to character
  isFac <- dapply(x, is.factor)
  if(any(isFac))
      ## This conditional is necessary because if x is a matrix, even if
      ## all(isFAC==FALSE), this assignment will coerce it to mode
      ## character.  This isn't a problem for dataframes.
      x[, isFac] <- sapply(x[, isFac, drop=FALSE], as.character)

  ## Collect information about how format() will format columns.
  ## We need to get this info now, since format will turn all columns to character
  tmp <- dapply(x, format.info, ..., simplify=FALSE)
  if(is.matrix(x)) tmp <- as.data.frame(tmp)
  tmp1 <- sapply(tmp, length)
  tmp <- t(as.data.frame(tmp))
  retFormat$width <- tmp[, 1]
  ## Collect other details for numeric columns
  if(any(isNum)) {
    ## Numeric columns with digits
    test <- tmp1 > 1
    if(any(test)) {
      retFormat[test, c("digits", "exp")] <- tmp[test, c(2, 3)]
      ## Numeric columns with scientific notation
      test2 <- tmp[test, 3] > 0
      if(any(test2)) ## adding +1; see ?format.info
        retFormat[test, ][test2, "exp"] <- retFormat[test, ][test2, "exp"] + 1
    }
  }

  ## --- Format ---

  ## store original object in 'y'
  y <- x

  ## Formatting (to character)
  for(i in 1:nCol) {
    if(widthNULL) {
      tmp <- NULL
    } else {
      tmp <- width[i]
    }
    ## Due to na.encode bug in format() in 2.7.1; na.encode=TRUE should
    ## return NA values and not "NA", but even then we rely on the
    ## following test to "fiddle" with the value in 'na' argument since -
    ## NA should not increase the width of column with width 1, while wider
    ## value for 'na' should increase the width
    test <- is.na(y[, i])
    ## Make a copy to make sure we get character after first format() - Date class caused problems
    x2 <- character(length=nRow)
    ## Add formatted values
    x2[!test] <- format(y[!test, i], justify=justify, width=tmp, ...)
    ## Add 'na' value
    x2[test] <- na
    ## Replace the original
    x[, i] <- x2
    ## Collect width (again)
    tmp2 <- format.info(x2, ...)[1]
    ## Reformat if 'na' value change the width of the column
    if(tmp2 != retFormat[i, "width"]) {
      retFormat[i, "width"] <- tmp2
      ## ifelse() makes sure that numeric columns are justified to right
      x[, i] <- format(x[, i], justify=ifelse(isNum[i], "right", justify),
                       width=tmp, ...)
    }
    ## Reformat 'na' value if it is narrower than the width of the column
    if(nchar(na) < retFormat[i, "width"]) {
      x[test, i] <- format(na, justify=ifelse(isNum[i], "right", justify),
                           width=retFormat[i, "width"], ...)
    }
  }

  ## Number of levels for "non-numeric"" columns
  if(any(!isNum)) {
    retFormat[!isNum, "nlevels"] <- dapply(x[, !isNum, drop=FALSE],
                                           function(z) length(unique(z)))
  }

  ## Check that width was not to small
  if(!widthNULL) {
    test <- retFormat$width > width
    if(any(test)) {
      tmpCol <- paste(colnamesMy[test], collapse=", ")
      tmpWidth <- paste(width[test], collapse=", ")
      tmpNeed <- paste(retFormat$width[test], collapse=", ")
      stop(paste("'width' (", tmpWidth, ") was too small for columns: ",
                 tmpCol, "\n 'width' should be at least (", tmpNeed, ")",
                 sep=""))
    }
  }

  ## --- Write ---

  if(colnames) {
    if(rownames && is.null(rowCol)) colnamesMy <- colnamesMy[-1]
    write.table(t(as.matrix(colnamesMy)),
                file=file,
                append=append,
                quote=quote,
                sep=sep,
                eol=eol,
                na=na,
                row.names=FALSE,
                col.names=FALSE,
                qmethod=qmethod)
 }

  write.table(x=x,
              file=file,
              append=(colnames || append),
              quote=quote,
              sep=sep,
              eol=eol,
              na=na,
              row.names=FALSE,
              col.names=FALSE,
              qmethod=qmethod)

  ## --- Return format and fixed width information ---

  if(formatInfo) {
    ## be carefull with these ifelse constructs
    retFormat$position[1] <- ifelse(quote, ifelse(quoteInfo, 1, 2), 1)
    if(ifelse(quote, quoteInfo, FALSE)) retFormat$width <- retFormat$width + 2
    N <- nrow(retFormat)
    if(N > 1) {
      for(i in 2:N) {
        retFormat$position[i] <- retFormat$position[i - 1] +
          retFormat$width[i - 1] + nchar(x=sep, type="chars") +
            ifelse(quote, ifelse(quoteInfo, 0, 1), 0)
      }
    }
    if(rownames && is.null(rowCol)) {
      retFormat <- retFormat[-1,]
      rownames(retFormat) <- 1:(N-1)
    }
    return(retFormat)
  }
}

###------------------------------------------------------------------------
### write.fwf.R ends here

## Test setup
if(FALSE) {
  library("RUnit")
  library("gdata")
}

test.write.fwf <- function()
{
  ## 'x' must be a data.frame or matrix
  checkException(write.fwf(1:10))
  checkException(write.fwf(list(1:10)))

  ## Only single value is allowed in 'na'
  checkException(write.fwf(data.frame(1:10, letters[1:10]), na=c("", " ")))

  ## Example dataset
  num <- round(c(733070.345678, 1214213.78765456, 553823.798765678,
                 1085022.8876545678,  571063.88765456, 606718.3876545678,
                 1053686.6, 971024.187656, 631193.398765456, 879431.1),
               digits=3)

  testData <- data.frame(num1=c(1:10, NA),
                         num2=c(NA, seq(from=1, to=5.5, by=0.5)),
                         num3=c(NA, num),
                         int1=c(as.integer(1:4), NA, as.integer(4:9)),
                         fac1=factor(c(NA, letters[1:9], "hjh")),
                         fac2=factor(c(letters[6:15], NA)),
                         cha1=c(letters[17:26], NA),
                         cha2=c(NA, "longer", letters[25:17]),
                         stringsAsFactors=FALSE)
  levels(testData$fac1) <- c(levels(testData$fac1), "unusedLevel")
  testData$Date <- as.Date("1900-1-1")
  testData$Date[2] <- NA
  testData$POSIXt <- as.POSIXct(strptime("1900-1-1 01:01:01",
                                         format="%Y-%m-%d %H:%M:%S"))
  testData$POSIXt[5] <- NA

  ## Output
  ## is tested with regular tests

  ## formatInfo

  ## Default output
  formatInfoT <- data.frame(colname=c("num1", "num2"),
                            nlevels=c(0, 0),
                            position=c(1, 4),
                            width=c(2, 3),
                            digits=c(0, 1),
                            exp=c(0, 0),
                            stringsAsFactors=FALSE)

  testData1 <- testData[, c("num1", "num2")]
  testData1M <- as.matrix(testData1)

  formatInfo <- write.fwf(testData1, formatInfo=TRUE)
  checkEquals(formatInfo, formatInfoT)

  formatInfoM <- write.fwf(testData1M, formatInfo=TRUE)
  checkEquals(formatInfoM, formatInfoT)

  ## Scientific notation
  dd <- options("digits"); options(digits = 7)
  testData2 <- data.frame(a=123, b=pi, c=1e8, d=1e222)
  formatInfo <- write.fwf(x=testData2, formatInfo=TRUE)
  checkEquals(formatInfo$width, c(3, 8, 5, 6))
  checkEquals(formatInfo$digits, c(0, 6, 0, 0))
  checkEquals(formatInfo$exp, c(0, 0, 2, 3))
  options(dd) ## reset old options

  ## 'na' can either decrease or increase the width
  ## --> values of int1 have width 1 and using na="" should not increase
  ##     the width
  formatInfo <- write.fwf(testData[, "int1", drop=FALSE], formatInfo=TRUE,
                          na="")
  checkEquals(formatInfo$width, 1)
  ## --> values of int1 have width 1 and using na="1234" should increase
  ##     the width to 4
  formatInfo <- write.fwf(testData[, "int1", drop=FALSE], formatInfo=TRUE,
                          na="1234")
  checkEquals(formatInfo$width, 4)

  ## rowCol
  formatInfoTR <- data.frame(colname=c("row", "num1", "num2"),
                             nlevels=c(11, 0, 0),
                             position=c(1, 4, 7),
                             width=c(2, 2, 3),
                             digits=c(0, 0, 1),
                             exp=c(0, 0, 0),
                             stringsAsFactors=FALSE)
  testData3 <- testData[, c("num1", "num2")]
  testData3M <- as.matrix(testData3)

  formatInfoR <- write.fwf(testData3, formatInfo=TRUE, rownames=TRUE,
                           rowCol="row")
  checkEquals(formatInfoR, formatInfoTR)

  formatInfoR <- write.fwf(testData3M, formatInfo=TRUE, rownames=TRUE,
                           rowCol="row")
  checkEquals(formatInfoR, formatInfoTR)


  ## QuoteInfo alone does not have any effect
  formatInfoI <- write.fwf(testData3,  formatInfo=TRUE, quoteInfo=TRUE)
  checkEquals(formatInfoI, formatInfoT)

  formatInfoI <- write.fwf(testData3M, formatInfo=TRUE, quoteInfo=TRUE)
  checkEquals(formatInfoI, formatInfoT)

  ## Quote
  formatInfoTQ <-  formatInfoT
  formatInfoTQ$position <- c(1, 6)
  formatInfoTQ$width <- c(4, 5)

  formatInfoQ <- write.fwf(testData3, formatInfo=TRUE, quote=TRUE)
  checkEquals(formatInfoQ, formatInfoTQ)

  formatInfoQ <- write.fwf(testData3M, formatInfo=TRUE, quote=TRUE)
  checkEquals(formatInfoQ, formatInfoTQ)

  ## Quote without quoteInfo
  formatInfoTQI <-  formatInfoT
  formatInfoTQI$position <- c(2, 6)

  formatInfoQI <- write.fwf(testData3, formatInfo=TRUE, quote=TRUE,
                            quoteInfo=FALSE)
  checkEquals(formatInfoQI, formatInfoTQI)

  formatInfoQI <- write.fwf(testData3M, formatInfo=TRUE, quote=TRUE,
                            quoteInfo=FALSE)
  checkEquals(formatInfoQI, formatInfoTQI)

  ## Width
  ## --> default width for num1 is 2
  testData4 <- testData[, "num1", drop=FALSE]
  testData4M <- as.matrix(testData[, "num1", drop=FALSE])

  formatInfo <- write.fwf(testData4, width=10, formatInfo=TRUE)
  checkEquals(formatInfo$width, 10)

  formatInfo <- write.fwf(testData4M, width=10, formatInfo=TRUE)
  checkEquals(formatInfo$width, 10)

  ## Too small value in width (this also tests recycling)
  ## --> proper width for num1 is 2, while for num2 it is 3
  checkException(write.fwf(testData[, c("num1", "num2")], width=2))
  checkException(write.fwf(testData[, c("num1", "num2")], width=c(2, 1)))

  ## Done
  cat("\nDONE.\n\n")
}

### runit.write.fwf.R
###------------------------------------------------------------------------
### What: Unit tests for write.fwf
### $Id: runit.write.fwf.R 1966 2015-04-25 16:23:31Z warnes $
### Time-stamp: <2008-08-05 11:58:50 ggorjan>
###------------------------------------------------------------------------

### {{{ --- Test setup ---

if(FALSE) {
  library("RUnit")
  library("gdata")
}

### }}}
### {{{ --- write.fwf ---

test.write.fwf <- function()
{

  ## 'x' must be a data.frame or matrix
  checkException(write.fwf(1:10))
  checkException(write.fwf(list(1:10)))

  ## only single value is allowed in 'na'
  checkException(write.fwf(data.frame(1:10, letters[1:10]), na=c("", " ")))

  ## Example dataset
  num <- round(c(733070.345678, 1214213.78765456, 553823.798765678,
                 1085022.8876545678,  571063.88765456, 606718.3876545678,
                 1053686.6, 971024.187656, 631193.398765456, 879431.1),
               digits=3)

  testData <- data.frame(num1=c(1:10, NA),
                         num2=c(NA, seq(from=1, to=5.5, by=0.5)),
                         num3=c(NA, num),
                         int1=c(as.integer(1:4), NA, as.integer(4:9)),
                         fac1=factor(c(NA, letters[1:9], "hjh")),
                         fac2=factor(c(letters[6:15], NA)),
                         cha1=c(letters[17:26], NA),
                         cha2=c(NA, "longer", letters[25:17]),
                         stringsAsFactors=FALSE)
  levels(testData$fac1) <- c(levels(testData$fac1), "unusedLevel")
  testData$Date <- as.Date("1900-1-1")
  testData$Date[2] <- NA
  testData$POSIXt <- as.POSIXct(strptime("1900-1-1 01:01:01", format="%Y-%m-%d %H:%M:%S"))
  testData$POSIXt[5] <- NA

  ## --- output ---
  ## is tested with regular tests

  ## --- formatInfo ---

  ## default output
  formatInfoT <- data.frame(colname=c("num1", "num2"),
                            nlevels=c(0, 0),
                            position=c(1, 4),
                            width=c(2, 3),
                            digits=c(0, 1),
                            exp=c(0, 0),
                            stringsAsFactors=FALSE)

  testData1 <- testData[, c("num1", "num2")]
  testData1M <- as.matrix(testData1)

  formatInfo  <- write.fwf(testData1, formatInfo=TRUE)
  checkEquals(formatInfo, formatInfoT)

  formatInfoM  <- write.fwf(testData1M, formatInfo=TRUE)
  checkEquals(formatInfoM, formatInfoT)


  ## scientific notation
  dd <- options("digits"); options(digits = 7)
  testData2 <- data.frame(a=123, b=pi, c=1e8, d=1e222)
  formatInfo <- write.fwf(x=testData2, formatInfo=TRUE)
  checkEquals(formatInfo$width, c(3, 8, 5, 6))
  checkEquals(formatInfo$digits, c(0, 6, 0, 0))
  checkEquals(formatInfo$exp, c(0, 0, 2, 3))
  options(dd) ## reset old options

  ## 'na' can either decrease or increase the width
  ## --> values of int1 have width 1 and using na="" should not increase
  ##     the width
  formatInfo  <- write.fwf(testData[, "int1", drop=FALSE], formatInfo=TRUE,
                           na="")
  checkEquals(formatInfo$width, 1)
  ## --> values of int1 have width 1 and using na="1234" should increase
  ##     the width to 4
  formatInfo  <- write.fwf(testData[, "int1", drop=FALSE], formatInfo=TRUE,
                           na="1234")
  checkEquals(formatInfo$width, 4)

  ## rowCol
  formatInfoTR <- data.frame(colname=c("row", "num1", "num2"),
                             nlevels=c(11, 0, 0),
                             position=c(1, 4, 7),
                             width=c(2, 2, 3),
                             digits=c(0, 0, 1),
                             exp=c(0, 0, 0),
                             stringsAsFactors=FALSE)
  testData3 <- testData[, c("num1", "num2")]
  testData3M <- as.matrix(testData3)

  formatInfoR <- write.fwf(testData3, formatInfo=TRUE, rownames=TRUE,
                           rowCol="row")
  checkEquals(formatInfoR, formatInfoTR)

  formatInfoR <- write.fwf(testData3M, formatInfo=TRUE, rownames=TRUE,
                           rowCol="row")
  checkEquals(formatInfoR, formatInfoTR)


  ## quoteInfo alone does not have any effect
  formatInfoI <- write.fwf(testData3,  formatInfo=TRUE, quoteInfo=TRUE)
  checkEquals(formatInfoI, formatInfoT)

  formatInfoI <- write.fwf(testData3M, formatInfo=TRUE, quoteInfo=TRUE)
  checkEquals(formatInfoI, formatInfoT)

  ## quote
  formatInfoTQ <-  formatInfoT
  formatInfoTQ$position <- c(1, 6)
  formatInfoTQ$width <- c(4, 5)

  formatInfoQ <- write.fwf(testData3, formatInfo=TRUE, quote=TRUE)
  checkEquals(formatInfoQ, formatInfoTQ)

  formatInfoQ <- write.fwf(testData3M, formatInfo=TRUE, quote=TRUE)
  checkEquals(formatInfoQ, formatInfoTQ)

  ## quote without quoteInfo
  formatInfoTQI <-  formatInfoT
  formatInfoTQI$position <- c(2, 6)

  formatInfoQI <- write.fwf(testData3, formatInfo=TRUE, quote=TRUE,
                            quoteInfo=FALSE)
  checkEquals(formatInfoQI, formatInfoTQI)

  formatInfoQI <- write.fwf(testData3M, formatInfo=TRUE, quote=TRUE,
                            quoteInfo=FALSE)
  checkEquals(formatInfoQI, formatInfoTQI)

  ## width
  ## --> default width for num1 is 2
  testData4 <- testData[, "num1", drop=FALSE]
  testData4M <- as.matrix(testData[, "num1", drop=FALSE])

  formatInfo <- write.fwf(testData4, width=10, formatInfo=TRUE)
  checkEquals(formatInfo$width, 10)

  formatInfo <- write.fwf(testData4M, width=10, formatInfo=TRUE)
  checkEquals(formatInfo$width, 10)

  ## too small value in width (this also tests recycling)
  ## --> proper width for num1 is 2, while for num2 it is 3
  checkException(write.fwf(testData[, c("num1", "num2")], width=2))
  checkException(write.fwf(testData[, c("num1", "num2")], width=c(2, 1)))

  ## Done
  cat("\nDONE.\n\n")
}

### }}}
### {{{ Dear Emacs
## Local variables:
## folded-file: t
## End:
### }}}

###------------------------------------------------------------------------
### runit.write.fwf.R ends here

### runRUnitTests.R
###------------------------------------------------------------------------
### What: Run RUnit tests (the core)- R code
### $Id$
### Time-stamp: <2008-12-30 12:52:51 ggorjan>
###------------------------------------------------------------------------

## The setup seems to be quite messy, but it is so to enable use of this in
## several ways as shown bellow.

## "R CMD check" way should be the most authoritative way to run the RUnit
## tests for a developer. RUnit tests are issued during R CMD check of the
## package due to example section of .runRUnitTests() function. If any test
## fails (failure) or if there are any R errors during RUnit testing, R CMD
## check fails. These are variable values specific for this way:
##  - .path DEVEL/PATH/PKG.Rcheck/PKG/unitTests
##  - .way  function

## ".runRUnitTests()" way from within R after library(PKG) is handy for
## package useRs, since it enables useRs to be sure that all tests pass for
## their installation. This is just a convenient wrapper function to run
## the RUnit testing suite. These are variable values specific for this
## way:
##  - .path INSTALL/PATH/PKG/unitTests
##  - .way  function

## "Shell" way is another possibility mainly for a developer in order to
## skip possibly lengthy R CMD check and perform just RUnit testing with an
## installed version of a pcakage. These are variable values specific for
## this way:
##  - .path DEVEL/PATH/PKG/inst/unitTests
##  - .way  shell
##
## Rscript runRUnitTests.R
## R CMD BATCH runRUnitTests.R
## make
## make all

PKG  <- 'gdata'

if(require("RUnit", quietly=TRUE))
{

    path <- normalizePath("unitTests")

    cat("\nRunning unit tests\n")
    print(list(pkg=PKG, getwd=getwd(), pathToUnitTests=path))

    library(package=PKG, character.only=TRUE)

    testFileRegexp <- "^runit.+\\.[rR]$"

    ## Debugging echo
    cat("\nRunning RUnit tests\n")
    print(list(pkg=PKG,
               getwd=getwd(),
               pathToRUnitTests=path))

    ## Define tests
    testSuite <- defineTestSuite(name=paste(PKG, "RUnit testing"),
                                 dirs=path,
                                 testFileRegexp=testFileRegexp
                                 )

    ## Run
    tests <- runTestSuite(testSuite)

    if(file.access(path, 02) != 0)
        {
            ## cannot write to path -> use writable one
            tdir <- tempfile(paste(PKG, "RUnitTests", sep="_"))
            dir.create(tdir)
            pathReport <- file.path(tdir, "report")
        }
    else
        {
            pathReport <- file.path(path, "report")
        }

    ## Print results:
    printTextProtocol(tests)
    printTextProtocol(tests,
                      fileName=paste(pathReport, ".txt", sep=""))

    ## Print HTML Version of results:
    printHTMLProtocol(tests,
                      fileName=paste(pathReport, ".html", sep=""))

    cat("\nRUnit reports also written to\n",
        pathReport, ".(txt|html)\n\n", sep="")

    ## Return stop() to cause R CMD check stop in case of
    ##  - failures i.e. FALSE to RUnit tests or
    ##  - errors i.e. R errors
    tmp <- getErrors(tests)
    if(tmp$nFail > 0 || tmp$nErr > 0)
        {
            stop(paste("\n\nRUnit testing failed:\n",
                       " - #test failures: ", tmp$nFail, "\n",
                       " - #R errors: ",  tmp$nErr, "\n\n", sep="")
                 )
        }
} else {

    cat("R package 'RUnit' cannot be loaded - no unit tests run\n",
        "for package", PKG,"\n")

}

###------------------------------------------------------------------------
### runRUnitTests.R ends here

%\VignetteIndexEntry{Mapping levels of a factor}
%\VignettePackage{gdata}
%\VignetteKeywords{levels, factor, manip}

\documentclass[a4paper]{report}
\usepackage{Rnews}
\usepackage[round]{natbib}
\bibliographystyle{abbrvnat}

\usepackage{Sweave}
\SweaveOpts{strip.white=all, keep.source=TRUE}
\SweaveOpts{concordance=TRUE}

\begin{document}
\SweaveOpts{concordance=TRUE}

\begin{article}

\title{Mapping levels of a factor}
\subtitle{The \pkg{gdata} package}
\author{by Gregor Gorjanc}

\maketitle

\section{Introduction}

Factors use levels attribute to store information on mapping between
internal integer codes and character values i.e. levels. First level is
mapped to internal integer code 1 and so on. Although some users do not
like factors, their use is more efficient in terms of storage than for
character vectors. Additionally, there are many functions in base \R{} that
provide additional value for factors. Sometimes users need to work with
internal integer codes and mapping them back to factor, especially when
interfacing external programs. Mapping information is also of interest if
there are many factors that should have the same set of levels. This note
describes \code{mapLevels} function, which is an utility function for
mapping the levels of a factor in \pkg{gdata} \footnote{from version 2.3.1}
package \citep{WarnesGdata}.

\section{Description with examples}

Function \code{mapLevels()} is an (S3) generic function and works on
\code{factor} and \code{character} atomic classes. It also works on
\code{list} and \code{data.frame} objects with previously mentioned atomic
classes. Function \code{mapLevels} produces a so called ``map'' with names
and values. Names are levels, while values can be internal integer codes or
(possibly other) levels. This will be clarified later on.  Class of this
``map'' is \code{levelsMap}, if \code{x} in \code{mapLevels()} was atomic
or \code{listLevelsMap} otherwise - for \code{list} and \code{data.frame}
classes. The following example shows the creation and printout of such a
``map''.

<<ex01>>=
library(gdata)
(fac <- factor(c("B", "A", "Z", "D")))
(map <- mapLevels(x=fac))
@

If we have to work with internal integer codes, we can transform factor to
integer and still get ``back the original factor'' with ``map'' used as
argument in \code{mapLevels<-} function as shown bellow. \code{mapLevels<-}
is also an (S3) generic function and works on same classes as
\code{mapLevels} plus \code{integer} atomic class.

<<ex02>>=
(int <- as.integer(fac))
mapLevels(x=int) <- map
int
identical(fac, int)
@

Internally ``map'' (\code{levelsMap} class) is a \code{list} (see bellow),
but its print method unlists it for ease of inspection. ``Map'' from
example has all components of length 1. This is not mandatory as
\code{mapLevels<-} function is only a wrapper around workhorse function
\code{levels<-} and the later can accept \code{list} with components of
various lengths.

<<ex03>>=
str(map)
@

Although not of primary importance, this ``map'' can also be used to remap
factor levels as shown bellow.  Components ``later'' in the map take over
the ``previous'' ones. Since this is not optimal I would rather recommend
other approaches for ``remapping'' the levels of a \code{factor}, say
\code{recode} in \pkg{car} package \citep{FoxCar}.

<<ex04>>=
map[[2]] <- as.integer(c(1, 2))
map
int <- as.integer(fac)
mapLevels(x=int) <- map
int
@

Up to now examples showed ``map'' with internal integer codes for values
and levels for names. I call this integer ``map''. On the other hand
character ``map'' uses levels for values and (possibly other) levels for
names. This feature is a bit odd at first sight, but can be used to easily
unify levels and internal integer codes across several factors.  Imagine
you have a factor that is for some reason split into two factors \code{f1}
and \code{f2} and that each factor does not have all levels. This is not
uncommon situation.

<<ex05>>=
(f1 <- factor(c("A", "D", "C")))
(f2 <- factor(c("B", "D", "C")))
@

If we work with this factors, we need to be careful as they do not have the
same set of levels. This can be solved with appropriately specifying
\code{levels} argument in creation of factors i.e. \code{levels=c("A", "B",
  "C", "D")} or with proper use of \code{levels<-} function. I say proper
as it is very tempting to use:

<<ex06>>=
fTest <- f1
levels(fTest) <- c("A", "B", "C", "D")
fTest
@

Above example extends set of levels, but also changes level of 2nd and 3rd
element in \code{fTest}! Proper use of \code{levels<-} (as shown in
\code{levels} help page) would be:

<<ex07>>=
fTest <- f1
levels(fTest) <- list(A="A", B="B",
                      C="C", D="D")
fTest
@

Function \code{mapLevels} with character ``map'' can help us in such
scenarios to unify levels and internal integer codes across several
factors. Again the workhorse under this process is \code{levels<-} function
from base \R{}! Function \code{mapLevels<-} just controls the assignment of
(integer or character) ``map'' to \code{x}. Levels in \code{x} that match
``map'' values (internal integer codes or levels) are changed to ``map''
names (possibly other levels) as shown in \code{levels} help page. Levels
that do not match are converted to \code{NA}. Integer ``map'' can be
applied to \code{integer} or \code{factor}, while character ``map'' can be
applied to \code{character} or \code{factor}. Result of \code{mapLevels<-}
is always a \code{factor} with possibly ``remapped'' levels.

To get one joint character ``map'' for several factors, we need to put
factors in a \code{list} or \code{data.frame} and use arguments
\code{codes=FALSE} and \code{combine=TRUE}. Such map can then be used to
unify levels and internal integer codes.

<<ex08>>=
(bigMap <- mapLevels(x=list(f1, f2),
                     codes=FALSE,
                     combine=TRUE))
mapLevels(f1) <- bigMap
mapLevels(f2) <- bigMap
f1
f2
cbind(as.character(f1), as.integer(f1),
      as.character(f2), as.integer(f2))
@

If we do not specify \code{combine=TRUE} (which is the default behaviour)
and \code{x} is a \code{list} or \code{data.frame}, \code{mapLevels}
returns ``map'' of class \code{listLevelsMap}. This is internally a
\code{list} of ``maps'' (\code{levelsMap} objects). Both
\code{listLevelsMap} and \code{levelsMap} objects can be passed to
\code{mapLevels<-} for \code{list}/\code{data.frame}. Recycling occurs when
length of \code{listLevelsMap} is not the same as number of
components/columns of a \code{list}/\code{data.frame}.

Additional convenience methods are also implemented to ease the work with
``maps'':

\begin{itemize}

\item \code{is.levelsMap}, \code{is.listLevelsMap}, \code{as.levelsMap} and
  \code{as.listLevelsMap} for testing and coercion of user defined
  ``maps'',

\item \code{"["} for subsetting,

\item \code{c} for combining \code{levelsMap} or \code{listLevelsMap}
  objects; argument \code{recursive=TRUE} can be used to coerce
  \code{listLevelsMap} to \code{levelsMap}, for example \code{c(llm1, llm2,
    recursive=TRUE)} and

\item \code{unique} and \code{sort} for \code{levelsMap}.

\end{itemize}

\section{Summary}

Functions \code{mapLevels} and \code{mapLevels<-} can help users to map
internal integer codes to factor levels and unify levels as well as
internal integer codes among several factors. I welcome any comments or
suggestions.

% \bibliography{refs}
\begin{thebibliography}{1}
\providecommand{\natexlab}[1]{#1}
\providecommand{\url}[1]{\texttt{#1}}
\expandafter\ifx\csname urlstyle\endcsname\relax
  \providecommand{\doi}[1]{doi: #1}\else
  \providecommand{\doi}{doi: \begingroup \urlstyle{rm}\Url}\fi

\bibitem[Fox(2006)]{FoxCar}
J.~Fox.
\newblock \emph{car: Companion to Applied Regression}, 2006.
\newblock URL \url{http://socserv.socsci.mcmaster.ca/jfox/}.
\newblock R package version 1.1-1.

\bibitem[Warnes(2006)]{WarnesGdata}
G.~R. Warnes.
\newblock \emph{gdata: Various R programming tools for data manipulation},
  2006.
\newblock URL
  \url{http://cran.r-project.org/src/contrib/Descriptions/gdata.html}.
\newblock R package version 2.3.1. Includes R source code and/or documentation
  contributed by Ben Bolker, Gregor Gorjanc and Thomas Lumley.

\end{thebibliography}

\address{Gregor Gorjanc\\
  University of Ljubljana, Slovenia\\
\email{gregor.gorjanc@bfro.uni-lj.si}}

\end{article}

\end{document}

%\VignetteIndexEntry{Working with Unknown Values}
%\VignettePackage{gdata}
%\VignetteKeywords{unknown, missing, manip}

\documentclass[a4paper]{report}
\usepackage{Rnews}
\usepackage[round]{natbib}
\bibliographystyle{abbrvnat}

\usepackage{Sweave}
\SweaveOpts{strip.white=all, keep.source=TRUE}
\SweaveOpts{concordance=TRUE}

\begin{document}

\begin{article}

\title{Working with Unknown Values}
\subtitle{The \pkg{gdata} package}
\author{by Gregor Gorjanc}

\maketitle

This vignette has been published as \cite{Gorjanc}.

\section{Introduction}

Unknown or missing values can be represented in various ways. For example
SAS uses \code{.}~(dot), while \R{} uses \code{NA}, which we can read as
Not Available. When we import data into \R{}, say via \code{read.table} or
its derivatives, conversion of blank fields to \code{NA} (according to
\code{read.table} help) is done for \code{logical}, \code{integer},
\code{numeric} and \code{complex} classes. Additionally, the
\code{na.strings} argument can be used to specify values that should also
be converted to \code{NA}. Inversely, there is an argument \code{na} in
\code{write.table} and its derivatives to define value that will replace
\code{NA} in exported data. There are also other ways to import/export data
into \R{} as described in the {\emph R Data Import/Export} manual
\citep{RImportExportManual}.  However, all approaches lack the possibility
to define unknown value(s) for some particular column. It is possible that
an unknown value in one column is a valid value in another column. For
example, I have seen many datasets where values such as 0, -9, 999 and
specific dates are used as column specific unknown values.

This note describes a set of functions in package \pkg{gdata}\footnote{
  package version 2.3.1} \citep{WarnesGdata}: \code{isUnknown},
\code{unknownToNA} and \code{NAToUnknown}, which can help with testing for
unknown values and conversions between unknown values and \code{NA}. All
three functions are generic (S3) and were tested (at the time of writing)
to work with: \code{integer}, \code{numeric}, \code{character},
\code{factor}, \code{Date}, \code{POSIXct}, \code{POSIXlt}, \code{list},
\code{data.frame} and \code{matrix} classes.

\section{Description with examples}

The following examples show simple usage of these functions on
\code{numeric} and \code{factor} classes, where value \code{0} (beside
\code{NA}) should be treated as an unknown value:

<<ex01>>=
library("gdata")
xNum <- c(0, 6, 0, 7, 8, 9, NA)
isUnknown(x=xNum)
@

The default unknown value in \code{isUnknown} is \code{NA}, which means
that output is the same as \code{is.na} --- at least for atomic
classes. However, we can pass the argument \code{unknown} to define which
values should be treated as unknown:

<<ex02>>=
isUnknown(x=xNum, unknown=0)
@

This skipped \code{NA}, but we can get the expected answer after
appropriately adding \code{NA} into the argument \code{unknown}:

<<ex03>>=
isUnknown(x=xNum, unknown=c(0, NA))
@

Now, we can change all unknown values to \code{NA} with \code{unknownToNA}.
There is clearly no need to add \code{NA} here. This step is very handy
after importing data from an external source, where many different unknown
values might be used. Argument \code{warning=TRUE} can be used, if there is
a need to be warned about ``original'' \code{NA}s:

<<ex04>>=
(xNum2 <- unknownToNA(x=xNum, unknown=0))
@

Prior to export from \R{}, we might want to change unknown values
(\code{NA} in \R{}) to some other value. Function \code{NAToUnknown} can be
used for this:

<<ex05>>=
NAToUnknown(x=xNum2, unknown=999)
@

Converting \code{NA} to a value that already exists in \code{x} issues an
error, but \code{force=TRUE} can be used to overcome this if needed. But be
warned that there is no way back from this step:

<<ex06>>=
NAToUnknown(x=xNum2, unknown=7, force=TRUE)
@

Examples below show all peculiarities with class \code{factor}.
\code{unknownToNA} removes \code{unknown} value from levels and inversely
\code{NAToUnknown} adds it with a warning. Additionally, \code{"NA"} is
properly distinguished from \code{NA}. It can also be seen that the
argument \code{unknown} in functions \code{isUnknown} and
\code{unknownToNA} need not match the class of \code{x} (otherwise factor
should be used) as the test is internally done with \code{\%in\%}, which
nicely resolves coercing issues.

<<ex07>>=
(xFac <- factor(c(0, "BA", "RA", "BA", NA, "NA")))
isUnknown(x=xFac)
isUnknown(x=xFac, unknown=0)
isUnknown(x=xFac, unknown=c(0, NA))
isUnknown(x=xFac, unknown=c(0, "NA"))
isUnknown(x=xFac, unknown=c(0, "NA", NA))

(xFac <- unknownToNA(x=xFac, unknown=0))
(xFac <- NAToUnknown(x=xFac, unknown=0))
@

These two examples with classes \code{numeric} and \code{factor} are fairly
simple and we could get the same results with one or two lines of \R{}
code. The real benefit of the set of functions presented here is in
\code{list} and \code{data.frame} methods, where \code{data.frame} methods
are merely wrappers for \code{list} methods.

We need additional flexibility for \code{list}/\code{data.frame} methods,
due to possibly having multiple unknown values that can be different among
\code{list} components or \code{data.frame} columns. For these two methods,
the argument \code{unknown} can be either a \code{vector} or \code{list},
both possibly named. Of course, greater flexibility (defining multiple
unknown values per component/column) can be achieved with a \code{list}.

When a \code{vector}/\code{list} object passed to the argument
\code{unknown} is not named, the first value/component of a
\code{vector}/\code{list} matches the first component/column of a
\code{list}/\code{data.frame}. This can be quite error prone, especially
with \code{vectors}. Therefore, I encourage the use of a \code{list}. In
case \code{vector}/\code{list} passed to argument \code{unknown} is named,
names are matched to names of \code{list} or \code{data.frame}. If lengths
of \code{unknown} and \code{list} or \code{data.frame} do not match,
recycling occurs.

The example below illustrates the application of the described functions to
a list which is composed of previously defined and modified numeric
(\code{xNum}) and factor (\code{xFac}) classes. First, function
\code{isUnknown} is used with \code{0} as an unknown value. Note that we
get \code{FALSE} for \code{NA}s as has been the case in the first example.

<<ex08>>=
(xList <- list(a=xNum, b=xFac))
isUnknown(x=xList, unknown=0)
@

We need to add \code{NA} as an unknown value. However, we do not get the
expected result this way!

<<ex09>>=
isUnknown(x=xList, unknown=c(0, NA))
@

This is due to matching of values in the argument \code{unknown} and
components in a \code{list}; i.e., \code{0} is used for component \code{a}
and \code{NA} for component \code{b}.  Therefore, it is less error prone
and more flexible to pass a \code{list} (preferably a named list) to the
argument \code{unknown}, as shown below.

<<ex10>>=
(xList1 <- unknownToNA(x=xList,
                       unknown=list(b=c(0, "NA"),
                                    a=0)))
@

Changing \code{NA}s to some other value (only one per component/column) can
be accomplished as follows:

<<ex11>>=
NAToUnknown(x=xList1,
            unknown=list(b="no", a=0))
@

A named component \code{.default} of a \code{list} passed to argument
\code{unknown} has a special meaning as it will match a component/column
with that name and any other not defined in \code{unknown}. As such it is
very useful if the number of components/columns with the same unknown
value(s) is large. Consider a wide \code{data.frame} named \code{df}. Now
\code{.default} can be used to define unknown value for several columns:

<<ex12, echo=FALSE>>=
df <- data.frame(col1=c(0, 1, 999, 2),
                 col2=c("a", "b", "c", "unknown"),
                 col3=c(0, 1, 2, 3),
                 col4=c(0, 1, 2, 2))
@

<<ex13>>=
tmp <- list(.default=0,
            col1=999,
            col2="unknown")
(df2 <- unknownToNA(x=df,
                    unknown=tmp))
@

If there is a need to work only on some components/columns you can of
course ``skip'' columns with standard \R{} mechanisms, i.e.,
by subsetting \code{list} or \code{data.frame} objects:

<<ex14>>=
df2 <- df
cols <- c("col1", "col2")
tmp <- list(col1=999,
            col2="unknown")
df2[, cols] <- unknownToNA(x=df[, cols],
                           unknown=tmp)
df2
@

\section{Summary}

Functions \code{isUnknown}, \code{unknownToNA} and \code{NAToUnknown}
provide a useful interface to work with various representations of
unknown/missing values. Their use is meant primarily for shaping the data
after importing to or before exporting from \R{}. I welcome any comments or
suggestions.

% \bibliography{refs}

\begin{thebibliography}{1}
\providecommand{\natexlab}[1]{#1}
\providecommand{\url}[1]{\texttt{#1}}
\expandafter\ifx\csname urlstyle\endcsname\relax
  \providecommand{\doi}[1]{doi: #1}\else
  \providecommand{\doi}{doi: \begingroup \urlstyle{rm}\Url}\fi

\bibitem[Gorjanc(2007)]{Gorjanc}
G.~Gorjanc.
\newblock Working with unknown values: the gdata package.
\newblock \emph{R News}, 7\penalty0 (1):\penalty0 24--26, 2007.
\newblock URL \url{http://CRAN.R-project.org/doc/Rnews/Rnews_2007-1.pdf}.

\bibitem[{R Development Core Team}(2006)]{RImportExportManual}
{R Development Core Team}.
\newblock \emph{R Data Import/Export}, 2006.
\newblock URL \url{http://cran.r-project.org/manuals.html}.
\newblock ISBN 3-900051-10-0.

\bibitem[Warnes (2006)]{WarnesGdata}
G.~R. Warnes.
\newblock \emph{gdata: Various R programming tools for data manipulation},
  2006.
\newblock URL
  \url{http://cran.r-project.org/src/contrib/Descriptions/gdata.html}.
\newblock R package version 2.3.1. Includes R source code and/or documentation
  contributed by Ben Bolker, Gregor Gorjanc and Thomas Lumley.

\end{thebibliography}

\address{Gregor Gorjanc\\
  University of Ljubljana, Slovenia\\
\email{gregor.gorjanc@bfro.uni-lj.si}}

\end{article}

\end{document}

%\VignetteIndexEntry{Working with Unknown Values}
%\VignettePackage{gdata}
%\VignetteKeywords{unknown, missing, manip}

\documentclass[a4paper]{report}
\usepackage{Rnews}
\usepackage[round]{natbib}
\bibliographystyle{abbrvnat}

\usepackage{Sweave}
\SweaveOpts{strip.white=all, keep.source=TRUE}

\begin{document}

\begin{article}

\title{Working with Unknown Values}
\subtitle{The \pkg{gdata} package}
\author{by Gregor Gorjanc}

\maketitle

This vignette has been published as \cite{Gorjanc}.

\section{Introduction}

Unknown or missing values can be represented in various ways. For example
SAS uses \code{.}~(dot), while \R{} uses \code{NA}, which we can read as
Not Available. When we import data into \R{}, say via \code{read.table} or
its derivatives, conversion of blank fields to \code{NA} (according to
\code{read.table} help) is done for \code{logical}, \code{integer},
\code{numeric} and \code{complex} classes. Additionally, the
\code{na.strings} argument can be used to specify values that should also
be converted to \code{NA}. Inversely, there is an argument \code{na} in
\code{write.table} and its derivatives to define value that will replace
\code{NA} in exported data. There are also other ways to import/export data
into \R{} as described in the {\emph R Data Import/Export} manual
\citep{RImportExportManual}.  However, all approaches lack the possibility
to define unknown value(s) for some particular column. It is possible that
an unknown value in one column is a valid value in another column. For
example, I have seen many datasets where values such as 0, -9, 999 and
specific dates are used as column specific unknown values.

This note describes a set of functions in package \pkg{gdata}\footnote{
  package version 2.3.1} \citep{WarnesGdata}: \code{isUnknown},
\code{unknownToNA} and \code{NAToUnknown}, which can help with testing for
unknown values and conversions between unknown values and \code{NA}. All
three functions are generic (S3) and were tested (at the time of writing)
to work with: \code{integer}, \code{numeric}, \code{character},
\code{factor}, \code{Date}, \code{POSIXct}, \code{POSIXlt}, \code{list},
\code{data.frame} and \code{matrix} classes.

\section{Description with examples}

The following examples show simple usage of these functions on
\code{numeric} and \code{factor} classes, where value \code{0} (beside
\code{NA}) should be treated as an unknown value:

<<ex01>>=
library("gdata")
xNum <- c(0, 6, 0, 7, 8, 9, NA)
isUnknown(x=xNum)
@

The default unknown value in \code{isUnknown} is \code{NA}, which means
that output is the same as \code{is.na} --- at least for atomic
classes. However, we can pass the argument \code{unknown} to define which
values should be treated as unknown:

<<ex02>>=
isUnknown(x=xNum, unknown=0)
@

This skipped \code{NA}, but we can get the expected answer after
appropriately adding \code{NA} into the argument \code{unknown}:

<<ex03>>=
isUnknown(x=xNum, unknown=c(0, NA))
@

Now, we can change all unknown values to \code{NA} with \code{unknownToNA}.
There is clearly no need to add \code{NA} here. This step is very handy
after importing data from an external source, where many different unknown
values might be used. Argument \code{warning=TRUE} can be used, if there is
a need to be warned about ``original'' \code{NA}s:

<<ex04>>=
(xNum2 <- unknownToNA(x=xNum, unknown=0))
@

Prior to export from \R{}, we might want to change unknown values
(\code{NA} in \R{}) to some other value. Function \code{NAToUnknown} can be
used for this:

<<ex05>>=
NAToUnknown(x=xNum2, unknown=999)
@

Converting \code{NA} to a value that already exists in \code{x} issues an
error, but \code{force=TRUE} can be used to overcome this if needed. But be
warned that there is no way back from this step:

<<ex06>>=
NAToUnknown(x=xNum2, unknown=7, force=TRUE)
@

Examples below show all peculiarities with class \code{factor}.
\code{unknownToNA} removes \code{unknown} value from levels and inversely
\code{NAToUnknown} adds it with a warning. Additionally, \code{"NA"} is
properly distinguished from \code{NA}. It can also be seen that the
argument \code{unknown} in functions \code{isUnknown} and
\code{unknownToNA} need not match the class of \code{x} (otherwise factor
should be used) as the test is internally done with \code{\%in\%}, which
nicely resolves coercing issues.

<<ex07>>=
(xFac <- factor(c(0, "BA", "RA", "BA", NA, "NA")))
isUnknown(x=xFac)
isUnknown(x=xFac, unknown=0)
isUnknown(x=xFac, unknown=c(0, NA))
isUnknown(x=xFac, unknown=c(0, "NA"))
isUnknown(x=xFac, unknown=c(0, "NA", NA))

(xFac <- unknownToNA(x=xFac, unknown=0))
(xFac <- NAToUnknown(x=xFac, unknown=0))
@

These two examples with classes \code{numeric} and \code{factor} are fairly
simple and we could get the same results with one or two lines of \R{}
code. The real benefit of the set of functions presented here is in
\code{list} and \code{data.frame} methods, where \code{data.frame} methods
are merely wrappers for \code{list} methods.

We need additional flexibility for \code{list}/\code{data.frame} methods,
due to possibly having multiple unknown values that can be different among
\code{list} components or \code{data.frame} columns. For these two methods,
the argument \code{unknown} can be either a \code{vector} or \code{list},
both possibly named. Of course, greater flexibility (defining multiple
unknown values per component/column) can be achieved with a \code{list}.

When a \code{vector}/\code{list} object passed to the argument
\code{unknown} is not named, the first value/component of a
\code{vector}/\code{list} matches the first component/column of a
\code{list}/\code{data.frame}. This can be quite error prone, especially
with \code{vectors}. Therefore, I encourage the use of a \code{list}. In
case \code{vector}/\code{list} passed to argument \code{unknown} is named,
names are matched to names of \code{list} or \code{data.frame}. If lengths
of \code{unknown} and \code{list} or \code{data.frame} do not match,
recycling occurs.

The example below illustrates the application of the described functions to
a list which is composed of previously defined and modified numeric
(\code{xNum}) and factor (\code{xFac}) classes. First, function
\code{isUnknown} is used with \code{0} as an unknown value. Note that we
get \code{FALSE} for \code{NA}s as has been the case in the first example.

<<ex08>>=
(xList <- list(a=xNum, b=xFac))
isUnknown(x=xList, unknown=0)
@

We need to add \code{NA} as an unknown value. However, we do not get the
expected result this way!

<<ex09>>=
isUnknown(x=xList, unknown=c(0, NA))
@

This is due to matching of values in the argument \code{unknown} and
components in a \code{list}; i.e., \code{0} is used for component \code{a}
and \code{NA} for component \code{b}.  Therefore, it is less error prone
and more flexible to pass a \code{list} (preferably a named list) to the
argument \code{unknown}, as shown below.

<<ex10>>=
(xList1 <- unknownToNA(x=xList,
                       unknown=list(b=c(0, "NA"),
                                    a=0)))
@

Changing \code{NA}s to some other value (only one per component/column) can
be accomplished as follows:

<<ex11>>=
NAToUnknown(x=xList1,
            unknown=list(b="no", a=0))
@

A named component \code{.default} of a \code{list} passed to argument
\code{unknown} has a special meaning as it will match a component/column
with that name and any other not defined in \code{unknown}. As such it is
very useful if the number of components/columns with the same unknown
value(s) is large. Consider a wide \code{data.frame} named \code{df}. Now
\code{.default} can be used to define unknown value for several columns:

<<ex12, echo=FALSE>>=
df <- data.frame(col1=c(0, 1, 999, 2),
                 col2=c("a", "b", "c", "unknown"),
                 col3=c(0, 1, 2, 3),
                 col4=c(0, 1, 2, 2))
@

<<ex13>>=
tmp <- list(.default=0,
            col1=999,
            col2="unknown")
(df2 <- unknownToNA(x=df,
                    unknown=tmp))
@

If there is a need to work only on some components/columns you can of
course ``skip'' columns with standard \R{} mechanisms, i.e.,
by subsetting \code{list} or \code{data.frame} objects:

<<ex14>>=
df2 <- df
cols <- c("col1", "col2")
tmp <- list(col1=999,
            col2="unknown")
df2[, cols] <- unknownToNA(x=df[, cols],
                           unknown=tmp)
df2
@

\section{Summary}

Functions \code{isUnknown}, \code{unknownToNA} and \code{NAToUnknown}
provide a useful interface to work with various representations of
unknown/missing values. Their use is meant primarily for shaping the data
after importing to or before exporting from \R{}. I welcome any comments or
suggestions.

% \bibliography{refs}

\begin{thebibliography}{1}
\providecommand{\natexlab}[1]{#1}
\providecommand{\url}[1]{\texttt{#1}}
\expandafter\ifx\csname urlstyle\endcsname\relax
  \providecommand{\doi}[1]{doi: #1}\else
  \providecommand{\doi}{doi: \begingroup \urlstyle{rm}\Url}\fi

\bibitem[Gorjanc(2007)]{Gorjanc}
G.~Gorjanc.
\newblock Working with unknown values: the gdata package.
\newblock \emph{R News}, 7\penalty0 (1):\penalty0 24--26, 2007.
\newblock URL \url{http://CRAN.R-project.org/doc/Rnews/Rnews_2007-1.pdf}.

\bibitem[{R Development Core Team}(2006)]{RImportExportManual}
{R Development Core Team}.
\newblock \emph{R Data Import/Export}, 2006.
\newblock URL \url{http://cran.r-project.org/manuals.html}.
\newblock ISBN 3-900051-10-0.

\bibitem[Warnes (2006)]{WarnesGdata}
G.~R. Warnes.
\newblock \emph{gdata: Various R programming tools for data manipulation},
  2006.
\newblock URL
  \url{http://cran.r-project.org/src/contrib/Descriptions/gdata.html}.
\newblock R package version 2.3.1. Includes R source code and/or documentation
  contributed by Ben Bolker, Gregor Gorjanc and Thomas Lumley.

\end{thebibliography}

\address{Gregor Gorjanc\\
  University of Ljubljana, Slovenia\\
\email{gregor.gorjanc@bfro.uni-lj.si}}

\end{article}

\end{document}

\name{write.fwf}
\alias{write.fwf}
\concept{data output}
\concept{data export}
\title{Write object to file in fixed width format}
\description{
  Write object to file in fixed width (fwf) format.
}
\usage{
write.fwf(x, file="", append=FALSE, quote=FALSE, sep=" ", na="",
  rownames=FALSE, colnames=TRUE, rowCol=NULL, justify="left",
  formatInfo=FALSE, quoteInfo=TRUE, width=NULL, eol="\n",
  qmethod=c("escape", "double"),  scientific=TRUE, \dots)
}
\arguments{
  \item{x}{data.frame or matrix, the object to be written.}
  \item{file}{character, name of file or connection, look in
    \code{\link{write.table}} for more.}
  \item{append}{logical, append to existing data in \code{file}.}
  \item{quote}{logical, quote data in output.}
  \item{na}{character, the string to use for missing values
    (\code{NA}) in the output.}
  \item{sep}{character, separator between columns in output.}
  \item{rownames}{logical, print row names.}
  \item{colnames}{logical, print column names.}
  \item{rowCol}{character, rownames column name.}
  \item{justify}{character, alignment of character columns, see
    \code{\link{format}}.}
  \item{formatInfo}{logical, return information on number of levels,
    widths and format.}
  \item{quoteInfo}{logical, should \code{formatInfo} account for quotes.}
  \item{width}{numeric, width of the columns in the output.}
  \item{eol}{the character(s) to print at the end of each line (row).
    For example, \code{eol="\\r\\n"} will produce Windows line endings on a
    Unix-alike OS, and \code{eol="\\r"} will produce files as expected by Mac
    OS Excel 2004.}
  \item{qmethod}{a character string specifying how to deal with embedded
    double quote characters when quoting strings. Must be one of
    \code{"escape"} (default), in which case the quote character is
    escaped in C style by a backslash, or \code{"double"}, in which
    case it is doubled. You can specify just the initial letter.}
  \item{scientific}{logical, allow numeric values to be
    formatted using scientific notation.}
  \item{\dots}{further arguments to
    \code{\link{format.info}} and \code{\link{format}}.}
}
\details{
  Output is similar to \code{print(x)} or \code{format(x)}. Formatting is
  done completely by \code{\link{format}} on a column basis. Columns in
  the output are by default separated with a space i.e. empty column with
  a width of one character, but that can be changed with \code{sep}
  argument as passed to \code{\link{write.table}} via \dots.

  As mentioned formatting is done completely by
  \code{\link{format}}. Arguments can be passed to \code{format} via
  \code{\dots} to further modify the output. However, note that the
  returned \code{formatInfo} might not properly account for this, since
  \code{\link{format.info}} (which is used to collect information about
  formatting) lacks the arguments of \code{\link{format}}.

  \code{quote} can be used to quote fields in the output. Since all
  columns of \code{x} are converted to character (via
  \code{\link{format}}) during the output, all columns will be quoted! If
  quotes are used, \code{\link{read.table}} can be easily used to read the
  data back into \R. Check examples. Do read the details about
  \code{quoteInfo} argument.

  Use only true characters, i.e., avoid use of tabs, i.e., \code{"\\t"} or
  similar separators via argument \code{sep}. Width of the separator is taken as
  the number of characters evaluated via \code{\link{nchar}(sep)}.

  Use argument \code{na} to convert missing/unknown values. Only single value
  can be specified. Use \code{\link{NAToUnknown}} prior to export if you need
  greater flexibility.

  If \code{rowCol} is not \code{NULL} and \code{rownames=TRUE}, rownames
  will also have column name with \code{rowCol} value. This is mainly for
  flexibility with tools outside \R. Note that it may not be
  easy to import data back to \R with \code{\link{read.fwf}} if
  you also export rownames. This is the reason, that default is
  \code{rownames=FALSE}.

  Information about format of output will be returned if
  \code{formatInfo=TRUE}. Returned value is described in value
  section. This information is gathered by \code{\link{format.info}} and
  care was taken to handle numeric properly. If output contains rownames,
  values account for this. Additionally, if \code{rowCol} is not
  \code{NULL} returned values contain also information about format
  of rownames.

  If \code{quote=TRUE}, the output is of course wider due to
  quotes. Return value (with \code{formatInfo=TRUE}) can account for this
  in two ways; controlled with argument \code{quoteInfo}. However, note
  that there is no way to properly read the data back to \R if
  \code{quote=TRUE} and \code{quoteInfo=FALSE} arguments were used for
  export. \code{quoteInfo} applies only when \code{quote=TRUE}. Assume
  that there is a file with quoted data as shown below (column numbers in
  first three lines are only for demonstration of the values in the
  output).

\preformatted{
123456789 12345678 # for position
123 1234567 123456 # for width with quoteInfo=TRUE
 1   12345   1234  # for width with quoteInfo=FALSE
"a" "hsgdh" "   9"
" " "   bb" " 123"
}

  With \code{quoteInfo=TRUE} \code{write.fwf} will return

\preformatted{
colname position width
V1             1     3
V2             5     7
V3            13     6
}

  or (with \code{quoteInfo=FALSE})

\preformatted{
colname position width
V1             2     1
V2             6     5
V3            14     4
}

  Argument \code{width} can be used to increase the width of the columns
  in the output. This argument is passed to the width argument of
  \code{\link{format}} function. Values in \code{width} are recycled if
  there is less values than the number of columns. If the specified width
  is too short in comparison to the "width" of the data in particular
  column, error is issued.
}
\value{
  Besides its effect to write/export data \code{write.fwf} can provide
  information on format and width. If \code{formatInfo = FALSE}, then a
  data frame is returned with the following columns:
  \item{colname}{name of the column}
  \item{nlevels}{number of unique values (unused levels of factors are
    dropped), 0 for numeric column}
  \item{position}{starting column number in the output}
  \item{width}{width of the column}
  \item{digits}{number of digits after the decimal point}
  \item{exp}{width of exponent in exponential representation; 0 means
    there is no exponential representation, while 1 represents exponent
    of length one i.e. \code{1e+6} and 2 \code{1e+06} or \code{1e+16}}
}
\author{Gregor Gorjanc.}
\seealso{
  \code{\link{format.info}}, \code{\link{format}},
  \code{\link{NAToUnknown}}, \code{\link{write.table}},
  \code{\link{read.fwf}}, \code{\link{read.table}} and
  \code{\link{trim}}.
}
\examples{
## Some data
num <- round(c(733070.345678, 1214213.78765456, 553823.798765678,
               1085022.8876545678,  571063.88765456, 606718.3876545678,
               1053686.6, 971024.187656, 631193.398765456, 879431.1),
             digits=3)

testData <- data.frame(num1=c(1:10, NA),
                       num2=c(NA, seq(from=1, to=5.5, by=0.5)),
                       num3=c(NA, num),
                       int1=c(as.integer(1:4), NA, as.integer(4:9)),
                       fac1=factor(c(NA, letters[1:9], "hjh")),
                       fac2=factor(c(letters[6:15], NA)),
                       cha1=c(letters[17:26], NA),
                       cha2=c(NA, "longer", letters[25:17]),
                       stringsAsFactors=FALSE)
levels(testData$fac1) <- c(levels(testData$fac1), "unusedLevel")
testData$Date <- as.Date("1900-1-1")
testData$Date[2] <- NA
testData$POSIXt <- as.POSIXct(strptime("1900-1-1 01:01:01",
                                       format="\%Y-\%m-\%d \%H:\%M:\%S"))
testData$POSIXt[5] <- NA

## Default
write.fwf(x=testData)

## NA should be -
write.fwf(x=testData, na="-")
## NA should be -NA-
write.fwf(x=testData, na="-NA-")

## Some other separator than space
write.fwf(x=testData[, 1:4], sep="-mySep-")

## Force wider columns
write.fwf(x=testData[, 1:5], width=20)

## Show effect of 'scientific' option
testData$num3 <- testData$num3 * 1e8
write.fwf(testData, scientific=TRUE)
write.fwf(testData, scientific=FALSE)
testData$num3 <- testData$num3 / 1e8

## Write to file and report format and fixed width information
file <- tempfile()
formatInfo <- write.fwf(x=testData, file=file, formatInfo=TRUE)
formatInfo

## Read exported data back to R (note +1 due to separator)
## - without header
read.fwf(file=file, widths=formatInfo$width + 1, header=FALSE, skip=1,
         strip.white=TRUE)

## - with header, via postimport modfication
tmp <- read.fwf(file=file, widths=formatInfo$width + 1, skip=1,
                strip.white=TRUE)
colnames(tmp) <- read.table(file=file, nrow=1, as.is=TRUE)
tmp

## - with header, persuading read.fwf to accept header properly
## (thanks to Marc Schwartz)
read.fwf(file=file, widths=formatInfo$width + 1, strip.white=TRUE,
         skip=1, col.names=read.table(file=file, nrow=1, as.is=TRUE))

## - with header, using quotes
write.fwf(x=testData, file=file, quote=TRUE)
read.table(file=file, header=TRUE, strip.white=TRUE)

## Tidy up
unlink(file)
}
\keyword{print}
\keyword{file}

\name{nobs}
\alias{nobs}
\alias{n_obs}
\alias{nobs.data.frame}
\alias{nobs.default}
\alias{nobs.lm}
\title{Compute the Number of Non-Missing Observations}
\description{
  Compute the number of non-missing observations. Provides a new default
  method to handle numeric and logical vectors, and a method for data frames.
}
\usage{
nobs(object, \dots)
\method{nobs}{default}(object, \dots)
\method{nobs}{data.frame}(object, \dots)
\method{nobs}{lm}(object, \dots)
n_obs(object, \dots)
}
\arguments{
  \item{object}{Numeric or logical vector, data frame, or a model object.}
  \item{\dots}{Further arguments to be passed to methods.}
}
\value{
  Either single numeric value (for vectors) or a vector of numeric
  values (for data frames) giving the number of non-missing values.
}
\note{
  The base R package \code{stats} provides a generic \code{nobs}
  function with methods for fitted model objects. The \code{gdata}
  package adds methods for numeric and logical vectors, as well as data
  frames.

  An alias function \code{n_obs} is also provided, equivalent to
  \code{gdata::nobs}. Using \code{n_obs} in scripts makes it explicitly
  clear that the \code{gdata} implementation is being used.
}
\author{Gregory R. Warnes \email{greg@warnes.net}}
\seealso{
  \code{\link[stats]{nobs}} in package 'stats' for the base R implementation,
  \code{\link{is.na}}, \code{\link{length}}
}
\examples{
x <- c(1, 2, 3, 5, NA, 6, 7, 1, NA)
length(x)
nobs(x)

df <- data.frame(x=rnorm(100), y=rnorm(100))
df[1,1] <- NA
df[1,2] <- NA
df[2,1] <- NA
nobs(df)

fit <- lm(y~x, data=df)
nobs(fit)
n_obs(fit)

# Comparison
# gdata
nobs(x)
nobs(df)
# stats
length(na.omit(x))
sapply(df, function(x) length(na.omit(x)))
}
\keyword{attribute}

% bindData.Rd
%--------------------------------------------------------------------------
% What: Bind two data frames - help
% $Id$
% Time-stamp: <2008-12-30 13:49:50 ggorjan>
%--------------------------------------------------------------------------

\name{bindData}
\alias{bindData}

\title{Bind two data frames into a multivariate data frame}

\description{
  Usually data frames represent one set of variables and one needs to 
  bind/join them for multivariate analysis. When \code{\link{merge}} is not
  the approriate solution, \code{bindData} might perform an appropriate binding
  for two data frames. This is especially usefull when some variables are
  measured once, while others are repeated.
}

\usage{
  bindData(x, y, common)
}

\arguments{
  \item{x}{data.frame}
  \item{y}{data.frame}
  \item{common}{character, list of column names that are common to both
    input data frames}
}

\details{
  Data frames are joined in a such a way, that the new data frame has
  \eqn{c + (n_1 - c) + (n_2 - c)} columns, where \eqn{c} is the number of
  common columns, and \eqn{n_1} and \eqn{n_2} are the number of columns 
  in the first and in the second data frame, respectively.
}

\value{
  A data frame.
}

\author{Gregor Grojanc}

\seealso{
  \code{\link[base]{merge}},
  \code{\link{wideByFactor}}
}

\examples{
n1 <- 6
n2 <- 12
n3 <- 4
## Single trait 1
num <- c(5:n1, 10:13)
(tmp1 <- data.frame(y1=rnorm(n=n1),
                    f1=factor(rep(c("A", "B"), n1/2)),
                    ch=letters[num],
                    fa=factor(letters[num]),
                    nu=(num) + 0.5,
                    id=factor(num), stringsAsFactors=FALSE))

## Single trait 2 with repeated records, some subjects also in tmp1 
num <- 4:9
(tmp2 <- data.frame(y2=rnorm(n=n2),
                    f2=factor(rep(c("C", "D"), n2/2)),
                    ch=letters[rep(num, times=2)],
                    fa=factor(letters[rep(c(num), times=2)]),
                    nu=c((num) + 0.5, (num) + 0.25),
                    id=factor(rep(num, times=2)), stringsAsFactors=FALSE))

## Single trait 3 with completely distinct set of subjects
num <- 1:4
(tmp3 <- data.frame(y3=rnorm(n=n3),
                    f3=factor(rep(c("E", "F"), n3/2)),
                    ch=letters[num],
                    fa=factor(letters[num]),
                    nu=(num) + 0.5,
                    id=factor(num), stringsAsFactors=FALSE))

## Combine all datasets
(tmp12 <- bindData(x=tmp1, y=tmp2, common=c("id", "nu", "ch", "fa")))
(tmp123 <- bindData(x=tmp12, y=tmp3, common=c("id", "nu", "ch", "fa")))

## Sort by subject
tmp123[order(tmp123$ch), ]
}

\keyword{manip}
\keyword{misc}

%--------------------------------------------------------------------------
% bindData.Rd ends here

\name{wideByFactor}
\alias{wideByFactor}
\title{Create multivariate data by a given factor}
\description{
  Modify data frame in such a way that variables are
  \dQuote{separated} into several columns by factor levels.
}
\usage{
  wideByFactor(x, factor, common, sort=TRUE, keepFactor=TRUE)
}
\arguments{
  \item{x}{data frame}
  \item{factor}{character, column name of a factor by which variables will
    be divided}
  \item{common}{character, column names of (common) columns that should not
    be divided}
  \item{sort}{logical, sort resulting data frame by factor levels}
  \item{keepFactor}{logical, keep the \sQuote{factor} column}
}
\details{
  Given data frame is modified so that the output represents a data
  frame with \eqn{c + f + n * v} columns, where \eqn{c} is a number of common
  columns for all levels of a factor, \eqn{f} is a factor column, \eqn{n} is a
  number of levels in factor \eqn{f} and \eqn{v} is a number of variables that
  should be divided for each level of a factor. Number of rows stays the same.
}
\value{
  A data frame where divided variables have sort of \dQuote{diagonalized}
  structure.
}
\author{Gregor Gorjanc}
\seealso{
  \code{\link[stats]{reshape}} in the \pkg{stats} package.
}
\examples{
n <- 10
f <- 2
tmp <- data.frame(y1=rnorm(n=n),
                  y2=rnorm(n=n),
                  f1=factor(rep(letters[1:f], n/2)),
                  f2=factor(c(rep("M", n/2), rep("F", n/2))),
                  c1=1:n,
                  c2=2*(1:n))

wideByFactor(x=tmp, factor="f1", common=c("c1", "c2", "f2"))
wideByFactor(x=tmp, factor="f1", common=c("c1", "c2"))
}
\keyword{manip}
\keyword{misc}

\name{mv}
\alias{mv}
\title{
  Rename an Object
}
\description{
  Rename an object.
}
\usage{
mv(from, to, envir = parent.frame())
}
\arguments{
  \item{from}{Character scalar giving the source object name}
  \item{to}{Character scalar giving the desination object name}
  \item{envir}{Environment in which to do the rename}
}
\details{
  This function renames an object by the value of object \code{a} to the
  name \code{b}, and removing \code{a}.
}
\value{
  Invisibly returns the value of the object.
}
\author{
  Gregory R. Warnes \email{greg@warnes.net}
}
\seealso{
  \code{\link[base]{rm}}, \code{\link[base]{assign}}
}
\examples{
a <- 1:10
a
mv("a", "b")
b
exists("a")
}
\keyword{environment}
\keyword{data}

\name{centerText}
\alias{centerText}
\title{
Center Text Strings 
}
\description{
Function to center text strings for display on the text console
by prepending the necessary number of spaces to each element.
}
\usage{
centerText(x, width = getOption("width"))
}
\arguments{
  \item{x}{Character vector containing text strings to be centered.}
  \item{width}{Desired display width. Defaults to the R display width
    given by \code{getOption("width")}. }
}
\details{
    Each element will be centered individually by prepending the
    necessary number of spaces to center the text in the specified
    display width assuming a fixed width font.
}
\value{
Vector of character strings.
}
\author{
Gregory R. Warnes \email{greg@warnes.net}
}
\seealso{
  \code{\link[base]{strwrap}}
}
\examples{
cat(centerText("One Line Test"), "\n\n")

mText <-c("This", "is an example",
          " of a multiline text    ",
          "with ",
          "          leading",
          " and trailing         ",
          "spaces.")
cat("\n", centerText(mText), "\n", sep="\n")
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{manip}
\keyword{character}

\name{xlsFormats}
\Rdversion{1.1}
\alias{xlsFormats}
\title{
Check which file formats are supported by read.xls
}
\description{
Check which file formats are supported by read.xls
}
\usage{
xlsFormats(perl = "perl", verbose = FALSE)
}
\arguments{
  \item{perl}{Path to perl interpreter (optional).}
  \item{verbose}{If \code{TRUE}, show additional messages during
    processing.}
}
\value{
  Vector of supported formats, possible elements are 'XLS' and 'XLSX'.
}

\seealso{
	\code{\link{read.xls}}, \code{\link{xls2csv}}.
}
\examples{
   xlsFormats()
}
\keyword{ misc }

\name{matchcols}
\alias{matchcols}
\title{Select columns names matching certain critera}
\description{
  This function allows easy selection of the column names of an object
  using a set of inclusion and exclusion critera.
}
\usage{
matchcols(object, with, without, method=c("and","or"), ...)
}
\arguments{
  \item{object}{Matrix or dataframe}
  \item{with, without}{Vector of regular expression patterns}
  \item{method}{One of "and" or "or"}
  \item{\dots}{Optional arguments to \code{grep}}
}
\value{
  Vector of column names which match all (\code{method="and"}) or any
  (\code{method="or"}) of the patterns specified in \code{with}, but
  none of the patterns specified in \code{without}.
}

\author{Gregory R. Warnes \email{greg@warnes.net}}
\seealso{ \code{\link[base]{grep}} }
\examples{

# create a matrix with a lot of named columns
x <- matrix( ncol=30, nrow=5 )
colnames(x) <- c("AffyID","Overall Group Means: Control",
                     "Overall Group Means: Moderate",
                     "Overall Group Means: Marked",
                     "Overall Group Means: Severe",
                     "Overall Group StdDev: Control",
                     "Overall Group StdDev: Moderate",
                     "Overall Group StdDev: Marked",
                     "Overall Group StdDev: Severe",
                     "Overall Group CV: Control",
                     "Overall Group CV: Moderate",
                     "Overall Group CV: Marked",
                     "Overall Group CV: Severe",
                     "Overall Model P-value",
                     "Overall Model: (Intercept): Estimate",
                     "Overall Model: Moderate: Estimate",
                     "Overall Model: Marked: Estimate",
                     "Overall Model: Severe: Estimate",
                     "Overall Model: (Intercept): Std. Error",
                     "Overall Model: Moderate: Std. Error",
                     "Overall Model: Marked: Std. Error",
                     "Overall Model: Severe: Std. Error",
                     "Overall Model: (Intercept): t value",
                     "Overall Model: Moderate: t value",
                     "Overall Model: Marked: t value",
                     "Overall Model: Severe: t value",
                     "Overall Model: (Intercept): Pr(>|t|)",
                     "Overall Model: Moderate: Pr(>|t|)",
                     "Overall Model: Marked: Pr(>|t|)",
                     "Overall Model: Severe: Pr(>|t|)")

# Get the columns which give estimates or p-values
# only for marked and severe groups
matchcols(x, with=c("Pr", "Std. Error"),
             without=c("Intercept","Moderate"),
             method="or"
          )

# Get just the column which give the p-value for the intercept
matchcols(x, with=c("Intercept", "Pr") )
                     
}
\keyword{manip}

\name{duplicated2}
\alias{duplicated2}
\title{Determine Duplicate Elements}
\description{
  \code{duplicated2()} determines which elements of a vector or data 
  frame are duplicates, and returns a logical vector indicating which 
  elements (rows) are duplicates.
}
\usage{
duplicated2(x, bothWays=TRUE, ...)
}
\arguments{
  \item{x}{a vector or a data frame or an array or \code{NULL}.}
  \item{bothWays}{if \code{TRUE} (the default), duplication should be 
    considered from both sides. For more information see the argument \code{fromLast}
    to the function \code{\link{duplicated}}.}
  \item{\dots}{further arguments passed down to \code{duplicated()} and
    its methods.}
}
\details{
  The standard \code{\link{duplicated}} function (in \code{package:base}) 
  only returns \code{TRUE} for the second and following copies of each
  duplicated value (second-to-last and earlier when
  \code{fromLast=TRUE}).  This function returns all duplicated
  elementes, including the first (last) value.
  
  When \code{bothWays} is \code{FALSE}, \code{duplicated2()} defaults to 
  a \code{\link{duplicated}} call. When \code{bothWays} is \code{TRUE}, 
  the following call is being executed: 
  \code{duplicated(x, ...) | duplicated(x, fromLast=TRUE, ...)}
}
\value{
    For a vector input, a logical vector of the same length as
    \code{x}.  For a data frame, a logical vector with one element for
    each row.  For a matrix or array, and when \code{MARGIN = 0}, a
    logical array with the same dimensions and dimnames.
    
    For more details see \code{\link{duplicated}}.
}
\seealso{
  \code{\link[base]{duplicated}}, \code{\link[base]{unique}}
}
\author{Liviu Andronic}
\examples{
  data(iris)
  iris[duplicated(iris), ]                 # 2nd duplicated value
  iris[duplicated(iris, fromLast=TRUE), ]  # 1st duplicated value
  iris[duplicated2(iris), ]                # both duplicated values
}
\keyword{logic}
\keyword{manip}

\name{Args}
\alias{Args}
\title{Describe Function Arguments}
\description{
  Display function argument names and corresponding default values,
  formatted in two columns for easy reading.
}
\usage{
Args(name, sort=FALSE)
}
\arguments{
  \item{name}{a function or function name.}
  \item{sort}{whether arguments should be sorted.}
}
\value{
  A data frame with named rows and a single column called \code{value},
  containing the default value of each argument.
}
\note{
  Primitive functions like \code{sum} and \code{all} have no formal
  arguments. See the \code{\link{formals}} help page.
}
\author{Arni Magnusson}
\seealso{
  \code{Args} is a verbose alternative to \code{\link{args}}, based on
  \code{\link{formals}}.

  \code{\link{help}} also describes function arguments.
}
\examples{
Args(glm)
Args(scan)
Args(legend, sort=TRUE)
}
% Programming
\keyword{programming}
\keyword{utilities}
\keyword{documentation}

\name{startsWith}
\alias{startsWith}
\alias{starts_with}
\title{Does String Start or End With Another String?}
\description{
  Determines if entries of \code{x} start with a string \code{prefix},
  where strings are recycled to common lengths.
}
\usage{
startsWith(x, prefix, trim=FALSE, ignore.case=FALSE)
}
\arguments{
  \item{x}{character vector whose \dQuote{starts} are considered.}
  \item{prefix}{character vector, typicall of length one, i.e., a string.}
  \item{trim}{whether leading and trailing spaces should be removed from
    \code{x} before testing for a match.}
  \item{ignore.case}{whether case should be ignored when testing for a
    match.}
}
\value{
  A logical vector, of \dQuote{common length} of \code{x} and
  \code{prefix}, i.e., of the longer of the two lengths unless one of
  them is zero when the result is also of zero length.  A shorter input
  is recycled to the output length.
}
\note{
  The \code{base} package provides the underlying \code{startsWith}
  function that performs the string comparison. The \code{gdata} package
  adds the \code{trim} and \code{ignore.case} features.

  An alias function \code{starts_with} is also provided, equivalent to
  \code{gdata::startsWith}. Using \code{starts_with} in scripts makes it
  explicitly clear that the \code{gdata} implementation is being used.
}
\author{
  Gregory R. Warnes \email{greg@warnes.net}
}
\seealso{
  \code{\link[base]{startsWith}} for the 'base' package implementation,
  \code{\link[base]{grepl}}, \code{\link{substring}}
}
\examples{
## Simple example
startsWith("Testing", "Test")

## Vector examples
s <- c("Testing", " Testing", "testing", "Texting")
names(s) <- s

startsWith(s, "Test")        # " Testing", "testing", and "Texting" do not match
startsWith(s, "Test", trim=TRUE)        # Now " Testing" matches
startsWith(s, "Test", ignore.case=TRUE) # Now "testing" matches

# Comparison
# gdata
startsWith(s, "Test", trim=TRUE)
startsWith(s, "Test", ignore.case=TRUE)
# base
startsWith(trimws(s), "Test")
startsWith(tolower(s), tolower("Test"))
}
\keyword{character}

\name{resample}
\alias{resample}
\title{Consistent Random Samples and Permutations}
\description{
  \code{resample} takes a sample of the specified size from the
  elements of \code{x} using either with or without replacement.

}
\usage{
resample(x, size, replace = FALSE, prob = NULL)
}
\arguments{
  \item{x}{ A numeric, complex, character or logical vector from which
    to choose.}
  \item{size}{Non-negative integer giving the number of items to choose.}
  \item{replace}{Should sampling be with replacement?}
  \item{prob}{A vector of probability weights for obtaining the elements of
    the vector being sampled.}
  }
\details{
  \code{resample} differs from the S/R \code{sample} function in
  \code{resample} always considers \code{x} to be a vector of elements
  to select from, while \code{sample} treats a vector of length one as a
  special case and samples from \code{1:x}.  Otherwise, the functions
  have identical behavior.
}
\value{
  Vector of the same length as the input, with the elements permuted.
}
\author{Gregory R. Warnes \email{greg@warnes.net} }
\seealso{ \code{\link{sample}} }
\examples{

## sample behavior differs if first argument is scalar vs vector
sample( c(10) )
sample( c(10,10) )

## resample has the consistent behavior for both cases
resample( c(10) )
resample( c(10,10) )

}
\keyword{misc}

\name{combine}
\alias{combine}
\title{Combine R Objects With a Column Labeling the Source}
\description{
  Take a sequence of vector, matrix or data frames and
  combine into rows of a common data frame with an additional column
  \code{source} indicating the source object.
}
\usage{
combine(\dots, names=NULL)
}
\arguments{
  \item{\dots}{vectors or matrices to combine.}
  \item{names}{character vector of names to use when creating source column.}
}
\details{
  If there are several matrix arguments, they must all have the same
  number of columns.  The number of columns in the result will be one
  larger than the number of columns in the component matrixes. If all
  of the arguments are vectors, these are treated as single column
  matrixes.  In this case, the column containing the combineinated
  vector data is labeled \code{data}.

  When the arguments consist of a mix of matrices and vectors the
  number of columns of the result is determined by the number
  of columns of the matrix arguments.  Vectors are considered row
  vectors and have their values recycled or subsetted (if necessary)
  to achieve this length.

  The \code{source} column is created as a factor with levels
  corresponding to the name of the object from which the each row was
  obtained.  When the \code{names} argument is ommitted, the name of
  each object is obtained from the specified argument name in the
  call (if present) or from the name of the object.  See below for
  examples.
}
\author{Gregory R. Warnes \email{greg@warnes.net}}
\seealso{\code{\link{rbind}}, \code{\link{merge}}}
\examples{
a <- matrix(rnorm(12),ncol=4,nrow=3)
b <- 1:4
combine(a,b)

combine(x=a,b)
combine(x=a,y=b)
combine(a,b,names=c("one","two"))

c <- 1:6
combine(b,c)
}
\keyword{array}
\keyword{manip}

\name{unmatrix}
\alias{unmatrix}
\title{Convert a matrix into a vector, with appropriate names}
\description{
  Convert a matrix into a vector, with element names constructed from
  the row and column names of the matrix.
}
\usage{
unmatrix(x, byrow=FALSE)
}
\arguments{
  \item{x}{matrix}
  \item{byrow}{Logical. If \code{FALSE}, the elements within columns will be
    adjacent in the resulting vector, otherwise elements within rows
    will be adjacent.}
}
\value{
  A vector with names constructed from the row and column names from the
  matrix. If the the row or column names are missing, ('r1', 'r2', ..,) or
  ('c1', 'c2', ..) will be used as appropriate.
}
\author{Gregory R. Warnes \email{greg@warnes.net} }
\seealso{ \code{\link[base]{as.vector}} }
\examples{
# simple, useless example
m <- matrix( letters[1:10], ncol=5)
m
unmatrix(m)

# unroll model output
x <- rnorm(100)
y <- rnorm(100, mean=3+5*x, sd=0.25)
m <- coef( summary(lm( y ~ x )) )
unmatrix(m)
}
\keyword{manip}

\name{interleave}
\alias{interleave}
\title{Interleave Rows of Data Frames or Matrices}
\description{
  Interleave rows of data frames or matrices.
}
\usage{
interleave(\dots, append.source=TRUE, sep=": ", drop=FALSE)
}
\arguments{
  \item{\dots}{objects to be interleaved.}
  \item{append.source}{boolean flag. When \code{TRUE} (the default) the
    argument name will be appended to the row names to show the source of
    each row.}
  \item{sep}{separator between the original row name and the object name.}
  \item{drop}{boolean flag - when TRUE, matrices containing one column
    will be converted to vectors.}
}
\details{
  This function creates a new matrix or data frame from its arguments.

  The new object will have all of the rows from the source objects
  interleaved. Starting with row 1 of object 1, followed by row 1
  of object 2, \ldots, row 1 of object 'n', row 2 of object 1, row 2 of
  object 2, \ldots, row 2 of object 'n', etc.
}
\value{
  Matrix containing the interleaved rows of the function arguments.
}
\author{Gregory R. Warnes \email{greg@warnes.net}}
\seealso{\code{\link{cbind}}, \code{\link{rbind}}, \code{\link{combine}}}
\examples{
# Simple example
a <- matrix(1:10,ncol=2,byrow=TRUE)
b <- matrix(letters[1:10],ncol=2,byrow=TRUE)
c <- matrix(LETTERS[1:10],ncol=2,byrow=TRUE)
interleave(a,b,c)

# Create a 2-way table of means, standard errors, and nobs
g1 <- sample(letters[1:5], 1000, replace=TRUE)
g2 <- sample(LETTERS[1:3], 1000, replace=TRUE)
dat <- rnorm(1000)

stderr <- function(x) sqrt(var(x,na.rm=TRUE) / nobs(x))

means   <- tapply(dat, list(g1, g2), mean)
stderrs <- tapply(dat, list(g1, g2), stderr)
ns      <- tapply(dat, list(g1, g2), nobs)
blanks <- matrix(" ", nrow=5, ncol=3)

tab <- interleave("Mean"=round(means,2),
                  "Std Err"=round(stderrs,2),
                  "N"=ns, " "=blanks, sep=" ")
print(tab, quote=FALSE)

# Using drop to control coercion to a lower dimensions
m1 <- matrix(1:4)
m2 <- matrix(5:8)

interleave(m1, m2, drop=TRUE)   # this will be coerced to a vector
interleave(m1, m2, drop=FALSE)  # this will remain a matrix
}
\keyword{category}
\keyword{array}

\name{MedUnits}
\alias{MedUnits}
\docType{data}
\title{
  Table of conversions between Intertional
  Standard (SI) and US 'Conventional' Units for common medical
  measurements.
}
\description{
  Table of conversions between Intertional
  Standard (SI) and US 'Conventional' Units for common medical
  measurements.
}
\usage{data(MedUnits)}
\format{
  A data frame with the following 5 variables.
  \describe{
    \item{Abbreviation}{Common Abbreviation (mostly missing)}
    \item{Measurement}{Measurement Name}
    \item{ConventionalUnit}{Conventional Unit}
    \item{Conversion}{Conversion factor}
    \item{SIUnit}{SI Unit}
  }
}
\details{

  Medical laboratories and practitioners in the United States use one
  set of units (the so-called 'Conventional' units) for reporting the
  results of clinical laboratory measurements, while the rest of the
  world uses the International Standard (SI) units.  It often becomes
  necessary to translate between these units when participating in
  international collaborations.

  This data set provides constants for converting between SI and
  US 'Conventional' units.

  To perform the conversion from SI units to US 'Conventional' units do:

     Measurement in \code{ConventionalUnit} =
     (Measurement in \code{SIUnit}) /  \code{Conversion}

  To perform conversion from 'Conventional' to SI units do:

     Measurement in \code{SIUnit} =
     (Measurement in \code{ConventionalUnit}) * \code{Conversion}

}
\source{
  \url{https://globalrph.com/medical/conventional-units-international-units/}
}
\seealso{
  The function \code{\link{ConvertMedUnits}} automates the
  conversion task.
}
\examples{

data(MedUnits)

# show available conversions
MedUnits$Measurement

# utility function
matchUnits <- function(X) MedUnits[ grep(X, MedUnits$Measurement),]

# Convert SI Glucose measurement to 'Conventional' units
GlucoseSI = c(5, 5.4, 5, 5.1, 5.6, 5.1, 4.9, 5.2, 5.5) # in SI Units
GlucoseUS = GlucoseSI / matchUnits("Glucose")$Conversion 
cbind(GlucoseSI,GlucoseUS)

# also consider using ConvertMedUnits()
ConvertMedUnits( GlucoseSI, "Glucose", to="US" )
}
\keyword{datasets}

\name{keep}
\alias{keep}
\title{Remove All Objects, Except Those Specified}
\description{
  Remove all objects from the user workspace, except those specified.
}
\usage{
keep(\dots, list=character(), all=FALSE, sure=FALSE)
}
\arguments{
  \item{\dots}{objects to be kept, specified one by one, quoted or
    unquoted.}
  \item{list}{character vector of object names to be kept.}
  \item{all}{whether hidden objects (beginning with a \code{.}) should
    be removed, unless explicitly kept.}
  \item{sure}{whether to perform the removal, otherwise return names of
    objects that would be removed.}
}
\details{
  Implemented with safety caps: objects whose name starts with a
  \code{.} are not removed unless \code{all=TRUE}, and an explicit
  \code{sure=TRUE} is required to remove anything.
}
\value{
  A character vector containing object names that are deleted if
  \code{sure=TRUE}.
}
\author{Arni Magnusson}
\seealso{
  \code{keep} is a convenient interface to \code{\link{rm}} for removing
  most objects from the user workspace.
}
\examples{
data(trees, CO2)
keep(trees)
# To remove all objects except trees, run:
# keep(trees, sure=TRUE)
}
\keyword{data}
\keyword{environment}
\keyword{utilities}

\name{object_size}
\alias{object_size}
\alias{c.object_sizes}
\alias{as.object_sizes}
\alias{is.object_sizes}
\alias{format.object_sizes}
\alias{print.object_sizes}
\alias{object.size}
\title{Report the Space Allocated for Objects}
\description{
  Provides an estimate of the memory that is being used to store \R objects.
}
\usage{
object_size(\dots)

\method{is}{object_sizes}(x)

\method{as}{object_sizes}(x)

\method{c}{object_sizes}(\dots, recursive=FALSE)

\method{format}{object_sizes}(x, humanReadable=getOption("humanReadable"),
       standard="IEC", units, digits=1, width=NULL, sep=" ",
       justify=c("right", "left"), \dots)

\method{print}{object_sizes}(x, quote=FALSE,
      humanReadable=getOption("humanReadable"), standard="IEC", units, digits=1,
      width=NULL, sep=" ", justify=c("right", "left"), \dots)
}
\arguments{
  \item{\dots}{\code{object_size}: \R objects;
    \code{print} and \code{format}: arguments to be passed to other methods.}
  \item{x}{output from \code{object_size}.}
  \item{quote}{whether or not the result should be printed with
    surrounding quotes.}
  \item{humanReadable}{whether to use the \dQuote{human readable} format.}
  \item{standard,units,digits,width,sep,justify}{passed to
    \code{\link{humanReadable}}.}
  \item{recursive}{passed to the \code{\link[base]{c}} method.}
}
\value{
  A numeric vector class \code{c("object_sizes", "numeric")} containing
  estimated memory allocation attributable to the objects in bytes.
}
\details{
  The base R package \code{utils} provides an \code{object.size}
  function that handles a single object. The \code{gdata} package
  provides a similar \code{object_size} function that handles multiple
  objects.

  Both the \code{utils} and \code{gdata} implementations store the
  object size in bytes, but offer a slightly different user interface
  for showing the object size in other formats. The \code{gdata}
  implementation offers human readable format similar to \code{ls},
  \code{df} or \code{du} shell commands, by calling \code{humanReadable}
  directly, calling \code{print} with the argument
  \code{humanReadable=TRUE}, or by setting
  \code{options(humanReadable=TRUE)}.

  The 3.0.0 release of \code{gdata} renamed this function to
  \code{object_size}, allowing users to explicitly call the \code{gdata}
  implementation. This eliminates ambiguity and makes it less likely to
  get errors when running parts of an existing script without first
  loading the \code{gdata} package. The old \code{object.size} function
  name is now deprecated in the \code{gdata} package, pointing users to
  \code{object_size} and \code{utils::gdata} instead.
}
\seealso{
  \code{\link[utils]{object.size}} in package 'utils' for the base R
  implementation,
  \code{\link{Memory-limits}} for the design limitations on object size,
  \code{\link{humanReadable}} for human readable format.
}
\examples{
object_size(letters)
object_size(ls)

# Find the 10 largest objects in the base package
allObj <- sapply(ls("package:base"), function(x)
  object_size(get(x, envir=baseenv())))
(bigObj <- as.object_sizes(rev(sort(allObj))[1:10]))
print(bigObj, humanReadable=TRUE)

as.object_sizes(14567567)

oldopt <- options(humanReadable=TRUE)
(z <- object_size(letters,
                  c(letters, letters),
                  rep(letters, 100),
                  rep(letters, 10000)))
is.object_sizes(z)
as.object_sizes(14567567)
options(oldopt)

# Comparison
# gdata
print(object_size(loadNamespace), humanReadable=TRUE)
print(bigObj, humanReadable=TRUE)
# utils
print(utils::object.size(loadNamespace), units="auto")
sapply(bigObj, utils:::format.object_size, units="auto")
# ll
ll("package:base")[order(-ll("package:base")$KB)[1:10],]
}
\keyword{utilities}

\name{env}
\alias{env}
\title{Describe All Loaded Environments}
\description{
  Display name, number of objects, and size of all loaded environments.
}
\usage{
env(unit="KB", digits=0)
}
\arguments{
  \item{unit}{unit for displaying environment size: "bytes", "KB", "MB",
    or first letter.}
  \item{digits}{number of decimals to display when rounding environment
    size.}
}
\value{
  A data frame with the following columns:
  \item{Environment}{environment name.}
  \item{Objects}{number of objects in environment.}
  \item{KB}{environment size \emph{(see notes)}.}
}
\note{
  The name of the environment size column is the same as the unit used.
}
\author{Arni Magnusson}
\seealso{
  \code{env} is a verbose alternative to \code{\link{search}}.

  \code{\link{ll}} is a related function that describes objects in an
  environment.
}
\examples{
\dontrun{
env()
}
}
% Basics
\keyword{data}
% Programming
\keyword{environment}
\keyword{utilities}

\name{nPairs}
\alias{nPairs}
\concept{pairs}
\title{Number of variable pairs}
\description{
  Count the number of pairs between variables.
}
\usage{
nPairs(x, margin=FALSE, names=TRUE, abbrev=TRUE, \dots)
}
\arguments{
  \item{x}{data.frame or a matrix}
  \item{margin}{logical, calculate the cumulative number of \dQuote{pairs}}
  \item{names}{logical, add row/col-names to the output}
  \item{abbrev}{logical, abbreviate names}
  \item{\dots}{other arguments passed to \code{\link{abbreviate}}}
}
\details{
  The class of returned matrix is nPairs and matrix. There is a summary method,
  which shows the opposite information - counts how many times each variable is
  known, while the other variable of a pair is not. See examples.
}
\value{
  Matrix of order \eqn{k}, where \eqn{k} is the number of columns in \code{x}.
  Values in a matrix represent the number of pairs between columns/variables in
  \code{x}. If \code{margin=TRUE}, the number of columns is \eqn{k+1} and the
  last column represents the cumulative number of pairing all variables.
}
\author{Gregor Gorjanc}
\seealso{\code{\link{abbreviate}}}
\examples{
# Test data
test <- data.frame(V1=c(1, 2, 3, 4, 5),
                   V2=c(NA, 2, 3, 4, 5),
                   V3=c(1, NA, NA, NA, NA),
                   V4=c(1, 2, 3, NA, NA))

# Number of variable pairs
nPairs(x=test)

# Without names
nPairs(x=test, names=FALSE)

# Longer names
colnames(test) <- c("Variable1", "Variable2", "Variable3", "Variable4")
nPairs(x=test)

# Margin
nPairs(x=test, margin=TRUE)

# Summary
summary(object=nPairs(x=test))
}
\keyword{misc}

\name{elem}
\alias{elem}
\title{
  Display Information about Elements in a Given Object
}
\description{
  \emph{This function is depreciated.  Please use \code{\link{ll}}
    instead.}
  
  Display name, class, size, and dimensions of each element in a given
  object.
}
\usage{
elem(object=1, unit=c("KB","MB","bytes"), digits=0,
                 dimensions=FALSE)
}
\arguments{
  \item{object}{object containing named elements, perhaps a model or
    data frame.}
  \item{unit}{required unit for displaying element size: "KB", "MB",
    "bytes", or first letter.}
  \item{digits}{number of decimals to display when rounding element
    size.}
  \item{dimensions}{whether element dimensions should be returned.}
}
\details{
  A verbose alternative to \code{names()}.
}
\value{
  A data frame with named rows and the following columns:
  \item{Class}{element class.}
  \item{KB}{element size \emph{(see notes)}.}
  \item{Dim}{element dimensions \emph{(optional)}.}
}
\note{
  The name of the element size column is the same as the unit used.

  Elements of class \code{classRepresentation},
  \code{ClassUnionRepresentation}, and \code{grob} do not have a defined
  size, so 0 bytes are assumed for those.
}
\author{Arni Magnusson \email{arnima@u.washington.edu}}
\seealso{

  \code{\link[base]{names}}, \code{\link[utils]{str}}, and
  \code{\link[base]{summary}} display different information about object
  elements.

  \code{\link{ll}} and \code{\link{env}} are related to \code{elem}.
}
\examples{
\dontrun{
data(infert)
elem(infert)
model <- glm(case~spontaneous+induced, family=binomial, data=infert)
elem(model, dim=TRUE)
elem(model$family)
}
}
% Basics
\keyword{attribute}
\keyword{classes}
\keyword{list}
% Programming
\keyword{print}
\keyword{utilities}

\name{ans}
\alias{ans}
\title{Value of Last Evaluated Expression}
\usage{
ans()
}
\description{
  The functon returns the value of the last evaluated \emph{top-level}
  expression, which is always assigned to \code{.Last.value} (in
  \code{package:base}).
}
\details{
  This function retrieves \code{.Last.value}. For more details see 
  \code{\link[base]{.Last.value}}.  
}
\value{
  \code{.Last.value}
}
\seealso{
  \code{\link[base]{.Last.value}}, \code{\link[base]{eval}}
}
\author{Liviu Andronic}
\examples{
  2+2                  # Trivial calculation...
  ans()                # See the answer again

  gamma(1:15)          # Some intensive calculation...
  fac14 <- ans()       # store the results into a variable

  rnorm(20)            # Generate some standard normal values
  ans()^2              # Convert to Chi-square(1) values...
  stem(ans())          # Now show a stem-and-leaf table
}
\keyword{programming}

\name{installXLSXsupport}
\Rdversion{1.1}
\alias{installXLSXsupport}
\title{
Install perl modules needed for read.xls to support Excel 2007+ XLSX format
}
\description{
Install perl modules needed for read.xls to support Excel 2007+ XLSX format
}
\usage{
installXLSXsupport(perl = "perl", verbose = FALSE)
}
\arguments{
  \item{perl}{Path to perl interpreter (optional).}
  \item{verbose}{If \code{TRUE}, show additional messages during
    processing.}
}
\value{
  Either \code{TRUE} indicating that the necessary perl modules have
  been successfully installed, or \code{FALSE} indicating that an error
  has occured.
}
\details{

  This function calls the perl script 'install_modules.pl' located in
  the perl subdirectory of the gdata package directory (or inst/perl in
  the source package).  This perl script attempts to use the perl 'CPAN'
  package, which should be included as part of most perl installations,
  to automatically download, compile, and install the
  Compress::Raw::Zlib and Spreadsheet::XLSX perl modules needed for
  read.xls to support support Excel 2007+ XLSX files into the gdata
  perl subdirectory.

  Since the perl modules are installed into the gdata installation
  directory, the perl modules will be available until the gdata package
  is replaced or removed.  Since this occurs each time a new version of
  gdata is installed, \code{installXLSXsupport} will need to be run each
  time a new version of the gdata package is installed.

  Further, the binary Compress::Raw::Zlib package installed by
  \code{installXLSXsupport} is tied to the particular version of perl
  used to compile it, therefore, you will need to re-run
  \code{installXLSXsupport} if you install a different perl
  distribution.

  This installation process will fail if 1) perl is not available on the
  search path and the \code{perl} argument is not used to specify the
  path of the perl executable, 2) the perl installation is not properly
  configured for installing binary packages*, 3) if the CPAN module is
  not present, or 4) if the C compiler specified by the perl
  installation is   not present.

  \emph{In particular, \code{installXLSXsupport} will fail for the
  version of perl included with the current RTools.zip package, which is
  not correctly configured to allow installation of additional perl
  packages. (The RTools version of perl is installed in a different
  directory than the perl configuration files expect.)}

  The function \code{xlsFormats} can be used to see whether XLS and XLSX
  formats are supported by the currently available perl modules.
  }
\seealso{
	\code{\link{read.xls}}, \code{\link{xls2csv}}, \code{\link{xlsFormats}}
}
\examples{
\dontrun{
   installXLSXsupport()
}
}
\keyword{ misc }

\name{ls.funs}
\alias{ls.funs}
\title{List function objects}
\description{
  Return a character vector giving the names of function objects in the
  specified environment.
}
\usage{
ls.funs(...)
}
\arguments{
  \item{\dots}{Arguments passed to \code{ls}. See the help for
    \code{\link[base]{ls}} for details.}
}
\details{
  This function calls \code{ls} and then returns a character vector
  containing only the names of only function objects.
}
\value{
  character vector
}
\author{
  Gregory R. Warnes \email{greg@warnes.net}
}
\seealso{
  \code{\link[base]{ls}}, \code{\link[base]{is.function}}
}
\examples{
## List functions defined in the global environment:
ls.funs()

## List functions available in the base package:
ls.funs("package:base")
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{misc}
\keyword{environment}

\name{trimSum}
\alias{trimSum}
\title{Trim a vector such that the last/first value represents the sum of
  trimmed values}
\description{
  Trim (shorten) a vector in such a way that
  the last or first value represents the sum of trimmed values. User needs
  to specify the desired length of a trimmed vector.
}
\usage{trimSum(x, n, right=TRUE, na.rm=FALSE, \dots)}
\arguments{
  \item{x}{numeric, a vector of numeric values}
  \item{n}{numeric, desired length of the output}
  \item{right}{logical, trim on the right/bottom  or the left/top  side}
  \item{na.rm}{logical, remove \code{NA} values when applying a function}
  \item{\dots}{arguments passed to other methods - currently not used}
}
\value{
  Trimmed vector with a last/first value representing the sum of
  trimmed values
}
\author{Gregor Gorjanc}
\seealso{\code{\link[gdata]{trim}}}
\examples{
x <- 1:10
trimSum(x, n=5)
trimSum(x, n=5, right=FALSE)

x[9] <- NA
trimSum(x, n=5)
trimSum(x, n=5, na.rm=TRUE)
}
\keyword{manip}

% runRUnitTests.Rd
%--------------------------------------------------------------------------
% What: Run RUnit tests - help
% $Id$
% Time-stamp: <2008-12-30 20:58:26 ggorjan>
%--------------------------------------------------------------------------

\name{.runRUnitTestsGdata}
\alias{.runRUnitTestsGdata}

\title{Run RUnit tests for the gdata package}

\description{

Run \pkg{RUnit} tests to perform the validity/unit testing of installed
\pkg{gdata} package on your own computer.

}

\usage{.runRUnitTestsGdata(testFileRegexp="^runit.+\\\\.[rR]$")}

\arguments{
  \item{testFileRegexp}{regular expression; see details}
}

\details{

Argument \code{testFileRegexp} can be used to specify different sets of
tests provided by the package. The following values are sensible:
\itemize{
  \item \code{"^runit.+\\\\.[rR]$"} for basic tests
}

}

\value{

None, just the print out of \pkg{RUnit} testing.

}

\seealso{
  \code{\link[RUnit]{defineTestSuite}} in \pkg{RUnit} package
}

\examples{
  ## Basic testing
  .runRUnitTestsGdata()
}

\keyword{misc}

%--------------------------------------------------------------------------
% runRUnitTests.Rd ends here

\name{read.xls}
\alias{read.xls}
\alias{xls2csv}
\alias{xls2tab}
\alias{xls2tsv}
\alias{xls2sep}
\title{Read Excel files}
\description{Read a Microsoft Excel file into a data frame}
\usage{
read.xls(xls, sheet=1, verbose=FALSE, pattern, na.strings=c("NA","#DIV/0!"),
         ..., method=c("csv","tsv","tab"), perl="perl")
xls2csv(xls, sheet=1, verbose=FALSE, blank.lines.skip=TRUE, ..., perl="perl")
xls2tab(xls, sheet=1, verbose=FALSE, blank.lines.skip=TRUE, ..., perl="perl")
xls2tsv(xls, sheet=1, verbose=FALSE, blank.lines.skip=TRUE, ..., perl="perl")
xls2sep(xls, sheet=1, verbose=FALSE, blank.lines.skip=TRUE, ...,
        method=c("csv","tsv","tab"), perl="perl")
}
\arguments{
  \item{xls}{path to the Microsoft Excel file.  Supports "http://",
    "https://", and "ftp://" URLS.}
  \item{sheet}{name or number of the worksheet to read}
  \item{verbose}{logical flag indicating whether details should be
    printed as the file is processed.}
  \item{pattern}{if specified, them skip all lines before the first 
    containing this string}
  \item{perl}{name of the perl executable to be called.}
  \item{method}{intermediate file format, "csv" for comma-separated and
    "tab" for tab-separated}
  \item{na.strings}{a character vector of strings which are to be interpreted
          as 'NA' values. See \code{\link[utils]{read.table}} for
	  details.}
  \item{blank.lines.skip}{logical flag indicating whether blank lines in
    the orginal file should be ignored.}
  \item{...}{additional arguments to read.table. The defaults for
    read.csv() are used.}
}
\value{
  \code{"read.xls"} returns a data frame.

  \code{"xls2sep"} returns a temporary file in the specified format.  
  \code{"xls2csv"} and \code{"xls2tab"} are simply wrappers for
  \code{"xls2sep"} specifying method as "csv" or "tab", respectively.
}
\details{
  This function works translating the named Microsoft Excel file into a
  temporary .csv or .tab file, using the xls2csv or xls2tab
  Perl script installed as part of this (gdata) package.

  Caution: In the conversion to csv, strings will be quoted. This can be
    problem if you are trying to use the \code{comment.char} option of
  \code{read.table} since the first character of all lines (including
  comment lines) will be "\"" after conversion.

  If you have quotes in your data which confuse the process you may wish to use 
  \code{read.xls(..., quote = '')}.  This will cause the quotes to be regarded
  as data and you will have to then handle the quotes yourself after reading 
  the file in.

  Caution: If you call \code{"xls2csv"} directly, is your responsibility 
  to close and delete the file after using it.
}
\references{http://www.analytics.washington.edu/statcomp/downloads/xls2csv}
\note{ Either a working version of Perl must be present in the executable
  search path, or the exact path of the perl executable must be provided
  via the \code{perl} argument.  See the examples below for an illustration.}
\seealso{ \code{\link[utils]{read.csv}} }
\examples{

   # iris.xls is included in the gregmisc package for use as an example
   xlsfile <- file.path(path.package('gdata'),'xls','iris.xls')
   xlsfile

   iris <- read.xls(xlsfile) # defaults to csv format
   iris <- read.xls(xlsfile,method="csv") # specify csv format
   iris <- read.xls(xlsfile,method="tab") # specify tab format

   head(iris)  # look at the top few rows

   \dontshow{
   iris.1 <- read.xls(xlsfile) # defaults to csv format
   iris.2 <- read.xls(xlsfile,method="csv") # specify csv format
   iris.3 <- read.xls(xlsfile,method="tab") # specify tab format

   stopifnot(all.equal(iris.1, iris.2))
   stopifnot(all.equal(iris.1, iris.3))
   }


  \dontrun{
   ## Example specifying exact Perl path for default MS-Windows install of
   ## ActiveState perl
   iris <- read.xls(xlsfile, perl="C:/perl/bin/perl.exe")
   }

   \dontrun{
   ## Example specifying exact Perl path for Unix systems
   iris <- read.xls(xlsfile, perl="/usr/bin/perl")

   ## finding perl 
   ## (read.xls automatically calls findPerl so this is rarely needed)
   perl <- gdata:::findPerl("perl")
   iris <- read.xls(xlsfile, perl=perl)
   }

   \dontrun{
   ## read xls file from net
   nba.url <- "http://mgtclass.mgt.unm.edu/Bose/Excel/Tutorial.05/Cases/NBA.xls"
   nba <- read.xls(nba.url)
   }

   \dontrun{
   ## read xls file ignoring all lines prior to first containing State
   crime.url <- "http://www.jrsainfo.org/jabg/state_data2/Tribal_Data00.xls"
   crime <- read.xls(crime.url, pattern = "State")

   ## use of xls2csv - open con, print two lines, close con
   con <- xls2csv(crime.url)
   print(readLines(con, 2)) 
   file.remove(summary(con)$description)
   }

   ## Examples demonstrating selection of specific 'sheets'
   ## from the example XLS file 'ExampleExcelFile.xls'
   exampleFile <- file.path(path.package('gdata'),'xls',
                            'ExampleExcelFile.xls')
   exampleFile2007 <- file.path(path.package('gdata'),'xls',
                            'ExampleExcelFile.xlsx')

   ## see the number and names of sheets:
   sheetCount(exampleFile)
   if( 'XLSX' \%in\% xlsFormats() )  ## if XLSX is supported..
     sheetCount(exampleFile2007)


   ## show names of shets in the file
   sheetNames(exampleFile)
   if( 'XLSX' \%in\% xlsFormats() )  ## if XLSX is supported..
   sheetNames(exampleFile2007)

   data <- read.xls(exampleFile)          ## default is first worksheet
   data <- read.xls(exampleFile, sheet=2) ## second worksheet by number
   data <- read.xls(exampleFile, sheet="Sheet Second",v=TRUE) ## and by name

   ## load the third worksheet, skipping the first two non-data lines...
   if( 'XLSX' \%in\% xlsFormats() )  ## if XLSX is supported..
     data <- read.xls(exampleFile2007, sheet="Sheet with initial text", skip=2)

   ## load a file containing data and column names using latin-1
   ## characters
   latinFile <- file.path(path.package('gdata'),'xls','latin-1.xls')
   latin1 <- read.xls(latinFile, fileEncoding="latin1")
   colnames(latin1)


}
\author{
  Gregory R. Warnes \email{greg@warnes.net},
  Jim Rogers \email{james.a.rogers@pfizer.com}, and
  Gabor Grothendiek \email{ggrothendieck@gmail.com}.
}
\keyword{file}

\name{gdata-defunct}
\alias{aggregate.table}
\title{Defunct Functions in Package 'gdata'}
\description{
  The functions or variables listed here are no longer part of 'gdata'.
}
\usage{
aggregate.table(x, by1, by2, FUN=mean, \dots)
}
\arguments{
  \item{x}{data to be summarized.}
  \item{by1}{first grouping factor.}
  \item{by2}{second grouping factor.}
  \item{FUN}{a scalar function to compute the summary statistics which can
    be applied to all data subsets. Defaults to \code{mean}.}
  \item{\dots}{optional arguments for \code{FUN}.}
}
\details{
  \code{aggregate.table(x, by1, by2, FUN=mean, \dots)} should be replaced
  by \code{tapply(X=x, INDEX=list(by1, by2), FUN=FUN, \dots)}.
}

\name{reorder.factor}
\alias{reorder.factor}
\title{Reorder the Levels of a Factor}
\description{
  Reorder the levels of a factor
}
\usage{
\method{reorder}{factor}(x, X, FUN, ..., order=is.ordered(x), new.order, sort=mixedsort)
}
\arguments{
  \item{x}{factor}
  \item{X}{auxillary data vector}
  \item{FUN}{function to be applied to subsets of \code{X} determined by
    \code{x}, to determine factor order}
  \item{...}{optional parameters to \code{FUN}}
  \item{order}{logical value indicating whether the returned
    object should be an \code{\link{ordered}} factor}
  \item{new.order}{a vector of indexes or a vector of label names giving
    the order of the new factor levels}
  \item{sort}{function to use to sort the factor level names, used only
    when \code{new.order} is missing}
}
\details{
  This function changes the order of the levels of a factor. It can do
  so via three different mechanisms, depending on whether, \code{X}
  \emph{and} \code{FUN}, \code{new.order} or \code{sort} are provided.

  If \code{X} \emph{and} \code{Fun} are provided: The data in \code{X}
  is grouped by the levels of \code{x} and \code{FUN} is applied.
  The groups are then sorted by this value, and the resulting order is
  used for the new factor level names.

  If \code{new.order} is a numeric vector, the new factor level names
  are constructed by reordering the factor levels according to the
  numeric values. If \code{new.order} is a chraccter vector,
  \code{new.order} gives the list of new factor level names. In either
  case levels omitted from \code{new.order} will become missing
  (\code{NA}) values.

  If \code{sort} is provided (as it is by default): The new factor level
  names are generated by calling the function specified by \code{sort}
  to the existing factor level \emph{names}. With \code{sort=mixedsort}
  (the default) the factor levels are sorted so that combined numeric
  and character strings are sorted in according to character rules on
  the character sections (including ignoring case), and the numeric
  rules for the numeric sections. See \code{\link[gtools]{mixedsort}}
  for details.
}
\value{ A new factor with reordered levels }

\author{Gregory R. Warnes \email{greg@warnes.net}}

\seealso{
  \code{\link{factor}}
  and
  \code{\link[stats]{reorder}}
}

\examples{

\dontshow{
    set.seed(123456)
}

   # Create a 4 level example factor
   trt <- factor( sample( c("PLACEBO", "300 MG", "600 MG", "1200 MG"),
                  100, replace=TRUE ) )
   summary(trt)
   # Note that the levels are not in a meaningful order.

   # Change the order to something useful..
   # - default "mixedsort" ordering
   trt2 <- reorder(trt)
   summary(trt2)
   # - using indexes:
   trt3 <- reorder(trt, new.order=c(4, 2, 3, 1))
   summary(trt3)
   # - using label names:
   trt4 <- reorder(trt, new.order=c("PLACEBO", "300 MG", "600 MG", "1200 MG"))
   summary(trt4)
   # - using frequency
   trt5 <- reorder(trt, X=rnorm(100), FUN=mean)
   summary(trt5)

   # Drop out the '300 MG' level
   trt6 <- reorder(trt, new.order=c("PLACEBO", "600 MG", "1200 MG"))
   summary(trt6)
}
\keyword{ manip }

\name{case}
\alias{case}
\title{Map elements of a vector according to the provided 'cases'}
\description{
  Map elements of a vector according to the provided 'cases'.  This
  function is useful for mapping discrete values to factor labels and 
  is the vector equivalent to the \code{switch} function.
}
\usage{
case(x, ..., default = NA)
}
\arguments{
  \item{x}{Vector to be converted}
  \item{\dots}{Map of alternatives, specified as "name"=value}
  \item{default}{Value to be assigned to elements of \code{x} not
    matching any of the alternatives. Defaults to \code{NA}.}
}
\details{
  This function is to \code{switch} what \code{ifelse} is to \code{if},
  and is a convenience wrapper for \code{factor}.
}
\value{
  A factor variables with each element of \code{x} mapped into the
  corresponding level of specified in the mapping.
}
\author{Gregory R. Warnes \email{greg@warnes.net}}
\seealso{\code{factor}, \code{switch}, \code{ifelse}}
\examples{
## default = NA
case( c(1,1,4,3), "a"=1, "b"=2, "c"=3)

## default = "foo"
case( c(1,1,4,3), "a"=1, "b"=2, "c"=3, default="foo" )


}
\keyword{ manip }

\name{sheetCount}
\Rdversion{1.1}
\alias{sheetCount}
\alias{sheetNames}
\title{
Count or list sheet names in Excel spreadsheet files.
}
\description{
Count or list sheet names in Excel spreadsheet files.
}
\usage{
sheetCount(xls, verbose = FALSE, perl = "perl")
sheetNames(xls, verbose = FALSE, perl = "perl")
}
\arguments{
  \item{xls}{File path to spreadsheet.  Supports "http://",
    "https://", and "ftp://" URLS.}
  \item{verbose}{If \code{TRUE}, show additional messages during
    processing.}
  \item{perl}{Path to perl interpreter.}
}
\value{
  \code{sheetCount} returns the number of sheets in the spreadsheet.
  \code{sheetNames} returns the names of sheets  in the spreadsheet.
}

\seealso{
	\code{\link{read.xls}}, \code{\link{xls2csv}}.
}
\examples{
   xlsfile <- system.file("xls", "iris.xls", package = "gdata")
   xlsfile

   sheetCount(xlsfile)


   exampleFile <- file.path(path.package('gdata'),'xls',
                            'ExampleExcelFile.xls')
   exampleFile2007 <- file.path(path.package('gdata'),'xls',
                            'ExampleExcelFile.xlsx')

   sheetCount(exampleFile)

   if( 'XLSX' \%in\% xlsFormats() )  # if XLSX is supported..
     sheetNames(exampleFile2007)


}
\keyword{ misc }

\name{is.what}
\alias{is.what}
\title{Run Multiple is.* Tests on a Given Object}
\description{
  Run multiple \code{is.*} tests on a given object: \code{is.na},
  \code{is.numeric}, and many others.
}
\usage{
is.what(object, verbose=FALSE)
}
\arguments{
  \item{object}{any \R object.}
  \item{verbose}{whether negative tests should be included in output.}
}
\value{
  A character vector containing positive tests, or when \code{verbose}
  is \code{TRUE}, a data frame showing all test results.
}
\author{Arni Magnusson, inspired by \code{demo(is.things)}.}
\note{
  The following procedure is used to look for valid tests:
  \enumerate{
    \item{}{Find all objects named \code{is.*} in all loaded
      environments.}
    \item{}{Discard objects that are not functions.}
    \item{}{Include test result only if it is of class \code{"logical"},
      not an \code{NA}, and of length 1.}
  }
}
\seealso{
  \code{\link{is.na}} and \code{\link{is.numeric}} are commonly used
  tests.
}
\examples{
is.what(pi)
is.what(NA, verbose=TRUE)
is.what(lm(1~1))
is.what(is.what)
}
% Basics
\keyword{classes}
\keyword{NA}
% Programming
\keyword{programming}
\keyword{error}
\keyword{utilities}

\name{ConvertMedUnits}
\alias{ConvertMedUnits}
\title{Convert medical measurements between International Standard (SI)
  and US 'Conventional' Units.}
\description{
  Convert Medical measurements between International Standard (SI)
  and US 'Conventional' Units.
}
\usage{
ConvertMedUnits(x, measurement, abbreviation,
                to = c("Conventional", "SI", "US"),
                exact = !missing(abbreviation))
}
\arguments{
  \item{x}{Vector of measurement values}
  \item{measurement}{Name of the measurement}
  \item{abbreviation}{Measurement abbreviation}
  \item{to}{Target units}
  \item{exact}{Logicial indicating whether matching should be exact}
}
\details{

  Medical laboratories and practitioners in the United States use one
  set of units (the so-called 'Conventional' units) for reporting the
  results of clinical laboratory measurements, while the rest of the
  world uses the International Standard (SI) units.  It often becomes
  necessary to translate between these units when participating in
  international collaborations.

  This function converts between SI and US 'Conventional' units.

  If \code{exact=FALSE}, \code{grep} will be used to do a
  case-insensitive sub-string search for matching measurement names.  If
  more than one match is found, an error will be generated, along with a
  list of the matching entries.
  
}
\value{
  Returns a vector of converted values.  The attribute 'units' will
  contain the target units converted.
}
\seealso{
  The data set \code{\link{MedUnits}} provides the conversion
  factors.
}
\references{
  \url{https://globalrph.com/medical/conventional-units-international-units/}
  }
\author{ Gregory R. Warnes \email{greg@warnes.net} }
\examples{
data(MedUnits)

# show available conversions
MedUnits$Measurement

# Convert SI Glucose measurement to 'Conventional' units
GlucoseSI = c(5, 5.4, 5, 5.1, 5.6, 5.1, 4.9, 5.2, 5.5) # in SI Units
GlucoseUS = ConvertMedUnits( GlucoseSI, "Glucose", to="US" )
cbind(GlucoseSI,GlucoseUS)

\dontrun{
# See what happens when there is more than one match
ConvertMedUnits( 27.5, "Creatin", to="US")
}

# To solve the problem do:
ConvertMedUnits( 27.5, "Creatinine", to="US", exact=TRUE)
}
\keyword{manip}

\name{getYear}
\alias{getDateTimeParts}
\alias{getYear}
\alias{getYear.default}
\alias{getYear.Date}
\alias{getYear.POSIXct}
\alias{getYear.POSIXlt}
\alias{getMonth}
\alias{getMonth.default}
\alias{getMonth.Date}
\alias{getMonth.POSIXct}
\alias{getMonth.POSIXlt}
\alias{getDay}
\alias{getDay.default}
\alias{getDay.Date}
\alias{getDay.POSIXct}
\alias{getDay.POSIXlt}
\alias{getHour}
\alias{getHour.default}
\alias{getMin}
\alias{getMin.default}
\alias{getSec}
\alias{getSec.default}
\title{Get date/time parts from date and time objects}
\description{
  Experimental approach for
  extracting the date/time parts from objects of a date/time class.
  They are designed to be intiutive and thus lowering the learning
  curve for work with date and time classes in \R{}.}
\usage{
getYear(x, format, \dots)
getMonth(x, format, \dots)
getDay(x, format, \dots)
getHour(x, format, \dots)
getMin(x, format, \dots)
getSec(x, format, \dots)
}
\arguments{
  \item{x}{generic, date/time object}
  \item{format}{character, format}
  \item{\dots}{arguments pased to other methods}
}
\value{Character}
\author{Gregor Gorjanc}
\seealso{
  \code{\link{Date}},
  \code{\link{DateTimeClasses}},
  \code{\link{strptime}}
}
\examples{
## Date
tmp <- Sys.Date()
tmp
getYear(tmp)
getMonth(tmp)
getDay(tmp)

## POSIXct
tmp <- as.POSIXct(tmp)
getYear(tmp)
getMonth(tmp)
getDay(tmp)

## POSIXlt
tmp <- as.POSIXlt(tmp)
getYear(tmp)
getMonth(tmp)
getDay(tmp)
}
\keyword{manip}
\keyword{misc}

\name{rename.vars}
\alias{rename.vars}
\alias{remove.vars}
\title{Remove or rename variables in a data frame}
\description{
  Remove or rename a variables in a data frame.
}
\usage{
rename.vars(data, from="", to="", info=TRUE)
remove.vars(data, names="", info=TRUE)
}
\arguments{
  \item{data}{data frame to be modified.}
  \item{from}{character vector containing the current name of each
    variable to be renamed.}
  \item{to}{character vector containing the new name of each variable
    to be renamed.}
  \item{names}{character vector containing the names of variables to be
    removed.}
  \item{info}{boolean value indicating whether to print details of the
    removal/renaming. Defaults to TRUE.}
}
\value{
  The updated data frame with variables listed in \code{from} renamed to
  the corresponding element of \code{to}.
}
\author{Code by Don MacQueen \email{macq@llnl.gov}.
  Documentation by Gregory R. Warnes \email{greg@warnes.net}.}
\seealso{
  \code{\link{names}}, \code{\link{colnames}}, \code{\link{data.frame}}
}
\examples{
data <- data.frame(x=1:10,y=1:10,z=1:10)
names(data)
data <- rename.vars(data, c("x","y","z"), c("first","second","third"))
names(data)

data <- remove.vars(data, "second")
names(data)
}
\keyword{manip}

\name{cbindX}
\alias{cbindX}
\title{Column-bind objects with different number of rows}
\description{
  \code{cbindX} column-binds objects with different number of rows.
}
\usage{cbindX(\dots)}
\arguments{
  \item{\dots}{matrix and data.frame objects}
}
\details{
  First the object with maximal number of rows is found. Other objects
  that have less rows get (via \code{\link{rbind}}) additional rows with
  \code{NA} values.  Finally, all objects are column-binded (via
  \code{\link{cbind}}).
}
\value{See details.}
\author{Gregor Gorjanc}
\seealso{Regular \code{\link{cbind}} and \code{\link{rbind}}}
\examples{
df1 <- data.frame(a=1:3, b=c("A", "B", "C"))
df2 <- data.frame(c=as.character(1:5), a=5:1)

ma1 <- matrix(as.character(1:4), nrow=2, ncol=2)
ma2 <- matrix(1:6, nrow=3, ncol=2)

cbindX(df1, df2)
cbindX(ma1, ma2)
cbindX(df1, ma1)
cbindX(df1, df2, ma1, ma2)
cbindX(ma1, ma2, df1, df2)
}
\keyword{misc}

\name{left}
\alias{right}
\alias{left}
\alias{right.data.frame}
\alias{left.data.frame}
\alias{right.matrix}
\alias{left.matrix}
\title{Return the leftmost or rightmost columns of a matrix or data frame}
\description{
  Return the leftmost or rightmost or  columns of a matrix or data frame
}
\usage{
right(x, n = 6L, \dots)
left(x, n=6L, \dots)

\method{right}{matrix}(x, n=6L, add.col.nums=TRUE, \dots)
\method{left}{matrix}(x, n=6L, add.col.nums=TRUE, \dots)

\method{right}{data.frame}(x, n=6L, add.col.nums=TRUE, \dots)
\method{left}{data.frame}(x, n=6L, add.col.nums=TRUE, \dots)
}
\arguments{
  \item{x}{Matrix or data frame}
  \item{n}{If positive, number of columns to return. If negative, number
    of columns to omit. See examples.}
  \item{add.col.nums}{Logical. If no column names are present,
    add names giving original column number. (See example below.)}
  \item{\dots}{Additional arguments used by methods}
}
\value{
  An object consisting of the leftmost or rightmost  \code{n} columns
  of \code{x}.
}
\author{
  Gregory R. Warnes \email{greg@warnes.net}
}
\seealso{
  \code{\link{first}},
  \code{\link{last}},
  \code{\link[utils]{head}},
  \code{\link[utils]{tail}}
}
\examples{
m <- matrix(1:100, ncol=10)
colnames(m) <- paste("Col",1:10, sep="_")

left(m)
right(m)

# When no column names are present, they are added by default
colnames(m) <- NULL

left(m)
colnames(left(m))

right(m)
colnames(right(m))

# Prevent addition of column numbers
left(m, add.col.nums = FALSE)
colnames(left(m, add.col.nums = FALSE))

right(m, add.col.nums = FALSE)            # columns are labeled 1:6
colnames(right(m, add.col.nums = FALSE))  #   instead of 5:10

# Works for data frames too!
d <- data.frame(m)
left(d)
right(d)

# Use negative n to specify number of columns to omit
left(d, -3)
right(d, -3)
}
\keyword{manip}

% $Id: aggregate.table.Rd 1605 2012-09-12 17:39:42Z warnes $
%
% $Log$
% Revision 1.7  2005/09/12 15:42:45  nj7w
% Updated Greg's email
%
% Revision 1.6  2005/06/09 14:20:25  nj7w
% Updating the version number, and various help files to synchronize splitting of gregmisc bundle in 4 individual components.
%
% Revision 1.1.1.1  2005/05/25 22:07:33  nj7w
% Initial entry for individual package gdata
%
% Revision 1.5  2003/11/17 22:09:00  warnes
% Fix syntax error.
%
% Revision 1.4  2003/06/07 17:58:37  warnes
%
% - Fixed error in examples.  Had sqrt(var(x)/(n-1)) for the standard
%   error of the mean instead of sqrt(var(x)/n).
%
% Revision 1.3  2002/09/23 13:59:30  warnes
% - Modified all files to include CVS Id and Log tags.
%
%

\name{aggregate.table}
\alias{aggregate.table}
\title{Create 2-Way Table of Summary Statistics}
\description{
  Splits the data into subsets based on two factors, computes a summary
  statistic on each subset, and arranges the results in a 2-way table.
}
\usage{
aggregate.table(x, by1, by2, FUN=mean, ...)
}
%- maybe also `usage' for other objects documented here.
\arguments{
  \item{x}{ data to be summarized }
  \item{by1}{ first grouping factor.  }
  \item{by2}{ second grouping factor. }
  \item{FUN}{ a scalar function to compute the summary statistics which can
          be applied to all data subsets. Defaults to \code{mean}.}
  \item{\dots}{ Optional arguments for \code{FUN}. }
}
%\details{
%  ~~ If necessary, more details than the __description__  above ~~
%}
\value{
  Returns a matrix with one element for each combination of \code{by1}
  and \code{by2}.
}
\author{ Gregory R. Warnes \email{greg@warnes.net}}

\seealso{ \code{\link{aggregate}}, \code{\link{tapply}},
          \code{\link{interleave}} }
\note{This function is DEPRECIATED.  Please use \code{tapply} 
  instead. See example for illustration.}
\examples{
# Useful example:
#
# Create a 2-way table of means, standard errors, and # obs
set.seed(314159)
g1 <- sample(letters[1:5], 1000, replace=TRUE)
g2 <- sample(LETTERS[1:3], 1000, replace=TRUE )
dat <- rnorm(1000)

stderr <- function(x) sqrt( var(x,na.rm=TRUE) / nobs(x) )

## Depreciated:
means   <- aggregate.table( dat, g1, g2, mean )
## Instead use:
means   <- tapply( dat, list(g1, g2), mean )

## Depreciated
stderrs <- aggregate.table( dat, g1, g2, stderr )
## Instead use:
stderrs <- tapply( dat, list(g1, g2), stderr )

## Depreciated
ns      <- aggregate.table( dat, g1, g2, nobs )
## Instead use:
ns      <- tapply( dat, list(g1, g2), nobs )

blanks <- matrix( " ", nrow=5, ncol=3)

tab <- interleave( "Mean"=round(means,2),
                   "Std Err"=round(stderrs,2),
                   "N"=ns, " " = blanks, sep=" " )

print(tab, quote=FALSE)
}
\keyword{iteration}
\keyword{category}

\name{upperTriangle}
\alias{upperTriangle}
\alias{upperTriangle<-}
\alias{lowerTriangle}
\alias{lowerTriangle<-}
\title{Extract or replace the upper/lower triangular portion of a matrix}
\description{
  Extract or replace the upper/lower triangular portion of a matrix
}
\usage{
upperTriangle(x, diag=FALSE, byrow=FALSE)
upperTriangle(x, diag=FALSE, byrow=FALSE) <- value
lowerTriangle(x, diag=FALSE, byrow=FALSE)
lowerTriangle(x, diag=FALSE, byrow=FALSE) <- value
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{x}{Matrix}
  \item{diag}{Logical.  If \code{TRUE}, include the matrix diagonal.}
  \item{byrow}{Logical.  If \code{FALSE}, return/replace elements in 
    column-wise order. If \code{TRUE}, return/replace elements in 
    row-wise order.}
  \item{value}{Either a single value or a vector of length equal to that
    of the current upper/lower triangular.  Should be of a mode which 
    can be coerced to that of \code{x}.}
}

\value{
  \code{upperTriangle(x)} and \code{lowerTriangle(x)} return the upper
  or lower triangle of matrix x, respectively. The assignment forms
  replace the upper or lower triangular area of the
  matrix with the provided value(s).  
  }
\note{
  By default, the elements are returned/replaced in R's default column-wise order.  Thus 
  \preformatted{  lowerTriangle(x) <- upperTriangle(x)}
  will not yield a symmetric matrix.  Instead use:
  \preformatted{  lowerTriangle(x) <- upperTriangle(x, byrow=TRUE)} 
  or equivalently:
  \preformatted{  lowerTriangle(x, byrow=TRUE) <- upperTriangle(x)} 
}
  
\author{Gregory R. Warnes \email{greg@warnes.net}}
\seealso{ 
  \code{\link[base]{diag}},  
  \code{\link[base]{lower.tri}}, 
  \code{\link[base]{upper.tri}}
}
\examples{
  x <- matrix( 1:25, nrow=5, ncol=5)
  x
  upperTriangle(x)
  upperTriangle(x, diag=TRUE)
  upperTriangle(x, diag=TRUE, byrow=TRUE)


  lowerTriangle(x)
  lowerTriangle(x, diag=TRUE)
  lowerTriangle(x, diag=TRUE, byrow=TRUE)

  upperTriangle(x) <- NA
  x

  upperTriangle(x, diag=TRUE) <- 1:15
  x

  lowerTriangle(x) <- NA
  x

  lowerTriangle(x, diag=TRUE) <- 1:15
  x

  ## Copy lower triangle into upper triangle to make 
  ## the matrix (diagonally) symmetric
  x <- matrix(LETTERS[1:25], nrow=5, ncol=5, byrow=TRUE)
  x
  lowerTriangle(x) = upperTriangle(x, byrow=TRUE)
  x
}
\keyword{array}

\name{gdata-package}
\alias{gdata-package}
\alias{gdata}
\docType{package}
\title{Various R Programming Tools for Data Manipulation}
\description{
  Various R programming tools for data manipulation, including:
  \itemize{
    \item Medical unit conversions: \code{\link{ConvertMedUnits}},
      \code{\link{MedUnits}}
    \item Combining objects: \code{link{bindData}},
      \code{\link{cbindX}}, \code{\link{combine}},
      \code{\link{interleave}}
    \item Character vector operations: \code{\link{centerText}},
      \code{\link{startsWith}}, \code{\link{trim}}
    \item Factor manipulation: \code{\link{levels}},
      \code{\link{reorder.factor}}, \code{\link{mapLevels}}
    \item Obtaining information about R objects:
      \code{\link{object_size}}, \code{\link{env}},
      \code{\link{humanReadable}}, \code{\link{is.what}},
      \code{\link{ll}}, \code{\link{keep}}, \code{\link{ls.funs}},
      \code{\link{Args}}, \code{\link{nPairs}}, \code{\link{nobs}}
    \item Generating fixed-width format files: \code{\link{write.fwf}}
    \item Extracting components of date & time objects:
      \code{\link{getYear}}, \code{\link{getMonth}},
      \code{\link{getDay}}, \code{\link{getHour}}, \code{\link{getMin}},
      \code{\link{getSec}}
    \item Operations on columns of data frames: \code{\link{matchcols}},
      \code{\link{rename.vars}}
    \item Matrix operations: \code{\link{unmatrix}},
      \code{\link{upperTriangle}}, \code{\link{lowerTriangle}}
    \item Operations on vectors: \code{\link{case}},
      \code{\link{unknownToNA}}, \code{\link{duplicated2}},
      \code{\link{trimSum}}
    \item Operations on data frames: \code{\link{frameApply}},
      \code{\link{wideByFactor}}
    \item Value of last evaluated expression: \code{\link{ans}}
    \item Wrapper for \code{sample} that ensures consistent behavior for
      both scalar and vector arguments: \code{\link{resample}}
  }
}
\note{\code{browseVignettes()} shows package vignettes.}
\author{
  Gregory R. Warnes, Gregor Gorjanc, Arni Magnusson, Liviu Andronic, Jim
  Rogers, Don MacQueen, and Ales Korosec, with contributions by Ben
  Bolker, Michael Chirico, Gabor Grothendieck, Thomas Lumley, and Brian
  Ripley.
}
\keyword{package}

% unknown.Rd
%--------------------------------------------------------------------------
% What: Change given unknown value to NA and vice versa man page
% $Id: unknown.Rd 1357 2009-08-20 14:54:44Z warnes $
% Time-stamp: <2007-08-17 20:18:54 ggorjan>
%--------------------------------------------------------------------------

\name{unknownToNA}

\alias{isUnknown}
\alias{isUnknown.default}
\alias{isUnknown.POSIXlt}
\alias{isUnknown.list}
\alias{isUnknown.data.frame}
\alias{isUnknown.matrix}

\alias{unknownToNA}
\alias{unknownToNA.default}
\alias{unknownToNA.factor}
\alias{unknownToNA.list}
\alias{unknownToNA.data.frame}

\alias{NAToUnknown}
\alias{NAToUnknown.default}
\alias{NAToUnknown.factor}
\alias{NAToUnknown.list}
\alias{NAToUnknown.data.frame}

\concept{missing}

\title{Change unknown values to NA and vice versa}

\description{

Unknown or missing values (\code{NA} in \R) can be represented in
various ways (as 0, 999, etc.) in different programs. \code{isUnknown},
\code{unknownToNA}, and \code{NAToUnknown} can help to change unknown
values to \code{NA} and vice versa.

}

\usage{

isUnknown(x, unknown=NA, \dots)
unknownToNA(x, unknown, warning=FALSE, \dots)
NAToUnknown(x, unknown, force=FALSE, call.=FALSE, \dots)

}

\arguments{
  \item{x}{generic, object with unknown value(s)}
  \item{unknown}{generic, value used instead of \code{NA}}
  \item{warning}{logical, issue warning if \code{x} already has \code{NA}}
  \item{force}{logical, force to apply already existing value in \code{x}}
  \item{\dots}{arguments pased to other methods (as.character for POSIXlt
    in case of isUnknown)}
  \item{call.}{logical, look in \code{\link{warning}}}
}

\details{

This functions were written to handle different variants of
\dQuote{other \code{NA}} like representations that are usually used in
various external data sources. \code{unknownToNA} can help to change
unknown values to \code{NA} for work in \R, while \code{NAToUnknown} is
meant for the opposite and would usually be used prior to export of data
from \R. \code{isUnknown} is utility function for testing for unknown
values.

All functions are generic and the following classes were tested to work
with latest version: \dQuote{integer}, \dQuote{numeric},
\dQuote{character}, \dQuote{factor}, \dQuote{Date}, \dQuote{POSIXct},
\dQuote{POSIXlt}, \dQuote{list}, \dQuote{data.frame} and
\dQuote{matrix}. For others default method might work just fine.

\code{unknownToNA} and \code{isUnknown} can cope with multiple values in
\code{unknown}, but those should be given as a \dQuote{vector}. If not,
coercing to vector is applied. Argument \code{unknown} can be feed also
with \dQuote{list} in \dQuote{list} and \dQuote{data.frame} methods.

If named \dQuote{list} or \dQuote{vector} is passed to argument
\code{unknown} and \code{x} is also named, matching of names will occur.

Recycling occurs in all \dQuote{list} and \dQuote{data.frame} methods,
when \code{unknown} argument is not of the same length as \code{x} and
\code{unknown} is not named.

Argument \code{unknown} in \code{NAToUnknown} should hold value that is
not already present in \code{x}. If it does, error is produced and one
can bypass that with \code{force=TRUE}, but be warned that there is no
way to distinguish values after this action. Use at your own risk!
Anyway, warning is issued about new value in \code{x}. Additionally,
caution should be taken when using \code{NAToUnknown} on factors as
additional level (value of \code{unknown}) is introduced. Then, as
expected, \code{unknownToNA} removes defined level in \code{unknown}. If
\code{unknown="NA"}, then \code{"NA"} is removed from factor levels in
\code{unknownToNA} due to consistency with conversions back and forth.

Unknown representation in \code{unknown} should have the same class as
\code{x} in \code{NAToUnknown}, except in factors, where \code{unknown}
value is coerced to character anyway. Silent coercing is also applied,
when \dQuote{integer} and \dQuote{numeric} are in question. Otherwise
warning is issued and coercing is tried. If that fails, \R introduces
\code{NA} and the goal of \code{NAToUnknown} is not reached.

\code{NAToUnknown} accepts only single value in \code{unknown} if
\code{x} is atomic, while \dQuote{list} and \dQuote{data.frame} methods
accept also \dQuote{vector} and \dQuote{list}.

\dQuote{list/data.frame} methods can work on many components/columns. To
reduce the number of needed specifications in \code{unknown} argument,
default unknown value can be specified with component ".default". This
matches component/column ".default" as well as all other undefined
components/columns! Look in examples.

}

\value{

\code{unknownToNA} and \code{NAToUnknown} return modified
\code{x}. \code{isUnknown} returns logical values for object \code{x}.

}

\author{Gregor Gorjanc}

\seealso{\code{\link{is.na}}}

\examples{

xInt <- c(0, 1, 0, 5, 6, 7, 8, 9, NA)
isUnknown(x=xInt, unknown=0)
isUnknown(x=xInt, unknown=c(0, NA))
(xInt <- unknownToNA(x=xInt, unknown=0))
(xInt <- NAToUnknown(x=xInt, unknown=0))

xFac <- factor(c("0", 1, 2, 3, NA, "NA"))
isUnknown(x=xFac, unknown=0)
isUnknown(x=xFac, unknown=c(0, NA))
isUnknown(x=xFac, unknown=c(0, "NA"))
isUnknown(x=xFac, unknown=c(0, "NA", NA))
(xFac <- unknownToNA(x=xFac, unknown="NA"))
(xFac <- NAToUnknown(x=xFac, unknown="NA"))

xList <- list(xFac=xFac, xInt=xInt)
isUnknown(xList, unknown=c("NA", 0))
isUnknown(xList, unknown=list("NA", 0))
tmp <- c(0, "NA")
names(tmp) <- c(".default", "xFac")
isUnknown(xList, unknown=tmp)
tmp <- list(.default=0, xFac="NA")
isUnknown(xList, unknown=tmp)

(xList <- unknownToNA(xList, unknown=tmp))
(xList <- NAToUnknown(xList, unknown=999))

}

\keyword{manip}
\keyword{NA}

%--------------------------------------------------------------------------
% unknown.Rd ends here

% $Id: drop.levels.Rd 1096 2007-06-06 10:19:15Z ggorjan $

\name{drop.levels}
\alias{drop.levels}
\title{Drop unused factor levels}
\description{Drop unused levels in a factor}
\usage{
drop.levels(x, reorder=TRUE, ...)
}
\arguments{
  \item{x}{object to be processed}
  \item{reorder}{should factor levels be reordered using
    \code{\link{reorder.factor}}?}
  \item{...}{additional arguments to \code{\link{reorder.factor}}}
}

\details{

\code{drop.levels} is a generic function, where default method does
nothing, while method for factor \code{s} drops all unused levels. Drop
is done with \code{x[, drop=TRUE]}.

There are also convenient methods for \code{list} and \code{data.frame},
where all unused levels are dropped in all factors (one by one) in a
\code{list} or a \code{data.frame}.

}

\value{Input object without unused levels.}

\author{Jim Rogers \email{james.a.rogers@pfizer.com} and Gregor Gorjanc}

\examples{

f <- factor(c("A", "B", "C", "D"))[1:3]
drop.levels(f)

l <- list(f=f, i=1:3, c=c("A", "B", "D"))
drop.levels(l)

df <- as.data.frame(l)
str(df)
str(drop.levels(df))

}

\keyword{manip}

% $Id%
%
\name{frameApply}
\alias{frameApply}
\title{Subset analysis on data frames}
\description{Apply a function to row subsets of a data frame. 
}
\usage{
frameApply(x, by=NULL, on=by[1], fun=function(xi) c(Count=nrow(xi)), 
           subset=TRUE, simplify=TRUE, byvar.sep="\\\\$\\\\@\\\\$", ...)
}
\arguments{
  \item{x}{a data frame}
  \item{by}{names of columns in \code{x} specifying the variables to use
    to form the subgroups. 
    None of the \code{by} variables should have
    the name "sep" (you will get an error if one of them does; a bit of
    laziness in the code). Unused levels of 
    the \code{by} variables will be dropped. Use \code{by = NULL} (the
    default) to indicate that all of the data is to be treated as a
    single (trivial) subgroup.}
  \item{on}{names of columns in \code{x} specifying columns over which
    \code{fun} is to be applied. These can include columns specified in
    \code{by}, (as with the default) although that is not usually the case.}
  \item{fun}{a function that can operate on data frames that are row
    subsets of \code{x[on]}. If \code{simplify = TRUE},
    the return value of the function should always be either a try-error
    (see \code{\link{try}}), or a vector of
    fixed length (i.e. same length for every subset), preferably with
    named elements.}
  \item{subset}{logical vector (can be specified in terms of variables
    in data). This row subset of \code{x} is taken before doing anything
    else.}
  \item{simplify}{logical. If TRUE (the default), return value will
    be a data frame including the \code{by} columns and a column for
    each element of the return vector of \code{fun}. If FALSE, the
    return value will be a list, sometimes necessary for less structured
    output (see description of return value below).}
  \item{byvar.sep}{character. This can be any character string not
    found anywhere in the values of the \code{by} variables. The
    \code{by} variables will be pasted together using this as the
    separator, and the result will be used as the index to form the
    subgroups.  }
  \item{...}{additional arguments to \code{fun}.}
}
\value{a data frame if \code{simplify = TRUE} (the default), assuming
  there is sufficiently structured output from \code{fun}. If
  \code{simplify = FALSE} and \code{by} is not NULL, the return value will be a list with two
  elements. The first element, named "by", will be a data frame with the
  unique rows of \code{x[by]}, and the second element, named "result"
  will be a list where the ith 
  component gives the result for the ith row of the "by" element.  
}
\details{This function accomplishes something similar to
  \code{\link{by}}. The main difference is that \code{frameApply} is
  designed to return data frames and lists instead of objects of class
  'by'. Also, \code{frameApply} works only on the unique combinations of
  the \code{by} that are actually present in the data, not on the entire
  cartesian product of the \code{by} variables. In some cases this
  results in great gains in efficiency, although \code{frameApply} is
  hardly an efficient function.}

\examples{
data(ELISA, package="gtools")

# Default is slightly unintuitive, but commonly useful: 
frameApply(ELISA, by = c("PlateDay", "Read"))

# Wouldn't actually recommend this model! Just a demo:
frameApply(ELISA, on = c("Signal", "Concentration"), by = c("PlateDay", "Read"),
           fun = function(dat) coef(lm(Signal ~ Concentration, data =
dat)))

frameApply(ELISA, on = "Signal", by = "Concentration",
           fun = function(dat, ...) {
                    x <- dat[[1]]
                    out <- c(Mean = mean(x, ...),
                             SD = sd(x, ...),
                             N = sum(!is.na(x)))
                  },
           na.rm = TRUE,
           subset = !is.na(Concentration))
}
\author{Jim Rogers \email{james.a.rogers@pfizer.com}}
\keyword{manip}

\name{trim}
\alias{trim}
\title{Remove leading and trailing spaces from character strings}
\description{
 Remove leading and trailing spaces from character strings and other
 related objects.
}
\usage{
trim(s, recode.factor=TRUE, \dots)
}
\arguments{
  \item{s}{object to be processed}
  \item{recode.factor}{should levels of a factor be recoded, see below}
  \item{\dots}{arguments passed to other methods, currently only to
    \code{\link{reorder.factor}} for factors}
}

\details{

\code{trim} is a generic function, where default method does nothing,
while method for character \code{s} trims its elements and method for
factor \code{s} trims \code{\link{levels}}. There are also methods for
\code{list} and \code{data.frame}.

Trimming character strings can change the sort order in some locales.
For factors, this can affect the coding of levels. By default, factor
levels are recoded to match the trimmed sort order, but this can be
disabled by setting \code{recode.factor=FALSE}. Recoding is done with
\code{\link{reorder.factor}}.

}
\value{
  \code{s} with all leading and trailing spaces removed in its elements.
}
\author{Gregory R. Warnes \email{greg@warnes.net}  with
  contributions by Gregor Gorjanc}
\seealso{ \code{\link[base]{trimws}}, \code{\link[base]{sub}},\code{\link[base]{gsub}}
as well as argument \code{strip.white} in \code{\link{read.table}} and
  \code{\link{reorder.factor}}
}
\examples{
s <- "    this is an example string    "
trim(s)

f <- factor(c(s, s, " A", " B ", "  C ", "D "))
levels(f)

trim(f)
levels(trim(f))

trim(f, recode.factor=FALSE)
levels(trim(f, recode.factor=FALSE))

l <- list(s=rep(s, times=6), f=f, i=1:6)
trim(l)

df <- as.data.frame(l)
trim(df)
}
\keyword{manip}
\keyword{character}

\name{mapLevels}
\alias{mapLevels}
\alias{mapLevels.default}
\alias{mapLevels.factor}
\alias{mapLevels.character}
\alias{mapLevels.list}
\alias{mapLevels.data.frame}
\alias{print.levelsMap}
\alias{print.listLevelsMap}
\alias{is.levelsMap}
\alias{is.listLevelsMap}
\alias{as.levelsMap}
\alias{as.listLevelsMap}
\alias{.checkLevelsMap}
\alias{.checkListLevelsMap}
\alias{"[.levelsMap"}
\alias{"[.listLevelsMap"}
\alias{c.levelsMap}
\alias{c.listLevelsMap}
\alias{unique.levelsMap}
\alias{sort.levelsMap}
\alias{mapLevels<-}
\alias{mapLevels<-.default}
\alias{mapLevels<-.factor}
\alias{mapLevels<-.character}
\alias{mapLevels<-.list}
\alias{mapLevels<-.data.frame}
\title{Mapping levels}
\description{
  \code{mapLevels} produces a map with information on levels and/or
  internal integer codes. As such can be conveniently used to store level
  mapping when one needs to work with internal codes of a factor and later
  transfrorm back to factor or when working with several factors that
  should have the same levels and therefore the same internal coding.
}
\usage{
mapLevels(x, codes=TRUE, sort=TRUE, drop=FALSE, combine=FALSE, \dots)
mapLevels(x) <- value
}
\arguments{
  \item{x}{object whose levels will be mapped, look into details}
  \item{codes}{boolean, create integer levelsMap (with internal
    codes) or character levelsMap (with level names)}
  \item{sort}{boolean, sort levels of character \code{x}, look into
    details}
  \item{drop}{boolean, drop unused levels}
  \item{combine}{boolean, combine levels, look into details}
  \item{\dots}{additional arguments for \code{sort}}
  \item{value}{levelsMap or listLevelsMap, output of \code{mapLevels}
    methods or constructed by user, look into details}
}
\section{mapLevels}{
  The \code{mapLevels} function was written primarly for work with
  \dQuote{factors}, but is generic and can also be used with
  \dQuote{character}, \dQuote{list} and \dQuote{data.frame}, while
  \dQuote{default} method produces error. Here the term levels is also
  used for unique character values.

  When \code{codes=TRUE} \bold{integer \dQuote{levelsMap}} with
  information on mapping internal codes with levels is produced. Output
  can be used to transform integer to factor or remap factor levels as
  described below. With \code{codes=FALSE} \bold{character
    \dQuote{levelsMap}} is produced. The later is usefull, when one would
  like to remap factors or combine factors with some overlap in levels as
  described in \code{mapLevels<-} section and shown in examples.

  \code{sort} argument provides possibility to sort levels of
  \dQuote{character} \code{x} and has no effect when \code{x} is a
  \dQuote{factor}.

  Argument \code{combine} has effect only in \dQuote{list} and
  \dQuote{data.frame} methods and when \code{codes=FALSE} i.e. with
  \bold{character \dQuote{levelsMaps}}. The later condition is necesarry
  as it is not possible to combine maps with different mapping of level
  names and integer codes. It is assumed that passed \dQuote{list} and
  \dQuote{data.frame} have all components for which methods
  exist. Otherwise an error is produced.
}
\section{levelsMap and listLevelsMap}{
  Function \code{mapLevels} returns a map of levels. This map is of class
  \dQuote{levelsMap}, which is actually a list of length equal to number
  of levels and with each component of length 1. Components need not be of
  length 1. There can be either integer or character
  \dQuote{levelsMap}. \bold{Integer \dQuote{levelsMap}} (when
  \code{codes=TRUE}) has names equal to levels and components equal to
  internal codes. \bold{Character \dQuote{levelsMap}} (when
  \code{codes=FALSE}) has names and components equal to levels. When
  \code{mapLevels} is applied to \dQuote{list} or \dQuote{data.frame},
  result is of class \dQuote{listLevelsMap}, which is a list of
  \dQuote{levelsMap} components described previously. If
  \code{combine=TRUE}, result is a \dQuote{levelsMap} with all levels in
  \code{x} components.

  For ease of inspection, print methods unlists \dQuote{levelsMap} with
  proper names. \code{mapLevels<-} methods are fairly general and
  therefore additional convenience methods are implemented to ease the
  work with maps: \code{is.levelsMap} and \code{is.listLevelsMap};
  \code{as.levelsMap} and \code{as.listLevelsMap} for coercion of user
  defined maps; generic \code{"["} and \code{c} for both classes (argument
  \code{recursive} can be used in \code{c} to coerce
  \dQuote{listLevelsMap} to \dQuote{levelsMap}) and generic \code{unique}
  and \code{sort} (generic from \R 2.4) for \dQuote{levelsMap}.
}
\section{mapLevels<-}{
  Workhorse under \code{mapLevels<-} methods is
  \code{\link{levels<-}}. \code{mapLevels<-} just control the assignment
  of \dQuote{levelsMap} (integer or character) or \dQuote{listLevelsMap}
  to \code{x}. The idea is that map values are changed to map names as
  indicated in \code{\link{levels}} examples. \bold{Integer
    \dQuote{levelsMap}} can be applied to \dQuote{integer} or
  \dQuote{factor}, while \bold{character \dQuote{levelsMap}} can be
  applied to \dQuote{character} or \dQuote{factor}. Methods for
  \dQuote{list} and \dQuote{data.frame} can work only on mentioned atomic
  components/columns and can accept either \dQuote{levelsMap} or
  \dQuote{listLevelsMap}. Recycling occurs, if length of \code{value} is not
  the same as number of components/columns of a \dQuote{list/data.frame}.
}
\value{
  \code{mapLevels()} returns \dQuote{levelsMap} or \dQuote{listLevelsMap}
  objects as described in levelsMap and listLevelsMap section.

  Result of \code{mapLevels<-} is always a factor with remapped levels or
  a \dQuote{list/data.frame} with remapped factors.
}
\author{Gregor Gorjanc}
\seealso{
  \code{\link{factor}}, \code{\link{levels}} and \code{\link{unclass}}
}
\examples{
## Integer levelsMap
(f <- factor(sample(letters, size=20, replace=TRUE)))
(mapInt <- mapLevels(f))

## Integer to factor
(int <- as.integer(f))
(mapLevels(int) <- mapInt)
all.equal(int, f)

## Remap levels of a factor
(fac <- factor(as.integer(f)))
(mapLevels(fac) <- mapInt) # the same as levels(fac) <- mapInt
all.equal(fac, f)

## Character levelsMap
f1 <- factor(letters[1:10])
f2 <- factor(letters[5:14])

## Internal codes are the same, but levels are not
as.integer(f1)
as.integer(f2)

## Get character levelsMaps and combine them
mapCha1 <- mapLevels(f1, codes=FALSE)
mapCha2 <- mapLevels(f2, codes=FALSE)
(mapCha <- c(mapCha1, mapCha2))

## Remap factors
mapLevels(f1) <- mapCha # the same as levels(f1) <- mapCha
mapLevels(f2) <- mapCha # the same as levels(f2) <- mapCha

## Internal codes are now "consistent" among factors
as.integer(f1)
as.integer(f2)

## Remap characters to get factors
f1 <- as.character(f1); f2 <- as.character(f2)
mapLevels(f1) <- mapCha
mapLevels(f2) <- mapCha

## Internal codes are now "consistent" among factors
as.integer(f1)
as.integer(f2)
}
\keyword{misc}
\keyword{manip}

\name{first}
\alias{first}
\alias{last}
\alias{first<-}
\alias{last<-}
\title{Return first or last element of an object}
\description{
  Return first or last element of an object.  These functions are convenience
  wrappers for \code{head(x, n=1, ...)} and \code{tail(x, n=1, ...)}.
  }
\usage{
first(x, n=1, ...)
last(x, n=1, ...)
first(x, n=1, ...) <- value
last(x, n=1, ...) <- value
}
\arguments{
  \item{x}{data object}
  \item{n}{a single integer. If positive, size for the resulting object:
          number of elements for a vector (including lists), rows for a
          matrix or data frame or lines for a function. If negative,
          all but the 'n' last/first number of elements of 'x'.}
   \item{...}{arguments to be passed to or from other methods.}
   \item{value}{a vector of values to be assigned (should be of length \code{n})}
}
\value{
  An object (usually) like 'x' but generally smaller.
}
\author{
  Gregory R. Warnes \email{greg@warnes.net}
}
\seealso{
  \code{\link[utils]{head}},
  \code{\link[utils]{tail}},
  \code{\link{left}},
  \code{\link{right}}
}
\examples{
## works for vectors..
v <- 1:10
first(v)
last(v)

first(v) <- 9
v

last(v) <- 20
v

## and for lists
l <- list(a=1, b=2, c=3)
first(l)
last(l)

first(l) <- "apple"
last(l)  <- "bannana"
l

## and data.frames
df <- data.frame(a=1:2, b=3:4, c=5:6)
first(df)
last(df)

first(df) <- factor(c("red","green"))
last(df) <- list(c(20,30)) # note the enclosing list!
df

## and matrixes
m <- as.matrix(df)
first(m)
last(m)

first(m) <- "z"
last(m)  <- "q"
m
}
\keyword{ manip }

\name{update.list}
\alias{update.list}
\alias{update.data.frame}
\title{Update the elements of a list, or rows of a data frame}
\description{
  For a list, update the elements of a list to contain all of the named elements
  of a new list, overwriting elements with the same name, and (optionally)
  copying unnamed elements. For a data frame, replace the rows of a data frame
  by corresponding rows in 'new' with the same value for 'by'.
}
\usage{
\method{update}{list}(object, new, unnamed=FALSE, \dots)
\method{update}{data.frame}(object, new, by, by.x=by, by.y=by,
       append=TRUE, verbose=FALSE, \dots)
}
\arguments{
  \item{object}{List or data frame to be updated.}
  \item{new}{List or data frame containing new elements/rows.}
  \item{unnamed}{Logical. If \code{TRUE}, unnamed list elements of \code{new}
    will be appended to \code{object}.}
  \item{by, by.x, by.y}{Character. Name of column to use for matching
    data frame rows.}
  \item{append}{Logical. If \code{TRUE}, items in \code{new} with no match
    in \code{object} will be appended to the data frame.}
  \item{verbose}{Logical. If \code{TRUE} progress messages will be displayed.}
  \item{\dots}{optional method arguments (ignored).}
}
\note{
  These methods can be called directly or as via the S3 base method for
  \code{update}.
}
\value{
  \item{update.list}{a list a constructed from the elements of \code{object},
    with named elements of \code{new} replacing corresponding named elements
    from \code{object}, and non-corresponding elements of \code{new} appended.
    If \code{unnamed=TRUE}, unnamed elements of \code{new} will be appended.}
  \item{update.data.frame}{a data frame constructed from the rows of
    \code{object} with rows where values in \code{by.x} equal the values
    in \code{by.y} replaced by the corresponding row in \code{new}. If
    \code{append=TRUE}, any elements of \code{new} without no matching
    rows in \code{object} will be appended.}
}
\author{Gregory R. Warnes \email{greg@warnes.net}}
\seealso{
  \code{\link[stats]{update}}, \code{\link[base]{merge}}
}
\examples{
# Update list
old <- list(a=1,b="red",c=1.37)
new <- list(b="green",c=2.4)

update(old, new)
update.list(old,new)  # equivalent

older <- list(a=0, b="orange", 4, 5, 6)
newer <- list(b="purple", 7, 8, 9)
update(older, newer)               # ignores unnamed elements of newer
update(older, newer, unnamed=TRUE) # appends unnamed elements of newer

# Update data frame
old <- data.frame(letter=letters[1:5], number=1:5)
new <- data.frame(letter=letters[c(5, 1, 7)], number=c(-5, -1, -7))

update(old, new, by="letter") # default is append=TRUE
update(old, new, by="letter", append=FALSE)
update(old, new, by="letter", verbose=FALSE)
}
\keyword{data}
\keyword{manip}

\name{ll}
\alias{ll}
\title{Describe Objects or Elements}
\description{
  Display name, class, size, and dimensions of each object in a given
  environment. Alternatively, if the main argument is a list-like
  object, its elements are listed and described.
}
\usage{
ll(pos=1, unit="KB", digits=0, dim=FALSE, sort=FALSE, class=NULL,
   invert=FALSE, ...)
}
\arguments{
  \item{pos}{environment position number, environment name, data frame,
    list, model, or any object that \code{is.list}.}
  \item{unit}{unit for displaying object size: "bytes", "KB", "MB", or
    first letter.}
  \item{digits}{number of decimals to display when rounding object
    size.}
  \item{dim}{whether object dimensions should be returned.}
  \item{sort}{whether elements should be sorted by name.}
  \item{class}{character vector for limiting the output to specified
    classes.}
  \item{invert}{whether to invert the \code{class} filter, so specified
    classes are excluded.}
  \item{...}{passed to \code{ls}.}
}
\value{
  A data frame with named rows and the following columns:
  \item{Class}{object class.}
  \item{KB}{object size \emph{(see note)}.}
  \item{Dim}{object dimensions \emph{(optional)}.}
}
\note{The name of the object size column is the same as the unit used.}
\author{Arni Magnusson, with a contribution by Jim Rogers}
\seealso{
  \code{ll} is a verbose alternative to \code{\link{ls}} (objects in an
  environment) and \code{\link{names}} (elements in a list-like object).

  \code{\link{str}} and \code{\link{summary}} also describe elements in
  a list-like objects.

  \code{\link{env}} is a related function that describes all loaded
  environments.
}
\examples{
ll()
ll(all=TRUE)
ll("package:base")
ll("package:base", class="function", invert=TRUE)

data(infert)
ll(infert)
model <- glm(case~spontaneous+induced, family=binomial, data=infert)
ll(model, dim=TRUE)
ll(model, sort=TRUE)
ll(model$family)
}
% Basics
\keyword{data}
\keyword{attribute}
\keyword{classes}
\keyword{list}
% Programming
\keyword{environment}
\keyword{print}
\keyword{utilities}

\name{humanReadable}

\alias{humanReadable}

\title{Print Byte Size in Human Readable Format}

\description{

\code{humanReadable} converts integer byte sizes to a human readable units such as
kB, MB, GB, etc.

}

\usage{
humanReadable(x, units="auto", standard=c("IEC", "SI", "Unix"),
              digits=1, width=NULL, sep=" ", justify=c("right", "left") )
}

\arguments{
  \item{x}{integer, byte size}
  \item{standard}{character, "IEC" for powers of 1024 ('MiB'), "SI" for
    powers of 1000 ('MB'), or "Unix" for powers of 1024 ('M').  See
    details.}
  \item{units}{character, unit to use for all values (optional), one of
    "auto", "bytes", or an appropriate unit corresponding to
    \code{standard}.}
  \item{digits}{integer, number of digits after decimal point}
  \item{width}{integer, width of number string}
  \item{sep}{character, separator between number and unit}
  \item{justify}{two-element vector specifiy the alignment for the number
    and unit components of the size.  Each element should be one of
    "none", "left", "right", or "center"}
}

\details{

The basic unit used to store information in computers is a bit. Bits are
represented as zeroes and ones - binary number system. Although, the binary
number system is not the same as the decimal number system, decimal prefixes
for binary multiples such as kilo and mega are often used. In the decimal system
kilo represent 1000, which is close to \eqn{1024 = 2^{10}} in the binary system.
This sometimes causes problems as it is not clear which powers (2 or 10) are used
in a notation like 1 kB. To overcome this problem International Electrotechnical
Commission (IEC) has provided the following solution to this problem:

\tabular{lrcll}{
Name      \tab System  \tab Symbol \tab Size             \tab Conversion    \cr
byte      \tab binary  \tab B      \tab \eqn{2^3}        \tab    8 bits     \cr
kilobyte  \tab decimal \tab kB     \tab \eqn{10^3}       \tab 1000 bytes    \cr
kibibyte  \tab binary  \tab KiB    \tab \eqn{2^{10}}     \tab 1024 bytes    \cr
megabyte  \tab decimal \tab MB     \tab \eqn{(10^3)^2}   \tab 1000 kilobytes\cr
mebibyte  \tab binary  \tab MiB    \tab \eqn{(2^{10})^2} \tab 1024 kibibytes\cr
gigabyte  \tab decimal \tab GB     \tab \eqn{(10^3)^3}   \tab 1000 megabytes\cr
gibibyte  \tab binary  \tab GiB    \tab \eqn{(2^{10})^3} \tab 1024 mebibytes\cr
terabyte  \tab decimal \tab TB     \tab \eqn{(10^3)^4}   \tab 1000 gigabytes\cr
tebibyte  \tab binary  \tab TiB    \tab \eqn{(2^{10})^4} \tab 1024 gibibytes\cr
petabyte  \tab decimal \tab PB     \tab \eqn{(10^3)^5}   \tab 1000 terabytes\cr
pebibyte  \tab binary  \tab PiB    \tab \eqn{(2^{10})^5} \tab 1024 tebibytes\cr
exabyte   \tab decimal \tab EB     \tab \eqn{(10^3)^6}   \tab 1000 petabytes\cr
exbibyte  \tab binary  \tab EiB    \tab \eqn{(2^{10})^6} \tab 1024 pebibytes\cr
zettabyte \tab decimal \tab ZB     \tab \eqn{(10^3)^7}   \tab 1000 exabytes\cr
zebibyte  \tab binary  \tab ZiB    \tab \eqn{(2^{10})^7} \tab 1024 exbibytes\cr
yottabyte \tab decimal \tab YB     \tab \eqn{(10^3)^8}   \tab 1000 zettabytes\cr
yebibyte  \tab binary  \tab YiB    \tab \eqn{(2^{10})^8} \tab 1024 zebibytes\cr
}

where Zi and Yi are GNU extensions to IEC. To get the output in the decimal
system (powers of 1000) use \code{standard="SI"}. To obtain IEC standard
(powers of 1024) use \code{standard="IEC"}.

In addition, single-character units are provided that follow (and
extend) the Unix pattern (use \code{standard="Unix"}):

\tabular{lrcll}{
Name      \tab System  \tab Symbol \tab Size             \tab Conversion    \cr
byte      \tab binary  \tab B      \tab \eqn{2^3}        \tab    8 bits     \cr
kibibyte  \tab binary  \tab K      \tab \eqn{2^{10}}     \tab 1024 bytes    \cr
mebibyte  \tab binary  \tab M      \tab \eqn{(2^{10})^2} \tab 1024 kibibytes\cr
gibibyte  \tab binary  \tab G      \tab \eqn{(2^{10})^3} \tab 1024 mebibytes\cr
tebibyte  \tab binary  \tab T      \tab \eqn{(2^{10})^4} \tab 1024 gibibytes\cr
pebibyte  \tab binary  \tab P      \tab \eqn{(2^{10})^5} \tab 1024 tebibytes\cr
exbibyte  \tab binary  \tab E      \tab \eqn{(2^{10})^6} \tab 1024 pebibytes\cr
zebibyte  \tab binary  \tab Z      \tab \eqn{(2^{10})^7} \tab 1024 exbibytes\cr
yottabyte \tab binary  \tab Y      \tab \eqn{(2^{10})^8} \tab 1024 zebibytes\cr
}

For printout both \code{digits} and \code{width} can be specified. If
\code{width} is \code{NULL}, all values have given number of digits. If
\code{width} is not \code{NULL}, output is rounded to a given width and
formated similar to human readable format of the Unix \code{ls},
\code{df} or \code{du} shell commands.

}

\references{

Wikipedia:
\url{https://en.wikipedia.org/wiki/Byte}
\url{https://en.wikipedia.org/wiki/SI_prefix}
\url{https://en.wikipedia.org/wiki/Binary_prefix}

GNU manual for coreutils:
\url{http://www.gnu.org/software/coreutils/manual/html_node/Block-size.html#Block-size}

}

\value{

Byte size in human readable format as character with proper unit symbols
added at the end of the string.

}

\author{Ales Korosec, Gregor Gorjanc, and Gregory R. Warnes
  \email{greg@warnes.net}}


\seealso{
  \code{\link{object.size}} in package 'gdata',
  \code{\link[utils]{object.size}} in package 'utils',
  \code{\link[gdata]{ll}}
}

\examples{

# Simple example: maximum addressible size of 32 bit pointer
humanReadable(2^32-1)
humanReadable(2^32-1, standard="IEC")
humanReadable(2^32-1, standard="SI")
humanReadable(2^32-1, standard="Unix")

humanReadable(2^32-1, unit="MiB")
humanReadable(2^32-1, standard="IEC",  unit="MiB")
humanReadable(2^32-1, standard="SI",   unit="MB")
humanReadable(2^32-1, standard="Unix", unit="M")

# Vector of sizes
matrix(humanReadable(c(60810, 124141, 124, 13412513), width=4))
matrix(humanReadable(c(60810, 124141, 124, 13412513), width=4, unit="KiB"))

# Specify digits rather than width
matrix(humanReadable(c(60810, 124141, 124, 13412513), width=NULL, digits=2))

# Change the justification
matrix(humanReadable(c(60810, 124141, 124, 13412513), width=NULL,
                    justify=c("right", "right") ))

}

\keyword{misc}

%--------------------------------------------------------------------------
% humanReadable.Rd ends here

% Come material taken from src/library/utils/man/object.size.Rd
% Part of the R package, http://www.R-project.org
% Copyright 1995-2007 R Core Development Team
% Distributed under GPL 2 or later

\name{object.size}
\alias{object.size}
\alias{c.object_sizes}
\alias{as.object_sizes}
\alias{is.object_sizes}
\alias{format.object_sizes}
\alias{print.object_sizes}

\title{Report the Space Allocated for Objects}
\description{
  Provides an estimate of the memory that is being used to store \R objects.
}
\usage{
object.size(\dots)

\method{is}{object_sizes}(x)

\method{as}{object_sizes}(x)

\method{c}{object_sizes}(\dots, recursive=FALSE)

\method{format}{object_sizes}(x, humanReadable=getOption("humanReadable"), standard="IEC", units,
       digits=1, width=NULL, sep=" ", justify = c("right", "left"),
\dots)

\method{print}{object_sizes}(x, quote=FALSE, humanReadable=getOption("humanReadable"),
      standard="IEC", units, digits=1, width=NULL, sep=" ",
      justify = c("right", "left"), \dots)

}
\arguments{
  \item{\dots}{\code{object.size}: \R objects;
    \code{print} and \code{format}: arguments to be passed to other
    methods.
  }
  \item{x}{output from \code{object.size}}
  \item{quote}{logical, indicating whether or not the result should be
    printed with surrounding quotes.}
  \item{humanReadable}{logical, use the \dQuote{human readable} format.}
  \item{standard,units,digits,width,sep,justify}{See the man page for
    \code{\link{humanReadable}}.
   }
   \item{recursive}{See the man page for \code{\link[base]{c}}. }
}
\details{

  \emph{This is a modified copy of the man page for utils::object.size in R
     2.2.1.}

  Exactly which parts of the memory allocation should be attributed
  to which object is not clear-cut.  This function merely provides a
  rough indication: it should be reasonably accurate for atomic
  vectors, but does not detect if elements of a list are shared, for
  example.  (Sharing amongst elements of a character vector is taken
  into account, but not that between character vectors in a single
  object.)

  The calculation is of the size of the object, and excludes the
  space needed to store its name in the symbol table.

  Associated space (e.g., the environment of a function and what the
  pointer in a \code{EXTPTRSXP} points to) is not included in the
  calculation.

  Object sizes are larger on 64-bit builds than 32-bit ones, but
  will very likely be the same on different platforms with the same
  word length and pointer size.

  % Modificitaion start
  \emph{Changes}

  Class of returned object is \code{c("object_sizes", "numeric")} with
  appropriate \code{print} and \code{c} methods.

  By default \code{object.size} outputs size in bytes, but human
  readable format similar to \code{ls}, \code{df} or \code{du} shell
  commands can be displayed by calling \code{humanReadable} directly,
  calling \code{print} with the argument \code{humanReadable=TRUE}, or
  by setting \code{options(humanReadable=TRUE)}.

  % Modificitaion end

}
\value{
  A numeric vector class \code{c("object_sizes", "numeric")} containing
  estimated memory allocation attributable to the objects in bytes.
}
\seealso{
  \code{\link[utils]{object.size}} in package 'utils' for the standard
  version of this function,
  \code{\link{Memory-limits}} for the design limitations on object size,
  \code{\link{humanReadable}} for human readable format.
}

\examples{
object.size(letters)
object.size(ls)
## find the 10 largest objects in the base package
allObj <- sapply(ls("package:base"),
            function(x)
                object.size(get(x, envir = baseenv()))
            )

( bigObj <- as.object_sizes(rev(sort(allObj))[1:10] ) )
print(bigObj, humanReadable=TRUE)


as.object_sizes(14567567)

\dontshow{
    optionsOrig <- options("humanReadable")
}

options(humanReadable=TRUE)
(
    z <- object.size(letters,
                     c(letters, letters),
                     rep(letters, 100),
                     rep(letters, 10000)
                     )
)
is.object_sizes(z)
as.object_sizes(14567567)

\dontshow{
    options(optionsOrig)
}

}
\keyword{utilities}
